 /**********************************************************************
  *                                                                    *
  * Copyright (c) 2002 Andreas Pfeiffer, CERN/IT                       *
  *                    <andreas.pfeiffer@cern.ch>                      *
  *                                                                    *
  * This library is free software; you can redistribute it and/or      *
  * modify it under the terms of the GNU Lesser General Public         *
  * License as published by the Free Software Foundation; either       *
  * version 2.1 of the License, or (at your option) any later version. *
  *                                                                    *
  * This library is distributed in the hope that it will be useful,    *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of     *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
  * Lesser General Public License for more details.                    *
  *                                                                    *
  * You should have received a copy of the GNU Lesser General Public   *
  * License along with this library (see file COPYING); if not, write  *
  * to the Free Software Foundation, Inc., 59 Temple Place, Suite      *
  * 330, Boston, MA 02111-1307 USA, or contact the author              *
  **********************************************************************/


#ifndef INCLUDED_AIDAANNOTATION_H
#define INCLUDED_AIDAANNOTATION_H 1


// Include files
#include <map>
#include <vector>
#include <string>
#include <iostream>

#include "Interfaces/IAnnotation.h"

namespace Anaphe {

  // Forward declarations

  class AIDAAnnotation : virtual public IAnnotation {

  public:
    AIDAAnnotation()  { /* nop */ }
    ~AIDAAnnotation() { /* nop */ }

    // let the compiler deal with copy-constructor and assignment ...

  public:
    /// add a key/value pair with a given visibility 
    bool   add(std::string key, std::string value, bool _visibility = true);

    /// Retrieve the value for a given key
    std::string find(std::string key) const;

    /// Retrieve the visibility flag for a given key
    bool    visibilityFlag (std::string key) const;

    virtual void setVisibility(std::string key, bool value);

    /// For "iterative" access to the objects ... 
    /// in future releases we might have an iterator ...

    /// Get the number of items in the Annotation
    int size() const;
    /// Idividual access to the Annotation-items
    /// This probably should be replaced by a call which 
    /// returns a AnnotationItem object ...
    std::string key   (int index) const;
    std::string value (int index) const;
    bool    visibilityFlag (int index) const;

    /// Write to and read from Ascii stream
    std::ostream& toAscii   (std::ostream &os = std::cout) const;
    std::istream& fromAscii (std::istream &is = std::cin);

    /// Reset contents
    void reset();

  private:
    std::map<std::string, std::string > annotations;

    std::map<std::string, bool > visibility;

    std::vector<std::string>        keyOrder;
  };

  std::ostream & operator << (std::ostream &os, const AIDAAnnotation& ann);
  std::istream & operator >> (std::istream &is,       AIDAAnnotation& ann);


} //end namespace Anaphe

#endif    // INCLUDED_AIDAANNOTATION_H
