 /**********************************************************************
  *                                                                    *
  * Copyright (c) 2002 Andreas Pfeiffer, CERN/IT                       *
  *                    <andreas.pfeiffer@cern.ch>                      *
  *                                                                    *
  * This library is free software; you can redistribute it and/or      *
  * modify it under the terms of the GNU Lesser General Public         *
  * License as published by the Free Software Foundation; either       *
  * version 2.1 of the License, or (at your option) any later version. *
  *                                                                    *
  * This library is distributed in the hope that it will be useful,    *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of     *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
  * Lesser General Public License for more details.                    *
  *                                                                    *
  * You should have received a copy of the GNU Lesser General Public   *
  * License along with this library (see file COPYING); if not, write  *
  * to the Free Software Foundation, Inc., 59 Temple Place, Suite      *
  * 330, Boston, MA 02111-1307 USA, or contact the author              *
  **********************************************************************/


// 
// implementation file for class SimpleTokenizer 
// 
// Created by: Andreas PFEIFFER
// 
// Last update: 
// 

#ifndef INCLUDED_SIMPLETOKENIZER_H
#include "SimpleTokenizerCopy.h"
#endif

using Anaphe::SimpleTokenizer;

SimpleTokenizer::~SimpleTokenizer() 
{
}
SimpleTokenizer::SimpleTokenizer(String delim, String term) : delimiters(delim), terminators(term) 
{
}

SimpleTokenizer::SimpleTokenizer(const SimpleTokenizer &) 
{
}

SimpleTokenizer & SimpleTokenizer::operator = (const SimpleTokenizer &rhs) 
{
   if (this == &rhs) return *this;  // time saving self-test

// clean up old object, prepare new, copy over ...
   return *this;
}

std::vector<std::string> SimpleTokenizer::tokenize(String input) const 
{
  std::vector<std::string> tokens;

  int endOfSearch = input.size(); // pre-set
  if (terminators.size() != 0) {	  // iterate to find last pos to check
    String::size_type pos = input.find_first_of(terminators);
    if (pos != String::npos) endOfSearch = pos;
  }
  
  String newInput = input.substr(0, endOfSearch);

  String::size_type startPos = 0;
  String::size_type newPos   = 0;
  while (startPos != String::npos) {
    newPos = newInput.find_first_of(delimiters, startPos);
    if (newPos != String::npos) {
      tokens.push_back( newInput.substr( startPos, newPos-startPos ) );
      newPos++;
    } else {			// scanned up to terminator
	tokens.push_back( newInput.substr( startPos, endOfSearch-startPos ) );
    }
    startPos = newPos;
  }

  // I'm sacrificing efficiency for clarity here ... but since this class is 
  // basically only used in the context of interactive work, speed should
  // not be a critical issue.

  std::vector<std::string> newTokens;
  std::vector<std::string>::iterator i;
  i = tokens.begin();
  while(i != tokens.end()) { // clear out empty (i.e. only-space) tokens
    if (! ( (i->find_first_not_of(" ") == String::npos) || ((*i).size() == 0) ) ) {
      newTokens.push_back(*i);	// add to new vector
    }
    i++;
  }

#ifdef DEBUG_PRINT
  i = newTokens.begin();
  cerr << "found " << newTokens.size() << " newTokens " << endl;
  while(i != newTokens.end()) {
    cerr << "token = '" << *i << "' size:" << (*i).size() << endl;
    i++;
  }
#endif
  return newTokens;
}
