 /**********************************************************************
  *                                                                    *
  * Copyright (c) 2002 Andreas Pfeiffer, CERN/IT                       *
  *                    <andreas.pfeiffer@cern.ch>                      *
  *                                                                    *
  * This library is free software; you can redistribute it and/or      *
  * modify it under the terms of the GNU Lesser General Public         *
  * License as published by the Free Software Foundation; either       *
  * version 2.1 of the License, or (at your option) any later version. *
  *                                                                    *
  * This library is distributed in the hope that it will be useful,    *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of     *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
  * Lesser General Public License for more details.                    *
  *                                                                    *
  * You should have received a copy of the GNU Lesser General Public   *
  * License along with this library (see file COPYING); if not, write  *
  * to the Free Software Foundation, Inc., 59 Temple Place, Suite      *
  * 330, Boston, MA 02111-1307 USA, or contact the author              *
  **********************************************************************/


// 
// header file for class SimpleTokenizer 
// Created by: Andreas PFEIFFER  at Thu Apr 15 17:30:48 1999
// 
// Last update: 
//              
// 

#ifndef INCLUDED_SIMPLETOKENIZER_H
#define INCLUDED_SIMPLETOKENIZER_H

#include <string>
#include <vector>

typedef std::string String;

/**
   Class to tokenize ans STL string. 
   This class allows the user to tokenize (i.e. split in separate tokens)
   an STL string.
   The user can specify the set of delimiters as well as the terminator 
   character. The class splits the original string in a vector of tokens.
*/

namespace Anaphe {

  class SimpleTokenizer  { 

  public: 
    /// Constructor. Users can specify delimiters and terminator character.
    SimpleTokenizer(String delim=" ", String term="\n\0");
    /// Destructor
    virtual ~SimpleTokenizer(); 
  private: 
    // inhibit use of these
    SimpleTokenizer(const SimpleTokenizer &); 
    SimpleTokenizer & operator = (const SimpleTokenizer &); 
  public: 
    /// Redefine the set of delimiters
    void setDelimiters (String str) { delimiters  = str; }
    /// Redefine the set of terminators
    void setTerminators(String str) { terminators = str; }
    /// Split the string according to current delimiters/terminators
    std::vector<String > tokenize(String) const;
  private:		// private attributes
    String delimiters;
    String terminators;
  }; 

} //end namespace Anaphe

#endif /* INCLUDED_SIMPLETOKENIZER_H */ 
