// 
// Copyright (C) CERN, Geneva 
// 
// header file for class IHistogramFactory 
// Created by: Andreas Pfeiffer  at Thu Jan 13 16:17:26 2000
// 
// Last update: 
//              
// 
#ifndef INTERFACE_IHISTOGRAMFACTORY_H
#define INTERFACE_IHISTOGRAMFACTORY_H 1

#include <string>

namespace Anaphe {

  class IHistogram;
  class IHistogram1D;
  class IHistogram2D;

  //-ap start
  //
  // Q: what about the "openness" of the bins (rightOpen vs. leftOpen) ? 
  //    Should this be added to the creation methods (after/before ID) ?
  //
  //-ap end

  class IHistogramFactory  { 

  public: 
    virtual ~IHistogramFactory() { /* nop */; }

  public:
    //
    // Creation of histograms with known boundaries.
    // The name of the methods is simply "create*D", 
    // as a HistogramFactory should create only Histograms. :-)
    //
    virtual IHistogram1D * create1D( std::string title
				     , int nBinsx, double xmin, double xmax
				     , const int ID = 0) = 0;

    virtual IHistogram2D * create2D(std::string title
				    , int nBinsx, double xmin, double xmax
				    , int nBinsy, double ymin, double ymax
				    , const int ID = 0) = 0;

    /// remove the histogram from the list of active histos
    virtual void unregister(IHistogram * h) = 0;
    /// unregister() and delete histogram
    virtual void destroy(IHistogram * h) = 0;

  }; 

} //end namespace Anaphe

extern "C" {
  Anaphe::IHistogramFactory * createIHistogramFactory();
}

#endif /* INTERFACE_IHISTOFACTORY_H */ 
