// 
// Copyright (C) CERN, Geneva 
// 
// header file for class IHistogramManager 
// Created by: Andreas Pfeiffer  at Thu Sep  2 14:29:35 1999
// 
// Last update: 
//              
//  ap, 16-jan-01 : added createIHistogramManager() method, removed default arguments
// 
#ifndef INTERFACE_IHISTOGRAMMANAGER_H
#define INTERFACE_IHISTOGRAMMANAGER_H 1

#include <iostream>

namespace Anaphe {

  class IHistogram1D;
  class IHistogram2D;
  class IHistoFactory;

  class IHistogramManager { 

  public: 
    virtual ~IHistogramManager() { /* nop */; }

  public: 		// public methods
    virtual bool register1D(IHistogram1D *) = 0;
    virtual bool register2D(IHistogram2D *) = 0;

    virtual IHistogram1D * retrieveHisto1D(int ) = 0;
    virtual IHistogram2D * retrieveHisto2D(int ) = 0;
  
    virtual void deleteHisto(const char * label) = 0;

#ifdef AIDA_DONT_USE_STD
    virtual void list(     ostream& os =      cout) = 0;
#else
    virtual void list(std::ostream& os = std::cout) = 0;
#endif

    virtual void disableOverwrite() = 0;
    virtual void enableOverwrite () = 0;
    virtual void disableWarnOverwrite() = 0;
    virtual void enableWarnOverwrite () = 0;

  public: 		// public methods
    virtual void setFactory(IHistoFactory *f) = 0;

    virtual IHistogram1D * create1D(std::string title, int nBinsx, double xmin, double xmax, int ID) = 0;
    virtual IHistogram2D * create2D(std::string title
				    , int nBinsx, double xmin, double xmax
				    , int nBinsy, double ymin, double ymax, int ID) = 0;

  }; 

} //end namespace Anaphe

extern "C" {
  Anaphe::IHistogramManager * createIHistogramManager();
}

#endif /* INTERFACE_IHISTOGRAMMANAGER_H */ 
