// 
// Header file for IModelFunction interface
// Created by: Jakub T. Moscicki, Oct 2000
//
// Time-stamp: <2002-01-21 14:23:53 moscicki>
// 

# ifndef INTERFACE_IMODELFUNCTION_H__included__
# define INTERFACE_IMODELFUNCTION_H__included__ 1

# include "IFunction.h"
# include <string>

namespace Anaphe
{

/*************************************************************/
/**
 * Interface of model function for fitting.
 *  
 * It supports setting and retrieving of the (double) parameters.
 * PENDING: to be investigated -> it should also contain parameter 
 * constraint information 
 * (the valid range of parameter values, if applicable).
 */

class IModelFunction : virtual public IFunction
{
 public:
  /**
   * Set new vector of parameters.
   */
#ifdef AIDA_DONT_USE_STD
  virtual void setParameters(const      vector<double>& params) = 0;
#else
  virtual void setParameters(const std::vector<double>& params) = 0;
#endif

  /**
   * Retrieve current vector of parameters.
   */
#ifdef AIDA_DONT_USE_STD
  virtual const      vector<double>& parameters() const = 0;
#else
  virtual const std::vector<double>& parameters() const = 0;
#endif

  /**
   * Compute gradient with respect to parameters.
   */
#ifdef AIDA_DONT_USE_STD
  virtual const      vector<double>& parameterGradient(const      vector<double>& x) const = 0;
#else
  virtual const std::vector<double>& parameterGradient(const std::vector<double>& x) const = 0;
#endif

  /**
   * Return true if has parameter gradient.
   */
  virtual bool hasParameterGradient() const = 0;
  
  /**
   * Number of parameters.
   */
  virtual int numberOfParameters() const = 0;

  /**
   * Names of paramaters.
   * Names should be unique. Case-insensitive for comparison. 
   * Case-sensitive for storage and output.
   * All or several names may be left blank "" if the implementation
   * does not want to provide them.
   */
  virtual string nameOfParameter(int ix) const = 0;
};

} //  end namespace Anaphe

# endif
