// -*- C++ -*-

//
//  Copyright (C) CERN, Geneva
//
//  IRandomManager interface
//
//  Author: Max Sang 
//  Created: 16 May 2001
// 



#ifndef INTERFACE_IRANDOMMANAGER_H
#define INTERFACE_IRANDOMMANAGER_H 



// std includes...
#include <iostream>
#include <string>
#include <vector>



// ...and a kludge
#ifdef AIDA_STD
# undef AIDA_STD
#endif
#ifdef AIDA_NOT_USE_STD
# define AIDA_STD
#else
# define AIDA_STD std
#endif



class IRandomGenerator;



class IRandomManager
{
public:
  /// Produce new generator from one of (usually) several
  /// options such as Gaussian, exponential, etc.
  /// Non-default parameters can be passed and will
  /// be used if reasonable. 
  virtual IRandomGenerator* 
    newGenerator(const AIDA_STD::string& = "flat", 
		 const double& = 0.0,
		 const double& = 0.0,
		 const double& = 0.0 ) = 0;

  /// Produce a new generator from a vector of positive real
  /// numbers representing the function. Bad vectors are 
  /// ignored and a default (flat) generator is created instead.
  /// The lower and upper bounds of the vector may be specified
  /// if required (defaults 0 and 1, repectively)
  virtual IRandomGenerator* 
    newGenerator(const AIDA_STD::vector<double>&,
		 const AIDA_STD::string& = "",
		 const double& = 0.0,
		 const double& = 1.0 ) = 0;
  
  /// how many generators?
  virtual int nGenerators(void) const = 0;

  /// Retrieve a generator by ID. Passing a non-existent ID will
  /// create a new generator with default parameters and return
  /// a pointer to that.
  virtual IRandomGenerator* gen(const int& id) = 0;

  /// delete one generator by ID
  virtual void destroy(const int& victim) = 0;

  /// Delete all generators (all pointers to generators invalidated)
  virtual void destroyAll(void) = 0;

  /// Dump list of current generators
  virtual void listGenerators(AIDA_STD::ostream& = AIDA_STD::cout) const = 0;

public:
  virtual ~IRandomManager() { }

}; // End of class IRandomManager





/// pseudo-new function
extern "C" {
  IRandomManager* createIRandomManager(void);
}




#endif /* INTERFACE_IRANDOMMANAGER_H */ 

