 /**********************************************************************
  *                                                                    *
  * Copyright (c) 2002 Andreas Pfeiffer, CERN/IT                       *
  *                    <andreas.pfeiffer@cern.ch>                      *
  *                                                                    *
  * This library is free software; you can redistribute it and/or      *
  * modify it under the terms of the GNU Lesser General Public         *
  * License as published by the Free Software Foundation; either       *
  * version 2.1 of the License, or (at your option) any later version. *
  *                                                                    *
  * This library is distributed in the hope that it will be useful,    *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of     *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
  * Lesser General Public License for more details.                    *
  *                                                                    *
  * You should have received a copy of the GNU Lesser General Public   *
  * License along with this library (see file COPYING); if not, write  *
  * to the Free Software Foundation, Inc., 59 Temple Place, Suite      *
  * 330, Boston, MA 02111-1307 USA, or contact the author              *
  **********************************************************************/

// 
// header file for class IVectorFactory 
// Created by: Andreas Pfeiffer  at Wed Jan 19 20:36:32 2000
// 
// Last update: 
//              
// 
#ifndef INTERFACE_IVECTORFACTORY_H
#define INTERFACE_IVECTORFACTORY_H 

namespace Anaphe {

//-ap  class IHistogram1D;
//-ap  class IHistogram2D;
//-ap  class IProfileHistogram;

  class IScalar;
  class IPoint;
  class IVector;

  class IVectorManager;

  class IVectorFactory  { 

  public: 
    virtual ~IVectorFactory() { /* nop */; }

  public:
    // let the compiler deal with these ...
    //  IVectorFactory(const IVectorFactory &); 
    //  IVectorFactory & operator = (const IVectorFactory &); 

  public: 		// public methods
    virtual IScalar * createScalar(double x=0, double xp=0, double xm=0) = 0;
    virtual IPoint  * createPoint() = 0;

    virtual IVector * create() = 0;

//-ap	 virtual IVector * from1D(IHistogram1D *h) = 0;
//-ap	 virtual IVector * from2D(IHistogram2D *h) = 0;
//-ap	 virtual IVector * fromProfile(IProfileHistogram *h, const char * options  = 0) = 0;

    virtual void unregister(IVector * v) = 0;

    virtual void setManager(IVectorManager * vm) = 0;

  }; 

} //end namespace Anaphe

extern "C" {
  Anaphe::IVectorFactory * createIVectorFactory();
}


#endif /* INTERFACE_IVECTORFACTORY_H */ 
