/** \file INtuple.h
 * Declaration of INtuple class
 *
 * Copyright (C) CERN, Geneva
 *
 * \author Zsolt Moln&aacute;r
 */

#ifndef INTERFACES_INTUPLE_H
#define INTERFACES_INTUPLE_H 1

#include <iostream>
#include <utility>
#include <vector>

namespace Anaphe {

  class IHistogram1D;
  class IHistogram2D;
  class IScatter2D;

  class INtuple 
  {
  public: // Interface

    // Construct/destruct/copy
    virtual ~INtuple                    () { /* EMPTY */ }
    // Information retrieval
    virtual const char* name            ()                                    = 0;

#ifdef AIDA_DONT_USE_STD
    virtual void        listAttributes  ( ostream& aOs = cout )               = 0;
    // Analyzing
    virtual void        scan      ( const char* aAttributes, 
				    const char* aCut,
				    const unsigned long aFirst = 0,
				    const unsigned long aRows = LONG_MAX,
				    ostream& aOs = cout )                     = 0;
    virtual void        scan      ( const int aAttNum,
				    const char* aCut,
				    const unsigned long aFirst = 0,
				    const unsigned long aRows = LONG_MAX,
				    ostream& aOs = cout )                     = 0;
    virtual void        cscan     ( const char* aAttributes,
				    const char* aCut, 
				    const unsigned long aFirst = 0,
				    const unsigned long aRows = LONG_MAX,
				    ostream& aOs = cout )                     = 0;
    virtual void        cscan     ( const int aAttNum,
				    const char* aCut,
				    const unsigned long aFirst = 0,
				    const unsigned long aRows = LONG_MAX,
				    ostream& aOs = cout )                     = 0;
    virtual void        project1D ( IHistogram1D* aHist, 
				    const char* aAttribute, 
				    const char* aCut,
				    const unsigned long aFirst = 0,
				    const unsigned long aRows = LONG_MAX )    = 0;
    virtual void        cproject1D ( IHistogram1D* aHist, 
				     const char* aFunc, 
				     const char* aCut,
				     const unsigned long aFirst = 0,
				     const unsigned long aRows = LONG_MAX )   = 0;
    virtual void        project2D  ( IHistogram2D* aHist, 
				     const char* aAttribute1,
				     const char* aAttribute2,
				     const char* aCut,
				     const unsigned long aFirst = 0,
				     const unsigned long aRows = LONG_MAX )   = 0;
    virtual void        cproject2D ( IHistogram2D* aHist, 
				     const char* aFunc1,
				     const char* aFunc2,
				     const char* aCut,
				     const unsigned long aFirst = 0,
				     const unsigned long aRows = LONG_MAX )   = 0;
    virtual void        probe1D    ( const char* aFunc,
				     const char* aCut,
				     double& xMin, double& xMax, 
				     unsigned long& aMatcNum,
				     const unsigned long aFirst = 0,
				     const unsigned long aRows = LONG_MAX )   = 0;
    virtual void        probe2D    ( const char* aFuncX,
				     const char* aFuncY,
				     const char* aCut,
				     double & xMin, double & xMax, 
				     double & yMin, double & yMax, 
				     unsigned long& aMatcNum,
				     const unsigned long aFirst = 0,
				     const unsigned long aRows = LONG_MAX )   = 0;
#else   // #ifdef AIDA_DONT_USE_STD

    virtual void        listAttributes  ( std::ostream& aOs = std::cout )     = 0;
    // Analyzing
    virtual void        scan      ( const char* aAttributes, 
				    const char* aCut,
				    const unsigned long aFirst = 0,
				    const unsigned long aRows = LONG_MAX,
				    std::ostream& aOs = std::cout )           = 0;
    virtual void        scan      ( const int aAttNum,
				    const char* aCut,
				    const unsigned long aFirst = 0,
				    const unsigned long aRows = LONG_MAX,
				    std::ostream& aOs = std::cout )           = 0;
    virtual void        cscan     ( const char* aAttributes,
				    const char* aCut, 
				    const unsigned long aFirst = 0,
				    const unsigned long aRows = LONG_MAX,
				    std::ostream& aOs = std::cout )           = 0;
    virtual void        cscan     ( const int aAttNum,
				    const char* aCut,
				    const unsigned long aFirst = 0,
				    const unsigned long aRows = LONG_MAX,
				    std::ostream& aOs = std::cout )           = 0;
    virtual void        project1D ( IHistogram1D* aHist, 
				    const char* aAttribute, 
				    const char* aCut,
				    const unsigned long aFirst = 0,
				    const unsigned long aRows = LONG_MAX )    = 0;
    virtual void        cproject1D ( IHistogram1D* aHist, 
				     const char* aFunc, 
				     const char* aCut,
				     const unsigned long aFirst = 0,
				     const unsigned long aRows = LONG_MAX )   = 0;
    virtual void        project2D  ( IHistogram2D* aHist, 
				     const char* aAttribute1,
				     const char* aAttribute2,
				     const char* aCut,
				     const unsigned long aFirst = 0,
				     const unsigned long aRows = LONG_MAX )   = 0;
    virtual void        cproject2D ( IHistogram2D* aHist, 
				     const char* aFunc1,
				     const char* aFunc2,
				     const char* aCut,
				     const unsigned long aFirst = 0,
				     const unsigned long aRows = LONG_MAX )   = 0;
    virtual void        probe1D    ( const char* aFunc,
				     const char* aCut,
				     double & xMin, double & xMax,
				     unsigned long& aMatcNum,
				     const unsigned long aFirst = 0,
				     const unsigned long aRows = LONG_MAX )   = 0;
    virtual void        probe2D    ( const char* aFuncX,
				     const char* aFuncY,
				     const char* aCut,
				     double & xMin, double & xMax, 
				     double & yMin, double & yMax, 
				     unsigned long& aMatcNum,
				     const unsigned long aFirst = 0,
				     const unsigned long aRows = LONG_MAX )   = 0;
#endif   // #ifdef AIDA_DONT_USE_STD
    virtual void        scatter2D  ( IScatter2D* aPlot, 
				     const char* aFuncX,
				     const char* aFuncY,
				     const char* aCut,
				     const unsigned long aFirst = 0,
				     const unsigned long aRows = LONG_MAX )   = 0;

  }; 

} //end namespace Anaphe

#endif // #ifndef INTERFACES_INTUPLE_H
