/** \file INtupleManager.h
 * Declaration of INtupleManager class
 *
 * Copyright (C) CERN, Geneva
 *
 * \author Zsolt Moln&aacute;r
 */

#ifndef INTERFACES_INTUPLEMANAGER_H
#define INTERFACES_INTUPLEMANAGER_H 1

#include <iostream>

namespace Anaphe {

  class INtuple;
  class INtupleFactory;
  class INtupleParameters;

  class INtupleManager  
  { 
  public: // Interface

    // Construct/destruct/copy
    virtual ~INtupleManager            ()  { /* EMPTY */; }
    // Methods
    virtual void     setFactory        ( const INtupleFactory* aFactory )     = 0;
    virtual INtuple* retrieve          ( const unsigned int aIndex )          = 0;
    virtual void     erase             ( const unsigned int aIndex )          = 0;

#ifdef AIDA_DONT_USE_STD
    virtual void     review            (      ostream& aOs =      cout )      = 0;
    virtual void     listNtuples       (      ostream& aOs =      cout )      = 0;
    virtual void     listNtuples       ( const char* aStoreName,
					 ostream& aOs =      cout )      = 0;
#else   // #ifdef AIDA_DONT_USE_STD
    virtual void     review            ( std::ostream& aOs = std::cout )      = 0;
    virtual void     listNtuples       ( std::ostream& aOs = std::cout )      = 0;
    virtual void     listNtuples       ( const char* aStoreName,
					 std::ostream& aOs = std::cout )      = 0;
#endif   // #ifdef AIDA_DONT_USE_STD
    // Ntuple creation
    virtual INtuple* findNtuple        ( const char* aName )                  = 0;
    // NtupleParameter creation
    virtual INtupleParameters* 
    createParameters  ()                                     = 0;
    // Store selection
    virtual void     selectStore       ( const char* aName )                  = 0;
  }; 

} //end namespace Anaphe

// for access from dynamic loading
extern "C" {
  Anaphe::INtupleManager* createINtupleManager();
}

#endif // #ifndef INTERFACES_INTUPLEMANAGER_H
