// 
// Copyright (C) CERN, Geneva 
// 
// header file for class IVector 
// Created by: Andreas Pfeiffer  at ????????
// 
// Last update: 
//              
//   ap, 16-jan-01 : removed default arguments
// 

#ifndef INTERFACES_IPLOTTER_H
#define INTERFACES_IPLOTTER_H 1

#include <iostream>

namespace Anaphe {

  class IVector;
  class IScatter2D;

  class IPlotter {

  public:
    virtual ~IPlotter()   { /* nop */; }

  public:
    virtual void refresh() = 0;
    virtual void reset() = 0;

    virtual void plot(IVector *v1, IVector *v2) = 0;
    virtual void plot(IScatter2D *scat) = 0;
    virtual void overlay(IVector *v2, int selZone=-1) = 0;

    virtual void psPrint(const char *file) = 0;
    virtual void makePicture(const char *file) = 0;

    virtual void createRegion(double x, double y, double w, double h) = 0;
    virtual void zone(int, int, int iSel, const char *opt) = 0;

    virtual void zoneOption(const char *opt, const  char *val) = 0;
    virtual void dataOption(const char *opt, const  char *val) = 0;

    virtual void xAxisOption(const char *opt=0, const  char *val=0) = 0;
    virtual void yAxisOption(const char *opt=0, const  char *val=0) = 0;

    virtual void dataStyle(const char *opt, const  char *val) = 0;
    virtual void textStyle(const char *opt, const  char *val) = 0;

#ifdef AIDA_DONT_USE_STD
    virtual void listOptions(     ostream& os =      cout) = 0;
#else
    virtual void listOptions(std::ostream& os = std::cout) = 0;
#endif

    virtual void pageTitle(const char *title) = 0;
    virtual void zoneTitle(const char *title, int selZone=1) = 0;

    // Add text to page
    virtual void pageText(double xC, double yC, const char *text) = 0;
    // Add text to zone
    virtual void zoneText(double xC, double yC, const char *text, int selZone=1) = 0;

    virtual void setMinMaxX(double xMin, double xMax, int selZone=1) = 0;
    virtual void setMinMaxY(double yMin, double yMax, int selZone=1) = 0;

    virtual void resetMinMax() = 0;

    virtual void interact() = 0;
  };

} //end namespace Anaphe 

extern "C" {
  Anaphe::IPlotter * createIPlotter();
}

#endif   /* INTERFACES_IPLOTTER_H */
