// 
// Copyright (C) CERN, Geneva 
// 
// header file for class IVectorManager 
// Created by: Andreas Pfeiffer  at Wed Jan 19 20:35:12 2000
// 
// Last update: 
//              
//   ap, 16-jan-01 : removed default arguments, cleaned up std:: stuff
// 
#ifndef INTERFACE_IVECTORMANAGER_H
#define INTERFACE_IVECTORMANAGER_H

#include <iostream>

namespace Anaphe {

  class IScalar;
  class IPoint;
  class IVector;
  class IVectorFactory;

//-ap  class IHistogram1D;
//-ap  class IHistogram2D;
//-ap  class IProfileHistogram;

  class IVectorManager  { 

  public: 
    virtual ~IVectorManager()  { /* nop */; }

  public:
    // let these be done by the compiler
    //  IVectorManager(const IVectorManager &); 
    //  IVectorManager & operator = (const IVectorManager &); 

  public: 		// public methods
    virtual void setFactory(IVectorFactory * f) = 0;

    virtual void enroll(IVector * v) = 0;
    virtual void unregister(IVector * v) = 0;

    virtual IVector * retrieve1D(int i) = 0;
    virtual IVector * retrieve2D(int i) = 0;

    virtual void delete1D(int i) = 0;
    virtual void delete2D(int i) = 0;

    // returns number of vectors known to the manager
    virtual int size() const = 0;

    // destroys all vectors known to the manager
    virtual void clearAll() = 0;

#ifdef AIDA_DONT_USE_STD
    virtual void list(     ostream& os) = 0;
#else
    virtual void list(std::ostream& os) = 0;
#endif

  public:				// scalar- and point- creators
    virtual IScalar * createScalar(double x=0, double xp=0, double xm=0) = 0;
    virtual IPoint  * createPoint() = 0;

  public:				// vector-creators
    virtual IVector * create() = 0;

//-ap	 virtual IVector * from1D(IHistogram1D *h) = 0;
//-ap	 virtual IVector * from2D(IHistogram2D *h) = 0;
//-ap	 virtual IVector * fromProfile(IProfileHistogram *h, const char * options  = 0) = 0;

  }; 

} //end namespace Anaphe

extern "C" {
  Anaphe::IVectorManager* createIVectorManager();
}

#endif /* INTERFACE_IVECTORMANAGER_H */ 
