#include "VectorOfPoints/Vector.h"
#include "HTL/I_Histo.h"
#include "HTL/H_Statistics.h"
#include "Qplotter/adapters/HTLAdapter.h"
#include "Interfaces/IAnnotation.h"

#if defined __hpux || (defined __sun && __SUNPRO_CC < 0x500)
#define QPSTD
#else
#define QPSTD std
#endif

Anaphe::IVector& HTLAdapter::getVect() {
  return *histoVect;
}

HTLAdapter::~HTLAdapter () {
  delete histoVect;
}

HTLAdapter::HTLAdapter (I_Histo& histo) {
  switch (histo.dim()) {
  case 1:
    to1D(histo);
    break;
  case 2:
    to2D(histo);
    break;
  default:
    break;
  }
  Anaphe::IAnnotation *statistics = histoVect->annotation();
  // add statistics info from histo to annotation. 
  // ostrstream bug... create a new one every time.
  { QPSTD::ostrstream val;
  val << H_Statistics::mean(histo ) << QPSTD::ends;
  statistics->add("Mean"   , val.str(), true); }
  { QPSTD::ostrstream val;
  val << H_Statistics::rms(histo) << QPSTD::ends; 
  statistics->add("Rms"   , val.str(), true); }
  { QPSTD::ostrstream val;
  val << H_Statistics::entries_count(histo) << QPSTD::ends; 
  statistics->add("Entries"   , val.str(), true); }
}

void HTLAdapter::to1D (I_Histo& histo) {
  int nBins = histo.bin_count();
  QPSTD::vector<Anaphe::AIDAPoint *> points;
  for(int i = 0; i < nBins; i++ ) {
    QPSTD::vector<Anaphe::AIDAScalar> scalars;
    double binHeight = histo.i_bin(i).value();
    double error = histo.i_bin(i).error();
    Anaphe::AIDAScalar v(binHeight, error, error);
    double low  = histo.i_partition(0).i_lower_point(i);
    double high  = histo.i_partition(0).i_upper_point(i);
    //???double center = H_Bin_Helper::bin_center(histo,0)-low;
    double center = H_Bin_Helper::bin_center(histo,i);
    // for gravity binned histograms (default in HTL) we need to calculate 
    // the e+/e- ...
    Anaphe::AIDAScalar x(center, high-center, center-low);
    scalars.push_back(x);
    Anaphe::AIDAPoint *point = new Anaphe::AIDAPoint (1,scalars,v);
    points.push_back(point);
   }
  Anaphe::AIDAVector *hV = new Anaphe::AIDAVector (1 ,nBins ,0,points) ;
  hV->computeMinMax();
  histoVect = hV;
}

void HTLAdapter::to2D (I_Histo& histo) {
  I_Histo::I_Bin_Location bbL(2);
  int nBinsX = histo.i_partition(0).bin_count();
  int nBinsY = histo.i_partition(1).bin_count();
  double *bcX = new double [nBinsX]; // Array of bin centers in X
  double *hbwX = new double [nBinsX];// Array of bin centers in Y
  double *bcY = new double [nBinsY]; // Array of half bin widht in X
  double *hbwY = new double [nBinsY];// Array of half bin widht in Y
  int i,j;
  // Compute once for all bin centers and half bin widths
  for (i=0;i<nBinsX;i++) {
    bcX[i] = (histo.i_partition(0).i_lower_point(i)+
      histo.i_partition(0).i_upper_point(i))/2.;
    hbwX[i] = (histo.i_partition(0).i_upper_point(i)-
      histo.i_partition(0).i_lower_point(i))/2.;
  }
  for (j=0;j<nBinsY;j++) {
    bcY[j] = (histo.i_partition(1).i_lower_point(j)+
      histo.i_partition(1).i_upper_point(j))/2.;
    hbwY[j] = (histo.i_partition(1).i_upper_point(j)-
      histo.i_partition(1).i_lower_point(j))/2.;
  }
  // Reserve space for the STL vector
  QPSTD::vector<Anaphe::AIDAPoint *> points;
  points.reserve(nBinsX*nBinsY);
  for(i = 0; i < nBinsX; i++ ) {
    Anaphe::AIDAScalar x (bcX[i], hbwX[i], hbwX[i]);   
    for(j = 0; j < nBinsY; j++ ) {
      QPSTD::vector<Anaphe::AIDAScalar > scalars;
      Anaphe::AIDAScalar y (bcY[j],hbwY[j],hbwY[j]);
      bbL[0]=i; bbL[1]=j;
      double height = histo.i_bin(bbL).value();
      double error = histo.i_bin(bbL).error();
      Anaphe::AIDAScalar v (height , error, error);
      scalars.push_back(x);
      scalars.push_back(y);
      Anaphe::AIDAPoint *point = new Anaphe::AIDAPoint(2,scalars,v);
      points.push_back(point);
    }
  }
  Anaphe::AIDAVector *hV = new Anaphe::AIDAVector (2 ,nBinsX ,nBinsY ,points) ;
  hV->computeMinMax();
  histoVect = hV;
  delete [] bcX;
  delete [] hbwX;
  delete [] bcY;
  delete [] hbwY;
}

