#include "VectorOfPoints/Vector.h"
#include "HepFitting/HEPfitting.h"
#include "Qplotter/adapters/fitAdapter.h"
#include "Interfaces/IAnnotation.h"
#include <stdio.h>

#if defined __hpux || (defined __sun && __SUNPRO_CC < 0x500)
#define QPSTD
#else
#define QPSTD std
#endif

Anaphe::IVector& FitAdapter::getVect() {
  return *fitVect;
}

FitAdapter::FitAdapter(Anaphe::IVector& vH, HEPHistoFit &fitter, MODELfun& mf) {
  int dim = vH.dimension();
  int nP  = vH.nPoints();
  double estim, err, fval;
  double pars [512];
  array_n<double> derivs;
  int i;

  fitVect = vH.deepClone();	// clone original vector
  Anaphe::IAnnotation *fitInfo = fitVect->annotation();
  fitInfo->reset();
  // add fit info to annotation

  for ( i=0; i<mf.parmDim(); i++) {
    const char *parName;
    QPSTD::ostrstream val;
    fitter.getResult(i+1, estim, err, fval);
    parName = fitter.getParName(i+1);
    val << estim << QPSTD::ends; 
    fitInfo->add(parName  , val.str(), true);
    pars[i]  = estim;
  }

  QPSTD::ostrstream val;
  val << fval << "/" << fitter.includedPoints()-fitter.getNoOfParams() << QPSTD::ends;
  fitInfo->add("Chi2/ndf"  , val.str(), true);
 
  double *x = new double [dim]; 
  double value;

  Anaphe::IPoint *pd;
  for ( i=0; i<nP; i++) {
    pd = fitVect->point(i);
    if (dim == 1) {
      if (pd != 0) 
	x[0] = pd->coordinate(0);
      else 
	x[0] = 0;
    } else if (dim == 2) {
      if (pd != 0) {
	x[0] = pd->coordinate(0);
	x[1] = pd->coordinate(1);
      } else {
	x[0] = 0;
	x[1] = 0;
      }
    }
    value = (mf)(x, pars, derivs, 0);
    fitVect->replaceFromFunction(i, value);
  }
  delete []x;
}

FitAdapter::FitAdapter(GEminiContour gC) {
  // Number of points of the contour
  int nP  = gC.n;
  QPSTD::vector<Anaphe::AIDAPoint *> points;
  for(int i = 0; i < nP; i++ ) {
    QPSTD::vector<Anaphe::AIDAScalar> scalars;
    Anaphe::AIDAScalar v(gC.y[i], 0, 0);
    Anaphe::AIDAScalar x(gC.x[i], 0, 0);
    scalars.push_back(x);
    Anaphe::AIDAPoint *point = new Anaphe::AIDAPoint (1,scalars,v);
    points.push_back(point);
  }
  // Now "close" the contour
  points.push_back(points[0]);
  Anaphe::AIDAVector *hV = new Anaphe::AIDAVector (1 ,nP ,0,points) ;
  hV->computeMinMax();
  fitVect = hV;
  Anaphe::IAnnotation *contourInfo = fitVect->annotation();
  contourInfo->reset();
  contourInfo->add(gC.title.p(),"",true);
}
FitAdapter::~FitAdapter() {
  delete fitVect;
}

