#include "HTL/Histograms.h" // Transient histograms.
#include "CLHEP/Random/Randomize.h"
#include <qapplication.h>
#include "Qplotter/qp/qpplot.h"
#include "Qplotter/qp/qpcurve2D.h"
#include "Qplotter/adapters/HTLAdapter.h"
#include "VectorOfPoints/Vector.h"

const int nbx=100;
const int nby=100;

int main(int argc, char* argv[])
{
  int nbLevels = 8;
  int nbZones  = 2;
  if (argc >1)
    nbLevels=atoi(argv[1]);
  if (argc >2)
    nbZones=atoi(argv[2]);
  int i;
  // Fill 2D histogram
  Histo2D h2d ( "Test of Histo2D",nbx,0.,10.,nby,0.,10. );
  double x,y;
  for( i=0; i<50000; i++ ) {
    // 2D Gaussian
    x = RandGauss::shoot(5.,1.6);   // (meanX=5, stDevX=1.6)
    y = RandGauss::shoot(5.,0.8);   // (meanY=5, stDevY=0.8)
    h2d.fill(x,y);
  }
  // Capture histogram in a vector
  HTLAdapter hA (h2d);
  QApplication myapp(argc, argv);  
  QpPlot *plot = new QpPlot();
  QObject::connect(plot, SIGNAL(quit()), &myapp, SLOT(quit()));
  QpPage *page = plot->getPage();
  page->createZones(nbZones,nbZones);
  // Show the histogram as a color plot (geographical palette)
  QpZone *z1 = page->getZone(0);
  QpCurve2D *qpc = new QpCurve2D(z1);  
  qpc->changePalette(QpPalette::GEOGRAPHICAL,nbLevels);

  Anaphe::AIDAVector  vect ;
  vect.fromAscii("ex2D.dat");
  
  qpc->newCurve(vect, QpCurve2D::COLOR );
  //  qpc->newCurve(hA.getVect(), QpCurve2D::COLOR );  
  if (nbZones > 1) {
    // Show the histogram as a color plot (default palette)
    z1 = page->getZone(1);
    qpc = new QpCurve2D(z1,hA.getVect(), QpCurve2D::COLOR);  
    // Show the histogram as a color plot (greyscale palette)
    z1 = page->getZone(2);
    qpc = new QpCurve2D(z1);  
    qpc->changePalette(QpPalette::GREYSCALE,nbLevels);
    qpc->newCurve(hA.getVect(), QpCurve2D::COLOR );  
    // Show the histogram as a color plot (geographical palette)*4 colors
    QpZone *z1 = page->getZone(3);
    QpCurve2D *qpc = new QpCurve2D(z1);  
    qpc->changePalette(QpPalette::GEOGRAPHICAL,4*nbLevels);
    qpc->newCurve(hA.getVect(), QpCurve2D::COLOR );  
  }
  myapp.setMainWidget(plot);
  plot->show();
  myapp.exec();
  return 1;
}
 


