#include <qapplication.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qptext.h"

/** A Qplotter program to show how to use math stuff.
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->draw();}
  void print() {page->print("math.eps");}
  void makeMath();
private:
  QpPage  *page;
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(1,1);
}

void MyWidget::makeMath () {
  QpZone *z = page->getZone(0);
  double x =0.1, yd=0.7, y=9.2-yd;
  z->setXRange(0.0,9.0);
  z->setYRange(0.0,9.0);
  z->setQpFont( QFont( "helvetica", 18 ));
  new QpText("<qpmath><mi>alpha=</mi><mi>&alpha;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>beta=</mi><mi>&beta;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>chi=</mi><mi>&chi;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>delta=</mi><mi>&delta;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>Delta=</mi><mi>&Delta;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>epsi=</mi><mi>&epsi;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>eta=</mi><mi>&eta;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>gamma=</mi><mi>&gamma;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>Gamma=</mi><mi>&Gamma;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>iota=</mi><mi>&iota;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>kappa=</mi><mi>&kappa;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>lambda=</mi><mi>&lambda;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>Lambda=</mi><mi>&Lambda;</mi></qpmath>",z,x,y);y-=yd;
  x =1.9; y=9.2-yd;
  new QpText("<qpmath><mi>mu=</mi><mi>&mu;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>nu=</mi><mi>&nu;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>omega=</mi><mi>&omega;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>Omega=</mi><mi>&Omega;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>omicron=</mi><mi>&omicron;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>phi=</mi><mi>&phi;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>Phi=</mi><mi>&Phi;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>phiv=</mi><mi>&phiv;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>pi=</mi><mi>&pi;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>Pi=</mi><mi>&Pi;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>piv=</mi><mi>&piv;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>psi=</mi><mi>&psi;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>Psi=</mi><mi>&Psi;</mi></qpmath>",z,x,y);y-=yd;
  x =3.8; y=9.2-yd;
  new QpText("<qpmath><mi>rho=</mi><mi>&rho;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>sigma=</mi><mi>&sigma;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>Sigma=</mi><mi>&Sigma;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>tau=</mi><mi>&tau;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>theta=</mi><mi>&theta;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>Theta=</mi><mi>&Theta;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>thetav=</mi><mi>&thetav;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>upsi=</mi><mi>&upsi;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>Upsi=</mi><mi>&Upsi;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>xi=</mi><mi>&xi;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>Xi=</mi><mi>&Xi;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>zeta=</mi><mi>&zeta;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>infin=</mi><mi>&infin;</mi></qpmath>",z,x,y);y-=yd;
  x =5.5; y=9.2-yd;
  new QpText("<qpmath><mi>int=</mi><mi>&int;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>mnplus=</mi><mi>&mnplus;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>nabla=</mi><mi>&nabla;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>sum=</mi><mi>&sum;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>nbsp=</mi><mi>&nbsp;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>larr=</mi><mi>&larr;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>rarr=</mi><mi>&rarr;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>horbar=</mi><mi>&horbar;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>prod=</mi><mi>&prod;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>ape=</mi><mi>&ape;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>ap=</mi><mi>&ap;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>lt=</mi><mi>&lt;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>gt=</mi><mi>&gt;</mi></qpmath>",z,x,y);y-=yd;
  x =7.3; y=9.2-yd;
  new QpText("<qpmath><mi>le=</mi><mi>&le;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>ge=</mi><mi>&ge;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>exist=</mi><mi>&exist;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>forall=</mi><mi>&forall;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>plusmn=</mi><mi>&plusmn;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>part=</mi><mi>&part;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>middot=</mi><mi>&middot;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>hArr=</mi><mi>&hArr;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>prop=</mi><mi>&prop;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>radic=</mi><mi>&radic;</mi></qpmath>",z,x,y);y-=yd;
  new QpText("<qpmath><mi>OverBar=</mi><mi>&OverBar;</mi></qpmath>",z,x,y);
}


int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);

  myFrame.makeMath();
  myFrame.show();
  myFrame.print();

  // Enter event loop
  myapp.exec();
  return 1;
}
