#include "fitTagsApp.h"
// #include "command.h"

/*
void fitTagsApp::runCommand(QString command)
{
  cout << "fitTagsApp::runCommand  " << command << endl ;
  
  if (command == "quit")
    exit(0);
}
*/


int fitTagsApp::run(const char *name) {
  Histo1D xhist = Histo1D(" Random Numbers ",40,0.0,10.0);
  for(int i = 1;i <= 10000;i++) {
    double n1 = RandGauss::shoot(5.,1.6);   // (mean=5, stDev=1.6)
    xhist.fill(n1,1.);
  }
  HPrinter hp( std::cout );
  hp.print(xhist);
  fitGauss (xhist, name);
  return 0;
}

void fitTagsApp::fitGauss (Histo1D& histToFit, const char *name) {
  HEPHistoFit fitter;
  MODELfun model("G");
  fitter.setHistogram(histToFit);
  fitter.setTitle("Gaussian fit");
  fitter.setModel(&model);
  // Set parameters for the gaussian
  fitter.parmDef(1,"max",  HInfo::in_range_max_value(histToFit),0.,MINF ,INF);
  fitter.parmDef(2,"mean", HStat::mean(histToFit) ,0.,MINF ,INF );
  fitter.parmDef(3,"rms",  HStat::rms(histToFit) ,0.,MINF ,INF );
  fitter.setPrintLevel(-1);
  fitter.perform(Chi2fit | IgnoreBinSize);
  double neg,pos;
  for(int i=1;i<=3;i++)fitter.minosErrors(i,neg,pos);
  fitter.printResults();

  // compare Minos and Hessian-based confidence regions for mean and rms
  GEminiContour c1('*'), c2('+'), c3;
  fitter.ellipticalConfidenceRegion(3,1,c1,0.90);
  fitter.minosConfidenceRegion(3,1,c2,0.90);
  c3 = c1 + c2;
  c3.setTitle("Hessian (*) and Minos (+) 90% confidence regions for rms and max");
  c3.plot();
  HTLAdapter hA (histToFit);
  FitAdapter fA (hA.getVect(),fitter,model);

  IViewer *myV = ViewerFactory::Viewer();
//  myV->addCommandInterpreter(new CommandRunner());
  
  IPage *pg = myV->createPage(1,1);

  IZone *myZ = pg->selectZone(1);
  myZ->xMinMax(2.,8.);
  myZ->yMinMax(0.,888.);
  myZ->setProperty("Coordinates","free");
  myZ->setProperty("Option","xAxisGrid");
  myZ->setProperty("Option","yAxisGrid");
  myZ->setProperty("mirrorAxis","");

  IDataset *fitCurve =  myZ->addDataset(fA.getVect());
  IDataset *histCurve = myZ->addDataset(hA.getVect());
  histCurve->setProperty("Representation","errormark");
  histCurve->setProperty("msize","8");
  histCurve->setProperty("mtype","rect");
  histCurve->setProperty("Legend","Data");
  IDrawStyle *idx = histCurve->getStyle();
  // Solid blue markers
  idx->setProperty("fillcolor","blue");
  idx->setProperty("fillstyle","solid");

  fitCurve->setProperty("Representation","hfilled");
  fitCurve->setProperty("Legend","Fit");
  idx = fitCurve->getStyle();
  idx->setProperty("linecolor","red");
  idx->setProperty("fillcolor","yellow");
  idx->setProperty("fillstyle","diagcross");

  myZ->getAxis("X")->setProperty("Title","Energy");
  myZ->getAxis("Y")->setProperty("Title","Frequency");
  myZ->getAxis("X")->setProperty("Label","MeV");
  myZ->getAxis("Y")->setProperty("Label","Counts");
  myZ->setTitle("Zone title");

  // Greek letter in page title
  pg->setTitle("<qpmath><mi>&gamma;</mi><mi> energy</mi></qpmath>");
  // Put some text in page coordinates
  IText *pgT = pg->addText(100,100,"Preliminary");
  // Rotate the text
  pgT->setProperty("angle","45");
  // Change text properties
  ITextStyle *itx = pgT->getStyle();
  itx->setProperty("fontName","Courier");
  itx->setProperty("fontSize","32");
  itx->setProperty("color","red");
  itx->setProperty("bold","");
  itx->setProperty("italic","");

  if (strlen(name) == 0) {
    // When myV draws, the common QApplication will enter exec() method
    // but no timer will interrupt 
#ifdef NO_THREADED_PLOTTER
    myV->setProperty("timer", "-1");
    std::cout << "Type CTRL+C to exit"<< std::endl;
    myV->draw();
#else
    char tmp[111];
    std::cout << "THREADED_PLOTTER Type CTRL+C to exit"<< std::endl;
    myV->draw();
    std::cin >> tmp;
#endif
  }
  else
    myV->makePS(name);
  delete myV;
}

int main(int argc , char **argv)
{
  fitTagsApp myApp;   // create an application object
  if (argc > 1 )
    return myApp.run(argv[1]); 
  else 
    return myApp.run(""); 
}

