#include "HTL/Histograms.h" // Transient histograms.
#include "HTL/H_Statistics.h" // Transient histograms.
#include "CLHEP/Random/Randomize.h"
#include <qapplication.h>
#include "Qplotter/qp/qpplot.h"
#include "Qplotter/qp/qpcurve1D.h"
#include "Qplotter/adapters/HTLAdapter.h"

const int nbx=40;

int main(int argc, char* argv[])
{
  int i;
  // Fill 1D histogram
  Histo1D h1d1 ( "Test of Histo1D",nbx,0.,10. );
  Histo1D h1d2 ( "Test of Histo1D",nbx,0.,10. );
  double x;
  for( i=0; i<2500; i++ ) {
    x = RandGauss::shoot(5.,1.6);   // (meanX=5, stDevX=1.6)
    h1d1.fill(x);
    h1d2.fill(x);
  }
  for( i=0; i<2500; i++ ) {
    x = RandGauss::shoot(5.,1.6);   // (meanX=5, stDevX=1.6)
    h1d2.fill(x);
  }
  // Capture histogram in a vector
  HTLAdapter hA (h1d1);
  HTLAdapter hB (h1d2);
  QApplication myapp(argc, argv);  
  QpPlot *plot = new QpPlot();
  QObject::connect(plot, SIGNAL(quit()), &myapp, SLOT(quit()));
  QpPage *page = plot->getPage();
  page->createZones(1,1);
  QpZone *z1 = page->getZone(0);

  // Show the histogram as Marker+error
  QpCurve1D *qpc1 = new QpCurve1D(z1, hA.getVect(),QpCurve1D::MARKER_ERROR);  
  qpc1->setSymSize(7);
  qpc1->setQpBrush(QBrush(Qt::blue));
  SummaryTable* st1 = new SummaryTable;
  Anaphe::IAnnotation *anno = hA.getVect().annotation();
  int index;
  for (index=0; index<anno->size(); index++) {
    if ( anno->visibilityFlag(index) ) {
      st1->append( new StPair(anno->key(index).c_str(), 
			      anno->value(index).c_str()) );
    }
  }
  st1->append(new StPair("legend", "Monte Carlo"));
  qpc1->setSummaryTable(st1);

  QpCurve1D *qpc4 = new QpCurve1D(z1, hB.getVect(),QpCurve1D::MARKER_ERROR); 
  qpc4->setSymStyle(QpMarker::Triangle);
  SummaryTable* st4 = new SummaryTable;
  anno = hB.getVect().annotation();
  for (index=0; index<anno->size(); index++) {
    if ( anno->visibilityFlag(index) ) {
      st4->append( new StPair(anno->key(index).c_str(), 
			      anno->value(index).c_str()) );
    }
  }
  st4->append(new StPair("legend", "Data"));
  qpc4->setSummaryTable(st4);

  //myapp.setMainWidget(plot);
  page->draw();
  plot->show();
  plot->raise();
  myapp.exec();
  return 1;
}
 


