#include "HTL/Histograms.h"   // Transient histograms.
#include "HTL/PHistograms.h"
#include "HTL/PIterators.h"
#include "HTL/H_Statistics.h" // Transient histograms.
#include "VectorOfPoints/Vector.h"
#include "Qplotter/adapters/HTLAdapter.h"
#include "string"
#include "Qplotter/viewer/viewerFactory.h"
// Uncomment if using multithreaded plotter
//#include "Qplotter/viewer/qpViewer.h"

class Histo_App : public HepDbApplication
{
public:
  Histo_App() {  myV = ViewerFactory::Viewer();} 
  void readHisto(char *dbName, char *contName);
  void init();
  void show(HTLAdapter& hA,const char *xTitle);
  void show1D(Histo1D& hd);
  void show2D(Histo2D& hd);
private:
  IViewer *myV ;
};


void Histo_App::init()
{    
  Init();
}

void Histo_App::show(HTLAdapter& hA, const  char *xTitle)
{ 
  IPage *pg = myV->createPage(1,1);
  IZone *myZ =  pg->selectZone(1);
  myZ->setProperty("XAxisTitle",xTitle);
  myZ->addDataset(hA.getVect());
  myV->draw();
  // Uncomment if using multithreaded plotter
  /* QpViewer *qV=(QpViewer *)myV;
  qV->join();*/
  myV->clear();
}

void Histo_App::show1D(Histo1D& hd)
{ 
  // Capture histogram in a vector
  HTLAdapter hA (hd);
  show(hA,hd.name());
}

void Histo_App::show2D(Histo2D& hd)
{ 
  // Capture histogram in a vector
  HTLAdapter hA (hd);
  show(hA,hd.name());
}
 
void Histo_App::readHisto(char * dbName, char * contName)
{
  db(dbName);
  {
    // Declare an iterator for 1D histograms and initialize it
    HistoIterator<P_I_Histo_1D> myItr(container(contName));
    while (++myItr ) {
      Histo1D h(*(*myItr));
      show1D(h);
    }
  }
  {
    // Declare an iterator for 2D histograms and initialize it
    HistoIterator<P_I_Histo_2D> myItr(container(contName));
    while (++myItr ) {
      Histo2D h(*(*myItr));
      show2D(h);
    }
  }
  delete myV;
}

int main( int argc, char **argv)
{
  // Create application object:
  Histo_App my_app;
  // Retrieve histograms 
  my_app.init();
  my_app.startRead();
  my_app.readHisto("demo","Histograms");
  my_app.commit();
}
