#include <iostream>
#include "BasicVector2D.hh"

namespace gblib {
  //--------------------------------------------------------------------------
  std::ostream &
  operator<<(std::ostream & os, const BasicVector2D<float> & a)
  {
    return os << "(" << a.x() << "," << a.y() << ")";
  }

  //--------------------------------------------------------------------------
  std::istream &
  operator>> (std::istream & is, BasicVector2D<float> & a)
  {
    // Required format is ( a, b ) that is, two numbers, preceded by
    // (, followed by ), and separated by comma.  The two numbers are
    // taken as x, y.

    float x, y;
    char c;

    is >> std::ws >> c;
    // ws is defined to invoke eatwhite(istream & )
    // see (Stroustrup gray book) page 333 and 345.
    if (is.fail() || c != '(' ) {
      std::cerr
	<< "Could not find required opening parenthesis "
	<< "in input of a BasicVector2D<float>"
	<< std::endl;
      return is;
    }

    is >> x >> std::ws >> c;
    if (is.fail() || c != ',' ) {
      std::cerr
	<< "Could not find x value and required trailing comma "
	<< "in input of a BasicVector2D<float>"
	<< std::endl; 
      return is;
    }

    is >> y >> std::ws >> c;
    if (is.fail() || c != ')' ) {
      std::cerr
	<< "Could not find y value and required close parenthesis "
	<< "in input of a BasicVector2D<float>"
	<< std::endl;
      return is;
    }

    a.setX(x);
    a.setY(y);
    return is;
  }
  
  //--------------------------------------------------------------------------
  std::ostream &
  operator<< (std::ostream & os, const BasicVector2D<double> & a)
  {
    return os << "(" << a.x() << "," << a.y() << ")";
  }
  
  //--------------------------------------------------------------------------
  std::istream &
  operator>> (std::istream & is, BasicVector2D<double> & a)
  {
    // Required format is ( a, b ) that is, two numbers, preceded by
    // (, followed by ), and separated by comma.  The two numbers are
    // taken as x, y.
    
    double x, y;
    char c;
    
    is >> std::ws >> c;
    // ws is defined to invoke eatwhite(istream & )
    // see (Stroustrup gray book) page 333 and 345.
    if (is.fail() || c != '(' ) {
      std::cerr
	<< "Could not find required opening parenthesis "
	<< "in input of a BasicVector2D<double>"
	<< std::endl;
      return is;
    }

    is >> x >> std::ws >> c;
    if (is.fail() || c != ',' ) {
      std::cerr
	<< "Could not find x value and required trailing comma "
	<< "in input of a BasicVector2D<double>"
	<< std::endl; 
      return is;
    }

    is >> y >> std::ws >> c;
    if (is.fail() || c != ')' ) {
      std::cerr
	<< "Could not find y value and required close parenthesis "
	<< "in input of a BasicVector2D<double>"
	<< std::endl;
      return is;
    }

    a.setX(x);
    a.setY(y);
    return is;
  }
} /* namespace gblib */
