#ifndef GBLIB_NORMAL2D_H
#define GBLIB_NORMAL2D_H

#include <math.h>
#include <iosfwd>
#include "BasicVector2D.hh"

namespace gblib {

  class Translation2D;
  class Rotation2D;
  class Transformation2D;

  /**
   * Geometrical 2D Normal.
   *
   * @author Evgeni Chernyaev <Evgueni.Tcherniaev@cern.ch>
   */
  template<class T>
  class Normal2D : public BasicVector2D<T> {};

  template<>
  class Normal2D<float> : public BasicVector2D<float> {
  public:
    // Default constructor
    Normal2D() {}

    // Constructor from three numbers.
    Normal2D(float x, float y) : BasicVector2D<float>(x,y) {}

    // Constructor from base class.
    Normal2D(const BasicVector2D<float> & v) : BasicVector2D<float>(v) {}

    // Copy constructor.
    Normal2D(const Normal2D<float> & v) : BasicVector2D<float>(v) {}

    // Destructor.
    ~Normal2D() {}

    // Assignment.
    Normal2D<float> & operator=(const Normal2D<float> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Assignment from BasicVector2D<float>.
    Normal2D<float> & operator=(const BasicVector2D<float> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Transformations.
    inline Normal2D<float> & transform(const Translation2D &);
    inline Normal2D<float> & transform(const Rotation2D &);
    inline Normal2D<float> & transform(const Transformation2D &);
  };

  inline Normal2D<float>
  operator*(const Translation2D &, const Normal2D<float> &);

  inline Normal2D<float>
  operator*(const Rotation2D &, const Normal2D<float> &);

  inline Normal2D<float>
  operator*(const Transformation2D &, const Normal2D<float> &);

  template<>
  class Normal2D<double> : public BasicVector2D<double> {
  public:
    // Default constructor
    Normal2D() {}

    // Constructor from three numbers.
    Normal2D(double x, double y) : BasicVector2D<double>(x,y) {}

    // Constructor from base class.
    Normal2D(const BasicVector2D<double> & v) : BasicVector2D<double>(v) {}

    // Constructor from Normal2D<float>.
    Normal2D(const Normal2D<float> & v) : BasicVector2D<double>(v) {}

    // Copy constructor.
    Normal2D(const Normal2D<double> & v) : BasicVector2D<double>(v) {}

    // Destructor
    ~Normal2D() {}

    // Assignment
    Normal2D<double> & operator=(const Normal2D<double> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Assignment from BasicVector2D<float>
    Normal2D<double> & operator=(const BasicVector2D<float> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Assignment from BasicVector2D<double>
    Normal2D<double> & operator=(const BasicVector2D<double> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Transformations.
    inline Normal2D<double> & transform(const Translation2D &);
    inline Normal2D<double> & transform(const Rotation2D &);
    inline Normal2D<double> & transform(const Transformation2D &);
  };

  inline Normal2D<double>
  operator*(const Translation2D &, const Normal2D<double> &);

  inline Normal2D<double>
  operator*(const Rotation2D &, const Normal2D<double> &);

  inline Normal2D<double>
  operator*(const Transformation2D &, const Normal2D<double> &);

} /* namespace gblib */

#include "Normal2D.icc"

#endif /* GBLIB_NORMAL2D_H */
