#include "Normal3D.hh"

namespace gblib {
//----------------------------------------------------------------------------
  Normal3D<float> & Normal3D<float>::transform(const Transformation3D & m)
  {
    double xx = m.xx(), xy = m.xy(), xz = m.xz();
    double yx = m.yx(), yy = m.yy(), yz = m.yz();
    double zx = m.zx(), zy = m.zy(), zz = m.zz();
    set(float(x()*(yy*zz-yz*zy)+y()*(yz*zx-yx*zz)+z()*(yx*zy-yy*zx)),
	float(x()*(zy*xz-zz*xy)+y()*(zz*xx-zx*xz)+z()*(zx*xy-zy*xx)),
	float(x()*(xy*yz-xz*yy)+y()*(xz*yx-xx*yz)+z()*(xx*yy-xy*yx)));
    return *this;
  }

//----------------------------------------------------------------------------
  Normal3D<float>
  operator*(const Transformation3D & m, const Normal3D<float> & v)
  {
    double x = v.x(), y = v.y(), z = v.z(); 
    double xx = m.xx(), xy = m.xy(), xz = m.xz();
    double yx = m.yx(), yy = m.yy(), yz = m.yz();
    double zx = m.zx(), zy = m.zy(), zz = m.zz();
    return Normal3D<float>
      (float(x*(yy*zz-yz*zy)+y*(yz*zx-yx*zz)+z*(yx*zy-yy*zx)),
       float(x*(zy*xz-zz*xy)+y*(zz*xx-zx*xz)+z*(zx*xy-zy*xx)),
       float(x*(xy*yz-xz*yy)+y*(xz*yx-xx*yz)+z*(xx*yy-xy*yx)));
  }

//----------------------------------------------------------------------------
  Normal3D<double> & Normal3D<double>::transform(const Transformation3D & m)
  {
    double xx = m.xx(), xy = m.xy(), xz = m.xz();
    double yx = m.yx(), yy = m.yy(), yz = m.yz();
    double zx = m.zx(), zy = m.zy(), zz = m.zz();
    set(x()*(yy*zz-yz*zy)+y()*(yz*zx-yx*zz)+z()*(yx*zy-yy*zx),
	x()*(zy*xz-zz*xy)+y()*(zz*xx-zx*xz)+z()*(zx*xy-zy*xx),
	x()*(xy*yz-xz*yy)+y()*(xz*yx-xx*yz)+z()*(xx*yy-xy*yx));
    return *this;
  }

//----------------------------------------------------------------------------
  Normal3D<double>
  operator*(const Transformation3D & m, const Normal3D<double> & v)
  {
    double x = v.x(), y = v.y(), z = v.z(); 
    double xx = m.xx(), xy = m.xy(), xz = m.xz();
    double yx = m.yx(), yy = m.yy(), yz = m.yz();
    double zx = m.zx(), zy = m.zy(), zz = m.zz();
    return Normal3D<double>
      (x*(yy*zz-yz*zy)+y*(yz*zx-yx*zz)+z*(yx*zy-yy*zx),
       x*(zy*xz-zz*xy)+y*(zz*xx-zx*xz)+z*(zx*xy-zy*xx),
       x*(xy*yz-xz*yy)+y*(xz*yx-xx*yz)+z*(xx*yy-xy*yx));
  }
} /* namespace gblib */
