#ifndef GBLIB_NORMAL3D_H
#define GBLIB_NORMAL3D_H

#include <math.h>
#include <iosfwd>
#include "BasicVector3D.hh"

namespace gblib {

  class Translation3D;
  class Rotation3D;
  class Transformation3D;

  /**
   * Geometrical 3D Normal.
   *
   * @author Evgeni Chernyaev <Evgueni.Tcherniaev@cern.ch>
   */
  template<class T>
  class Normal3D : public BasicVector3D<T> {};

  template<>
  class Normal3D<float> : public BasicVector3D<float> {
  public:
    // Default constructor
    Normal3D() {}

    // Constructor from three numbers.
    Normal3D(float x, float y, float z) : BasicVector3D<float>(x,y,z) {}

    // Constructor from base class.
    Normal3D(const BasicVector3D<float> & v) : BasicVector3D<float>(v) {}

    // Copy constructor.
    Normal3D(const Normal3D<float> & v) : BasicVector3D<float>(v) {}

    // Destructor.
    ~Normal3D() {}

    // Assignment.
    Normal3D<float> & operator=(const Normal3D<float> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Assignment from BasicVector3D<float>.
    Normal3D<float> & operator=(const BasicVector3D<float> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Transformations.
    inline Normal3D<float> & transform(const Translation3D &);
    inline Normal3D<float> & transform(const Rotation3D &);
           Normal3D<float> & transform(const Transformation3D &);
  };

  inline Normal3D<float>
  operator*(const Translation3D &, const Normal3D<float> &);

  inline Normal3D<float>
  operator*(const Rotation3D &, const Normal3D<float> &);

  Normal3D<float>
  operator*(const Transformation3D &, const Normal3D<float> &);

  template<>
  class Normal3D<double> : public BasicVector3D<double> {
  public:
    // Default constructor
    Normal3D() {}

    // Constructor from three numbers.
    Normal3D(double x, double y, double z) : BasicVector3D<double>(x,y,z) {}

    // Constructor from base class.
    Normal3D(const BasicVector3D<double> & v) : BasicVector3D<double>(v) {}

    // Constructor from Normal3D<float>.
    Normal3D(const Normal3D<float> & v) : BasicVector3D<double>(v) {}

    // Copy constructor.
    Normal3D(const Normal3D<double> & v) : BasicVector3D<double>(v) {}

    // Destructor
    ~Normal3D() {}

    // Assignment
    Normal3D<double> & operator=(const Normal3D<double> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Assignment from BasicVector3D<float>
    Normal3D<double> & operator=(const BasicVector3D<float> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Assignment from BasicVector3D<double>
    Normal3D<double> & operator=(const BasicVector3D<double> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Transformations.
    inline Normal3D<double> & transform(const Translation3D &);
    inline Normal3D<double> & transform(const Rotation3D &);
           Normal3D<double> & transform(const Transformation3D &);
  };

  inline Normal3D<double>
  operator*(const Translation3D &, const Normal3D<double> &);

  inline Normal3D<double>
  operator*(const Rotation3D &, const Normal3D<double> &);

  Normal3D<double>
  operator*(const Transformation3D &, const Normal3D<double> &);

} /* namespace gblib */

#include "Normal3D.icc"

#endif /* GBLIB_NORMAL3D_H */
