#ifndef GBLIB_POINT2D_H
#define GBLIB_POINT2D_H

#include <math.h>
#include <iosfwd>
#include "BasicVector2D.hh"

namespace gblib {

  class Translation2D;
  class Rotation2D;
  class Transformation2D;

  /**
   * Geometrical 2D Point.
   *
   * @author Evgeni Chernyaev <Evgueni.Tcherniaev@cern.ch>
   */
  template<class T>
  class Point2D : public BasicVector2D<T> {};

  template<>
  class Point2D<float> : public BasicVector2D<float> {
  public:
    // Default constructor
    Point2D() {}

    // Constructor from three numbers.
    Point2D(float x, float y) : BasicVector2D<float>(x,y) {}

    // Constructor from base class.
    Point2D(const BasicVector2D<float> & v) : BasicVector2D<float>(v) {}

    // Copy constructor.
    Point2D(const Point2D<float> & v) : BasicVector2D<float>(v) {}

    // Destructor.
    ~Point2D() {}

    // Assignment.
    Point2D<float> & operator=(const Point2D<float> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Assignment from BasicVector2D<float>.
    Point2D<float> & operator=(const BasicVector2D<float> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Transformations.
    inline Point2D<float> & transform(const Translation2D &);
    inline Point2D<float> & transform(const Rotation2D &);
    inline Point2D<float> & transform(const Transformation2D &);
  };

  inline Point2D<float>
  operator*(const Translation2D &, const Point2D<float> &);

  inline Point2D<float>
  operator*(const Rotation2D &, const Point2D<float> &);

  inline Point2D<float>
  operator*(const Transformation2D &, const Point2D<float> &);

  template<>
  class Point2D<double> : public BasicVector2D<double> {
  public:
    // Default constructor
    Point2D() {}

    // Constructor from three numbers.
    Point2D(double x, double y) : BasicVector2D<double>(x,y) {}

    // Constructor from base class.
    Point2D(const BasicVector2D<double> & v) : BasicVector2D<double>(v) {}

    // Constructor from Point2D<float>.
    Point2D(const Point2D<float> & v) : BasicVector2D<double>(v) {}

    // Copy constructor.
    Point2D(const Point2D<double> & v) : BasicVector2D<double>(v) {}

    // Destructor
    ~Point2D() {}

    // Assignment
    Point2D<double> & operator=(const Point2D<double> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Assignment from BasicVector2D<float>
    Point2D<double> & operator=(const BasicVector2D<float> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Assignment from BasicVector2D<double>
    Point2D<double> & operator=(const BasicVector2D<double> & v) {
      set(v.x(),v.y()); return *this;
    }
    // Transformations.
    inline Point2D<double> & transform(const Translation2D &);
    inline Point2D<double> & transform(const Rotation2D &);
    inline Point2D<double> & transform(const Transformation2D &);
  };

  inline Point2D<double>
  operator*(const Translation2D &, const Point2D<double> &);

  inline Point2D<double>
  operator*(const Rotation2D &, const Point2D<double> &);

  inline Point2D<double>
  operator*(const Transformation2D &, const Point2D<double> &);

} /* namespace gblib */

#include "Point2D.icc"

#endif /* GBLIB_POINT2D_H */
