#ifndef GBLIB_POINT3D_H
#define GBLIB_POINT3D_H

#include <math.h>
#include <iosfwd>
#include "BasicVector3D.hh"

namespace gblib {

  class Translation3D;
  class Rotation3D;
  class Transformation3D;

  /**
   * Geometrical 3D Point.
   *
   * @author Evgeni Chernyaev <Evgueni.Tcherniaev@cern.ch>
   */
  template<class T>
  class Point3D : public BasicVector3D<T> {};

  template<>
  class Point3D<float> : public BasicVector3D<float> {
  public:
    // Default constructor
    Point3D() {}

    // Constructor from three numbers.
    Point3D(float x, float y, float z) : BasicVector3D<float>(x,y,z) {}

    // Constructor from base class.
    Point3D(const BasicVector3D<float> & v) : BasicVector3D<float>(v) {}

    // Copy constructor.
    Point3D(const Point3D<float> & v) : BasicVector3D<float>(v) {}

    // Destructor.
    ~Point3D() {}

    // Assignment.
    Point3D<float> & operator=(const Point3D<float> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Assignment from BasicVector3D<float>.
    Point3D<float> & operator=(const BasicVector3D<float> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Transformations.
    inline Point3D<float> & transform(const Translation3D &);
    inline Point3D<float> & transform(const Rotation3D &);
    inline Point3D<float> & transform(const Transformation3D &);
  };

  inline Point3D<float>
  operator*(const Translation3D &, const Point3D<float> &);

  inline Point3D<float>
  operator*(const Rotation3D &, const Point3D<float> &);

  inline Point3D<float>
  operator*(const Transformation3D &, const Point3D<float> &);

  template<>
  class Point3D<double> : public BasicVector3D<double> {
  public:
    // Default constructor
    Point3D() {}

    // Constructor from three numbers.
    Point3D(double x, double y, double z) : BasicVector3D<double>(x,y,z) {}

    // Constructor from base class.
    Point3D(const BasicVector3D<double> & v) : BasicVector3D<double>(v) {}

    // Constructor from Point3D<float>.
    Point3D(const Point3D<float> & v) : BasicVector3D<double>(v) {}

    // Copy constructor.
    Point3D(const Point3D<double> & v) : BasicVector3D<double>(v) {}

    // Destructor
    ~Point3D() {}

    // Assignment
    Point3D<double> & operator=(const Point3D<double> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Assignment from BasicVector3D<float>
    Point3D<double> & operator=(const BasicVector3D<float> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Assignment from BasicVector3D<double>
    Point3D<double> & operator=(const BasicVector3D<double> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }
    // Transformations.
    inline Point3D<double> & transform(const Translation3D &);
    inline Point3D<double> & transform(const Rotation3D &);
    inline Point3D<double> & transform(const Transformation3D &);
  };

  inline Point3D<double>
  operator*(const Translation3D &, const Point3D<double> &);

  inline Point3D<double>
  operator*(const Rotation3D &, const Point3D<double> &);

  inline Point3D<double>
  operator*(const Transformation3D &, const Point3D<double> &);

} /* namespace gblib */

#include "Point3D.icc"

#endif /* GBLIB_POINT3D_H */
