#include <math.h>
#include <iostream>
#include "ScreenProjection3D.hh"

namespace gblib {

  static const double pi = 3.14159265358979323846;

//----------------------------------------------------------------------------
  void ScreenProjection3D::normSetViewPoint(double x, double y, double z)
  {
    double a = 0, b = 0, c = 0;
    if (axdir == ZXY) { a = x; b = y; c = z; }
    if (axdir == XYZ) { a = y; b = z; c = x; }
    if (axdir == YZX) { a = z; b = x; c = y; }

    double lat = atan2(b, a);
    double lon = atan2(sqrt(a*a+b*b), c);
    setNormToView(lon, lat);
    setViewToProj();
  }

//----------------------------------------------------------------------------
  void ScreenProjection3D::setUserToNorm()
  {
    Projection3D m;
    if (stype == NORMALIZED) {
      double d = 2/sqrt(3.);
      m = Projection3D().setScale(d/sizeX, d/sizeY, d/sizeZ);
    }
    if (stype == PROPORTIONAL) {
      double d = 2/sqrt(sizeX*sizeX+sizeY*sizeY+sizeZ*sizeZ);
      m = Projection3D().setScale(d, d, d);
    } 
    u2n = m * Translation3D(-oX, -oY, -oZ);
  }

//----------------------------------------------------------------------------
  void ScreenProjection3D::setNormToView(double lon, double lat)
  {
    if (axdir == ZXY) n2v = Projection3D();
    if (axdir == XYZ) n2v = Projection3D().setRotationY(-pi/2) *
			    Projection3D().setRotationX(-pi/2);
    if (axdir == YZX) n2v = Projection3D().setRotationY(+pi/2) *
			    Projection3D().setRotationZ(+pi/2);
    n2v = Projection3D().setRotationX(lat)  *
          Projection3D().setRotationY(-lon) * n2v;
  }

//----------------------------------------------------------------------------
  void ScreenProjection3D::setViewToProj()
  {
    if (vdist <= 0) vdist = 3; 
    double l = sqrt((vdist-1)/(vdist+1));
    v2p = Projection3D().setFrustum(-l, l, -l, l, vdist-1, vdist+1) *
          Translation3D(0, 0, -vdist);
  }

} /* namespace gblib */
