#ifndef GBLIB_TRANSLATION2D_HH
#define GBLIB_TRANSLATION2D_HH

#include "BasicVector2D.hh"

namespace gblib {

  /**
   * Class for translation of 2D objects.
   *
   * @author Evgeni Chernyaev <Evgueni.Tcherniaev@cern.ch>
   */
  class Translation2D {
  protected:
    BasicVector2D<double> v_;
    
  public:
    // Default constructor.
    Translation2D() {}

    // Constructor from three numbers.
    Translation2D(double x, double y) : v_(x, y) {}

    // Constructor from BasicVector2D.
    explicit Translation2D(const BasicVector2D<float>  & v) : v_(v) {}
    explicit Translation2D(const BasicVector2D<double> & v) : v_(v) {}

    // Copy constructor.
    Translation2D(const Translation2D & m) : v_(m.v_) {}

    // Destructor.
    ~Translation2D() {}
    
    // Assignment
    Translation2D & operator=(const Translation2D & m) {
      v_ = m.v_; return *this;
    }

    // Get components
    double dx() const { return v_.x(); }
    double dy() const { return v_.y(); }

    // Invert
    Translation2D   inverse() const { return Translation2D(-v_); }
    Translation2D & invert()        { v_ = -v_; return *this; }

    // Returns true if zero translation
    bool isIdentity() const { return v_ == BasicVector2D<double>(); }

    // Set the Identity (zero) translation;
    Translation2D & setIdentity() { v_.set(0.,0.); return *this; }

    // Set translations
    Translation2D & setTranslationX(double x) { v_.set(x, 0.); return *this; }
    Translation2D & setTranslationY(double y) { v_.set(0., y); return *this; }
    Translation2D &
    setTranslation(double x, double y) { v_.set(x, y); return *this; }

    // Transformation
    Translation2D & transform(const Translation2D & m) {
      v_ += m.v_; return *this;
    }
  };

  // Composite translation
  inline Translation2D
  operator*(const Translation2D & a, const Translation2D & b) {
    return Translation2D(a.dx()+b.dx(), a.dy()+b.dy());
  }

} /* namespace gblib */

#endif /* GBLIB_TRANSLATION2D_HH */
