#ifndef GBLIB_TRANSLATION3D_HH
#define GBLIB_TRANSLATION3D_HH

#include "BasicVector3D.hh"

namespace gblib {

  /**
   * Class for translation of 3D objects.
   *
   * @author Evgeni Chernyaev <Evgueni.Tcherniaev@cern.ch>
   */
  class Translation3D {
  protected:
    BasicVector3D<double> v_;
    
  public:
    // Default constructor.
    Translation3D() {}

    // Constructor from three numbers.
    Translation3D(double x, double y, double z) : v_(x, y, z) {}

    // Constructor from BasicVector3D.
    explicit Translation3D(const BasicVector3D<float>  & v) : v_(v) {}
    explicit Translation3D(const BasicVector3D<double> & v) : v_(v) {}

    // Copy constructor.
    Translation3D(const Translation3D & m) : v_(m.v_) {}

    // Destructor.
    ~Translation3D() {}
    
    // Assignment
    Translation3D & operator=(const Translation3D & m) {
      v_ = m.v_; return *this;
    }

    // Get components
    double dx() const { return v_.x(); }
    double dy() const { return v_.y(); }
    double dz() const { return v_.z(); }

    // Invert
    Translation3D inverse() const { return Translation3D(-v_); }
    Translation3D & invert() { v_ = -v_; return *this; }

    // Returns true if zero translation
    bool isIdentity() const { return v_ == BasicVector3D<double>(); }

    // Set the Identity (zero) translation;
    Translation3D & setIdentity() { v_.set(0.,0.,0.); return *this; }

    // Set translations
    Translation3D &
    setTranslationX(double x) { v_.set(x, 0., 0.); return *this; }
    Translation3D &
    setTranslationY(double y) { v_.set(0., y, 0.); return *this; }
    Translation3D &
    setTranslationZ(double z) { v_.set(0., 0., z); return *this; }
    Translation3D &
    setTranslation(double x, double y, double z) {
      v_.set(x, y, z); return *this;
    }

    // Transformation
    Translation3D & transform(const Translation3D & m) {
      v_ += m.v_; return *this;
    }
  };

  // Composite translation
  inline Translation3D
  operator*(const Translation3D & a, const Translation3D & b) {
    return Translation3D(a.dx()+b.dx(), a.dy()+b.dy(), a.dz()+b.dz());
  }

} /* namespace gblib */

#endif /* GBLIB_TRANSLATION3D_HH */
