#ifndef GBLIB_VECTOR2D_H
#define GBLIB_VECTOR2D_H

#include <math.h>
#include <iosfwd>
#include "BasicVector2D.hh"

namespace gblib {

  class Translation2D;
  class Rotation2D;
  class Transformation2D;

  /**
   * Geometrical 2D Vector.
   *
   * @author Evgeni Chernyaev <Evgueni.Tcherniaev@cern.ch>
   */
  template<class T>
  class Vector2D : public BasicVector2D<T> {};

  template<>
  class Vector2D<float> : public BasicVector2D<float> {
  public:
    // Default constructor
    Vector2D() {}

    // Constructor from three numbers.
    Vector2D(float x, float y) : BasicVector2D<float>(x,y) {}

    // Constructor from base class.
    Vector2D(const BasicVector2D<float> & v) : BasicVector2D<float>(v) {}

    // Copy constructor.
    Vector2D(const Vector2D<float> & v) : BasicVector2D<float>(v) {}

    // Destructor.
    ~Vector2D() {}

    // Assignment.
    Vector2D<float> & operator=(const Vector2D<float> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Assignment from BasicVector2D<float>.
    Vector2D<float> & operator=(const BasicVector2D<float> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Transformations.
    inline Vector2D<float> & transform(const Translation2D &);
    inline Vector2D<float> & transform(const Rotation2D &);
    inline Vector2D<float> & transform(const Transformation2D &);
  };

  inline Vector2D<float>
  operator*(const Translation2D &, const Vector2D<float> &);

  inline Vector2D<float>
  operator*(const Rotation2D &, const Vector2D<float> &);

  inline Vector2D<float>
  operator*(const Transformation2D &, const Vector2D<float> &);

  template<>
  class Vector2D<double> : public BasicVector2D<double> {
  public:
    // Default constructor
    Vector2D() {}

    // Constructor from three numbers.
    Vector2D(double x, double y) : BasicVector2D<double>(x,y) {}

    // Constructor from base class.
    Vector2D(const BasicVector2D<double> & v) : BasicVector2D<double>(v) {}

    // Constructor from Vector2D<float>.
    Vector2D(const Vector2D<float> & v) : BasicVector2D<double>(v) {}

    // Copy constructor.
    Vector2D(const Vector2D<double> & v) : BasicVector2D<double>(v) {}

    // Destructor
    ~Vector2D() {}

    // Assignment
    Vector2D<double> & operator=(const Vector2D<double> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Assignment from BasicVector2D<float>
    Vector2D<double> & operator=(const BasicVector2D<float> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Assignment from BasicVector2D<double>
    Vector2D<double> & operator=(const BasicVector2D<double> & v) {
      set(v.x(),v.y()); return *this;
    }

    // Transformations.
    inline Vector2D<double> & transform(const Translation2D &);
    inline Vector2D<double> & transform(const Rotation2D &);
    inline Vector2D<double> & transform(const Transformation2D &);
  };

  inline Vector2D<double>
  operator*(const Translation2D &, const Vector2D<double> &);

  inline Vector2D<double>
  operator*(const Rotation2D &, const Vector2D<double> &);

  inline Vector2D<double>
  operator*(const Transformation2D &, const Vector2D<double> &);

} /* namespace gblib */

#include "Vector2D.icc"

#endif /* GBLIB_VECTOR2D_H */
