#ifndef GBLIB_VECTOR3D_H
#define GBLIB_VECTOR3D_H

#include <math.h>
#include <iosfwd>
#include "BasicVector3D.hh"

namespace gblib {

  class Translation3D;
  class Rotation3D;
  class Transformation3D;

  /**
   * Geometrical 3D Vector.
   *
   * @author Evgeni Chernyaev <Evgueni.Tcherniaev@cern.ch>
   */
  template<class T>
  class Vector3D : public BasicVector3D<T> {};

  template<>
  class Vector3D<float> : public BasicVector3D<float> {
  public:
    // Default constructor
    Vector3D() {}

    // Constructor from three numbers.
    Vector3D(float x, float y, float z) : BasicVector3D<float>(x,y,z) {}

    // Constructor from base class.
    Vector3D(const BasicVector3D<float> & v) : BasicVector3D<float>(v) {}

    // Copy constructor.
    Vector3D(const Vector3D<float> & v) : BasicVector3D<float>(v) {}

    // Destructor.
    ~Vector3D() {}

    // Assignment.
    Vector3D<float> & operator=(const Vector3D<float> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Assignment from BasicVector3D<float>.
    Vector3D<float> & operator=(const BasicVector3D<float> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Transformations.
    inline Vector3D<float> & transform(const Translation3D &);
    inline Vector3D<float> & transform(const Rotation3D &);
    inline Vector3D<float> & transform(const Transformation3D &);
  };

  inline Vector3D<float>
  operator*(const Translation3D &, const Vector3D<float> &);

  inline Vector3D<float>
  operator*(const Rotation3D &, const Vector3D<float> &);

  inline Vector3D<float>
  operator*(const Transformation3D &, const Vector3D<float> &);

  template<>
  class Vector3D<double> : public BasicVector3D<double> {
  public:
    // Default constructor
    Vector3D() {}

    // Constructor from three numbers.
    Vector3D(double x, double y, double z) : BasicVector3D<double>(x,y,z) {}

    // Constructor from base class.
    Vector3D(const BasicVector3D<double> & v) : BasicVector3D<double>(v) {}

    // Constructor from Vector3D<float>.
    Vector3D(const Vector3D<float> & v) : BasicVector3D<double>(v) {}

    // Copy constructor.
    Vector3D(const Vector3D<double> & v) : BasicVector3D<double>(v) {}

    // Destructor
    ~Vector3D() {}

    // Assignment
    Vector3D<double> & operator=(const Vector3D<double> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Assignment from BasicVector3D<float>
    Vector3D<double> & operator=(const BasicVector3D<float> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Assignment from BasicVector3D<double>
    Vector3D<double> & operator=(const BasicVector3D<double> & v) {
      set(v.x(),v.y(),v.z()); return *this;
    }

    // Transformations.
    inline Vector3D<double> & transform(const Translation3D &);
    inline Vector3D<double> & transform(const Rotation3D &);
    inline Vector3D<double> & transform(const Transformation3D &);
  };

  inline Vector3D<double>
  operator*(const Translation3D &, const Vector3D<double> &);

  inline Vector3D<double>
  operator*(const Rotation3D &, const Vector3D<double> &);

  inline Vector3D<double>
  operator*(const Transformation3D &, const Vector3D<double> &);

} /* namespace gblib */

#include "Vector3D.icc"

#endif /* GBLIB_VECTOR3D_H */
