#include <iostream>

#include "Qplotter/qp/qpqwt_math.h"
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpaxis.h"
#include "Qplotter/qp/qptextmetrics.h"
#include <stdio.h>

#include "float.h"

#if defined __hpux || (defined __sun && __SUNPRO_CC < 0x500)
#define QPSTD
#else
#define QPSTD std
#endif

QpAxis::QpAxis(double _x1, double _y1, double _x2, double _y2,
               double _minVal, double _maxVal, int opt,
	       unsigned int nbdiv,QpObject *z, bool addOnParent, 
	       QpTransformation::tStrategy st) :
  QpNode<QpObject>(new QString("QpAxis"),z,addOnParent,st)
{ 
  init();
  setLimits(_x1, _y1, _x2, _y2);
  setValues(_minVal, _maxVal);

  if (x1!=x2 || y1!=y2) {
    setMaxL1Divs(nbdiv % 100);
    setMaxL2Divs((nbdiv % 10000 - getMaxL1Divs())/100);
    setMaxL3Divs((nbdiv - getMaxL1Divs() - getMaxL2Divs()*100)/10000);
    if (opt != 0)
      setOptions(opt);
    else
      changed();
  }
}
 
QpAxis::QpAxis(double _x1, double _y1, double _x2, double _y2,
               double _minVal, double _maxVal, 
	       double _step, int opt ,QpObject *z, bool addOnParent, 
	       QpTransformation::tStrategy st) :
  QpNode<QpObject>(new QString("QpAxis"),z,addOnParent,st)
{
  init();
  setLimits(_x1, _y1, _x2, _y2);
  setValues(_minVal, _maxVal);

  setMaxL1Divs(int(fabs(maxVal-minVal)/_step));
  setStep(int(_step));
  if (opt != 0)
    setOptions(opt);
  else
    changed();
}

QpAxis::QpAxis(double _x1, double _y1, double _x2, double _y2,
               QPtrList<QString> *labels_, int opt,
               QpObject *z, bool addOnParent, QpTransformation::tStrategy st) :
  QpNode<QpObject>(new QString("QpAxis"),z,addOnParent,st)
{
  init();
  setLimits(_x1, _y1, _x2, _y2);
  setMaxL1Divs(labels->count());
  setLabels(labels_);
  if (opt != 0)
    setOptions(opt);
  else
    changed();
}

QpAxis::~QpAxis() 
{
  delete labels;
}

//-----------------------------------------------------------------------------
//
// Axis variables initialisation. This procedure initialise variables like
// The number of division, the default options used etc ...
//
void QpAxis::init()
{
  stepl1 = stepl2 = stepl3 = x1 = x2 = y1 = y2 = 0.0;

  minVal           = 0.;
  maxVal           = 1.;
  Optimized_minVal = minVal;
  Optimized_maxVal = maxVal;
  maxL1Divs        = 10;
  maxL2Divs        = 5;
  maxL3Divs        = 0;
  L1Divs           = maxL1Divs;
  L2Divs           = maxL2Divs;
  L3Divs           = maxL3Divs;
  options          = (Options)((int)TickPositive +
                             (int)Optimized);
  labelMode        = Normal;
  labels           = 0;
  labelDist        = 4;
  titleDist        = 14;
  legendDist       = 14;
  multiDist        = 4;

  l1Height         = 3.0;
  l2Height         = 1.5;
  l3Height         = 0.75;
  arrowLength      = 3.0;
  arrowWidth       = 0.75;
  gridLength       = 100.;

  title            = QString("");
  legend           = QString("");
  multi            = QString("");
  findHint         = QString("bottom");
  labelAlignment   = 0;
  centerVar        = 10.0;


  TickLevel1X      = 0;
  TickLevel1Y      = 0;
  TickLevel2X      = 0;
  TickLevel2Y      = 0;
  TickLevel3X      = 0;
  TickLevel3Y      = 0;
  GridX1           = 0;
  GridY1           = 0;
  GridX2           = 0;
  GridY2           = 0;

  NbTickLevel2     = 0;
  NbTickLevel3     = 0;
}

//-----------------------------------------------------------------------------
//
// Sets axis limits (origin and end). And compute the angle Phi
//
void QpAxis::setLimits(double _x1, double _y1, double _x2, double _y2)
{
  x1     = _x1;
  y1     = _y1;
  x2     = _x2;
  y2     = _y2;
  Phi    = atan2(y2-y1, x2-x1);
  PhiDeg = Phi*(180/3.14159);
  cosPhi = cos(Phi);
  sinPhi = sin(Phi);
}

//-----------------------------------------------------------------------------
//
// Sets Labels alignment on the tick marks.
//
void QpAxis::setLabelAlignment(int _align)
{
  labelAlignment = _align;
  if (labelAlignment && 
    !(labelAlignment & Qt::AlignHCenter) && 
    !(labelAlignment & Qt::AlignVCenter)) { 
    centerVar = 0.0;
  } else {
    centerVar = 10.0;
  }
  changed();
}

//-----------------------------------------------------------------------------
//
// Override options (version with casting)
//
void QpAxis::setOptions(int opt) {
  Options opTyped = static_cast<Options>(opt);
  setOptions(opTyped);
}

//-----------------------------------------------------------------------------
//
// Sets the axis options.
//
void QpAxis::setOption(Options opt, bool value) 
{
  if ((options & opt) && !value) {
    options = (Options)(options - opt); 
  } else if (!(options & opt) && value) {
    options = (Options)(options + opt);
  }
  changed();
}

//-----------------------------------------------------------------------------
//
// Rotate and translate x and y
//
void QpAxis::rotation(double &x, double &y)
{
  double xb, yb;
  xb = cosPhi * x - sinPhi * y + x1;
  yb = sinPhi * x + cosPhi * y + y1;
  x  = xb; 
  y  = yb;
}

//-----------------------------------------------------------------------------
//
// Compute the end point of tick marks
//
void QpAxis::computeEndPoint(double &x, double &y)
{
   double xb, yb, offset;

   if (options & TickNegative) offset = -y;
   if (options & TickPositive) offset = y;

   if (options & TickVertical) {
      xb = x;
      yb = 0.0;
      rotation(xb, yb);
      x = xb; 
      y = yb+offset;
   } else if (options & TickHorizontal) {
      xb = x;
      yb = 0.0;
      rotation(xb, yb);
      x = xb+offset; 
      y = yb;
   } else {
      xb = x;
      yb = offset;
      rotation(xb, yb);
      x = xb; 
      y = yb;
   }
}

//-----------------------------------------------------------------------------
//
// Fills the arrays TickX and TickY with the tick mark number i 
//
void QpAxis::fillTickArray(double xi, double yi, int i, 
                           double *TickX, double *TickY)
{
  double x, y;
//
// Compute the first coordinates of the "i" level one tick mark ie: 
// fill TickX[i*2] and TickY[i*2].
//
    x = xi;
    y = 0;
    rotation(x, y);
    TickX[i*2] = x; 
    TickY[i*2] = y;
//
// Compute the second coordinates of the "i" level one tick mark ie:
// fill TickX[i*2+1] and TickY[i*2+1].
//
    y = yi;
    x = xi;
    computeEndPoint(x, y);
    TickX[i*2+1] = x; 
    TickY[i*2+1] = y;      
}

//-----------------------------------------------------------------------------
//
// Draw the axis
//
void QpAxis::drawAxis()
{
  double *tabx = new double[2], *taby = new double[2];
//
// Remove the created multilines and qptexts
//
  clearList();
//
// Draw the grid on level 1 tick marks
//
  if (GridX1) {
   QpMultiLine* ml = createMultiLine(2*(L1Divs+1), GridX1, GridY1);
   ml->setQpPen(QPen(Qt::lightGray,1,Qt::DashLine));
  }
//
// Draw the grid on level 2 tick marks
//
  if (GridX2) {
   QpMultiLine* ml = createMultiLine(2*NbTickLevel2, GridX2, GridY2);
   ml->setQpPen(QPen(Qt::lightGray,1,Qt::DotLine));
  }
//
// Draw the level 1 tick-marks
//
  if (TickLevel1X) createMultiLine(2*(L1Divs+1), TickLevel1X, TickLevel1Y);
//
// Draw the level 2 tick-marks
//
  if (TickLevel2X) createMultiLine(2*NbTickLevel2, TickLevel2X, TickLevel2Y);
//
// Draw the level 3 tick-marks
//
  if (TickLevel3X) createMultiLine(2*NbTickLevel3, TickLevel3X, TickLevel3Y);
//
// Create the tick mark labels
//
  drawLabels(2*(L1Divs+1), TickLevel1X, TickLevel1Y);
//
// Create the axis legend
//
  if (legend != "") {
    QpText *t = new QpText(legend, this, x1-legendDist, y1-legendDist, 
                           axisLength+legendDist, 0. , true, getTransfStrategy());
    t->getTextObject()->setAngle(PhiDeg);
    t->getTextObject()->setAlignment(Qt::AlignRight);
  }
//
// Create the axis title
//
  if (title != "") {
    QpText *t = new QpText(title, this, x1-titleDist, y1-titleDist, 
                           axisLength+titleDist, 0. , true, getTransfStrategy());
    t->getTextObject()->setAngle(PhiDeg);
    t->getTextObject()->setAlignment(Qt::AlignCenter);
  }
//
// Draw the axis body
// 
  tabx[0] = x1; 
  taby[0] = y1; 
  tabx[1] = x2; 
  taby[1] = y2;
  createMultiLine(2, tabx, taby);
  delete []tabx;
  delete []taby;
//
// Draw the arrow if needed
//
  if (options & Arrow) drawArrow();
//
// Delete the tick marks and grid arrays if they exist
//
  if (TickLevel1X) delete []TickLevel1X; TickLevel1X = 0;
  if (TickLevel1Y) delete []TickLevel1Y; TickLevel1Y = 0;
  if (TickLevel2X) delete []TickLevel2X; TickLevel2X = 0;
  if (TickLevel2Y) delete []TickLevel2Y; TickLevel2Y = 0;
  if (TickLevel3X) delete []TickLevel3X; TickLevel3X = 0;
  if (TickLevel3Y) delete []TickLevel3Y; TickLevel3Y = 0;
  if (GridX1     ) delete []GridX1     ; GridX1      = 0;
  if (GridY1     ) delete []GridY1     ; GridY1      = 0;
  if (GridX2     ) delete []GridX2     ; GridX2      = 0;
  if (GridY2     ) delete []GridY2     ; GridY2      = 0;
}

//-----------------------------------------------------------------------------
//
// Create the tick marks according to the options. Before calling the tick
// marks creation procedures, this function compute the length of the 3 kind
// of tick marks and the axis lenght in page coordinates.
//
void QpAxis::createTickMarks()
{

//-------------------------------------
//
// Compute tick mark's height (percent of page coordinates system)
//
  if ( x1==x2 ) {
    double Y2, Y1;
    Y1 = y1;
    Y2 = y2;
    if (Y1 > Y2) {
      Y1 = y2;
      Y2 = y1;
    }
    l1TickHeight = (Y2-Y1)*l1Height/100;
    l2TickHeight = (Y2-Y1)*l2Height/100;
    l3TickHeight = (Y2-Y1)*l3Height/100;
    projection   = (Y2-Y1)/L1Divs;
  } else {
    QpTransformation *ps = getTransformation(getTransfStrategy());
    l1TickHeight = y1 - ps->invYTransform(ps->yTransform(y1)+ps->xTransform(x1+(x2-x1)/cosPhi*l1Height/100)-ps->xTransform(x1));
    l2TickHeight = y1 - ps->invYTransform(ps->yTransform(y1)+ps->xTransform(x1+(x2-x1)/cosPhi*l2Height/100)-ps->xTransform(x1));
    l3TickHeight = y1 - ps->invYTransform(ps->yTransform(y1)+ps->xTransform(x1+(x2-x1)/cosPhi*l3Height/100)-ps->xTransform(x1));
    projection   = ((x2-x1)/cosPhi)/L1Divs;
  }
  axisLength  = projection*L1Divs;

//-------------------------------------
//
// Create the arrays to draw tick marks according to the options
//
  if (options & Logarithmic) {
    createLogTickMarks();
  } else {
    createLinearTickMarks();
  }
}

//-----------------------------------------------------------------------------
//
// Create the arrays to draw log tick marks
//
void QpAxis::createLogTickMarks()
{

  int firstDecade = (int)log10(minVal);
  int lastDecade  = (int)log10(maxVal);

//-------------------------------------
//
// Fill the arrays TickLevel1X and TickLevel1Y according to the array "values". 
// TickLevel1X and TickLevel1Y are use to draw the primary tick marks (decades).
//
  TickLevel1X = new double [(L1Divs+1)*2];
  TickLevel1Y = new double [(L1Divs+1)*2];
  if (options & GridMajor) {
    GridX1 = new double [(L1Divs+1)*2];
    GridY1 = new double [(L1Divs+1)*2];
  }
  double xc1;
  int j=0;

  for ( int i=firstDecade; i<=lastDecade; i++) {
    xc1 = ((i-log10(minVal))/(log10(maxVal)-log10(minVal)))*axisLength;
    fillTickArray(xc1, l1TickHeight, j, TickLevel1X, TickLevel1Y);
    if (options & GridMajor) fillTickArray(xc1, gridLength, j, GridX1, GridY1);
    j++;
  }

//-------------------------------------
//
// Fill the arrays TickLevel2X and TickLevel2Y.
// TickLevel2X and TickLevel2Y are use to draw the level two tick marks.
//

  TickLevel2X = new double [(L1Divs+2)*(L2Divs-1)*2];
  TickLevel2Y = new double [(L1Divs+2)*(L2Divs-1)*2];
  if (options & GridMinor) {
    GridX2 = new double [(L1Divs+2)*(L2Divs-1)*2];
    GridY2 = new double [(L1Divs+2)*(L2Divs-1)*2];
  }

  NbTickLevel2 = 0;
  double xc2, xcc2, dx2;

  for ( int i=firstDecade; i<lastDecade; i++) {
    dx2 = pow(10.,i);
    xc2 = dx2;
    for ( int j=1; j<=L2Divs-1; j++) {
      ++NbTickLevel2;
      xc2  = xc2+dx2;
      xcc2 = ((log10(xc2)-log10(minVal))/
              (log10(maxVal)-log10(minVal)))*axisLength;
      fillTickArray(xcc2, l2TickHeight, NbTickLevel2-1, 
                    TickLevel2X, TickLevel2Y);
      if (options & GridMinor)
        fillTickArray(xcc2, gridLength, NbTickLevel2-1, GridX2, GridY2);
    }
  }
//
// Make the remaining heading and tailing secondary tick marks.
//
  dx2  = pow(10.,lastDecade);
  xc2  = 2*dx2;
  xcc2 = ((log10(xc2)-log10(minVal))/
          (log10(maxVal)-log10(minVal)))*axisLength;
  while (xcc2 <= axisLength) {
    ++NbTickLevel2;
    fillTickArray(xcc2, l2TickHeight, NbTickLevel2-1, TickLevel2X, TickLevel2Y);
    if (options & GridMinor)
      fillTickArray(xcc2, gridLength, NbTickLevel2-1, GridX2, GridY2);
    xc2  = xc2+dx2;
    xcc2 = ((log10(xc2)-log10(minVal))/
            (log10(maxVal)-log10(minVal)))*axisLength;
  }
  dx2  = pow(10.,firstDecade-1);
  xc2  = pow(10.,firstDecade)-dx2;
  xcc2 = ((log10(xc2)-log10(minVal))/
          (log10(maxVal)-log10(minVal)))*axisLength;
  while (xcc2 >= 0) {
    ++NbTickLevel2;
    fillTickArray(xcc2, l2TickHeight, NbTickLevel2-1, TickLevel2X, TickLevel2Y);
    if (options & GridMinor)
      fillTickArray(xcc2, gridLength, NbTickLevel2-1, GridX2, GridY2);
    xc2  = xc2-dx2;
    xcc2 = ((log10(xc2)-log10(minVal))/
            (log10(maxVal)-log10(minVal)))*axisLength;
  }
}

//-----------------------------------------------------------------------------
//
// Create the arrays to draw linear tick marks
//
void QpAxis::createLinearTickMarks()
{
  double *values = new double [L1Divs+1];

//-------------------------------------
//
// Fill the tick position array according to the horizontal projection
// of the axis:
//
//                             
//                          +
//     Axis position       /:
//                        / :
//                       /  :
//                      /   :
//                     /    :
//                    /     :
//                   /      :  The axis is projection on the horizontal
//                  /       :  before being "tick marked". The length
//                 /        :  of the axis is: axisLength = (x2-x1)/cos(Phi)
//                /         :
//               /          :
//              /           :
//             /            :
//            / Phi         :
//           +-------       :
//           :              :
//           +--------------+---------------+
//           x1             x2
//             
//                  ^                 ^
//                  |                 |
//
//                      Optimized_minVal     Optimized_maxVal
//                      :                    :
//   Axis Value     +---+--------------------+----+
//                minVal                        maxVal
//
//
//
//
  double xmin, xmax, DVal;
  DVal = maxVal-minVal;
  xmin = axisLength*((Optimized_minVal-minVal)/DVal);
  xmax = axisLength*((Optimized_maxVal-minVal)/DVal);
  qwtLinSpace(values, L1Divs+1, xmin, xmax);    

//-------------------------------------
//
// Fill the arrays TickLevel1X and TickLevel1Y according to the array "values". 
// TickLevel1X and TickLevel1Y are use to draw the primary tick marks.
//
  TickLevel1X = new double [(L1Divs+1)*2];
  TickLevel1Y = new double [(L1Divs+1)*2];
  if (options & GridMajor) {
    GridX1 = new double [(L1Divs+1)*2];
    GridY1 = new double [(L1Divs+1)*2];
  }

  for ( int i=0; i<L1Divs+1; i++) {
    fillTickArray(values[i], l1TickHeight, i, TickLevel1X, TickLevel1Y);
    if (options & GridMajor) fillTickArray(values[i], gridLength, i, GridX1, GridY1);
  }

//-------------------------------------
//
// Fill the arrays TickLevel2X and TickLevel2Y.
// TickLevel2X and TickLevel2Y are use to draw the level two tick marks.
//
  double dx2 = (values[1]-values[0])/L2Divs;
  double dx3 = dx2/L3Divs;
  double xc2, xc3;

  if (L2Divs >= 2) {
    TickLevel2X = new double [(L1Divs+2)*(L2Divs-1)*2];
    TickLevel2Y = new double [(L1Divs+2)*(L2Divs-1)*2];
    if (options & GridMinor) {
      GridX2 = new double [(L1Divs+2)*(L2Divs-1)*2];
      GridY2 = new double [(L1Divs+2)*(L2Divs-1)*2];
    }

    NbTickLevel2 = 0;
    double xc2;
  
    for ( int i=0; i<L1Divs; i++) {
      xc2 = values[i];
      for ( int j=1; j<=L2Divs-1; j++) {
        ++NbTickLevel2;
        xc2 = xc2+dx2;
        fillTickArray(xc2, l2TickHeight, NbTickLevel2-1, 
                      TickLevel2X, TickLevel2Y);
        if (options & GridMinor) 
	  fillTickArray(xc2, gridLength, NbTickLevel2-1, GridX2, GridY2);
      }  
    }
//
// Make the remaining heading and tailing secondary tick marks.
//
    xc2 = values[L1Divs]+dx2;
    while (xc2 <= axisLength) {
      ++NbTickLevel2;
      fillTickArray(xc2, l2TickHeight, NbTickLevel2-1, 
                    TickLevel2X, TickLevel2Y);
      if (options & GridMinor) 
        fillTickArray(xc2, gridLength, NbTickLevel2-1, 
                      GridX2, GridY2);
      xc2 = xc2+dx2;
    }
    xc2 = values[0]-dx2;
    while (xc2 >= 0) {
      ++NbTickLevel2;
      fillTickArray(xc2, l2TickHeight, NbTickLevel2-1, 
                    TickLevel2X, TickLevel2Y);
      if (options & GridMinor) 
        fillTickArray(xc2, gridLength, NbTickLevel2-1, 
                      GridX2, GridY2);
      xc2 = xc2-dx2;
    }
  }

//-------------------------------------
//
// Fill the arrays TickLevel3X and TickLevel3Y.
// TickLevel3X and TickLevel3Y are use to draw the level two tick marks.
//
  if (L3Divs >= 2) {
    TickLevel3X = new double [(NbTickLevel2+L1Divs+2)*(L3Divs-1)*2];
    TickLevel3Y = new double [(NbTickLevel2+L1Divs+2)*(L3Divs-1)*2];

    NbTickLevel3 = 0;

    for ( int i=0; i<L1Divs; i++) {
      xc2 = values[i];
      for ( int j=1; j<=L2Divs; j++) {
	xc3 = xc2;
        for ( int k=1; k<=L3Divs-1; k++) {
	  xc3 = xc3+dx3;
          ++NbTickLevel3;
          fillTickArray(xc3, l3TickHeight, NbTickLevel3-1, 
                        TickLevel3X, TickLevel3Y);
        }
        xc2 = xc2+dx2;
      }
    }
//
// Make the remaining heading and tailing secondary tick marks.
//
    xc3 = values[L1Divs]+dx3;
    int L3count = 0;
    while (xc3 <= axisLength) {
      ++L3count;
      if (L3count == L3Divs) { // do not draw a L3 tick on a L2 one
         L3count = 0 ;
      } else {
        ++NbTickLevel3;
        fillTickArray(xc3, l3TickHeight, NbTickLevel3-1, 
                      TickLevel3X, TickLevel3Y);
      }
      xc3 = xc3+dx3;
    }

    xc3 = values[0]-dx3;
    L3count = 0;
    while (xc3 >= 0) {
      ++L3count;
      if (L3count == L3Divs) { // do not draw a L3 tick on a L2 one
         L3count = 0 ;
      } else {
        ++NbTickLevel3;
        fillTickArray(xc3, l3TickHeight, NbTickLevel3-1, 
                      TickLevel3X, TickLevel3Y);
      }
      xc3 = xc3-dx3;
    }
  }

  delete []values;
}

//-----------------------------------------------------------------------------
//
// Draws the arrow
// Uses arrowLength, arrowWidth
//
void QpAxis::drawArrow()
{
  PageStrategy *ps = (PageStrategy*)getTransformation(getTransfStrategy());
  double *tabx = new double[6], *taby = new double[6];
  
  tabx[0] = x2;
  taby[0] = y2; 
  tabx[2] = x2;
  taby[2] = y2;

  if (sinPhi==1 || sinPhi==-1) {
    double yA = (y2-y1)*arrowLength/100;
    double xA = ps->invXTransform(ps->xTransform(x1)+ps->yTransform(y1-fabs(y2-y1)*arrowWidth/100)-ps->yTransform(y1)) - x1;
    tabx[1] = x2-xA; 
    taby[1] = y2-yA;
    tabx[3] = x2+xA; 
    taby[3] = y2-yA;
  } else {
    double dx = x2-x1;
    double xA = dx/cosPhi*arrowLength/100;
    double yA = ps->invYTransform(ps->yTransform(y1)-
                ps->xTransform(dx/cosPhi*arrowWidth/100)) - y1;
    tabx[1] = dx/cosPhi-xA; 
    taby[1] = -yA; 
    rotation(tabx[1],taby[1]);
    tabx[3] = dx/cosPhi-xA; 
    taby[3] = yA; 
    rotation(tabx[3],taby[3]);
  }
  
  tabx[4] = tabx[1];
  taby[4] = taby[1]; 
  tabx[5] = tabx[3];
  taby[5] = taby[3];

  createMultiLine(6,tabx,taby);
  
  delete []tabx; tabx=0;
  delete []taby; taby=0;
}

//-----------------------------------------------------------------------------
//
// Draws the axis labels
//
// This procedure uses the multiline created to draw the tick marks
// to position the labels. That is why the loop over the label drawing
// is done with a step of 2 (only every 2 points in the tab arrays are needed) 
//
void QpAxis::drawLabels(int nbPoints, double *tabx, double *taby)
{
  if (getPainter() != 0) {    
    
    if (options & Unlabeled) return;
  
    int side;
    int mul = 1;
    double angle = 0.;
    int align = Qt::AlignLeft;

    double dx = labelDist*sinPhi;
    double dy = labelDist*cosPhi;

    if (options & TickPositive) {
      if (options & SameSide) {
	side = 1;
      } else {
	side = -1;
      }
    } else {
      if (options & SameSide) {
	side = -1;
      } else {
	side = 1;
	mul = 0;
      }
    }
    if (options & Orthogonal) angle = PhiDeg-90;
    if (x1 == x2)  {
      align = Qt::AlignLeft+Qt::AlignVCenter;
    }
    
    // Todo: set the label font size so that the text is readable
    for (int i=0; i<nbPoints; i+=2) {
      QString textLabel = *(labels->at(i/2));
      if (textLabel.findRev(".") != -1)
	textLabel = textLabel.mid(0,textLabel.findRev(".")+2);
	
      QpText *t = new QpText(textLabel, this, 0., 0., 0., 0. , true, 
			     getTransfStrategy());
      
      
      // BUG: width and height seem not correct for a Zone transformation
      /*	  
	QFontMetrics fm(*(t->getQpFont()));
	int w = fm.width("What's the width of this text");
	int h = fm.height();
      */
      double width  = t->getTextObject()->getWidth(getPainter());
      double height = t->getTextObject()->getHeight(getPainter());

      if (options & Axis3d) {
	if (tabx[i]+width > tabx[i+1])
	  {
	    t->setFontSize(8);
	  }
	
	t->getTextObject()->setCoords(tabx[i], taby[i], 0.0, 0.0);
      } else {
	
	if (x1 == x2) {
	  if (options & TickPositive) {
	    t->getTextObject()->setCoords(x1+labelDist, 
					  taby[i]-height/2, 
					  width, height);
	  } else {
	    t->getTextObject()->setCoords(x1-labelDist-width, 
					  taby[i]-height/2, 
					  width, height);
	  }
	} else {
	  if (options & Orthogonal) {
	    t->getTextObject()->setCoords(tabx[i]+dx-height/2, taby[i]+side*dy, 
					  width, height);
	  } else {
	    t->getTextObject()->setCoords(tabx[i]-width/2+dx, 
					  taby[i]+side*(dy+mul*height/2), 
					  width, height);
	  }
	}
	t->getTextObject()->setAngle(angle);
	t->getTextObject()->setAlignment(align);
      }
    }
  }
}

//-----------------------------------------------------------------------------
//
// Creates a nbPoints QpMultiLine with coordinates tabx and taby
//  Parameters : nbPoints      : number of points 
//               tabx and taby : arrays of coordinates
//
QpMultiLine* QpAxis::createMultiLine(int nbPoints, double *tabx, double *taby)
{
  return (new QpMultiLine(nbPoints, tabx, taby, this, true, getTransfStrategy()));
}


//-----------------------------------------------------------------------------
//
// Creates the labels strings for an axis with numeric equidistant labels.
// By default the labeling is optimized
//
void QpAxis::computeLabels()
{
  char tmp[80];

  stepl1 = 0;
  stepl2 = 0;
  stepl3 = 0;
  
  if (options & Logarithmic) {
    if (maxVal <= 0.) {
      QPSTD::cerr << "Negative max value. Cannot draw Log axis. Max is set to 1." 
           << QPSTD::endl;
      maxVal = 1.;
    }
    if (minVal <= 0.) minVal = 0.001*maxVal;
    L1Divs = (int)log10(maxVal)-(int)log10(minVal);
    L2Divs = 9;
  } else {
    if (options & Optimized) {
      double dummy1, dummy2;
      optimize(minVal, maxVal, maxL1Divs, 
               Optimized_minVal, Optimized_maxVal, 
               L1Divs, stepl1);
      optimize(Optimized_minVal, Optimized_minVal+stepl1, maxL2Divs, 
               dummy1, dummy2, 
               L2Divs, stepl2);
      optimize(Optimized_minVal, Optimized_minVal+stepl2, maxL3Divs, 
               dummy1, dummy2, 
               L3Divs, stepl3);
    } else {
      L1Divs             = maxL1Divs;
      L2Divs             = maxL2Divs;
      L3Divs             = maxL3Divs;
      Optimized_minVal   = minVal;
      Optimized_maxVal   = maxVal;
      if (L1Divs) stepl1 = (Optimized_maxVal-Optimized_minVal)/L1Divs;
      if (L2Divs) stepl2 = stepl1/L2Divs;
      if (L3Divs) stepl3 = stepl2/L3Divs;
    }
  }

  if (!labels) {
    labels = new QPtrList<QString>();
    labels->setAutoDelete( TRUE );
  } else {
    labels->clear();
  }

  if (options & Logarithmic) {
    int j = (int)log10(minVal);
    for (int i=0;i<L1Divs+1;i++) {
      sprintf(tmp,"%g",pow(10., (double)j));
      labels->append(new QString(tmp));
      j++;
    }
  } else {
    for (int i=0;i<L1Divs+1;i++) {
      sprintf(tmp,"%g",Optimized_minVal+i*stepl1);
      labels->append(new QString(tmp));
    }
  }
}

//-----------------------------------------------------------------------------
//
// Update the axis
//
void QpAxis::update(double _minVal, double _maxVal)
{
  if (x1!=x2 || y1!=y2) {
    setValues(_minVal, _maxVal);
    changed();
  } 
}

//-----------------------------------------------------------------------------
//
// Axis is "touched" (dummy for the time being)
//
QpObject *QpAxis::touched(double /*_x*/, double /*_y*/, double /*errorX*/, 
                          double /*errorY*/, QpObject *&/*node*/)
{
  return 0;
}

//-----------------------------------------------------------------------------
//
// QpAxisNode::getAxis returns the first Axis which has this orientation
//
QpAxis* const QpAxisNode::getAxis(QString findHint)
{ 
  QpAxis *res = 0;

  QPtrList<QpObject> objectList = findType(QString("QpAxis"));
  for (objectList.first();
       objectList.current() != 0;
       objectList.next())
    {
      if (((QpAxis *) objectList.current())->getFindHint() == findHint)
	{
	  res = ((QpAxis *) objectList.current());
	  break;
	}
    }

  return res; 
}

//-----------------------------------------------------------------------------
//                       Reasonable Axis labels optimisation                 
//
//     Get reasonable values for tick marks & ensure they are
//     not plotted beyond allowed limits
//
//     Parameters:
//
//     oldMin,oldMax  : Old WMIN,WMAX .
//     BinLow,BinHigh : New WMIN,WMAX .
//     oldN           : Old NDIV .
//     nbins          : New NDIV .
//
void QpAxis::optimize(double oldMin, double oldMax, int oldN, double &BinLow, 
                      double &BinHigh, int &nbins, double &BinWidth)
{
   int lwid, kwid;
   int ntemp = 0;
   int jlog  = 0;
   double siground = 0;
   double alb, awidth, sigfig;
   double timemulti = 1;
   int roundmode =0;

   int OptionTime = 0;
   //if(strchr(fChopt.Data(),'t')) OptionTime = 1;  else OptionTime = 0;

   double AL = qwtMin(oldMin,oldMax);
   double AH = qwtMax(oldMin,oldMax);
   if (AL == AH) AH = AL+1;
//*-*-      IF oldN  ==  -1 , program uses binwidth input from calling routine
   if (oldN == -1 && BinWidth > 0 ) goto L90;
   ntemp = qwtMax(oldN,2);
   if (ntemp < 1) ntemp = 1;


L20:
   awidth = (AH-AL)/float(ntemp);
   if (awidth >= FLT_MAX) goto LOK;  //in float.h
   if (awidth <= 0)       goto LOK;

//*-*-      If time representation, bin width should be rounded to seconds
//          minutes, hours or days

   if (OptionTime && awidth>=60) { // if width in seconds, treat it as normal
//   width in minutes
      awidth /= 60; timemulti *=60;
      roundmode = 1; // round minutes (60)
//   width in hours ?
      if (awidth>=60) {
         awidth /= 60; timemulti *= 60;
         roundmode = 2; // round hours (24)
//   width in days ?
         if (awidth>=24) {
            awidth /= 24; timemulti *= 24;
            roundmode = 3; // round days (30)
//   width in months ?
            if (awidth>=30.43685) { // Mean month length in 1900.
               awidth /= 30.43685; timemulti *= 30.43685;
               roundmode = 2; // round months (12)
//   width in years ?
               if (awidth>=12) {
                  awidth /= 12; timemulti *= 12;
                  roundmode = 0; // round years (10)
               }
            }
         }
      }
   }
//*-*-      Get nominal bin width in exponential form

   jlog   = int(log10(awidth));
   if (jlog <-30 || jlog > 30) {
      BinLow   = 0;
      BinHigh  = 1;
      BinWidth = 0.01;
      nbins    = 100;
      return;
   }
   if (awidth <= 1 && (!OptionTime || timemulti==1) ) jlog--;
   sigfig = awidth*pow(10,-jlog);
//*-*-      Round mantissa

   switch (roundmode) {

//*-*-      Round mantissa up to 1, 1.5, 2, 3, or 6 in case of minutes
      case 1: // case 60
         if      (sigfig <= 1)    siground = 1;
         else if (sigfig <= 1.5 && jlog==1)    siground = 1.5;
         else if (sigfig <= 2)    siground = 2;
         else if (sigfig <= 3 && jlog ==1)    siground = 3;
         else if (jlog==0)        {siground = 1; jlog++;}
         else                     siground = 6;
         break;
      case 2: // case 12 and 24

//*-*-      Round mantissa up to 1, 1.2, 2, 2.4, 3 or 6 in case of hours or months
         if      (sigfig <= 1 && jlog==0)    siground = 1;
         else if (sigfig <= 1.2 && jlog==1)    siground = 1.2;
         else if (sigfig <= 2 && jlog==0)    siground = 2;
         else if (sigfig <= 2.4 && jlog==1)    siground = 2.4;
         else if (sigfig <= 3)    siground = 3;
         else if (sigfig <= 6)    siground = 6;
         else if (jlog==0)        siground = 12;
         else                     siground = 2.4;
         break;

//*-*-      Round mantissa up to 1, 1.4, 2, or 7 in case of days (weeks)
      case 3: // case 30
         if      (sigfig <= 1 && jlog==0)    siground = 1;
         else if (sigfig <= 1.4 && jlog==1)    siground = 1.4;
         else if (sigfig <= 3 && jlog ==1)    siground = 3;
         else                     siground = 7;
         break;
      default :

//*-*-      Round mantissa up to 1, 2, 2.5, 5, or 10 in case of decimal number
         if      (sigfig <= 1)    siground = 1;
         else if (sigfig <= 2)    siground = 2;
//         else if (sigfig <= 2.5)  siground = 2.5;
         else if (sigfig <= 5 && (!OptionTime || jlog<1))    siground = 5;
         else if (sigfig <= 6 && OptionTime && jlog==1)    siground = 6;
         else                    {siground = 1;   jlog++; }
         break;
   }

   BinWidth = siground*pow(10,jlog);
   if (OptionTime) BinWidth *= timemulti;

//*-*-      Get new bounds from new width BinWidth

L90:
   alb  = AL/BinWidth;
   lwid   = int(alb);
   if (alb < 0) lwid--;
   BinLow     = BinWidth*float(lwid);
   alb        = AH/BinWidth + 1.00001;
   kwid = int(alb);
   if (alb < 0) kwid--;
   BinHigh = BinWidth*float(kwid);
   nbins = kwid - lwid;
   if (oldN == -1) goto LOK;
   if (oldN <= 5) {          //*-*-    Request for one bin is difficult case
      if (oldN > 1 || nbins == 1)goto LOK;
      BinWidth = BinWidth*2;
      nbins    = 1;
      goto LOK;
   }
   if (2*nbins == oldN) { ntemp++; goto L20; }

LOK:
   double oldBinLow = BinLow;
   double oldBinHigh = BinHigh;
   int oldnbins = nbins;

   double atest = BinWidth*0.0001;
   if (fabs(BinLow-oldMin)  >= atest) { BinLow  += BinWidth;  nbins--; }
   if (fabs(BinHigh-oldMax) >= atest) { BinHigh -= BinWidth;  nbins--; }

   if (OptionTime && nbins==0) {
      nbins = 2*oldnbins;
      BinHigh = oldBinHigh;
      BinLow = oldBinLow;
      BinWidth = (oldBinHigh - oldBinLow)/nbins;
      double atest = BinWidth*0.0001;
      if (fabs(BinLow-oldMin)  >= atest) { BinLow  += BinWidth;  nbins--; }
      if (fabs(BinHigh-oldMax) >= atest) { BinHigh -= BinWidth;  nbins--; }
   }
}
