#include "Qplotter/qp/qpcloudplot2d.h"
#include "Qplotter/qp/qpzone.h"

QpCloudPlot2D::QpCloudPlot2D(QpObject *_z):
  QpNode<QpObject>(new QString("QpCloudPlot2D"),_z), st (0) , 
  markerSize(0) 
{
}

QpCloudPlot2D::QpCloudPlot2D(QpObject *_z, ICloud2D *_v):
  QpNode<QpObject>(new QString("QpCloudPlot2D"),_z), st (0), 
  markerSize(0) {
    init(_v);
}

void QpCloudPlot2D::newCurve(ICloud2D *_v) {
    init(_v);
}

void QpCloudPlot2D::init(ICloud2D *cl2)
{
   theData = cl2;
   double xmin = cl2->lowerEdgeX();
   double xmax = cl2->upperEdgeX();
   double ymin = cl2->lowerEdgeY();
   double ymax = cl2->upperEdgeY();
   // Give the zone a chance to update axis
   getZone()->updateX(xmin,xmax);
   getZone()->updateY(ymin,ymax);
}


/// The draw/print function
void QpCloudPlot2D::paint (QPainter *painter)
{
  changeStyle(painter);

  // Getting zone limits
  ZoneStrategy *zt = (ZoneStrategy*)getTransformation(QpTransformation::Zone);
  double xmin = zt->getXzMap()->getContent1();
  double xmax = zt->getXzMap()->getContent2();
  double ymin = zt->getYzMap()->getContent1();
  double ymax = zt->getYzMap()->getContent2();

  double xC,yC;
  int pixX, pixY;
  // Transform coords in pixel 
  QpTransformation *transf = getTransformation(getTransfStrategy());  

  int dataEntries = theData->entries();
  int i;

  for (i=0;i<dataEntries;i++)
    {
      xC = theData->valueX(i);
      yC = theData->valueY(i);
      
      if (!((yC < ymin) || (yC > ymax) || (xC < xmin) || (xC > xmax ))) {
	pixX = transf->xTransform(xC);
	pixY = transf->yTransform(yC);
	if (markerSize != 0) {
	  d_pa.resize(4);
	  d_pa.setPoint(0, pixX-markerSize, pixY-markerSize);
	  d_pa.setPoint(1, pixX+markerSize, pixY-markerSize);
	  d_pa.setPoint(2, pixX+markerSize, pixY+markerSize);
	  d_pa.setPoint(3, pixX-markerSize, pixY+markerSize);
	  painter->drawPolygon(d_pa);
	}
	else
	  painter->drawPoint(pixX , pixY);
      }
    }
  // Restore default pen/brush/font
  restoreStyle(painter);
}

/// The draw function
void QpCloudPlot2D::draw()
{
  paint(getPainter());
}

/// The print function
void QpCloudPlot2D::print()
{
  paint(getPrinter());
}

/// Determine the point [x +/- errorX, y +/- errorY] is in the object
QpObject *QpCloudPlot2D::touched(double _x, double _y, double errorX, double errorY, QpObject *&node)
{
  QpObject *res = QpNode<QpObject>::touched(_x,_y,errorX,errorY,node);
  if (res != 0) 
    node=this; // The returned node will be the curve
  return res;
}

void QpCloudPlot2D::updateSt()
{
  if (st != 0) 
    st->setPen(*(getQpPen()));
}



