#include "Qplotter/qp/qpcolorplot.h"

QpPalette::QpPalette(){
  noOfLevels = defaultColorLevels;
  defaultPalette();
}

QpPalette::QpPalette(TYPE_PALETTE paletteType, int levels) {
  if (levels>240) 
    levels = 240 ;
  noOfLevels = levels;
  
  switch (paletteType) {
   case DEFAULT:
     defaultPalette();
     break;
   case GREYSCALE :
     greyScale();
     break;
   case  GEOGRAPHICAL:
     geographical();
     break;
   default:
     break;
   }
 
}

QpPalette::~QpPalette() {
  delete [] palette;
}

void QpPalette::defaultPalette() {
  palette = new QColor[defaultColorLevels];
  palette[0]= Qt::white ;
  palette[1]= Qt::yellow ;
  palette[2]= Qt::cyan ;
  palette[3]= Qt::green ;
  palette[4]= Qt::magenta ;
  palette[5]= Qt::red ;
  palette[6]= Qt::blue ;
  palette[7]= Qt::black ;
}

void QpPalette::greyScale(){
  // Is going from black to white. RGB color components increase in lockstep
  double component=0;
  // Subtract -1 to noOfLevels so that the last color is white (R=G=B=255)
  double step = 255./static_cast<double>(noOfLevels-1);
 
  palette = new QColor[noOfLevels];
  // It has to go from black to white, in the number of steps I want.
  for (int i=0;i<noOfLevels;i++) {
    int intComponent = static_cast<int>(component);
    palette[i]= QColor(intComponent,intComponent,intComponent);
    component += step;
  }
}
    
void QpPalette::geographical() {
  // This kind of Palette is going from blue to green and after to red.
  double realHue = 240.;
  int S=255;
  int V=200;
  // Subtract -1 to noOfLevels so that the last color is red (Hue=240)
  double step = realHue/static_cast<double>(noOfLevels-1) ;
 
  palette = new QColor[noOfLevels];
  // Using HSV coding rather than RGB to have constant luminosity
  for (int i=0;i<noOfLevels;i++) {
    QColor qc;
    qc.setHsv(static_cast<int>(realHue),S,V);
    palette[i]= qc;
    realHue -= step;
  }
}


QColor QpPalette::getColor (int i) {
  assert((i>=0) && (i<noOfLevels));
  return palette[i];
}

Level::Level (int length) {
  xS = new double[length];
  yS = new double[length];  
  wS = new double[length];
  hS = new double[length];  
  noOfFaces = 0;
}

Level::~Level () {
  delete[] xS;
  delete[] yS;
  delete[] wS;
  delete[] hS;
}

void Level::addFace(double x,double y,double w,double h) {
  xS[noOfFaces] = x;
  yS[noOfFaces] = y;
  wS[noOfFaces] = w;
  hS[noOfFaces] = h;
  noOfFaces++;
}




