#include <qstring.h>
#include <qnamespace.h>
#include "Qplotter/qp/qpcommontext.h"
#include "Qplotter/qp/qptextmetrics.h"

QpCommonText::QpCommonText(QString _data,QpObject *z,double _x,double _y, 
			   double _w, double _h , bool addOnParent,
			   QpTransformation::tStrategy st) : 
  QpObject(new QString("QpCommonText"),z,addOnParent,st), data(_data)
{
  setCoords(_x, _y, _w, _h);
  align = Qt::AlignTop | Qt::AlignLeft | Qt::DontClip;
  angle=0;
  addBaseLine = false;
}

void QpCommonText::setCoords(double _x, double _y, double _w, double _h) {
  xC=_x; 
  yC=_y;
  bRectWidth  = _w; 
  bRectHeight = _h;
}

/* Append text string to the current text string of this commontext object.
 */
void QpCommonText::appendData(QString more) {
  data += more;
}

void QpCommonText::rotatePainter(QPainter *painter,int &transfX, int &transfY)
{
  if (getAngle() != 0) {
    /** The point to start drawing from must be transformed according
      to the rotation. Unfortunately doing this on the painter gives
      back a "wrong" point. This is due to the fact that we set the
      printer viewport and window so to keep the screen ratio and the
      same resolution as on screen. In order to get the same transformed
      point on screen and on paper, the drawing coordinates are transformed
      using the screen painter in both cases. This is a workaround, if 
      anybody can figure out to take care of viewport and window changes
      in the proper manner that would be much better...
    */
    {
      QPoint point1(transfX,transfY);
      QPainter *screenP = getPainter();
      screenP->save();
      screenP->rotate(getAngle());
      point1 = screenP->xFormDev(point1);
      screenP->restore();
      transfX = point1.x(); 
      transfY=point1.y();
    }

    // Rotate painter
    painter->save();
    painter->rotate(getAngle());
  }
}

void QpCommonText::restorePainter(QPainter *painter)
{
    if (getAngle() != 0)
      // Restore painter
      painter->restore();
}

// Alignment approximation...
int QpCommonText::getXOffset (int boundWidth, QPainter *painter) const {
  int offset=0;
  // Bounding rect defined?
  if ((boundWidth != 0) && (boundWidth > getPixWidth(painter))) {
    // Left alignment -> no changes
    if (( getAlignment() & Qt::AlignLeft) == 0) {
      if (( getAlignment() & Qt::AlignRight) != 0) {
	// Right alignment
	offset = boundWidth - getPixWidth(painter);
      } else {
	// Center alignment
	if ((( getAlignment() & Qt::AlignHCenter) != 0) ||
	    (( getAlignment() & Qt::AlignCenter) != 0))
	  offset = (boundWidth - getPixWidth(painter))/2;
      }
    }
  }
  return offset;
}

int QpCommonText::getYOffset (int boundHeight, QPainter *painter) const {
  int offset = 0;
  // Bounding rect defined?
  if (getH() != 0 ) {   
    if (boundHeight >= getPixHeight(painter)) {
      // Bottom alignment -> no changes
      if (( getAlignment() & Qt::AlignBottom) == 0) {
	if (( getAlignment() & Qt::AlignTop) != 0) {
	  // Top alignment
	  offset = -(boundHeight - getPixHeight(painter));
	} else {
	  // Center alignment
	  if ((( getAlignment() & Qt::AlignVCenter) != 0) ||
	    (( getAlignment() & Qt::AlignCenter) != 0))
	    offset = -( boundHeight - getPixHeight(painter))/2;
	}
      }
    } 
  }
  if (addBaseLine)
    offset += -(QpTextMetrics::getLineSpacing(painter) - 
		QpTextMetrics::getAscent(painter));
  return offset;
}


double QpCommonText::getWidth(QPainter *painter) const {
  QpTransformation *tr = getTransformation(getTransfStrategy());
  return tr->getXpMap()->invWidthTransform(getPixWidth(painter));
}

double QpCommonText::getHeight(QPainter *painter) const {
  QpTransformation *tr = getTransformation(getTransfStrategy());
  return tr->getYpMap()->invWidthTransform(getPixHeight(painter));
}
