#include "Qplotter/qp/qpcurve1D.h"
#include "Qplotter/qp/qpzone.h"

const int defaultMarkerSize = 10;

QpMarker * QpErrorShaper::shape (QpObject *_z, double x, double y, 
				 double width, double height,double xCross, 
				 double yCross, QpMarker::SymStyle /* styl */)
{
  QpMarker *m = new QpScalableMarker(_z, false);
  m->setGeometry(x,y,width,height);
  m->setLineStyle(QpMarker::ErrorCross);
  m->setLineGeometry(xCross,yCross, width, height);
  m->setSymStyle(QpMarker::None);
  return m;
}

QpMarker * QpSymbolShaper::shape (QpObject *_z, double x, double y,
				  double width, double height,
				  double /* xCross */ , double /* yCross */, 
				  QpMarker::SymStyle styl)
{
  QpMarker *m = new QpScalableMarker(_z, false);
  m->setGeometry(x,y,width,height);
  m->setSymStyle(styl);
  return m;
}

QpMarker *QpFixedMarkerShaper::shape(QpObject *_z, double x, double y, 
				     double /* width */, double /* height */,
				     double /* xCross */, double /* yCross */,
				     QpMarker::SymStyle styl)
{
  QpMarker *m = new QpFixedSizeMarker(_z, false);
  m->setGeometry(x,y,msize,msize);
  m->setSymStyle(styl);
  return m;
}

QpMarker *QpErrorSymbolShaper::shape(QpObject *_z, double x, double y, double width, double height, double xCross, double yCross, QpMarker::SymStyle styl)
{
  
  QpMarker *m = new QpFixedSizeMarker(_z, false);
  m->setGeometry(x,y,msize,msize);
  m->setSymStyle(styl);
  m->setLineStyle(QpMarker::ErrorCross);
  m->setLineGeometry(xCross, yCross, width, height);
  return m;
}

// This constructor does not add the curve to the parent zone's list
// (although it sets the parent relation)
QpCurve1D::QpCurve1D(QpObject *_z) : 
  QpNode<QpObject>(new QString("QpCurve1D"),_z,false), st(0)
{
  pl=0;
  pm=0;
  style=QpMarker::Ellipse;
  mSize = defaultMarkerSize;
  type = HISTO;
  clipping=true;
  mSize = defaultMarkerSize;
}

QpCurve1D::QpCurve1D(QpObject *_z, Anaphe::IVector &VoP ,TYPE _t): 
  QpNode<QpObject>(new QString("QpCurve1D"),_z), st(0)
{
  pl=0;
  pm=0;
  style=QpMarker::Ellipse;
  clipping=true;
  mSize = defaultMarkerSize;
  type = _t;
  init(VoP);
}

/*
QpCurve1D::QpCurve1D(QpObject *_z, QVector<double> *&VoP ,TYPE _t): 
  QpNode<QpObject>(new QString("QpCurve1D"),_z), st(0)
{
  pl=0;
  pm=0;
  style=QpMarker::Ellipse;
  clipping=true;
  mSize = defaultMarkerSize;
  type = _t;
  // Todo: convert Anaphe::IVector to QVector
  //  init(VoP);
}
*/

QpCurve1D::~QpCurve1D()
{
  // Since summaryTable st is not created in these class, we don't have to delete it there.
}


void QpCurve1D::newCurve(Anaphe::IVector &VoP ,TYPE _t, bool first) {
  type = _t;
  newCurve(VoP,first);
  init(VoP);
}

void QpCurve1D::setType( TYPE _t) {
  type = _t;
}

void QpCurve1D::newCurve(Anaphe::IVector &VoP, bool first ) {
  // First add the curve to its parent zone list
  if(first)
    ((QpNode<QpObject> *) getParent() )->insertElementFirst(this);
  else 
    ((QpNode<QpObject> *) getParent() )->addElement(this);
  init(VoP);
}

void QpCurve1D::setSymSize(int scf) {
  mSize = scf;
  if (pm != 0) 
    pm->setSymbolSize(mSize,mSize);
}

void QpCurve1D::makeLine(Anaphe::IVector &VoP,double &xmin,double &xmax, double &ymin,
			 double &ymax)
{
  double *xS, *yS;
  size_t ptNbr = VoP.nPoints();  
  size_t i;
  xS = new double[ptNbr];
  yS = new double[ptNbr];
  // Retrieve coordinates in a form suitable for polyline
  for (i=0;i<ptNbr;i++) {
    Anaphe::IPoint *pt1D = VoP.point(i);
    xS[i] = pt1D->coordinate(0);
    yS[i] = pt1D->value();
  }
  // Remove previous polyline if existing.auto-deletion must be set 
  if (pl != 0) {
    removeElement(pl); 
    pl=0;
  }
  // Create a QpPolyline node first (so that we can assign it a pen attribute)
  pl=new QpNode<QpPolyShape>(new QString("QpNode<QpPolyShape>"),this,true);
  // Now create a new polyline and insert it in the QpNode
  QpPolyShape *qpl ;
  if (type != FILLED_SMOOTH) 
    qpl = new QpPolyLine(ptNbr, xS, yS, pl, true);
  else
    qpl = new QpPolygon(ptNbr, xS, yS, pl, true);
  if (type & SMOOTH)
    qpl->setSmooth();
  
  // Find min/max on X
  xmin = xS[0];
  xmax = xS[ptNbr-1];
  for (i=0;i<ptNbr;i++) {
    if (xS[i] < xmin) xmin = xS[i];
    if (xS[i] > xmax) xmax = xS[i];
  }

  ymin = VoP.minValue()->value();
  ymax = VoP.maxValue()->value();

  delete [] xS; xS=0; 
  delete [] yS; yS=0;
}

void QpCurve1D::makeHisto(Anaphe::IVector &VoP,double &xmin,double &xmax,double &ymin,
			  double &ymax) 
{
  double *xS, *yS;
  size_t ptNbr = VoP.nPoints();  
  size_t i=0;
  // We need (at least) N*2 points to make the stair curve.
  // We keep room for 2 additional points so that the curve can be closed
  // if that is required.
  xS = new double[ptNbr*2+2];
  yS = new double[ptNbr*2+2];

  // Retrieve coordinates in a form suitable for stairs polyline
  while ( i < ptNbr ) {
    Anaphe::IPoint *pt1D = VoP.point(i);
    xS[2*i+1] = pt1D->coordinate(0)-pt1D->coordMinus(0);
    xS[2*i+2] = pt1D->coordinate(0)+pt1D->coordPlus(0);
    yS[2*i+1] = pt1D->value();
    yS[2*i+2] = yS[2*i+1];
    i++;
  }
  // min/max on X are defined by the order of the vector (left to right)
  xmin = xS[1];
  xmax = xS[2*ptNbr];
  ymin = VoP.minValue()->value();
  ymax = VoP.maxValue()->value();
  // Additional points to close the curve
  xS[0]=xS[1];
  yS[0]=0;
  xS[2*ptNbr+1]=xS[2*ptNbr];
  yS[2*ptNbr+1]=0;

  // Remove previous polyline if existing.auto-deletion must be set 
  if (pl != 0) removeElement(pl);
  pl=new QpNode<QpPolyShape>(new QString("QpNode<QpPolyShape>"),this,true);
  if (type & FILLED)
    pl->addElement(new QpPolygon(2*ptNbr+2, xS, yS, pl, false));
  else 
    pl->addElement(new QpPolyLine(2*ptNbr+2, xS, yS, pl, false));

  delete [] xS; 
  delete [] yS;
}

void QpCurve1D::makeMarker(Anaphe::IVector &VoP,double &xmin,double &xmax,double &ymin,
			   double &ymax,QpSymbolShaper &symMaker)
{
  size_t ptNbr = VoP.nPoints();  
  size_t i;
  // Remove previous markers if existing. auto-deletion must be set 
  if (pm != 0) { pm->clearList(); }
  else pm = new QpPolyMarker(this, true);

  ymin = VoP.minValue()->value() - VoP.minValue()->vMinus();
  ymax = VoP.maxValue()->value() - VoP.minValue()->vPlus();
  // Retrieve coordinates in a form suitable for a markers set
  for(i=0; i<ptNbr; i++){
    double x,y,w,h,xc,yc;
    Anaphe::IPoint *pt1D = VoP.point(i);
    x = pt1D->coordinate(0);
    y = pt1D->value();
    w = pt1D->coordMinus(0)+pt1D->coordPlus(0);
    h = pt1D->vMinus()+pt1D->vPlus();
    xc = x-pt1D->coordMinus(0);
    yc = y+pt1D->vPlus();
    pm->addElement(symMaker.shape(this,x,y,w,h,xc,yc,style));
    // min/max on Y take care of error bars. If a marker without error
    // bars is used, these min/max values are actually overestimating the
    // limits.
    if(y+pt1D->vPlus()>ymax) ymax = y+pt1D->vPlus();
    if(y-pt1D->vMinus()<ymin) ymin = y-pt1D->vMinus();
  }
  // min/max on X are defined by the order of the vector (left to right)
  // Might be modified if we allow non-ordered vector drawn as polymarker
  Anaphe::IPoint *pt1D =  VoP.point(0);
  xmin = pt1D->coordinate(0)-pt1D->coordMinus(0);
  pt1D = VoP.point(ptNbr-1);
  xmax = pt1D->coordinate(0)+pt1D->coordPlus(0);
}

void QpCurve1D::setSymStyle(QpMarker::SymStyle _style) {
  style=_style;
  if (pm != 0)
    pm->setMarkerStyle(style);
}

void QpCurve1D::init(Anaphe::IVector &VoP)
{
  if ((VoP.dimension() == 1) && (VoP.nPoints() != 0)) {
   if (type == MARKER_ERROR) {
     QpErrorSymbolShaper shaper(mSize);
     makeMarker(VoP,xMin ,xMax, yMin, yMax, shaper);
   }
   else {
     if ((type & LINE) || (type & SMOOTH))
       makeLine(VoP,xMin ,xMax, yMin, yMax); 
     if (type & HISTO)
       makeHisto(VoP,xMin ,xMax, yMin, yMax); 
     if (type & ERROR) {
       QpErrorShaper shaper;     
       makeMarker(VoP,xMin ,xMax, yMin, yMax, shaper);
     }
     if (type & MARKER) {
       QpFixedMarkerShaper shaper(mSize);
       makeMarker(VoP,xMin ,xMax, yMin, yMax, shaper);
       // Recompute ymin/ymax to ignore error bars size
       yMin = VoP.minValue()->value();
       yMax = VoP.maxValue()->value();
     }
   }
   // Give the zone a chance to update X axis
   if (getZone()) getZone()->updateX(xMin,xMax);
   // Special case when ymin==ymax, i.e. the curve is constant
   // Is the curve constant ?
   if (yMax == yMin) {
     if (yMin == 0) {
       // The curve is all zeros
       yMin = -1;
       yMax = 1;
     } else {
       // The curve is a constant != 0
       if (yMax > 0) {
	 // The constant is > 0
	 yMin = 0;
	 yMax = yMax *2;
       } else {
	 // The constant is <  0
	 yMin = yMin*2;
	 yMax = 0;
       }
     }     
   } else {
     if  (yMax > 0) {   
       if  (yMin > 0 )
	 yMin = 0; // Positive values: anchor min to zero
       // Adjust yMax to have some 10% space on top of the curve
       yMax = yMax + fabs((yMax-yMin)/10);
     }
     if  (yMin < 0) {
       if  (yMax < 0 )
	 yMax = 0; // Negative values: anchor max to zero
       // Adjust yMin to have some 10% space on bottom of the curve
       yMin = yMin - fabs((yMax-yMin)/10);
     }
   }
   // Give the zone a chance to update Y axis
   if (getZone() != 0) 
     getZone()->updateY(yMin,yMax);
   updateSt();
  }
}

void QpCurve1D::updateSt()
{
  if (st != 0) {
    if (pl != 0 ) 
      st->setPen(*(pl->getQpPen()));
    else 
      st->setPen(*(getQpPen()));
    if (type & (MARKER | FILLED | FILLED_HISTO | FILLED_SMOOTH))  {
      st->setMarker(style);
      if (getQpBrush())
	st->setBrush(*(getQpBrush()));
    }
    if (type & MARKER) {
      st->isMarker = true;      
    } else {
      st->isMarker = false;       
    }    
    if (type & (FILLED | FILLED_HISTO | FILLED_SMOOTH)) {
      st->isFilled = true ;      
    } else {
      st->isFilled = false ;      
    }    
  }
}

// The draw function, improved to use clipping
void QpCurve1D::draw()
{
  if (clipping) { 
    getPainter()->setClipRect(getZone()->getLimits()); 
    getPainter()->setClipping(true); 
  }
  QpNode<QpObject>::draw();
  if (clipping) 
    getPainter()->setClipping(false); 
}
/// The print function, improved to use clipping
void QpCurve1D::print()
{
  /** The clipping is different in the printing case... 
    In fact, the region for clipping must be given to the painter in device coordinates.
    So you must transform the user coordinates (equal to pixels) to device/ps coordinates
    by using xForm(QRect).
  */ 
  if (clipping) { 
    getPrinter()->setClipRect(getPrinter()->xForm(getZone()->getLimits())); 
    getPrinter()->setClipping(true); 
  }
  QpNode<QpObject>::print();
  if (clipping) 
    getPrinter()->setClipping(false);
}


/// Determine the point [x +/- errorX, y +/- errorY] is in the object
QpObject *QpCurve1D::touched(double _x, double _y, double errorX, double errorY, QpObject *&node)
{
  QpObject *res = QpNode<QpObject>::touched(_x,_y,errorX,errorY,node);
  if (res != 0) 
    node=this; // The returned node will be the curve
  return res;
}





