#ifndef QPCURVE1D_H
#define QPCURVE1D_H

/** %%%
    
  Usage:

  TODO:

*/

/*
namespace Anaphe 
{
}
*/

#include "Interfaces/IVector.h"
#include <qvector.h>
#include "Qplotter/qp/qpnode.cpp"
#include "Qplotter/qp/qpsummaryinfo.h"
#include "Qplotter/qp/qppolyline.h"
#include "Qplotter/qp/qppolymarker.h"

/** A pure virtual class to accomodate different strategies of marker 
  definition 
*/
class QpSymbolShaper {
public:
  QpSymbolShaper() {};
  virtual QpMarker *shape (QpObject *_z, double x, double y, double width, 
      double height,double xCross, double yCross, QpMarker::SymStyle styl) = 0;
};

/** A class defining the strategy to create an error cross marker
*/
class QpErrorShaper : public QpSymbolShaper {
public:
  virtual QpMarker *shape (QpObject *_z, double x, double y, double width, 
      double height,double xCross, double yCross, QpMarker::SymStyle styl);
};

/** A class defining the strategy to create an error cross + symbol marker
*/
class QpErrorSymbolShaper : public QpSymbolShaper {
public:
  QpErrorSymbolShaper (int scaleFact) {
    msize = scaleFact;
  }
  virtual QpMarker *shape (QpObject *_z, double x, double y, double width, 
      double height,double xCross, double yCross, QpMarker::SymStyle styl);
private:
  // Marker size
  int msize;
};

/** A class defining the strategy to create a scalable "symbol" marker
*/
class QpMarkerShaper : public QpSymbolShaper {
public:
  virtual QpMarker *shape (QpObject *_z, double x, double y, double width, 
      double height,double xCross, double yCross, QpMarker::SymStyle styl);
};

/** A class defining the strategy to create a fixed size "symbol" marker
*/ 

class QpFixedMarkerShaper : public QpSymbolShaper {
public:
  QpFixedMarkerShaper (int size) {
    msize = size;
  }
  virtual QpMarker *shape (QpObject *_z, double x, double y, double width, 
      double height,double xCross, double yCross, QpMarker::SymStyle styl);
private:
  // Marker size
  int msize;
};

/** Class for 1D curves as histogram / line / error bars / markers.
*/

class QpCurve1D : public QpNode<QpObject>
{
public:
  enum TYPE {HISTO=1, LINE=2, ERROR=4, HISTO_ERROR=5, LINE_ERROR=6, MARKER=8 ,
  HISTO_MARKER=9,LINE_MARKER=10,MARKER_ERROR=12,SMOOTH=16,SMOOTH_ERROR=20,
  SMOOTH_MARKER=24,FILLED=32,FILLED_HISTO=33,FILLED_SMOOTH=48};
public:
  /// Constructors
  QpCurve1D(QpObject *_z);
  QpCurve1D(QpObject *_z, Anaphe::IVector &_v, TYPE _t=HISTO);
  QpCurve1D(QpObject *_z, QVector<double> *&_v, TYPE _t=HISTO);
  /// Destructor
  ~QpCurve1D() ;
  
  /// Insert a new curve
  void newCurve( Anaphe::IVector &_v, TYPE _t, bool first=false);
  void newCurve( Anaphe::IVector &_v, bool first=false);
  /// Set the curve type
  void setType (TYPE _t);
  /// Get the type
  const TYPE getType () const {return type;};
  /// Change the markers symbol
  void setSymStyle(QpMarker::SymStyle _style);
  /// Get the markers symbol
  QpMarker::SymStyle getSymStyle() const { return style; }
  /// Change the fixed markers size
  void setSymSize(int scf);
  /// Return the summary table
  SummaryTable *getSummaryTable() const { return st; }
  /// Set a SummaryTable
  void setSummaryTable(SummaryTable *_st) {
    delete st ;
    st=_st;
  }
  /// Set clipping on/off
  void setClipping(bool c)  {clipping=c;}
  /// Check whether clipping is on.
  bool hasClipping() const {return clipping;}
  /// Draw the curve
  virtual void draw();
  /// Print the curve
  virtual void print();
  /// Determine the point [x +/- errorX, y +/- errorY] is in the object
  virtual QpObject *touched(double _x, double _y, double errorX, double errorY, QpObject *&node);
  /// Update summary table with pen/marker information
  void updateSt();
protected:
  /// Create the polyline for a (almost) continuos line.
  void makeLine(Anaphe::IVector &VoP,double &xmin, double &xmax, double &ymin, 
		double &ymax);
  /// Create the polyline for histogram representation
  void makeHisto(Anaphe::IVector &VoP,double &xmin, double &xmax, double &ymin, 
		 double &ymax);
  /// Create the set of markers for marker or error representations
  void makeMarker(Anaphe::IVector &VoP,double &xmin, double &xmax, double &ymin, 
		  double &ymax, QpSymbolShaper &symMaker);
private:
  void init(Anaphe::IVector &VoP);
  TYPE type;
  SummaryTable *st;
  // Pointer on the list of polyline used in case of HISTO/LINE type
  QpNode<QpPolyShape> *pl;
  // Pointer on the list of markers used in case of ERROR type
  QpPolyMarker *pm;
  // Symbol used for the marker
  QpMarker::SymStyle style;
  // Size for fixed markers
  int  mSize;
  // Clipping enabled/disabled
  bool clipping;
  // Curve min and max
  double xMin;
  double xMax;
  double yMin;
  double yMax;
};

#endif





