#include <iostream>

#include "Qplotter/qp/qpcurve2D.h"
#include "Qplotter/qp/qpcurve3D.h"
#include "Qplotter/qp/qppolyline.h"

#if defined __hpux || (defined __sun && __SUNPRO_CC < 0x500)
#define QPSTD
#else
#define QPSTD std
#endif
#include <vector>

#include <qvector.h>


// Used to split the Z range in color levels so that the max value is included
const double eps=1e-6;

/******
const int MAXCONTOUR  = 104;

#define true  1
#define false 0

float ZMIN;
float ZMAX;
float DZ;
 ******/

// This constructor does not add the curve to the parent zone's list
// (although it sets the parent relation)
QpCurve2D::QpCurve2D(QpObject *_z):
  QpNode<QpObject>(new QString("QpCurve2D"),_z,false), st (0)  {
    mk=0;
/******
    pl=0;
 ******/
    clipping=true;
    type = BOX;
    thePalette = new QpPalette();
}

QpCurve2D::QpCurve2D(QpObject *_z, Anaphe::IVector &_v, TYPE2D _t):
  QpNode<QpObject>(new QString("QpCurve2D"),_z), type(_t), st (0)
{
    mk=0;
    thePalette = new QpPalette();
    init(_v);
    clipping=true;
}

/*
QpCurve2D::QpCurve2D (QpObject *_z,
	   QVector<double> *&_vx,
	   QVector<double> *&_vy,
	   TYPE2D _t)
  : QpNode<QpObject>(new QString("QpCurve2D"),_z), type(_t), st (0)
{
    mk=0;
    thePalette = new QpPalette();
    // Todo convert VOP to QVector 
    //    init(_v);
    clipping=true;
}
*/

QpCurve2D::~QpCurve2D(){
  delete thePalette;
}

void QpCurve2D::setType(TYPE2D _t) {
    type = _t;
}

void QpCurve2D::newCurve(Anaphe::IVector &_v, TYPE2D _t, bool first) {
    type = _t;
    newCurve(_v,first);
}

void QpCurve2D::newCurve(Anaphe::IVector &_v, bool first) {
  // First add the curve to its parent zone list
  if (first)
    ((QpNode<QpObject> *) getParent() )->insertElementFirst(this);
   else
     ((QpNode<QpObject> *) getParent() )->addElement(this);
  init(_v);
}

void QpCurve2D::changePalette(QpPalette::TYPE_PALETTE typ, int noOfLevels) {
  delete thePalette;
  thePalette = new QpPalette(typ,noOfLevels);
}

void QpCurve2D::init(Anaphe::IVector &VoP)
{
  if ((VoP.dimension() == 2) && (VoP.nPoints() != 0)) {
    double xmin, xmax, ymin, ymax;
    switch (type) {
    case BOX:
      makeBox(VoP,xmin, xmax, ymin, ymax, 0);
      break;
    case ELLIPSE:
      makeBox(VoP,xmin, xmax, ymin, ymax, 1);
      break;
    case COLOR:
      makeColor(VoP,xmin, xmax, ymin, ymax);
      break;
    default:
      break;
    }
    // Give the zone a chance to update axis
    getZone()->updateX(xmin,xmax);
    getZone()->updateY(ymin,ymax);
  }
}

void QpCurve2D::makeBox(Anaphe::IVector &VoP, double &xmin, double &xmax, 
			double &ymin, double &ymax, int opt)
{
   double *xS, *yS, *wS, *hS;
   double zratio,zmin,zmax;
   size_t i;
   int box=0;
   size_t boxNbr=VoP.nPoints();
   xS = new double[boxNbr];
   yS = new double[boxNbr];
   wS = new double[boxNbr];
   hS = new double[boxNbr];

   zmin = VoP.minValue()->value();
   zmax = VoP.maxValue()->value();
   for (i=0;i<boxNbr;i++) {
     Anaphe::IPoint *pt2D = VoP.point(i);
     // Zero values are suppressed
     if(pt2D->value() != 0.) {
       if (zmax != zmin) // To avoid division by zero...
	 zratio = sqrt((pt2D->value()-zmin)/(zmax-zmin));
       else
	 zratio = .98; // Avoid all boxes touching each other ...
       xS[box] = pt2D->coordinate(0)+
	 zratio*((pt2D->coordPlus(0)-pt2D->coordMinus(0))/2);
       yS[box] = pt2D->coordinate(1)+
	 zratio*((pt2D->coordPlus(1)-pt2D->coordMinus(1))/2);
       wS[box] = zratio*(pt2D->coordPlus(0)+pt2D->coordMinus(0));
       hS[box] = zratio*(pt2D->coordPlus(1)+pt2D->coordMinus(1));
       box++;
     }
   }
   // box is the number of non-empty boxes
   // Compute min/max along X & Y 
   limit(VoP,xmin,xmax,ymin,ymax);

   // Warning : "zone list"'s auto-deletion must be set
   if (mk != 0) 
     getZone()->removeElement(mk);
   mk = new QpPolyFace(box, xS, yS, wS, hS, true, this, true,QpTransformation::Zone);
   mk->FaceMode = opt;

   delete [] xS; delete [] yS; delete [] wS; delete [] hS;
}




// Color plot
void QpCurve2D::makeColor(Anaphe::IVector &VoP, double &xmin, double &xmax, 
			double &ymin, double &ymax)
{
  double zmin,zmax,zstep;
  int i;
  // The palette defineds the no. of levels so far...
  int noOfLevels = thePalette->size();

  int boxNbr=VoP.nPoints();
  // Allocate store for face dimensions for each level
  QPSTD::vector<Level *> dataLevels;
  for (i=0;i<noOfLevels;i++) 
    dataLevels.push_back(new Level(boxNbr));

  // Compute parameters to evaluate which level the value belongs to
  zmin = VoP.minValue()->value();
  zmax = VoP.maxValue()->value();
  // Use eps to make the max value fit in the upper level
  zstep = (zmax+eps-zmin)/noOfLevels;
  // For each value find out which level belongs to and create face features
  for (i=0;i<boxNbr;i++) {
    Anaphe::IPoint *pt2D = VoP.point(i);
    // Which level does the point belong to?
    int colorIndex = static_cast<int>((pt2D->value()-zmin)/zstep);
    dataLevels[colorIndex]->
      addFace(pt2D->coordinate(0)+(pt2D->coordPlus(0)-pt2D->coordMinus(0))/2,
	      pt2D->coordinate(1)+(pt2D->coordPlus(1)-pt2D->coordMinus(1))/2,
	      pt2D->coordPlus(0)+pt2D->coordMinus(0) ,
	      pt2D->coordPlus(1)+pt2D->coordMinus(1) );
  }
  // All non-empty levels result in a QpPolyface instance
  for (i=0;i<noOfLevels;i++) {
    if (dataLevels[i]->size() > 0)
      // "Zero-suppression"
      if (thePalette->getColor(i) != Qt::white) {
	QpPolyFace *pf = 
	  new QpPolyFace(dataLevels[i]->size(), dataLevels[i]->getX(), 
			 dataLevels[i]->getY(), dataLevels[i]->getW(), 
			 dataLevels[i]->getH(), true, this, true,
			 QpTransformation::Zone);
	pf->setQpPen(QPen(thePalette->getColor(i),1));
	pf->setQpBrush(QBrush(thePalette->getColor(i)));
      }
  }
  // Compute min/max along X & Y 
  limit(VoP,xmin,xmax,ymin,ymax);

  // Cleanup
  for (i=0;i<static_cast<int>(dataLevels.size());i++) 
   delete  dataLevels[i];
}


/// The draw function, improved to use clipping
void QpCurve2D::draw()
{
  if (clipping) { 
    getPainter()->setClipRect(getZone()->getLimits()); 
    getPainter()->setClipping(true); 
  }
  QpNode<QpObject>::draw();
  if (clipping)
    getPainter()->setClipping(false); 
}
/// The print function, improved to use cliiping
void QpCurve2D::print()
{
  /** The clipping is different in the printing case... 
    In fact, the region for clipping must be given to the painter in device coordinates.
    So you must transform the user coordinates (equal to pixels) to device/ps coordinates
    by using xForm(QRect).
  */ 
  if (clipping) { 
    getPrinter()->setClipRect(getPrinter()->xForm(getZone()->getLimits())); 
    getPrinter()->setClipping(true); 
  }
  QpNode<QpObject>::print();
  if (clipping) 
    getPrinter()->setClipping(false);
}

/// Determine the point [x +/- errorX, y +/- errorY] is in the object
QpObject *QpCurve2D::touched(double _x, double _y, double errorX, double errorY, QpObject *&node)
{
  QpObject *res = QpNode<QpObject>::touched(_x,_y,errorX,errorY,node);
  if (res != 0) 
    node=this; // The returned node will be the curve
  return res;
}

void QpCurve2D::updateSt()
{
  if (st != 0) {
    if (mk != 0 ) 
      st->setPen(*(mk->getQpPen()));
    else 
      st->setPen(*(getQpPen()));
    st->setMarker(QpMarker::Rect);
    st->setBrush(*(getQpBrush()));
  }
}


void QpCurve2D::limit(Anaphe::IVector &VoP,
		      double &xmin,double &xmax,double &ymin,double &ymax) {
  Anaphe::IPoint *pt2D =  VoP.point(0);
  xmin = pt2D->coordinate(0)-pt2D->coordMinus(0);
  ymin = pt2D->coordinate(1)-pt2D->coordMinus(1);
  pt2D = VoP.point(VoP.nPoints()-1);
  xmax = pt2D->coordinate(0)+pt2D->coordPlus(0);
  ymax = pt2D->coordinate(1)+pt2D->coordPlus(1);
}


/******
void QpCurve2D::makeEquiContour(Anaphe::IVector &VoP, double &xmin, double &xmax,
                                double &ymin, double &ymax, int NCONT)
{
   int I, J, IX, LJ, ITARS, JX, IR[4], M, N, ITYPE, MODE, ICOL, IEND, ICONT;
   double XSAVE, YSAVE;
   float  X[4], Y[4], ZC[4], XSTP, YSTP;
//______________________________________
//
   size_t NX = VoP.nPoints(0);
   size_t NY = VoP.nPoints(1);

   Anaphe::IPoint *pt2D =  VoP.point(0);
   xmin = pt2D->coordinate(0)-pt2D->coordMinus(0);
   ymin = pt2D->coordinate(1)-pt2D->coordMinus(1);
   pt2D = VoP.point(VoP.nPoints()-1);
   xmax = pt2D->coordinate(0)+pt2D->coordPlus(0);
   ymax = pt2D->coordinate(1)+pt2D->coordPlus(1);

   double *XARR    = new double[MAXCONTOUR];
   double *YARR    = new double[MAXCONTOUR];
   int    *ITARR   = new int[MAXCONTOUR];

   ZMIN = VoP.minValue()->value();
   ZMAX = VoP.maxValue()->value();
   DZ   = ZMAX-ZMIN;
//
// Remove previous QpPolyLine if it exist and create a new one
//
   if (pl != 0) {
      removeElement(pl);
      pl=0;
   }
   pl = new QpNode<QpPolyLine>(new QString("QpNode<QpPolyLine>"),this,true);

//______________________________________
//
//             Draw the table as a contour plot
//
//      ILNSAV = ILN
//      CALL ISLN(1)
   if (NCONT > MAXCONTOUR) NCONT = MAXCONTOUR;
   ITYPE = 0;
   ICOL  = 1;
//      IF(NPAR.GE.2)THEN
//         IF(PAR(2).GE.1..AND.PAR(2).LT.3.)THEN
//            ITYPE = INT(PAR(2))
//            ICOL  = INT((PAR(2)-ITYPE)*1000.+0.5)
//            IF(ICOL.LE.0)ICOL=1
//            CALL ISPLCI(ICOL)
//            IF(ITYPE.EQ.2)CALL ISLN(ILNSAV)
//         ENDIF
//      ENDIF
   XSTP = (xmax-xmin)/NX;
   YSTP = (ymax-ymin)/NY;
   IEND = 1;
   for (ICONT=1; ICONT<=IEND; ICONT++) {
      for (J=1; J<=(int)(NY-1); J++) {
         Y[0] = (float)J*YSTP+ymin+YSTP/2.;
         Y[1] = Y[0];
         Y[2] = (float)(J-1)*YSTP+ymin+YSTP/2.;
         Y[3] = Y[2];
         for (I=1; I<=(int)(NX-1); I++) {
////            ZC[0]=IGCELL(NX,NY,V,I,J,1)
////            ZC[1]=IGCELL(NX,NY,V,I+1,J,1)
////            ZC[2]=IGCELL(NX,NY,V,I+1,J+1,1)
////            ZC[3]=IGCELL(NX,NY,V,I,J+1,1)
            IR[0] = (int)((ZC[0]-ZMIN)*NCONT/DZ);
            IR[1] = (int)((ZC[1]-ZMIN)*NCONT/DZ);
            IR[2] = (int)((ZC[2]-ZMIN)*NCONT/DZ);
            IR[3] = (int)((ZC[3]-ZMIN)*NCONT/DZ);
            if (IR[0] != IR[1] || IR[1] != IR[2] || 
	        IR[2] != IR[3] || IR[3] != IR[0]) {
               X[0] = (float)(I-1)*XSTP+xmin+XSTP/2.;
               X[3] = X[0];
               X[1] = (float)I*XSTP+xmin+XSTP/2.;
               X[2] = X[1];
               if (ZC[0] < ZC[1]) N = 0; else N = 1;
               if (ZC[2] < ZC[3]) M = 2; else M = 3;
               if (ZC[N] > ZC[M]) N = M;
               N++;
               LJ = 1;
               for (IX=1;IX<=4;IX++) {
                  M = N%4 + 1;
                  LJ = LJ+2*Fillxyarr(ZC[N-1],IR[N-1],X[N-1],Y[N-1],ZC[M-1],
                  IR[M-1],X[M-1],Y[M-1],&XARR[LJ-1],&YARR[LJ-1],&ITARR[LJ-1],
                  NCONT);
                  N = M;
               }
               if (ZC[0] < ZC[1]) N = 0; else N = 1;
               if (ZC[2] < ZC[3]) M = 2; else M = 3;
               if (ZC[N] > ZC[M]) N = M;
               N++;
               LJ = 2;
               for (IX=1;IX<=4;IX++) {
                  if (N == 1) {
                     M = 4;
                  } else {
                     M = N-1;
                  }
                  LJ = LJ+2*Fillxyarr(ZC[N-1],IR[N-1],X[N-1],Y[N-1],ZC[M-1],
                  IR[M-1],X[M-1],Y[M-1],&XARR[LJ-1],&YARR[LJ-1],&ITARR[LJ-1],
                  NCONT);
                  N = M;
               }
//
// Re-order endpoints
//
               for (IX=1; IX<=LJ-5; IX +=2) {
L180:
                  if (ITARR[IX-1] != ITARR[IX]) {
                     XSAVE = XARR[IX];
                     YSAVE = YARR[IX];
                     ITARS = ITARR[IX];
                     for (JX=IX; JX<=LJ-5; JX +=2) {
                        XARR[JX]  = XARR[JX+2];
                        YARR[JX]  = YARR[JX+2];
                        ITARR[JX] = ITARR[JX+2];
                     }
                     XARR[LJ-3]  = XSAVE;
                     YARR[LJ-3]  = YSAVE;
                     ITARR[LJ-3] = ITARS;
                     goto L180;
                  }
               }

               for (IX=1; IX<=LJ-2; IX +=2) {
                  ICOL = (4*ITARR[IX-1])/NCONT+1;
                  if (ITYPE == 0) MODE=ICOL;
                  if (ITYPE == 1) {
                     MODE = ICOL%5;
                     if (MODE == 0) MODE=5;
                  }
//                     IF(ITYPE.EQ.0)CALL ISPLCI(MODE)
//                     IF(ITYPE.EQ.1)CALL ISLN(MODE)
                  pl->addElement(new QpPolyLine(2, XARR, YARR, pl, false));
               }
            }
         }
      }
   }
   delete [] XARR;
   delete [] YARR;
   delete [] ITARR;   
}

int QpCurve2D::Fillxyarr(float ELEV1, int ICONT1, float X1, float Y1, 
                         float ELEV2, int ICONT2, float X2, float Y2,
                         double *XARR, double *YARR, int *ITARR, int NCONT)
{
//
// Fill the matrix XARR YARR for Contour Plot
//
   bool VERT;
   float TLEN, TDIF, ELEV, DIFF, PDIF, XLEN;
   int   N, I, ICOUNT;
//______________________________________
//
   if (X1 == X2) {
      VERT = true;
      TLEN = Y2-Y1;
   } else {
      VERT = false;
      TLEN = X2-X1;
   }
   N = ICONT1+1;
   TDIF = ELEV2-ELEV1;
   I = 1;
   ICOUNT = 0;
L10:
   if (N > ICONT2 && I > 102) goto L20;
   ELEV = N*DZ/NCONT+ZMIN;
   DIFF = ELEV-ELEV1;
   PDIF = DIFF/TDIF;
   XLEN = TLEN*PDIF;
   if (VERT) {
      XARR[I-1] = (double)(X1);
      YARR[I-1] = (double)(Y1+XLEN);
   } else {
      XARR[I-1] = (double)(X1+XLEN);
      YARR[I-1] = (double)(Y1);
   }
   ITARR[I-1] = N;
   ICOUNT = ICOUNT+1;
   I = I+2;
   N = N+1;
   goto L10;
L20:
   return ICOUNT;
}
 ******/
