#ifndef QPCURVE2D_H
#define QPCURVE2D_H

/** %%%

     Class for 2D curves boxes.
     
  Usage:

  TODO:

*/


#include <qvector.h>
#include "Qplotter/qp/qpnode.cpp"
#include "Qplotter/qp/qpzone.h"
#include "Qplotter/qp/qpsummaryinfo.h"
#include "Qplotter/qp/qppolyface.h"
#include "Qplotter/qp/qpcolorplot.h"

#include <iostream>

#include "Interfaces/IVector.h"

class QpCurve2D : public QpNode<QpObject>
{
 public:
  enum TYPE2D { BOX, ELLIPSE, COLOR, CLOUD };
 public:
  /// Constructors
  QpCurve2D( QpObject *_z);
  QpCurve2D( QpObject *_z, Anaphe::IVector &_v, TYPE2D _t=BOX);
  QpCurve2D( QpObject *_z, QVector<double> *&_vx, QVector<double> *&_vy, TYPE2D _t=BOX);
  /// Destructor
  ~QpCurve2D();
  /// Insert a new curve
  void newCurve( Anaphe::IVector &_v, TYPE2D _t, bool first=false);
  void newCurve( Anaphe::IVector &_v, bool first=false);
  /// Set the type of representation
  void setType(TYPE2D _t);
  /// Retrieve the type of representation
  TYPE2D        getType()     const {return type;    }
  /// Check whether clipping is on.
  bool          hasClipping() const {return clipping;}
  /// Set clipping on/off
  void setClipping(bool c)  {clipping=c;}
  /// Return the summary table
  SummaryTable *getSummaryTable() const { return st; }
  /// Set a SummaryTable
  void setSummaryTable(SummaryTable *_st) {st=_st;}
  /// Draw the curve
  virtual void draw();
  /// Print the curve
  virtual void print();
  /// Determine if the point [x +/- errorX, y +/- errorY] is in the object

  virtual QpObject *touched(double _x, double _y, double errorX, double errorY, QpObject *&node);
  /// Update summary table with pen/marker information
  void updateSt();
  /// Change the palette
  void changePalette(QpPalette::TYPE_PALETTE typ, int noOfLevels=8);
  /// To create a scale bar on the right of the zone
  void drawScale();
 protected:
  /// Create the faces for box plot
  void makeBox(Anaphe::IVector &VoP,double &xmin, double &xmax, double &ymin, 
	       double &ymax, int opt);
  /// Create the faces for color plot
  void makeColor(Anaphe::IVector &VoP,double &xmin, double &xmax, double &ymin, 
		 double &ymax);
  /// Create the faces for lego plot
  void makeLego(Anaphe::IVector &VoP, double norme, double longitude, double latitude);
  
 private:
  
  /******
	 /// Create the polylines for contour plot
	 void makeEquiContour(Anaphe::IVector &VoP,double &xmin, double &xmax, double &ymin,
	 double &ymax, int NCONT=20);
	 int Fillxyarr(float ELEV1, int ICONT1, float X1, float Y1,
	 float ELEV2, int ICONT2, float X2, float Y2,
	 double *XARR, double *YARR, int *ITARR, int NCONT);
  ******/
 private:
  void init(Anaphe::IVector &VoP);
  void limit(Anaphe::IVector &VoP,double &xmin,double &xmax,double &ymin,double &ymax);

  TYPE2D type;
  SummaryTable *st;

  /******
	 // Pointer on the list of polyline used in case of CONTOUR
	 QpNode<QpPolyLine> *pl;
  ******/

  // Pointer on the list of faces used in case of BOX plot
  QpPolyFace *mk;
  // Clipping enabled/disabled
  bool clipping;
  QpPalette *thePalette;
};

#endif






