#include <iostream>
#include "Qplotter/qp/qpcurve3D.h"
#include "Qplotter/qp/qppage.h"

#include "Qplotter/gblib/Projection3D.hh"
#include "Qplotter/gblib/ScreenProjection3D.hh"
#include "Qplotter/gblib/Point3D.hh"


QpCurve3D::QpCurve3D (QpZone *_z, Anaphe::IVector &VoP) :
  QpNode<QpObject>(new QString("QpCurve3D"),_z,true)
{
  type = NONE ;
  clipping = true ;
    
  _z->is2D = false ;

  if (VoP.dimension() != 2)
    {
      std::cerr << "QpCurve3D::QpCurve3D>  Bad dimension in vector: "
		<< VoP.dimension() << std::endl ;
    }
  
  numberOfFaces = VoP.nPoints(0)*VoP.nPoints(1);
  

  //  surface = new Qp3dSolid[numberOfFaces];
  boxes   = new Qp3dSolid[numberOfFaces];
  
  xS = new double[numberOfFaces];
  yS = new double[numberOfFaces];
  zS = new double[numberOfFaces];
  wS = new double[numberOfFaces];
  hS = new double[numberOfFaces];

  // Convert IVector to a set of faces
  vectorToGrid(&VoP);

  areaLimit();

  for (row = 0;((row < numberOfFaces) && (xS[row] == xS[0])); row++);

  axis_x = axis_y = axis_z = 0;
  
}

QpCurve3D::~QpCurve3D ()
{
}

void QpCurve3D::createBoxes()
{
  int i;
  
  for (i=0;i<numberOfFaces;i++) {
    boxes[i] = Qp3dSolid(xS[i], yS[i], minz,
			 wS[i], 0.0,   0.0,
			 0.0,   hS[i], 0.0,
			 0.0,   0.0,   zS[i] - minz);
  }
}

void QpCurve3D::createSurface()
{
  int i;
  
  for (i=0;i<numberOfFaces;i++) {
    boxes[i] = Qp3dSolid(xS[i], yS[i], minz,
			 wS[i], 0.0,   0.0,
			 0.0,   hS[i], 0.0,
			 0.0,   0.0,   zS[i] - minz);
  }

  return ;
  
  pointList *plist = new pointList(pointList::X);

  for (i=0;i<numberOfFaces;i++) {
    plist->append(new Point (xS[i], yS[i], zS[i]));
  }
  plist->sort();
  i = 0;

  /*-ap int */ row = 0;
  Point *p = plist->first();
  double xline = p->x();
  pointList *buffer = new  pointList(pointList::Y);
  pvector = new Point[numberOfFaces] ;

  for (p = plist->current(); p != 0; p = plist->next())
    {
      if (p->x() > xline)
	{
	  row ++;
	  xline = p->x();

	  buffer->sort();
	  Point *q = buffer->first();
	  for (q = buffer->current(); q != 0; q = buffer->next())
	    {
	      pvector[i] = Point(q->x(),q->y(),q->z()) ;
	      i++;
	    }
	  
	  buffer->clear();
	} else {
	  buffer->append(p);
	}
    }


  buffer->sort();
  Point *q = buffer->first();
  for (q = buffer->current(); q != 0; q = buffer->next())
    {
      row ++;
      pvector[i++] = Point(q->x(),q->y(),q->z()) ;
    }
	  
  buffer->clear();

  for (;i<numberOfFaces;i++)
    {
      pvector[i] =  Point(0.0,0.0,0.0);
    }
  
}


void QpCurve3D::moveView(double deltaLongitude, double deltaLatitude, double deltaNorme)
{
  longitude += deltaLongitude;
  latitude += deltaLatitude;
  norme += deltaNorme;
  
  viewPointx = norme*cos(longitude)*cos(latitude);
  viewPointy = norme*sin(longitude)*cos(latitude);
  viewPointz = norme*sin(latitude);
  clearList();

  // Projection [X,Y,Z] -> [x,y]
  proj = computeProjection();
  m = proj.userToProj();

  draw();
}
  
void QpCurve3D::computeView(double longitude_,double latitude_,double norme_)
{
  longitude = longitude_ ;
  latitude = latitude_ ;
  norme = norme_ ;
  
  viewPointx = norme*cos(longitude)*cos(latitude);
  viewPointy = norme*sin(longitude)*cos(latitude);
  viewPointz = norme*sin(latitude);
}

void QpCurve3D::draw3dAxis(double xmin,double xmax,
			   double ymin,double ymax,
			   double zmin,double zmax,
			   Transformation mIn)
{
  //-ap grab the Transformation (should be passed by ref ?)
  m = mIn;

  /* Drawing is inside a cube area, find the front faces on that cube
    on draw only the border edges by removing the common point on all
    3 faces */

  /*
    For each faces
     if faces is in front
       mark the 4 edges

    For each edges
      if edge have only one mark
        draw it
  */

  int i,j;
  int edgeCount[12];
  QPtrList<Qp3dEdge> edgeList;
  
  Qp3dSolid boundingBox (xmin,ymin,zmin,
			 xmax-xmin,0.0,0.0,
			 0.0,ymax-ymin,0.0,
			 0.0,0.0,zmax-zmin);

  Qp3dArray array (&boundingBox,1,viewPointx,viewPointy,viewPointz);

  for (i=0;i<12;i++)
    edgeCount[i] = 0;
  
  for (i=0;i<6;i++)
    if (array[0]->face[i]->illumination(viewPointx,viewPointy,viewPointz) < 0.0)
      for (j=0;j<4;j++)
	edgeList.append((array[0]->face[i])->getEdge(j));

  QPtrListIterator<Qp3dEdge> it(edgeList);
  for (; it.current(); ++it )
    if (edgeList.contains(it.current()) == 1)
      {
	double axisMin, axisMax;

	switch ((it.current())->type)
	  {
	  case Qp3dEdge::X :
	    axisMin = ((it.current())->p1).x();
	    axisMax = ((it.current())->p2).x();
	    break;
	  case Qp3dEdge::Y :
	    axisMin = ((it.current())->p1).y();
	    axisMax = ((it.current())->p2).y();
	    break;
	  case Qp3dEdge::Z :
	    axisMin = ((it.current())->p1).z();
	    axisMax = ((it.current())->p2).z();
	    break;
	  }
	
	createAxis(m*((it.current())->p1),m*((it.current())->p2),axisMax,axisMin);      
      }
  
}


Projection QpCurve3D::computeProjection (Projection::ProjectionType projType)
{
  Projection Proj3d(projType,
		    // recenter to the unit sphere
		    Projection::NORMALIZED,
		    Projection::XYZ);

  computeView(longitude, latitude, norme);
      
  Proj3d.normSetViewDirection(longitude, latitude);
  Proj3d.normSetViewDistance(norme);

  // From [minx/maxx,miny/maxy,minz/maxz] to the unit sphere

  // User data maximum view
  Proj3d.userSetMinMax(minx,maxx,miny,maxy,minz,maxz);
  // to keep the aspect ratio:
  //  Proj3d.userSetMinMax(min,max,min,max,min,max);

  return Proj3d ;      
}


void QpCurve3D::areaLimit () 
{
  int i;
      
  minx = xS[0]; maxx = xS[0];
  miny = yS[0]; maxy = yS[0];
  minz = zS[0]; maxz = zS[0];

  // The "floor" must be minz instead of 0.0

  for (i=0;i<numberOfFaces;i++) {
    if (xS[i] < minx) minx = xS[i];
    if (yS[i] < miny) miny = yS[i];
    if (zS[i] < minz) minz = zS[i];
	 		      	   
    if (xS[i] > maxx) maxx = xS[i] + wS[i];    
    if (yS[i] > maxy) maxy = yS[i] + hS[i];
    if (zS[i] > maxz) maxz = zS[i];
  }

  min_xyz = minx ;  
  if (miny < min_xyz) min_xyz = miny;
  if (minz < min_xyz) min_xyz = minz;
   
  max_xyz = maxx ;
  if (maxy > max_xyz) max_xyz = maxy;
  if (maxz > max_xyz) max_xyz = maxz;
  
}



void QpCurve3D::vectorToGrid (Anaphe::IVector *VoP) 
{
  if (VoP->dimension() == 2)
    {
      int k;
      
      int NpointX = VoP->nPoints(0) ;
      int NpointY = VoP->nPoints(1) ;

      k = 0;
      
      for (int i=0;i<NpointX;i++) {
	for (int j=0;j<NpointY;j++) {
	  Anaphe::IPoint *point = VoP->point(i,j);
	  //  	  std::cout << "QpCurve3D::vectorToGrid> Iteration " << k << "  " << i << " " << j << std::endl ;
	  //  	  std::cout << "  Value " << point->value() << std::endl;
	  //  	  std::cout << "  Coord 0  " << point->coordinate(0) << std::endl ;
	  //	  std::cout << "  Coord 1  " << point->coordinate(1) << std::endl ;
	  
	  xS[k] = point->coordinate(0)+(point->coordPlus(0)-point->coordMinus(0))/2;
	  yS[k] = point->coordinate(1)+(point->coordPlus(1)-point->coordMinus(1))/2;
	  zS[k] = point->value();
	  wS[k] = point->coordPlus(0)+point->coordMinus(0);
	  hS[k] = point->coordPlus(1)+point->coordMinus(1);
	  k ++;
	}
      }
    } else {
      std::cerr << "QpCurve3D::vectorToGrid> Bad dimension in vector: " << VoP->dimension() << std::endl ;
    }
}
  
void QpCurve3D::drawCubeLimit(double xmin,double xmax,
			      double ymin,double ymax,
			      double zmin,double zmax,
			      Transformation mIn)
{
  //-ap grab the trans (see above)
  m = mIn;

  Point p000(xmin,ymin,zmin);
  Point p001(xmin,ymin,zmax);
  Point p010(xmin,ymax,zmin);
  Point p011(xmin,ymax,zmax);
  Point p100(xmax,ymin,zmin);
  Point p101(xmax,ymin,zmax);
  Point p110(xmax,ymax,zmin);
  Point p111(xmax,ymax,zmax);


  p000 = m*p000;
  p001 = m*p001;
  p010 = m*p010;
  p011 = m*p011;
  p100 = m*p100;
  p101 = m*p101;
  p110 = m*p110;
  p111 = m*p111;


  QpNode<QpPolyShape> *pl=new QpNode<QpPolyShape>(new QString("QpMultiLine"),this,true);
  pl->setQpPen(Qt::black);

  drawLine(pl,p000,p100);
  drawLine(pl,p001,p101);
  drawLine(pl,p010,p110);
  drawLine(pl,p011,p111);

  drawLine(pl,p000,p010);
  drawLine(pl,p001,p011);
  drawLine(pl,p100,p110);
  drawLine(pl,p101,p111);

  drawLine(pl,p000,p001);
  drawLine(pl,p010,p011);
  drawLine(pl,p100,p101);
  drawLine(pl,p110,p111);
}



/************/

void QpCurve3D::draw()
{
  
  switch (type) {
  case LEGO:
    drawLego();
    
    break ;
  case SURFACE:
    drawSurface();
    
    break;
  case NONE:
  default:
    break;
  }

  if (clipping) { 
    getPainter()->setClipRect(getZone()->getLimits()); 
    getPainter()->setClipping(true); 
  }
  QpNode<QpObject>::draw();
  if (clipping) 
    getPainter()->setClipping(false); 
}


void QpCurve3D::drawLego()
{
  // Draw every object with the current viewpoint
  //  QpNode<QpObject>::draw();

  /*
    Foreach point in (x,y,z)
    sort them with the distance and create a face

    For each face from far to near
    draw the rectangle with specific color (depend on angle)
  */

  //  getZone()->draw3dLine(0.0,0.0,0.0,viewPointx,viewPointy,viewPointz,m,Qt::red);

  // We have to sort the boxes to draw them in correct order
  // Only works if each box does not intersect another one.
  Qp3dArray faceArray (boxes,numberOfFaces,
		       viewPointx,viewPointy,viewPointz);
  faceArray.sort();


  int grayValue;
  double light ;

  int i;
  
  for (i=0;i<numberOfFaces;i++)
    for (int j=0;j<6;j++)
      {
	// draw only front faces
	if ((light = faceArray[i]->face[j]->illumination (viewPointx,viewPointy,viewPointz)) < 0.0)
	  {
	    int ambient = 64 ;
	    
	    grayValue = (int) (-256 * light) ;
	    grayValue += ambient ;
	    
	    if (grayValue > 255) grayValue = 255;
	    faceArray[i]->face[j]->drawBoxFace(this,m,grayValue);
	  }
      }

  // Draw axis
  draw3dAxis(minx,maxx,miny,maxy,minz,maxz,m);

  // Draw bounding box
  // Draw it like other boxes but with a transparent brush
  //  drawCubeLimit(minx,maxx,miny,maxy,minz,maxz,m);  

}

void QpCurve3D::drawSurface()
{
  int i;

  draw3dAxis(minx,maxx,miny,maxy,minz,maxz,m);

  QpNode<QpPolyShape> *pl=new QpNode<QpPolyShape>(new QString("QpMultiLine"),this,true);
  pl->setQpPen(Qt::black);

  
  Point p1,p2,p3 ;
  bool isAFace = true ;
  for (i=0;i<numberOfFaces-1;i++)
    {
      isAFace = true ;
      
      p1 = Point(xS[i],yS[i],zS[i]);
      p1 = m*p1;

      if ((i+1) % row == 0) {
	p2 = Point(xS[i+1],yS[i+1],zS[i+1]);
	p2 = m*p2;
	drawLine(pl,p1,p2);
      } else {
	p2 = p1;
	isAFace = false;
      }
      
      if (row + i < numberOfFaces) {
	p3 = Point(xS[i+row],yS[i+row],zS[i+row]);
	p3 = m*p3;
	drawLine(pl,p1,p3);
      } else {
	p3 = p1;
	isAFace = false;
      }

      /*
      if (isAFace) {
	double LineX[3];
	double LineY[3];
	
	LineX[0] = p1.x();
	LineX[1] = p2.x();
	LineX[2] = p3.x();
	LineY[0] = p1.y();
	LineY[1] = p2.y();
	LineY[2] = p3.y();
	
	QpNode<QpPolyShape> *pg=new QpNode<QpPolyShape>(new QString("QpMultiLine"),this,true);
	pg->setQpBrush(QBrush(Qt::white));
	pg->setQpPen(QPen(Qt::black));
	new QpPolygon (2, LineX, LineY, pg, true);
	QpNode<QpPolyShape> *pl=new QpNode<QpPolyShape>(new QString("QpMultiLine"),this,true);
	pg->setQpBrush(QBrush(Qt::white));
	pl->setQpPen(QPen(Qt::white));
	new QpPolyLine(3, LineX, LineY, pl, true);

      } else {
	drawLine(p1,p2);
	drawLine(p1,p3);
      }
      */
      // Todo: replace lines with faces to sort 
    }

  return ;
  
  Qp3dArray faceArray = Qp3dArray(boxes,numberOfFaces,
				  viewPointx,viewPointy,viewPointz);
  faceArray.sort();


  int grayValue;
  double light ;

  
  for (i=0;i<numberOfFaces;i++)
    for (int j=0;j<6;j++)
      {
	// draw only front faces
	if ((light = faceArray[i]->face[j]->illumination (viewPointx,viewPointy,viewPointz)) < 0.0)
	  {
	    int ambient = 64 ;
	    
	    grayValue = (int) (-256 * light) ;
	    grayValue += ambient ;
	    
	    if (grayValue > 255)  grayValue = 255;
	    faceArray[i]->face[j]->drawBoxFace(this,m,grayValue);
	  }
      }
}

void QpCurve3D::print()
{
  QpNode<QpObject>::print();
}

QpObject *QpCurve3D::touched(double , double , double , double , QpObject *&)
{
  return 0 ;
}

// Beware when filling histograms not to write twice in one cell
// How to have the cell size and grid dimension for a histo2D ?

void QpCurve3D::makeSurface(double norme_, double longitude_, double latitude_,
			    Projection::ProjectionType projType)
{
  type = SURFACE;
  // Link every point on a grid with a face

  createSurface();
  
  prepareView(norme_, longitude_, latitude_, projType);
}

void QpCurve3D::makeLego(double norme_, double longitude_, double latitude_,
			 Projection::ProjectionType projType)
{
  type = LEGO ;
  // Make a box for each point on a grid
    
  createBoxes();

  prepareView(norme_, longitude_, latitude_, projType);
}

void QpCurve3D::prepareView(double norme_, double longitude_, double latitude_,
			    Projection::ProjectionType projType)
{
  
  norme = norme_;
  latitude = latitude_;
  longitude = longitude_;
  
  // We draw on a [-1.2;1.2]x[-1.2;1.2] zone
  // No 2d Axis for 3d graphics
  getZone()->setXRange(-1.2,1.2);
  getZone()->setYRange(-1.2,1.2);
  
  // Compute Projection to screen
  double deg = 3.14159265358979323846/180.;

  longitude *= deg ;
  latitude *= deg ;
  
  // Projection [X,Y,Z] -> [x,y]
  proj = computeProjection(projType);
  m = proj.userToProj();

  computeView(longitude, latitude, norme);

}






