#ifndef QPCURVE3D_H
#define QPCURVE3D_H

/** %%%

    Class for 3D plotting.
    
  Usage:

  TODO:

  (!!) This is a bit slow, need some profiling here.
  (--) Add some drawing options.
*/


#include "Qplotter/qp/qpzone.h"
#include "Qplotter/qp/qpaxis.h"
#include <qvector.h> 
#include "Interfaces/IVector.h"



typedef gblib::Point3D<float> Point;
typedef gblib::ScreenProjection3D Projection;
typedef gblib::Projection3D Transformation;


class pointList : public QPtrList<Point>
{
 public:
  enum COORDTYPE {X, Y, Z};

 public:
  pointList(COORDTYPE type_) {type = type_;}
  ~pointList() {;}

  void setType (COORDTYPE type_) {type = type_;}
  
  int compareItems( QCollection::Item el1, QCollection::Item el2)
    {
      Point *obj1 = (Point *) el1;
      Point *obj2 = (Point *) el2;

      switch (type)
	{
	case X:
      
	  if (obj1->x() > obj2->x())
	    return 1;
	  else
	    return -1;
	  break;
	  
	case Y:
      
	  if (obj1->y() > obj2->y())
	    return 1;
	  else
	    return -1;
	  break;
	  
	case Z:
      
	  if (obj1->z() > obj2->z())
	    return 1;
	  else
	    return -1;
	  break;
	  
	}

      return 0;
    }

  COORDTYPE type ;
  
};

class Qp3dFace;

class Qp3dEdge
{
 public:
  enum AxisType {X,Y,Z};
  
  Qp3dEdge() {;}
  Qp3dEdge(Point a,Point b)
    {
      p1 = a;
      p2 = b;
    }
  Qp3dEdge(double x1,double y1,double z1,
	   double x2,double y2,double z2,
	   AxisType t = X)
    {
      p1 = Point(x1,y1,z1);
      p2 = Point(x2,y2,z2);
      type = t;
    }
  
  ~Qp3dEdge() {;}

  void addFace(Qp3dFace *f)
    {
      faces.append(f);
    }

  Qp3dFace *getFace(int i)
    {
      return faces.at(i);
    }
  
  
 public:
  Point p1,p2;
  QPtrList<Qp3dFace> faces;
  AxisType type;
  
};

class Qp3dFace
{
 public:
  Qp3dFace() {;}
  Qp3dFace(double x_,double y_,double z_,
	   double ux_,double uy_,double uz_,
	   double vx_,double vy_,double vz_) 
    {
      x = x_;
      y = y_;
      z = z_;
      ux = ux_;
      uy = uy_;
      uz = uz_;
      vx = vx_;
      vy = vy_;
      vz = vz_;

      // Normal unit vector
      nx = uy*vz - vy*uz ;
      ny = uz*vx - vz*ux ;
      nz = ux*vy - vx*uy ;

      double nn = sqrt(nx*nx + ny*ny + nz*nz) ;

      if (nn == 0.0) {
	nx = ny = nz = 0.0 ;	
      } else {
	nx = nx / nn ;
	ny = ny / nn ;
	nz = nz / nn ;
      }
    }
  
  ~Qp3dFace() {;}

  void drawFace (QpObject *obj, Transformation m,int grayValue = 255)
    {
      double xx[4], yy[4], zz[4];

      xx[0] = x;
      yy[0] = y;
      zz[0] = z;
		  
      xx[1] = x + ux;
      yy[1] = y + uy;
      zz[1] = z + uz;
		      
      xx[2] = x + ux + vx;
      yy[2] = y + uy + vy;
      zz[2] = z + uz + vz;
    		      
      xx[3] = x + vx;
      yy[3] = y + vy;
      zz[3] = z + vz;

      QColor color = QColor(grayValue,grayValue,grayValue);
      
      Point p1(xx[0], yy[0], zz[0]);
      Point p2(xx[1], yy[1], zz[1]);
      Point p3(xx[2], yy[2], zz[2]);
      Point p4(xx[3], yy[3], zz[3]);

      double LineX[5];
      double LineY[5];

      p1 = m*p1;
      p2 = m*p2;
      p3 = m*p3;
      p4 = m*p4;

      LineX[0] = p1.x();
      LineX[1] = p2.x();
      LineX[2] = p3.x();
      LineX[3] = p4.x();
      LineX[4] = p1.x();
      LineY[0] = p1.y();
      LineY[1] = p2.y();
      LineY[2] = p3.y();
      LineY[3] = p4.y();
      LineY[4] = p1.y();

      QpNode<QpPolyShape> *pg=new QpNode<QpPolyShape>(new QString("QpMultiLine"),obj,true);
      /*
	pg->setQpBrush(QBrush(Qt::white));
	pg->setQpPen(QPen(Qt::black));
      */
      pg->setQpBrush(QBrush(color));
      pg->setQpPen(QPen(Qt::black));
      new QpPolygon (4, LineX, LineY, pg, true);
      QpNode<QpPolyShape> *pl=new QpNode<QpPolyShape>(new QString("QpMultiLine"),obj,true);
      pg->setQpBrush(QBrush(color));
      pl->setQpPen(QPen(color));
      new QpPolyLine(5, LineX, LineY, pl, true);

    }
  
  void drawBoxFace (QpObject *obj, Transformation m,int grayValue = 255)
    {
      double xx[4], yy[4], zz[4];

      xx[0] = x;
      yy[0] = y;
      zz[0] = z;
		  
      xx[1] = x + ux;
      yy[1] = y + uy;
      zz[1] = z + uz;
		      
      xx[2] = x + ux + vx;
      yy[2] = y + uy + vy;
      zz[2] = z + uz + vz;
    		      
      xx[3] = x + vx;
      yy[3] = y + vy;
      zz[3] = z + vz;

      //      zone->draw3dPolygon(xx,yy,zz,m,QColor(grayValue,grayValue,grayValue));
      /******/

      QColor color = QColor(grayValue,grayValue,grayValue);
      
      Point p1(xx[0], yy[0], zz[0]);
      Point p2(xx[1], yy[1], zz[1]);
      Point p3(xx[2], yy[2], zz[2]);
      Point p4(xx[3], yy[3], zz[3]);

      double LineX[5];
      double LineY[5];

      p1 = m*p1;
      p2 = m*p2;
      p3 = m*p3;
      p4 = m*p4;

      LineX[0] = p1.x();
      LineX[1] = p2.x();
      LineX[2] = p3.x();
      LineX[3] = p4.x();
      LineX[4] = p1.x();
      LineY[0] = p1.y();
      LineY[1] = p2.y();
      LineY[2] = p3.y();
      LineY[3] = p4.y();
      LineY[4] = p1.y();

      QpNode<QpPolyShape> *pg=new QpNode<QpPolyShape>(new QString("QpMultiLine"),obj,true);
      /*
	pg->setQpBrush(QBrush(Qt::white));
	pg->setQpPen(QPen(Qt::black));
      */
      pg->setQpBrush(QBrush(color));
      pg->setQpPen(QPen(Qt::black));
      new QpPolygon (4, LineX, LineY, pg, true);
      QpNode<QpPolyShape> *pl=new QpNode<QpPolyShape>(new QString("QpMultiLine"),obj,true);
      pg->setQpBrush(QBrush(color));
      pl->setQpPen(QPen(color));
      new QpPolyLine(5, LineX, LineY, pl, true);

    }
  
  
  
  double scalarProduct (double _x, double _y, double _z)
    {
      return (_x*nx + _y*ny + _z*nz) / sqrt(_x*_x + _y*_y + _z*_z) ;      
    }

  double illumination (double _x, double _y, double _z)
    {
      return (_x*nx + _y*ny + _z*nz) / sqrt(_x*_x + _y*_y + _z*_z) ;      
    }

  double planarDistance (double _x, double _y, double _z)
    {
      return (_x - x)*nx + (_y - y)*ny + (_z - z)*nz ;      
    }
  
  double distance(double _x, double _y, double _z) 
    {
      double m_x = x + (ux + vx)/2.0;
      double m_y = y + (uy + vy)/2.0;
      double m_z = z + (uz + vz)/2.0;

      m_x -= _x;
      m_y -= _y;
      m_z -= _z;

      return (m_x*m_x + m_y*m_y + m_z*m_z);
    }

  void addEdge(Qp3dEdge *e)
    {
      edges.append(e);
    }

  Qp3dEdge *getEdge(int i)
    {
      return edges.at(i);
    }
  
  
  
 public:
  double x,y,z;
  double ux,uy,uz;
  double vx,vy,vz;

  // normal unit vector
  double nx,ny,nz;

  QPtrList<Qp3dEdge> edges;

};


//  Class to manage 3d solid, currently only 3d boxes

class Qp3dSolid
{
 public:
  Qp3dSolid() {;}
  
  Qp3dSolid(double x_,double y_,double z_,
	    double ux_,double uy_,double uz_,
	    double vx_,double vy_,double vz_,
	    double wx_,double wy_,double wz_) 
    {
      p = new Point(x_,y_,z_);
      u = new Point(ux_,uy_,uz_);
      v = new Point(vx_,vy_,vz_);
      w = new Point(wx_,wy_,wz_);

      face[0] = new Qp3dFace(x_,y_,z_,vx_,vy_,vz_,ux_,uy_,uz_);       
      face[1] = new Qp3dFace(x_,y_,z_,ux_,uy_,uz_,wx_,wy_,wz_);       
      face[2] = new Qp3dFace(x_,y_,z_,wx_,wy_,wz_,vx_,vy_,vz_); 
      face[3] = new Qp3dFace(x_+wx_,y_+wy_,z_+wz_,ux_,uy_,uz_,vx_,vy_,vz_); 
      face[4] = new Qp3dFace(x_+vx_,y_+vy_,z_+vz_,wx_,wy_,wz_,ux_,uy_,uz_);       
      face[5] = new Qp3dFace(x_+ux_,y_+uy_,z_+uz_,vx_,vy_,vz_,wx_,wy_,wz_); 

      faceEdge[0] = new Qp3dEdge(x_,y_,z_, x_+ux_,y_+uy_,z_+uz_, Qp3dEdge::X);
      faceEdge[1] = new Qp3dEdge(x_,y_,z_, x_+vx_,y_+vy_,z_+vz_, Qp3dEdge::Y);
      faceEdge[2] = new Qp3dEdge(x_,y_,z_, x_+wx_,y_+wy_,z_+wz_, Qp3dEdge::Z);
      faceEdge[3] = new Qp3dEdge(x_+ux_,y_+uy_,z_+uz_, x_+ux_+vx_,y_+uy_+vy_,z_+uz_+vz_, Qp3dEdge::Y);
      faceEdge[4] = new Qp3dEdge(x_+ux_,y_+uy_,z_+uz_, x_+ux_+wx_,y_+uy_+wy_,z_+uz_+wz_, Qp3dEdge::Z);
      faceEdge[5] = new Qp3dEdge(x_+vx_,y_+vy_,z_+vz_, x_+ux_+vx_,y_+uy_+vy_,z_+uz_+vz_, Qp3dEdge::X);
      faceEdge[6] = new Qp3dEdge(x_+vx_,y_+vy_,z_+vz_, x_+wx_+vx_,y_+wy_+vy_,z_+wz_+vz_, Qp3dEdge::Z);
      faceEdge[7] = new Qp3dEdge(x_+wx_,y_+wy_,z_+wz_, x_+wx_+ux_,y_+wy_+uy_,z_+wz_+uz_, Qp3dEdge::X);
      faceEdge[8] = new Qp3dEdge(x_+wx_,y_+wy_,z_+wz_, x_+wx_+vx_,y_+wy_+vy_,z_+wz_+vz_, Qp3dEdge::Y);
      faceEdge[9] = new Qp3dEdge(x_+ux_+vx_,y_+uy_+vy_,z_+uz_+vz_, x_+ux_+vx_+wx_,y_+uy_+vy_+wy_,z_+uz_+vz_+wz_, Qp3dEdge::Z);
      faceEdge[10] = new Qp3dEdge(x_+ux_+wx_,y_+uy_+wy_,z_+uz_+wz_, x_+ux_+vx_+wx_,y_+uy_+vy_+wy_,z_+uz_+vz_+wz_, Qp3dEdge::Y);
      faceEdge[11] = new Qp3dEdge(x_+wx_+vx_,y_+wy_+vy_,z_+wz_+vz_, x_+ux_+vx_+wx_,y_+uy_+vy_+wy_,z_+uz_+vz_+wz_, Qp3dEdge::X);
      
      linkEdgetoFace(face[0],faceEdge[0]);
      linkEdgetoFace(face[0],faceEdge[1]);
      linkEdgetoFace(face[0],faceEdge[3]);
      linkEdgetoFace(face[0],faceEdge[5]);
      linkEdgetoFace(face[1],faceEdge[0]);
      linkEdgetoFace(face[1],faceEdge[2]);
      linkEdgetoFace(face[1],faceEdge[4]);
      linkEdgetoFace(face[1],faceEdge[7]);
      linkEdgetoFace(face[2],faceEdge[1]);
      linkEdgetoFace(face[2],faceEdge[2]);
      linkEdgetoFace(face[2],faceEdge[6]);
      linkEdgetoFace(face[2],faceEdge[8]);
      linkEdgetoFace(face[3],faceEdge[7]);
      linkEdgetoFace(face[3],faceEdge[8]);
      linkEdgetoFace(face[3],faceEdge[10]);
      linkEdgetoFace(face[3],faceEdge[11]);
      linkEdgetoFace(face[4],faceEdge[5]);
      linkEdgetoFace(face[4],faceEdge[6]);
      linkEdgetoFace(face[4],faceEdge[9]);
      linkEdgetoFace(face[4],faceEdge[11]);
      linkEdgetoFace(face[5],faceEdge[3]);
      linkEdgetoFace(face[5],faceEdge[4]);
      linkEdgetoFace(face[5],faceEdge[9]);
      linkEdgetoFace(face[5],faceEdge[10]);
      

      // Center of the base face
      baseX = x_ + (ux_ + vx_) / 2.0 ;
      baseY = y_ + (uy_ + vy_) / 2.0;
      baseZ = z_ + (uz_ + vz_) / 2.0;
      
    }
  
  ~Qp3dSolid()
    {;}

 private:
  void linkEdgetoFace(Qp3dFace *f,Qp3dEdge *e)
    {
      e->addFace(f);
      f->addEdge(e);
    }
		      
 public:
  Point *p,*u,*v,*w;
  
  double baseX,baseY,baseZ;
  Qp3dFace* face[6];
  //  int faceEdges[6][4];
  Qp3dEdge* faceEdge[12];
  
};


class Qp3dArray : public QVector<Qp3dSolid>
{
 public:
      
  Qp3dArray(Qp3dSolid *array, int n, double vx,double vy,double vz)
    {
      viewX = vx ;
      viewY = vy ;
      viewZ = vz ; 
      
      resize(n);
      
      // create vector from array
      uint i;
      for (i=0;i<size();i++)
	insert(i,&(array[i]));
    }

  ~Qp3dArray() {;}  

  
  int compareItems (QCollection::Item el1, QCollection::Item el2)
    {      
      Qp3dSolid	*p1 = (Qp3dSolid *) el1;
      Qp3dSolid	*p2 = (Qp3dSolid *) el2;

      if (((p1->baseX - p2->baseX)*viewX +
	   (p1->baseY - p2->baseY)*viewY +
	   (p1->baseZ - p2->baseZ)*viewZ) > 0.0)
	return 1 ;
      
      // if ((p1->value - p2->value) > 0.0) return -1;
      return -1;      
    }

  double viewX, viewY, viewZ ;
  
};

// Keep in mind that we want to move the viewpoint without to have to
// destroy the curve object

class QpCurve3D : public QpNode<QpObject> 
{
 public:
  enum TYPE3D { NONE, LEGO, SURFACE };
 public:
  /// Constructors
  QpCurve3D(QpZone *_z, Anaphe::IVector &VoP); // TODO: Add some drawing options here
  /// Destructor
  ~QpCurve3D();

  virtual void draw();
  virtual void print();
  virtual QpObject *touched(double _x, double _y, double errorX, double errorY, QpObject *&node);

  void prepareView (double norme_, double longitude_, double latitude_,
		    Projection::ProjectionType projType = Projection::PARALLEL);
  void makeLego    (double norme_, double longitude_, double latitude_,
		    Projection::ProjectionType projType = Projection::PARALLEL);
  void makeSurface (double norme_, double longitude_, double latitude_,
		    Projection::ProjectionType projType = Projection::PARALLEL);
  
  void moveView(double deltaLongitude, double deltaLatitude, double deltaNorme);

  /// Check whether clipping is on.
  bool          hasClipping() const {return clipping;}
  /// Set clipping on/off
  void setClipping(bool c)  {clipping=c;}

  
 private:
  void createBoxes();
  void createSurface();
  void drawLego();
  void drawSurface();
  void vectorToGrid (Anaphe::IVector *VoP) ;
  void areaLimit () ;
  Projection computeProjection (Projection::ProjectionType projType = Projection::PARALLEL);
  void computeView(double longitude_,double latitude_,double norme_);
  void draw3dAxis(double xmin,double xmax,
		  double ymin,double ymax,
		  double zmin,double zmax,
		  Transformation m);
  void drawCubeLimit(double xmin,double xmax,
		     double ymin,double ymax,
		     double zmin,double zmax,
		     Transformation m);
  void drawLine(QpNode<QpPolyShape> *pl,Point p1, Point p2) // ,QColor color = Qt::black
    {
      double LineX[2];
      double LineY[2];

      LineX[0] = p1.x();
      LineX[1] = p2.x();
      LineY[0] = p1.y();
      LineY[1] = p2.y();

      /*
	QpNode<QpPolyShape> *pl=new QpNode<QpPolyShape>(new QString("QpMultiLine"),this,true);
	pl->setQpPen(color);
      */
      new QpPolyLine(2, LineX, LineY, pl, true);
    }

  double direction (Point p1,Point p2)
    {
      return ((viewPointx - p1.x())*(p2.x() - p1.x()) +
	      (viewPointy - p1.y())*(p2.y() - p1.y()) +
	      (viewPointz - p1.z())*(p2.z() - p1.z()));
    }

  void createAxis(Point p1, Point p2, double max_, double min_)
    {
      // TODO: adapt the font size to show the labels correctly
      new QpAxis(p1.x(),p1.y(),p2.x(),p2.y(),
		 min_,max_,
		 QpAxis::Axis3d,10U,
		 this,true,QpTransformation::Zone);
    }
  
  
  TYPE3D type ;
  bool clipping ;
  
  
  int row;
  int numberOfFaces;
  
  // Faces coordinates
  double *xS,*yS,*zS,*wS,*hS;

  // Boxes to sort
  Qp3dSolid* boxes;
  //  Qp3dFace* surface;
  Point *pvector;
  
  
  // Area limits
  double  minx, maxx, miny, maxy, minz, maxz;
  double  min_xyz,max_xyz; // optional usage

  // ViewPoint
  double longitude, latitude, norme;
  double viewPointx, viewPointy, viewPointz;

  // Projection
  Projection proj;
  Transformation m;

  // Axis
  QpAxis *axis_x, *axis_y, *axis_z;
  
};



#endif // QPCURVE3D_H





