#include <stdlib.h>
#include "Qplotter/qp/qpface.h"

QpFace::QpFace(QpObject *z, bool addOnParent,QpTransformation::tStrategy st) : 
  QpObject(new QString("QpFace"),z,addOnParent,st) {
  x=y=w=h=0.0;
  FaceMode = 0;
  backgroundColor = Qt::white;  
}

//If the coordinates we receive are from the center of the face.
//This function changes them to the up&left square
void QpFace::setGeometry(double _x, double _y, double _w, double _h, bool center) 
{
  if (center) {
   x=_x-(_w/2.0);
   y=_y+(_h/2.0);
  }
  else {
    x=_x;
    y=_y;
  }
  w=_w;h=_h;
}

void QpFace::draw() 
{
  paint(getPainter());
}

void QpFace::print()
{
  paint(getPrinter());
}

void QpFace::paint(QPainter *p)
{
//
// Open a transformation to convert user coordinates into pixel coordinates.  
//
   QpTransformation *transf = getTransformation(getTransfStrategy());  
//
// Transform the face (box) paramaters from user coordinates into pixel
// coordinates. If the width or the height is equal to 0, they are set to
// one pixel.
//
   int _x = transf->xTransform(x);
   int _y = transf->yTransform(y);
   int _w = transf->xTransform(x+w) - _x;
   int _h = _y - transf->yTransform(y+h);
   if (_w < 1) _w = 1;
   if (_h < 1) _h = 1;
//
// Faces drawing.
//
  if ((_w == 1) && (_h == 1)) {
    // Width AND height are one: draw a point
    p->drawPoint(_x,_y);
  } else {
    // Width OR height are one: draw a line
    if ((_w == 1) || (_h == 1)) {
      if (_w == 1)
	p->drawLine(_x,_y,_x,_y+_h);
      else
	p->drawLine(_x,_y,_x+_w,_y);
    } else {
	if (FaceMode == 0 ) {
           // Array to draw polygons with QPainter.
           QPointArray d_pa;
           // Filling the array to use QPainter
           d_pa.resize(4);
           d_pa.setPoint(0, _x, _y);
           d_pa.setPoint(1, _x+_w, _y);
           d_pa.setPoint(2, _x+_w, _y+_h);
           d_pa.setPoint(3, _x, _y+_h);
           //Using QPainter to draw the face.
	   QBrush br = p->brush();	   
	   p->setBrush(QBrush(backgroundColor)); 
           p->drawPolygon(d_pa);
	   p->setBrush(br);
           p->drawPolygon(d_pa);
	   
	} else {
           p->drawEllipse(_x,_y,_w,_h); 
	}
    }
  }
}
