#include <stdlib.h>
#include "Qplotter/qp/qplistener.h"
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpcurve3D.h"

#include <qcursor.h> 

QpListener::QpListener() {
  
  listenerType=None;
  picked=0;
  pnode=0;
  observers = new QPtrList<QpEvObserver> ();  
  observers->setAutoDelete(false);
  x=y=0;
  xz=yz=xp=yp=0.0;
}

QpListener::~QpListener() 
{
  
  delete observers; 
}

void QpListener::addObserver(QpEvObserver *o)
{
  
  observers->append(o); 
  o->update();
}

void QpListener::notify()
{
  unsigned int i;  
  /* If this observer deals with the latest kind of event,
     it will be updated */  
  for (i=0;i<observers->count();i++) {
    if ((observers->at(i))->getType() & listenerType)
      (observers->at(i))->update(); 
  }
}

/* FIXME        */
void QpListener::end()
{
  unsigned int i;
  
  
  /* End every observer */
  for (i=0;i<observers->count();i++) {
    (observers->at(i))->end() ;
  }
}

QpEvObserver::QpEvObserver(QpListener::EventType t)
{
  
  listener=0;
  type=t;
}

void QpEvObserver::lRegister(QpPage *p)
{
  
  listener = p->getListener();
  if (listener == 0) {
    std::cerr << "QpEvObserver::lRegister  listener == 0" <<std::endl;    
    return ;    
  }
  
  listener->addObserver(this);
}

void QpEvObserver::lRegister(QpListener *l)
{
  
  listener = l;
  listener->addObserver(this);
}

void QpEvObserver::end()
{
  
  listener->removeObserver(this);
}

QpZoom::QpZoom(QpOutLine *o, bool fixedstate)
  : QpEvObserver((QpListener::EventType)
		 (QpListener::LeftPressed +
		  QpListener::LeftReleased +
		  QpListener::RightPressed)),
  ol(o)
{
  
  z1=0;
  x1=y1=0.0;
  fixed = fixedstate ;  
  setState(Off);
}

QpZoom::~QpZoom() 
{ }

void QpZoom::setState(State s)
{
  
  state=s;
  if (state == Zoom && ol) {
    // If the outline is not a rectangle, combine the existing one with
    // the rectangle used by QpZoom
    if (!(ol->getType() & QpOutLine::Rectangle)) {
      ol->setType((QpOutLine::Type)(ol->getType() + QpOutLine::Rectangle));
    }
    ol->setSelectionState(QpOutLine::Ready);   
  }
  else if (state == Off && ol) {
    // Remove any outline rectangle left over
    if (ol->getType() & QpOutLine::Rectangle ) 
      ol->setType((QpOutLine::Type)(ol->getType() - QpOutLine::Rectangle));
  }
}

void QpZoom::update()
{
  if (listener->getEventType() == QpListener::LeftPressed)
    {
      if (state == Zoom) {
	z1=0;
	if (listener->getPickedNode()) 
	  z1 = listener->getPickedNode()->getZone();
	if (z1 != 0) {
	  if (!z1->isInvalid()) {
	    // Zoom state & left button pressed on a 2d zone: 
	    // beginning the selection of zoom area
	    x1=listener->getXz();
	    y1=listener->getYz();
	    xm1=listener->getX();
	    ym1=listener->getY();
	    ym2=ym1; 
	    xm2=xm1;
	    state = PickedFirst;
	  } else {
	    // Zoom on a invalid zone
	  }	  
	}	
	else {
	  setState(Off);
	}	
      }
      else if (state == UnZoom) {
	// Unzoom state & left button pressed: unzoom picked zone (if any) to original

	if (fixed) {
	  setState(UnZoom);
	} else {
	  setState(Off);
	}		
	if (listener->getPickedNode()) 
	  z1=listener->getPickedNode()->getZone();
	if (z1 != 0) {
	  //	  z1->setThickness(1);
	  z1->unZoom(true); 
	  z1->getPage()->redraw();
	}
      }
    }
  else if (listener->getEventType() == QpListener::LeftReleased)
    {
      if (state == PickedFirst) {
	// PickedFirst state & left button released : 
	// finishing the selection of zoom area	 
	if (fixed) {
	  setState(Zoom);	  
	} else {
	  setState(Off);
	}
	 
	QpZone *z2 = 0;
	if (listener->getPickedNode()) 
	  z2=listener->getPickedNode()->getZone();

	if (z2)
	  if (z2->is2D)
	    {
	      double x2 = listener->getXz();
	      double y2 = listener->getYz();
	      xm2=listener->getX();
	      ym2=listener->getY();
	      // If the cursor has not moved enough it will do nothing
	      if (abs(xm1-xm2)>2 && abs(ym1-ym2)>2) {
		if (z1 != z2) {
		  // If the button release has been done outside of the initial zone
		  // We must choose the closest coordinates inside the zone
		  ZoneStrategy *zs = (ZoneStrategy*)z1->getTransformation(QpTransformation::Zone);
		  if (listener->getXp() >= zs->getXzMap()->getContainer2()) {
		    x2=zs->getXzMap()->getContent2();
		  }
		  else if (listener->getXp() <= zs->getXzMap()->getContainer1()) {
		    x2=zs->getXzMap()->getContent1();
		  }
		  else {
		    x2=zs->getXzMap()->invTransform(listener->getXp());
		  }
		  if (listener->getYp() >= zs->getYzMap()->getContainer2()) {
		    y2=zs->getYzMap()->getContent2();
		  }
		  else if (listener->getYp() <= zs->getYzMap()->getContainer1()) {
		    y2=zs->getYzMap()->getContent1();
		  }
		  else {
		    y2=zs->getYzMap()->invTransform(listener->getYp());
		  }
		}
		z1->setThickness(3);	  
		z1->zoom(x1,y1,x2,y2);
	      }
	      z1->getPage()->redraw();
	    } else {
	      /*
		for (int k=0;k<180;k++) {
		QpCurve3D *qpc = (QpCurve3D *) ((z1->search("QpCurve3D")).at(0));
		qpc->moveView(-(listener->getX() - xm1)*0.01,
		-(listener->getY() - ym1)*0.01,
		0.0);
		z1->getPage()->redraw();
		}
	      */
	    }
      }
    }
  if (listener->getEventType() == QpListener::RightPressed)
    {
      if (state == Zoom) {
	// Cancel a previous zoom
	if (listener->getPickedNode()) 
	  z1=listener->getPickedNode()->getZone();
	if (z1 != 0)
	  if (!z1->isInvalid()) {
	    //	  z1->setThickness(1);	  
	    z1->unZoom();
	    z1->getPage()->redraw();
	  }
      }      
    }
}

#include <iostream>
void QpZoom::end()
{
  setState(Off);
  QpEvObserver::end();
}



QpOutLine::QpOutLine(Type t) :
 QpEvObserver((QpListener::EventType)(QpListener::LeftPressed + QpListener::LeftReleased + QpListener::Move + QpListener::Display))
{
  z1=0;
  x1=y1=x2=y2=0;
  p=0;
  
  setType(t);
  outlineState=NoSelection;
}

QpOutLine::~QpOutLine()
{  
  if (p) { p->end(); delete p; }
}

void QpOutLine::setType(Type t)
{
  type=t;
  if (p) {
    p->end();
    delete p;
    p=0;
  }
  if (type==Off) {
    if (z1) {
      z1->getPage()->setCursor(QCursor(Qt::arrowCursor));
      z1->getPage()->update();
    }
  }
  z1=0;
  x1=y1=x2=y2=0;

}

void QpOutLine::init(QpZone *z)
{
  if (z) {
    // first update -> create the painter and set the cross cursor
    z->getPage()->setCursor(QCursor(Qt::crossCursor));
    if (p) { p->end(); delete p; }
    p = new QPainter(z->getPage());
    p->setRasterOp(Qt::NotXorROP);
  }
  visibleOutline = false ;
}

void QpOutLine::getLimit()
{
  ZoneStrategy *zs = (ZoneStrategy*)z1->getTransformation(QpTransformation::Zone);
  PageStrategy *ps = (PageStrategy*)z1->getTransformation(QpTransformation::Page);
	
  if (listener->getXp() >= zs->getXzMap()->getContainer2()) {
    x2=ps->xTransform(zs->getXzMap()->getContainer2());
  }
  else if (listener->getXp() <= zs->getXzMap()->getContainer1()) {
    x2=ps->xTransform(zs->getXzMap()->getContainer1());
  }
  if (listener->getYp() >= zs->getYzMap()->getContainer2()) {
    y2=ps->yTransform(zs->getYzMap()->getContainer2());
  }
  else if (listener->getYp() <= zs->getYzMap()->getContainer1()) {
    y2=ps->yTransform(zs->getYzMap()->getContainer1());
  }
}

void QpOutLine::updateMove()
{
  QpZone *z2 = 0;
  
  if (listener->getPickedNode()) 
    z2 = listener->getPickedNode()->getZone();

  if (z2 == 0) {
    // Out of a zone: Clear remaining Outline

    if (((type & Rectangle) || (type & Ellipse)) && (outlineState == PickedFirst))
      {
	// Stop selection as we leave the zone

	drawType(type,outlineState);
	if (!(type & Rectangle)) {
	  setType((Type) (type + Rectangle));
	}

	setSelectionState(QpOutLine::Ready);
	
	/*
	outlineState = NoSelection ;
	*/
      }
    
    /*
      if (visibleOutline)
      if (z1) {	
      if (!z1->isInvalid()) {
      drawType(listenerType,state);	
      z1->draw();
      visibleOutline = false ;	  
      }
      }
    */
  } else {
    if (z2->isInvalid()) {
      // Invalid zone: anything to do ?
    } else {
      // Valid zone

      if (z2->is2D) {

	// Are we selecting an area ?
	if (((type & Rectangle) || (type & Ellipse)) && (outlineState == PickedFirst))
	  {	
	  if (x1!=x2 || y1!=y2) {
	    // Clear the last selection
	    drawType(type,outlineState);
	  }
	
	  x2=listener->getX();
	  y2=listener->getY();

	  // Stop on zone limits
	  if (z2 != z1 || (x2==0 && y2==0))  getLimit();	    
	  
	  // Draw the new selection rectangle
	  drawType(type, outlineState);
	}
	else if ((type & VLine) || (type & HLine)) {
	  // Probably locating coordinates...
	  if (z1 == 0) {
	    // first update -> create the painter and set the cross cursor
	    init(z2);
	  } else  {
	    // Clear the old outline
	    if (x2!=0 || y2!=0) {
	      // All those values are kept in the class since the previous drawing
	      drawType(type, outlineState);	    
	    }
	  }

	  updateZone(z2);
	
	  // Draw the new outline
	  x2=listener->getX();
	  y2=listener->getY();
	
	  if (x2!=0 || y2!=0) {
	    drawType(type, outlineState);	  
	  }
	}
	visibleOutline = true ;      
      } else {
	// TODO : Don't draw for every event to have faster code
	if (outlineState == PickedFirst)
	  {
	    // 3d zone draw the bounding box 
	    QpCurve3D *qpc = (QpCurve3D *) ((z1->search("QpCurve3D")).at(0));
	    qpc->moveView(-(listener->getX() - x1)*0.01,
			  -(listener->getY() - y1)*0.01,
			  0.0);
	
	    x1 = listener->getX();
	    y1 = listener->getY();
	
	    z1->getPage()->redraw();
	  }
      }
    }      
  } 
}

void QpOutLine::updateLeftPressed()
{
  if (((type & Rectangle) || (type & Ellipse)) && (outlineState==Ready))
    {
      // Building zooming rectangle
      // Anchor the shape (either one angle or the center)
      z1 = 0;
      if (listener->getPickedNode()) 
	z1=listener->getPickedNode()->getZone();
      if (z1 != 0)
	if (!z1->isInvalid()) {
	  init(z1);
	  p->setPen(QPen(Qt::DashLine));
	  x1=listener->getX();
	  y1=listener->getY();
	  x2=x1; 
	  y2=y1;
	  outlineState=PickedFirst;
	}
    }
}


void QpOutLine::updateZone(QpZone *z)
{
  // Changing zone
  if (z1 != z)
    {
      ZoneStrategy *zs2 = (ZoneStrategy*)z->getTransformation(QpTransformation::Zone);
      PageStrategy *ps2 = (PageStrategy*)z->getTransformation(QpTransformation::Page);
      xmin=ps2->xTransform(zs2->getXzMap()->getContainer1());
      xmax=ps2->xTransform(zs2->getXzMap()->getContainer2());
      ymin=ps2->yTransform(zs2->getYzMap()->getContainer1());
      ymax=ps2->yTransform(zs2->getYzMap()->getContainer2());
    }
  
  // Save the zone for the next time ...
  z1 = z;
}

void QpOutLine::updateLimit()
{
  if (z1)
    {
      ZoneStrategy *zs2 = (ZoneStrategy*)z1->getTransformation(QpTransformation::Zone);
      PageStrategy *ps2 = (PageStrategy*)z1->getTransformation(QpTransformation::Page);
      xmin=ps2->xTransform(zs2->getXzMap()->getContainer1());
      xmax=ps2->xTransform(zs2->getXzMap()->getContainer2());
      ymin=ps2->yTransform(zs2->getYzMap()->getContainer1());
      ymax=ps2->yTransform(zs2->getYzMap()->getContainer2());
    }
}


void QpOutLine::update()
{
  if (listener == 0) {
    std::cerr << "QpZoom::update() listener == 0"<< std::endl ;
    return;
  }
  
  if (listener->getEventType()==QpListener::Display) {
    x2=0; y2=0; //z1=0;
  }
  else if (listener->getEventType()==QpListener::Move) {
    updateMove();
  }
  else if (listener->getEventType()==QpListener::LeftPressed) {
    updateLeftPressed();    
  }
  else if (listener->getEventType()==QpListener::LeftReleased) {
    if (((type & Rectangle) || (type & Ellipse)) && (outlineState == PickedFirst)) {
      x1=0;
      y1=0;
      // no need to clear the last Rect since it will be done by QpZoom
      //if (listenerType & Rectangle) listenerType-=Rectangle;
      //if (listenerType & Ellipse) listenerType-=Ellipse;
      outlineState=NoSelection;
    }
    if (p) p->setPen(QPen(Qt::SolidLine));
  }
}

void QpOutLine::drawType(Type listenerType, State state)
{
  switch (state) {
  case NoSelection:
  case Ready:
    /*
      Outline cross 
     */
    if (listenerType & VLine) {    
      p->drawLine(x2,ymin,x2,ymax);
    }
    if (listenerType & HLine) {
      p->drawLine(xmin,y2,xmax,y2);
    }
    break;
  case PickedFirst:
    /*
      Click + drag rectangle in zooming
     */
    if (listenerType & Rectangle) {    
      drawRect();
    }
    if (listenerType & Ellipse) {
      drawEllipse();
    }
    break;
    
  default:
    return ;    
  }
}

void QpOutLine::drawRect()
{  
  p->drawLine(x1,y1,x2,y1);
  p->drawLine(x1,y1,x1,y2);
  p->drawLine(x2,y1,x2,y2);
  p->drawLine(x1,y2,x2,y2);
}

void QpOutLine::drawEllipse()
{
  p->drawEllipse(QRect(x1,y1,abs(x2-x1),abs(y2-y1)));
}

void QpOutLine::end()
{
  
  setType(Off);
}


QpFrameLocator::QpFrameLocator(QWidget *p, const char *name, WFlags f) :
  QFrame(p,name,f), QpEvObserver((QpListener::EventType)(QpListener::LeftPressed + QpListener::Move))
{
  
  int w=100, h=100;
  setGeometry(1,1,w,h);
  setFrameStyle(Panel | Raised);
  setLineWidth(2);
  grid = new QGridLayout( this, 8, 2, 10 );

  xpl = new QLabel(QString("X Page : "), this, "xpl");
  ypl = new QLabel(QString("Y Page : "), this, "ypl");
  xzl = new QLabel(QString("X Zone : "), this, "xzl");
  yzl = new QLabel(QString("Y Zone : "), this, "yzl");
  zonel = new QLabel(QString("Zone : "), this, "zonel");
  pickedl = new QLabel(QString("Picking : "), this, "pickedl");
  xpl->setAlignment(Qt::AlignLeft);
  ypl->setAlignment(Qt::AlignLeft);
  xzl->setAlignment(Qt::AlignLeft);
  yzl->setAlignment(Qt::AlignLeft);
  zonel->setAlignment(Qt::AlignLeft);
  pickedl->setAlignment(Qt::AlignLeft);
  grid->addWidget(xpl,0,0);
  grid->addWidget(ypl,1,0);
  grid->addWidget(zonel,3,0);
  grid->addWidget(xzl,4,0);
  grid->addWidget(yzl,5,0);
  grid->addWidget(pickedl,7,0);

  xp = new QLabel(QString("0.00"), this, "xp");
  yp = new QLabel(QString("0.00"), this, "yp");
  xz = new QLabel(QString("-"), this, "xz");
  yz = new QLabel(QString("-"), this, "yz");
  zone = new QLabel(QString("-"), this, "zone");
  picked = new QLabel(QString("-"), this, "picked");
  xp->setAlignment(Qt::AlignRight);
  yp->setAlignment(Qt::AlignRight);
  xz->setAlignment(Qt::AlignRight);
  yz->setAlignment(Qt::AlignRight);
  zone->setAlignment(Qt::AlignRight);
  picked->setAlignment(Qt::AlignRight);
  grid->addWidget(xp,0,1);
  grid->addWidget(yp,1,1);
  grid->addWidget(zone,3,1);
  grid->addWidget(xz,4,1);
  grid->addWidget(yz,5,1);
  grid->addWidget(picked,7,1);

  show();
}

QpFrameLocator::~QpFrameLocator()
{
  
}

void QpFrameLocator::update( )
{
  if (!listener) return;
  setXp(listener->getXp());
  setYp(listener->getYp());
  setXz(listener->getXz());
  setYz(listener->getYz());
  setZone(listener->getZoneTitle());
  if (listener->getPicked()) setPicked(listener->getPicked()->isA());
  else setPicked("-");
}

void QpFrameLocator::end( )
{
  
  close();
}

void QpFrameLocator::closeEvent( QCloseEvent *e )
{
  
  listener->removeObserver(this);
  e->accept();
}


 


