#ifndef QP_MAP_H
#define QP_MAP_H

/** %%%

  Usage:

    QpMap maps a double interval (called the container interval) into another 
  one (called the content interval).
  This class is used in QpZone to map the zone system of coordinates and 
  the page one.
  It has been created from the QwtDiMap class which maps
  an double interval into an integer interval (used in QpPage).
  
  TODO:

*/


#include <qglobal.h>
#include <string.h>
#include <math.h>

 
class QpMap {
private:
  bool ContainerEmpty ;
  bool ContentEmpty ;

  // container interval low boundary	 
  double container1,container1_;
  // container interval high boundary	 
  double container2,container2_;  
  // content  interval low boundary	 
  double content1,content1_;
  // content interval high boundary	 
  double content2,content2_;  
  // conversion factor
  double d_cnv;
  // logarithmic scale?
  bool d_log;		
  void newFactor();	
public:	
  static const double LogMin;
  static const double LogMax;
  /// Empty constructor
  QpMap();
  /** Constructor taking the boundaries of the two interval and a log/linear
    option
  */
  QpMap(double _content1, double _content2, double _container1, double _container2, bool lg = FALSE);
  QpMap *copy();
  
  /// Destructor
  ~QpMap();
  // Value contained in one interval?
  //bool contains(double x, char *interval="content") const;

  bool isContainerEmpty() { return ContainerEmpty; }
  bool isContentEmpty()   { return ContentEmpty; }
  bool isInvalid() { return (ContainerEmpty || ContentEmpty);}
      
  /// Retrieve container range
  void setContainerRange(double i1, double i2);
  /// Retrieve content range
  void setContentRange(double d1, double d2, bool lg = FALSE);
  /// Retrieve log/linear option
  bool logarithmic() const {return d_log;}
  /// Set log/linear option
  void setLog( bool lg);
  /// Transform content -> container 
  double transform(double x) const;
  /// Transform container -> content
  double invTransform(double i) const;
  /** The function is similar to transform(), but limits the input value
    to the nearest border of the map's content interval if it lies outside
    that interval.
  */
  double limTransform(double x) const;
  /// Accessor to low limit of container interval
  double getContainer1() const {return container1;}
  /// Accessor to high limit of container interval
  double getContainer2() const {return container2;}
  /// Accessor to low limit of content interval
  double getContent1() const {return (d_log)? pow(10,content1) : content1;}
  /// Accessor to high limit of content interval
  double getContent2() const {return (d_log)? pow(10,content2) : content2;}
  /// Transformation of width from zone space to page space
  double widthTransform(double w) const;
  /// Inverse transformation of width from page space to zone space
  double invWidthTransform(double w) const;
  // Get conversion factor
  double getFactor() {return d_cnv;}

};

#endif






