#include <stdlib.h>
#include "Qplotter/qp/qpmarker.h"

QpMarker::QpMarker(QpObject *z,bool addOnParent,QpTransformation::tStrategy st)
  : QpObject(new QString("QpMarker"),z,addOnParent,st) {
    symStyle=None;
    lineStyle=NoLine;
    x=y=w=h=xc=yc=wc=hc=0.0;
}

QpMarker::QpMarker(SymStyle ss, LineStyle ls, QpObject *z, bool addOnParent,
	   QpTransformation::tStrategy st) : 
  QpObject(new QString("QpMarker"),z,addOnParent,st)
{
  symStyle=ss; 
  lineStyle=ls; 
  x=y=w=h=xc=yc=wc=hc=0.0; 
}

void QpMarker::setGeometry(double _x, double _y, double _w, double _h, bool center) 
{
  if (center) {
    x=_x; y=_y;
  }
  else {
    x=_x+(_w/2.0);
    y=_y+(_h/2.0);
  }
  w=_w;h=_h;
}

void QpMarker::setSymbolSize(double _w, double _h) 
{
  if ((_w>0) && (_h>0)) {
    w=_w;
    h=_h;
  }
}

void QpMarker::setLineGeometry(double _xc, double _yc, double _wc, double _hc) 
{
  wc=_wc;hc=_hc;
  xc=_xc; yc=_yc;
}

void QpMarker::drawSymbol(int _x, int _y, int _w, int _h, QPainter *p)
{
  // draw symbol
  if (symStyle != None) {
    if (_w < 1) _w = 1;
    if (_h < 1) _h = 1;
    if ((_w >= 2) || (_h >= 2)) {
      int w2 = _w / 2;
      int h2 = _h / 2;     
      switch(symStyle)
	{
	case Ellipse:
	  d_pa.makeEllipse(_x-w2,_y-h2,_w+1,_h+1);
	  //p->drawEllipse(_x-w2,_y-h2,_w,_h);
	  p->drawPolygon(d_pa);
	  break;
	case Rect:
	  d_pa.resize(4);
	  d_pa.setPoint(0, _x-w2, _y-h2);
	  d_pa.setPoint(1, _x+w2, _y-h2);
	  d_pa.setPoint(2, _x+w2, _y+h2);
	  d_pa.setPoint(3, _x-w2, _y+h2);
	  p->drawPolygon(d_pa);
	  break;
	case Diamond:
	  d_pa.resize(4);
	  d_pa.setPoint(0, _x   , _y-h2);
	  d_pa.setPoint(1, _x+w2, _y);
	  d_pa.setPoint(2, _x   , _y+h2);
	  d_pa.setPoint(3, _x-w2, _y);
	  p->drawPolygon(d_pa);
	  break;
	case Cross:
	  p->drawLine(_x-w2 + w2, _y-h2, _x-w2 + w2, _y+h2);
	  p->drawLine(_x-w2, _y-h2 + h2, _x+w2, _y-h2 + h2);
	  break;
	case XCross:
	  p->drawLine(_x-w2, _y-h2, _x+w2, _y+h2);
	  p->drawLine(_x-w2, _y+h2, _x+w2, _y-h2);
	  break;
	case Triangle:
	case UTriangle:
	  d_pa.resize(3);
	  d_pa.setPoint(0, _x-w2 + w2, _y-h2);
	  d_pa.setPoint(1, _x+w2, _y+h2);
	  d_pa.setPoint(2, _x-w2, _y+h2);
	  p->drawPolygon(d_pa);
	  break;
	case DTriangle:
	  d_pa.resize(3);
	  d_pa.setPoint(0, _x-w2, _y-h2);
	  d_pa.setPoint(1, _x+w2, _y-h2);
	  d_pa.setPoint(2, _x-w2 +  w2, _y+h2);
	  p->drawPolygon(d_pa);
	  break;
	case LTriangle:
	  d_pa.resize(3);
	  d_pa.setPoint(0, _x-w2, _y-h2);
	  d_pa.setPoint(1, _x+w2, _y-h2 + h2);
	  d_pa.setPoint(2, _x-w2, _y+h2);
	  p->drawPolygon(d_pa);
	  break;
	case RTriangle:
	  d_pa.resize(3);
	  d_pa.setPoint(0, _x+w2, _y-h2);
	  d_pa.setPoint(1, _x-w2, _y-h2 + h2);
	  d_pa.setPoint(2, _x+w2, _y+h2);
	  p->drawPolygon(d_pa);
	  break;
	default:
	  break;
	}
      d_pa.resize(0);  // avoids memory consumming ?
    }else
      // Marker too small, draw a point instead
      p->drawPoint(_x , _y);
  }
}

void QpMarker::drawLineMarker(int _x, int _y, QPainter *painter)
{
  QpTransformation *transf = getTransformation(getTransfStrategy());  
  int _xc = transf->xTransform(xc);
  int _yc = transf->yTransform(yc);
  int right = transf->xTransform(xc+wc);
  int bottom = transf->yTransform(yc-hc);
  // draw lines
  switch(lineStyle) {
  case ErrorCross :
    painter->drawLine(_x,_yc,_x,bottom); //_yc+_hc);
    painter->drawLine(_xc,_y,right,_y); //_xc+_wc,_y);
    break;
  case NoLine :
    break;
  default :
    if (lineStyle == HLine)
      painter->drawLine(_xc, _y, right, _y);
    if (lineStyle == VLine)
      painter->drawLine(_x, _yc, _x, bottom);
  }
}

void QpScalableMarker::draw() 
{
  paint(getPainter());
}

void QpScalableMarker::print()
{
  paint(getPrinter());
}


void QpScalableMarker::paint(QPainter *painter) 
{
  // This draw is called after the zone's one so the Transformation object exists
  QpTransformation *transf = getTransformation(getTransfStrategy());  
  // Transform coords in pixel 
  int _x = transf->xTransform(x);
  int _y = transf->yTransform(y);
  // Transform width and height in pixel coords
  int _w = transf->xTransform(x+w) - _x;
  int _h = _y - transf->yTransform(y+h);
       
  drawLineMarker(_x,_y,painter);
  drawSymbol(_x,_y,_w,_h,painter);
}

void QpFixedSizeMarker::paint(QPainter *painter) 
{
  QpTransformation *transf = getTransformation(getTransfStrategy());  
  // Transform coords in pixel 
  int _x = transf->xTransform(x);
  int _y = transf->yTransform(y);

  drawLineMarker(_x,_y,painter);
  drawSymbol(_x,_y,w,h,painter);
}

void QpFixedSizeMarker::draw()
{
  paint(getPainter());
}

QpObject *QpFixedSizeMarker::touched(double _x, double _y, double errorX, double errorY, QpObject *& /* node */)
{
  bool res = false;
  QpTransformation *transf = getTransformation(getTransfStrategy());  
  int xMarker = transf->xTransform(x);
  int yMarker = transf->yTransform(y);
  int _xc = transf->xTransform(xc);
  int _yc = transf->yTransform(yc);
  int right = transf->xTransform(xc+wc);
  int bottom = transf->yTransform(yc-hc);
  QpTransformation *tp = getTransformation(QpTransformation::Page);  
  int xt = tp->xTransform(_x), yt = tp->yTransform(_y);
  int errorXt = tp->xTransform(_x+errorX) - xt, errorYt = yt - tp->yTransform(_y+errorY);

  if (lineStyle==QpMarker::ErrorCross){ 
    res = ((abs(xt - _xc) <= errorXt) && ((yt-errorYt <= bottom) && (yt+errorYt >= _yc)));  // close to the y bar or on the x bar
    res = res || ((abs(yt - _yc) <= errorYt) && ((xt-errorXt <= right) && (xt+errorXt >= _xc)));
    if (res) return this;
  }
  res = res || ((xt-errorXt <= xMarker+w/2) && (xt+errorXt >= xMarker-w/2));
  res = res && (yt-errorYt <= yMarker+h/2) && (yt+errorYt >= yMarker-h/2);
  if (res) return this;

  return 0;
}

void QpFixedSizeMarker::print()
{
  paint(getPrinter());
}

QpObject *QpMarker::touched(double _x, double _y, double errorX, double errorY, QpObject *& /* node */)
{
  bool res = false;
  QpTransformation *transf = getTransformation(getTransfStrategy());  
  int xMarker = transf->xTransform(x);
  int yMarker = transf->yTransform(y);
  int _w = transf->xTransform(x+w) - xMarker;
  int _h = yMarker - transf->yTransform(y+h);
  int right = transf->xTransform(xc+wc);
  int bottom = transf->yTransform(yc-hc);
  int _xc = transf->xTransform(xc);
  int _yc = transf->yTransform(yc);
  QpTransformation *tp = getTransformation(QpTransformation::Page);  
  int xt = tp->xTransform(_x), yt = tp->yTransform(_y);
  int errorXt = tp->xTransform(_x+errorX) - xt, errorYt = yt - tp->yTransform(_y+errorY);

  if (lineStyle==QpMarker::ErrorCross){ 
    res = ((abs(xt - _xc) <= errorXt) && ((yt-errorYt <= bottom) && (yt+errorYt >= _yc)));  // close to the y bar or on the x bar
    res = res || ((abs(yt - _yc) <= errorYt) && ((xt-errorXt <= right) && (xt+errorXt >= _xc)));
    if (res) return this;
  }
  else {
    res = (xt-errorXt <= xMarker+_w/2) && (xt+errorXt >= xMarker-_w/2);
    res = res && (yt-errorYt <= yMarker+_h/2) && (yt+errorYt >= yMarker-_h/2);
    if (res) return this;
  }
  return 0;
}


