#ifndef QPMATH_H
#define QPMATH_H

#include <qdom.h>
#include <qfont.h>
#include "Qplotter/qp/qpmathboxlist.h"
#include "Qplotter/qp/qpcommontext.h"
#include <map>
#if defined __hpux || (defined __sun && __SUNPRO_CC < 0x500)
#define QPSTD
#else
#define QPSTD std
#endif

// The default reduction factor of subscript/superscript font size
const float defSubSupFact=0.5;

/** %%%
    
  Usage:

    A class to draw scientific formulas. It accepts a subset of MathML
  as the formual definition. It support subscript/superscript (1 level only),
  greek letters, mathematical symbols (as in the Symbol font) and square root.
  Elements are not stretchy (i.e. they're height is not adjusted according
  to what's coming after or among them).
  The formula is transformed in a linear list of boxes (QpMathBox) that
  are typeset in the pixel space. Before rendering the formula, the 
  coordinates are adjusted to the real place in the Qplotter system of 
  coordinates (taking into account alignment properties as well).

  TODO:

*/


class QpMath : public QpCommonText {
public:
  /// Constructor
  QpMath(const char *value, QpObject *z, bool addOnParent=true, 
	 QpTransformation::tStrategy st= QpTransformation::Zone);
  ///Destructor
  ~QpMath();
  /// Draw the formula
  void draw ();
  /// Print the formula
  void print ();
  // Print out the XML attributes (debug)
  void nlist(QDomNodeList qnl);
  /// Returns the XML root element list
  QDomNodeList getRoot();
  /// Return the width of the formula in pixels.
  int getPixWidth(QPainter *p) const {return listOfBoxes->getPixWidth(p);}
  /// Return the height of the text in pixels. 
  int getPixHeight(QPainter *p) const {return listOfBoxes->getPixHeight(p);}
  /// Set subscript/superscript factor
  void setSubSupFact(float ssF) {subSupFactor = ssF;}
  /// Get subscript/superscript factor
  float getSubSupFact() {return subSupFactor;}
private:
  enum tags {msub=1,msup=2,msubsup=3,munder=4,mover=5,munderover=6,msqrt=7};
  // ??
  QDomDocument doc;
  // Standard font
  QFont *stdfont;
  // Symbol font
  QFont *symfont;
  // Standard font for subscript/superscript
  QFont *stdsubsup;
  // Symbol font for subscript/superscript
  QFont *symsubsup;
  // Size of subscript/superscript font
  int subSupSize;
  int ySub;
  int ySup;
  int yUnder;
  int yOver;
  // Map used to convert element names (strings) in enums
  QPSTD::map<QString,QpMath::tags> dict;
  float subSupFactor;
private:
  // The container of all QpMathBox the fromula consists of.
  QpMathBoxList *listOfBoxes;
  bool twoArgs(QDomElement e, QDomElement& e1, QDomElement& e2);
  bool threeArgs(QDomElement e,QDomElement& e1,QDomElement& e2,
		 QDomElement& e3);
  void initFont(QFont &);
  void makeBoxes(int &x, int &y, QDomElement& e, QFont* stdF, QFont* symF) ;
  void makeBoxes (int &x, int &y, QDomNodeList qnl, QFont* stdF, QFont* symF);
  void makeBoxes ();
  void prepareBoxes(int& xCoord,int& yCoord, QPainter *);
};

#endif
