/*
  setFont(QFont("Symbol",32,QFont::Bold));

  p.setFont( font() ):
  p.setPen(..);
  p.drawText(..);
  
*/

#include <iostream>

#include <qfont.h>
#include <qpainter.h>
#include "qpmathbox.h"

QpMathBox::QpMathBox(QFont *theFont, QpObject *z, bool addOnParent) : 
    QpObject(new QString("QpMathBox"),z,addOnParent) , curFont(theFont) {
}

void QpMathBox::setCoords(int _xC, int _yC) {
  xC = _xC;
  yC = _yC;
}

void QpMathBox::draw() {
  paint(getPainter());
}

void QpMathBox::print() {
  paint(getPrinter());
}

void QpMathBox::paint(QPainter *painter) {
  QFont oldF = QFont(painter->font());
  // Change font only when required
  if (oldF != *curFont) {
    painter->setFont(*curFont);
    /*
    cout << "QpMathBox::QpMathBox.paint(0)  painter->font() = " <<
      painter->font().family() << " Current = " << curFont->family() << endl ;
    */
    painter->drawText(xC, yC, nodeVal);
    painter->setFont(oldF);
  } else {
    //    cout << "QpMathBox::QpMathBox.paint(1)  "  <<
    //      painter->font().family() << " Current = " << curFont->family() << endl ;
      painter->drawText(xC, yC, nodeVal);
    /*
    painter->setFont( QFont("Symbol",32,QFont::Bold) );
    painter->setPen(QColor(255,0,0));
    painter->drawText(xC, yC, nodeVal);
    */
  }
}

QpSymbolBox::QpSymbolBox(QFont *theFont, QString entityVal, QpObject *z, 
			 bool addOnParent) : QpMathBox(theFont,z,addOnParent){
  entityVal.remove(0,7);
  int nc=entityVal.toInt();
  QChar qc = nc;
  nodeVal = qc;
  QFontMetrics qfm (*theFont);
  width = qfm.width(nodeVal);
  height = qfm.height();
}

QpTextBox::QpTextBox(QFont *theFont, QString textVal, QpObject *z,
			 bool addOnParent) : QpMathBox(theFont,z,addOnParent){
   nodeVal = textVal;
  QFontMetrics qfm (*theFont);
  width = qfm.width(nodeVal);
  height = qfm.height();
}

QpRootBox::QpRootBox(QFont *theFont, int rootWidth, QpObject *z,
			 bool addOnParent) : QpMathBox(theFont,z,addOnParent){
  QFontMetrics qfm (*theFont);
  skew = qfm.width(QString(QChar(214)));
  nodeVal = QChar(96);
  width = qfm.width(nodeVal);
  while (width < rootWidth ) {
    nodeVal.append(QChar(96));
    width = qfm.width(nodeVal);
  }
  height = qfm.height();
  width += skew;
}

void QpRootBox::draw() {
  paint(getPainter(),true);
}

void QpRootBox::print() {
  /* Workaround: when printing with fonts where the leftbearing of radix
     is zero, we should not advance the x coordinate! */
  paint(getPrinter(),false);
}

void QpRootBox::paint(QPainter *painter, bool advanceX) {
  bool fontChanged = false;
  QFontMetrics qfm (*curFont);
  QFont oldF = QFont(painter->font());
  // Change font only when required
  if (oldF != *curFont) {
    fontChanged = true;
    painter->setFont(*curFont);
  }
  QString radic(QChar(214));
  /*
  cout << "QpRootBox::paint()  "  <<
    painter->font().family() << " Current = " << curFont->family() << endl ;
  */
  painter->drawText(xC, yC, radic);
  if (qfm.leftBearing(QChar(96)) != 0)
    xC += skew - qfm.leftBearing(QChar(96)) - qfm.rightBearing(QChar(214));
  else {
    /* Workaround: when printing with fonts where the leftbearing of radix
       is zero, we should not advance the x coordinate! */
    if (advanceX)
      xC += skew;
  }
  /*
  cout << "QpRootBox::paint()  "  <<
    painter->font().family() << " Current = " << curFont->family() << endl ;
  */
  painter->drawText(xC, yC, nodeVal);
  // Restore font if necessary
  if (fontChanged)
    painter->setFont(oldF);   
}


