#include "Qplotter/qp/qpmultiline.h"
#include "Qplotter/qp/qpqwt_math.h"

QpMultiLine::QpMultiLine(int nbElements, double *_x, double *_y, QpObject *z, bool addOnParent, QpTransformation::tStrategy st) : 
  QpNode<QpObject>(new QString("QpMultiLine"), z, addOnParent, st) 
{
  nb = nbElements;
  xVector= new double [nb]; 
  yVector= new double [nb];
  for(int i=0;i<nb;i++) {
    xVector[i]=_x[i]; 
    yVector[i]=_y[i];
  }
}

void QpMultiLine::paint(QPainter *painter)
{
  QpTransformation *transf = getTransformation(getTransfStrategy());  
  // Possibly change default pen/brush/font
  changeStyle(painter);

  QPointArray data(nb);
  for(int i=0;i<getNb();i++) 
    data.setPoint(i,transf->xTransform(getX(i)), transf->yTransform(getY(i)));
  painter->drawLineSegments(data, 0, getNb());  

  // Restore default pen/brush/font
  restoreStyle(painter);
}

void QpMultiLine::draw()
{
  paint(getPainter());
}

void QpMultiLine::print()
{
  paint(getPrinter());
} 

void QpMultiLine::computeCoeffs(double x1, double y1, double x2, double y2, double *a, double *b)
{
  *a = (y2-y1)/(x2-x1);
  *b = y1 - (*a * x1);
}

QpObject *QpMultiLine::touched(double xC, double yC, 
			       double errorX, double errorY, QpObject *& /*node*/)
{
  double x1, y1, x2, y2, a, b;
  double xz, yz, errorXz, errorYz;
  ZoneStrategy *t = (ZoneStrategy*)getZone()->getTransformation(QpTransformation::Zone);
  xz = t->getXzMap()->invTransform(xC);
  errorXz = t->getXzMap()->invTransform(xC+errorX) - xz;
  yz = t->getYzMap()->invTransform(yC);
  errorYz = t->getYzMap()->invTransform(yC+errorY) - yz;

  //x1 = getX(0); y1 = getY(0);
  for(int i=1;i<getNb();i+=2) {
    x1=getX(i-1); y1=getY(i-1);
    x2 = getX(i); y2 = getY(i);
    if ((x1==x2) && (fabs(x1-xz)<=errorXz)) {
      if ((yz-errorYz <= qwtMax(y1,y2)) && (yz+errorYz >= qwtMin(y1,y2))) return this;
    }
    else if ((y1==y2) && (fabs(y1-yz)<=errorYz)) {
      if ((xz-errorXz <= qwtMax(x1,x2)) && (xz+errorXz >= qwtMin(x1,x2))) return this;
    }
    else if ((qwtMin(x1,x2) <= xz+errorXz) && (xz-errorXz <= qwtMax(x1,x2))) {
      computeCoeffs(x1,y1,x2,y2,&a,&b);
      if (fabs(yz-(a*xz+b)) <= errorYz) return this;
    }
  }
  return 0;
}


