#include <stdlib.h>
#include <assert.h>
#include <qapplication.h>
#include <qpaintdevicemetrics.h>
#include <qimage.h> 
#include "Qplotter/qp/qpnode.cpp"
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpaxis.h"
#include "Qplotter/qp/qpqwt_dimap.h"
#include <qdict.h>

// QDict<int> ElementList::priority = QDict<int>(173);

QpPage::QpPage(QWidget *_w, QRect geo, double wp, double hp) : 
  QWidget(_w, "QpPage") , 
  QpNode<QpObject>(new QString("QpPage"),0,false,QpTransformation::Page), 
  pagetitle(0) , w(wp), h(hp), nbXZones(1), nbYZones(1)  
{ 
    setGeometry(geo);   
    setBackgroundMode(QWidget::NoBackground);
    initPage(geo.width(),geo.height());
}

void QpPage::initPage (int geo_w, int geo_h)
{
  printer=0;
  printdevice=0;
  
  screenWidth  = -1 ;
  screenHeight = -1 ;
  
  _buffer = new QPixmap(geo_w,geo_h);
  _buffer->fill(white);

  painter = new QPainter(_buffer);
  
  QpPage::setQpPen(QPen(Qt::black,1)); 
  QpPage::setQpBrush(QBrush(Qt::white));
  QpPage::setQpFont(QFont(QApplication::font()));

  if (pen)    getPainter()->setPen(*pen);
  if (brush)  getPainter()->setBrush(*brush);
  if (mFont)  getPainter()->setFont(*mFont);
  

  fixedRatio = true;
  finePicking = false;

  // Events managing
  locate=false;
  setMouseTracking(true); 
  zoom = 0;

  listener = new QpListener () ;
  outline  = new QpOutLine (QpOutLine::Off);
  outline->lRegister(listener);

  // Zones / margins variables init
  leftMargin = rint(getWidth() / 10);
  bottomMargin = rint(getHeight() / 10);

  QpQwtDiMap *xMap, *yMap;
  double rapport;
  // Create pixel<-->page transformation
  int ww = width()-1,hw=height()-1;
  if (h > w) { 
    rapport = (double)w/(double)h;
    int x = (int)rint((ww - hw*rapport)/2);
    xMap = new QpQwtDiMap(x,ww-x,0.0,w);
    yMap = new QpQwtDiMap(0,hw,0.0,h);    // y,(int)rint(ww*rapport)+y,0.0,h);
  }
  else {
    rapport = (double)h/(double)w;
    int y = (int)rint((hw - ww*rapport)/2);
    xMap = new QpQwtDiMap(0,ww,0.0,w);       //(int)rint(hw*rapport),0.0,w);
    yMap = new QpQwtDiMap(y,hw-y,0.0,h);
  }

  transformation = new PageStrategy(xMap, yMap);
  originalXmin = transformation->getXpMap()->i1();
  originalXmax = transformation->getXpMap()->i2();
  originalYmin = transformation->getYpMap()->i1();
  originalYmax = transformation->getYpMap()->i2();

}


QpPage::~QpPage() {
  if (painter != 0)   {
    if (painter->isActive())
      painter->end();
    delete painter;
  }
  if (printer != 0)   {
    if (printer->isActive())
      printer->end();
    delete printer;
  }
  delete printdevice;
  delete _buffer;
  if (zoom) {
    zoom->end();
    delete zoom;
  }
  delete listener;
  // Delete page-to-pixel maps
  delete (transformation->getXpMap());
  delete (transformation->getYpMap());
  // Delete page transformation
  delete transformation;
  /*
  if (zoom) {
    zoom->end();
    delete zoom;
  }
  */
  delete outline;  
}

void QpPage::init()
{
  
  painter=0; printer=0; printdevice=0;
  screenWidth  = -1 ;
  screenHeight = -1 ;
  QpPage::setQpPen(QPen(Qt::black,1));
  QpPage::setQpBrush(QBrush(Qt::white));
  QpPage::setQpFont(QFont(QApplication::font()));
  fixedRatio = true; // false
  finePicking = false;

  listener = new QpListener () ;

  // Events managing
  locate=false;
  setMouseTracking(true); 
  zoom = 0;
  outline = new QpOutLine (QpOutLine::Off);
  outline->lRegister(listener);

  // Zones / margins variables init
  leftMargin = rint(getWidth() / 10);
  bottomMargin = rint(getHeight() / 10);
}

 
/****************************************/
/****************************************/

QPrinter * const QpPage::getPrintDevice()
{
  if (!printdevice) {
    printdevice = new QPrinter();
    //printer->setOrientation( QPrinter::Landscape );
    printdevice->setOutputToFile( TRUE );
    printdevice->setOutputFileName("default.ps");
    printdevice->setPageSize(QPrinter::A4);
  }
  return printdevice;
}

void QpPage::setQpPen(QPen p)
{
  QpNode<QpObject>::setQpPen(p);
  if (painter) painter->setPen(p);
  if (printer) printer->setPen(p);
}

void QpPage::setQpFont(QFont f)
{
  QpNode<QpObject>::setQpFont(f);
  if (painter) painter->setFont(f);
  if (printer) printer->setFont(f);
}

void QpPage::setQpBrush(QBrush b)
{
  QpNode<QpObject>::setQpBrush(b);
  if (painter) painter->setBrush(b);
  if (printer) printer->setBrush(b);
}

/****************************************/
/************** PAINTING ****************/
/****************************************/

void QpPage::redraw()
{
  if (getPainter()) { 
    clear(); 
    draw(); 
    repaint(true);
  } 
}

void QpPage::initPainting(int widthPaint, int heigthPaint)
{
  QpQwtDiMap *xMap, *yMap;
  if (painter) return;
  double rapport;
  _buffer = new QPixmap(widthPaint,heigthPaint);
  _buffer->fill(white);
  painter = new QPainter(_buffer);

  if (pen)    getPainter()->setPen(*pen);
  if (brush)  getPainter()->setBrush(*brush);
  if (mFont)  getPainter()->setFont(*mFont);

  
  
  // Create pixel<-->page transformation
  int ww = width()-1,hw=height()-1;
  if (heigthPaint > widthPaint) { 
    rapport = (double)widthPaint/(double)heigthPaint;
    int x = (int)rint((ww - hw*rapport)/2);
    xMap = new QpQwtDiMap(x,ww-x,0.0,widthPaint);
    yMap = new QpQwtDiMap(0,hw,0.0,heigthPaint);    // y,(int)rint(ww*rapport)+y,0.0,h);
  }
  else {
    rapport = (double)heigthPaint/(double)widthPaint;
    int y = (int)rint((hw - ww*rapport)/2);
    xMap = new QpQwtDiMap(0,ww,0.0,widthPaint);       //(int)rint(hw*rapport),0.0,w);
    yMap = new QpQwtDiMap(y,hw-y,0.0,heigthPaint);
  }
  transformation = new PageStrategy(xMap, yMap);
  originalXmin = transformation->getXpMap()->i1();
  originalXmax = transformation->getXpMap()->i2();
  originalYmin = transformation->getYpMap()->i1();
  originalYmax = transformation->getYpMap()->i2();
}

void QpPage::draw() {
  QpNode<QpObject>::drawList("");
  /*
  QPtrListIterator<QpObject> iGO(list); 
  for( ; iGO.current(); ++iGO ) {
    iGO.current()->draw();
  }
  */
}

bool QpPage::save(const QString& file,const char* format)
{
  QImage img = _buffer->convertToImage ();
  return img.save(file,format); 
}


QpTransformation *QpPage::getTransformation(QpTransformation::tStrategy s) 
  const  { 
  if (s==QpTransformation::Page) 
    return transformation; 
  else 
    assert(
      "QpPage::getTransformation: can't return the requested transformation");
  return 0;
}

void QpPage::print(QString fichier, bool setup)
{
  int xmin=0,xmax=0,ymin=0,ymax=0;
  if (!printdevice) {
    printdevice = new QPrinter();
    //printer->setOrientation( QPrinter::Landscape );
    printdevice->setOutputToFile( TRUE );
    printdevice->setPageSize(QPrinter::A4);
  }
  // Update filename associated to print device
  if ((fichier != 0) && (strlen(fichier) > 0))
    printdevice->setOutputFileName( fichier );

  //QFont font = painter->font();
  if (setup) 
    if (!printdevice->setup()) return;

  QPaintDeviceMetrics metrics( printdevice );
  int ww = metrics.width(),hw=metrics.height();
  // Retrieve widget size
  int _w = width(), _h = height();
  double rapport = 1;
  int y=0,x=0; 
  
  if (!printer) printer = new QPainter( printdevice ); 
  else printer->begin( printdevice );

  if (fixedRatio) {
    // Ignore the real widget ratio and assume we want a squared image
    // anyhow.
    y = (hw-ww)/2;

    xmin = transformation->getXpMap()->i1();
    xmax = transformation->getXpMap()->i2();
    ymin = transformation->getYpMap()->i1();
    ymax = transformation->getYpMap()->i2();

    /// Use current ratio with maximum resolution
    int xminnew	= 0 ;
    int yminnew	= 0 ;
    int xmaxnew	= 800 ;
    int ymaxnew	= 1200 ;    

    rapport = ((double)(originalYmax - originalYmin)/
	       (double)(originalXmax - originalXmin));

    
    if (rapport > (29.7/21)) {
      xminnew = (int) (800 - ((ymaxnew - yminnew) / rapport))/2 ;
      xmaxnew = 800 - xminnew ;
    } else {
      yminnew = (int) (1200 - (rapport*(xmaxnew - xminnew)))/2 ;
      ymaxnew = 1200 - yminnew ;
    }    
    
    transformation->getXpMap()->setIntRange(xminnew, xmaxnew);
    transformation->getYpMap()->setIntRange(yminnew, ymaxnew);
    printer->setWindow( 0, 0, 800, 1200);    
  }
  else {
    // Define print coordinates so that we keep the same ratio and the
    // same pixel coordinates as on the page widget.
    // This reduces 
    rapport = ((double)_h) / ((double)_w);
    if (rapport <= (29.7/21)) {
      // Ratio less than A4 ratio (landscape like)
      // y is the distance from the top of QpaintDevice Postscript
      // to the top of area we're going to print
      y = (int)rint((hw - (ww*rapport))/2);
      // Restrict printing in a sub-area whose size keeps the ratio of the
      // page widget.
      printer->setViewport(x,y,ww,int( rint(ww*rapport) ));
    }
    else {
      // Ratio greater than A4 ratio (portrait like)
      rapport = ((double)_w) / ((double)_h);
      x = (int)rint((ww - (hw*rapport))/2);
      printer->setViewport(x,y,int(rint(hw*rapport)),hw);
    }
    // Now we change printer coordinates system (i.e. coordinates in the
    // sub-area) so that pixel coordinates are exactly as on screen.
    // This allows to re-use the transformation pixel<->page
    // but does not allow to exploit completely Postscript precision
    // e.g. Postscript would allow 1200X800, we use only 600X600
    printer->setWindow( 0, 0, width(), height());
  }
  
  // Since we're in the page we skip the get methods
  if (pen != 0) getPrinter()->setPen(*pen);
  if (brush != 0) getPrinter()->setBrush(*brush);
  if (mFont != 0) getPrinter()->setFont(*mFont);

  printList("QpZone");
  
  /*
    QPtrListIterator<QpObject> zones(list);
  
    for ( ; zones.current(); ++zones ) {
    zones.current()->print();
    }
  */
  
  if (printer->isActive()) printer->end();
  if (fixedRatio) {
    // Restore previous maps
    transformation->getXpMap()->setIntRange(xmin, xmax);
    transformation->getYpMap()->setIntRange(ymin, ymax);
  }

  //delete printer; printer = 0;
}


/****************************************/
/****************************************/

void QpPage::setZooming(bool activate, bool fixedstate, QpZoom::State state)
{
  if (!zoom && activate) {
    zoom = new QpZoom(outline,fixedstate);
    zoom->lRegister(listener);
    zoom->setState(state);
  }
  else if (zoom) {
    if (!activate) zoom->setState(QpZoom::Off);
    else zoom->setState(state);
  }
}


// Retrieve the zone indexed by index
QpZone * const QpPage::getZone(int index) const {

  return zones[index];
  
  /*
  int curZone = 0;
  QpZone *retval = 0;

  QPtrListIterator<QpObject> iGO(list); 
  for( ; iGO.current(); ++iGO ) {
    if (iGO.current()->isA(QString("QpZone"))) {
      if (curZone == index)
	retval = (QpZone*)iGO.current();
      curZone++;
    }
  }
  return retval;
  */
}

QpZone *QpPage::touchedZone(double _x, double _y) 
{
  QpZone *retval = 0;
  QPtrList<QpObject> zoneList = findType("QpZone");

  for (zoneList.first();
       zoneList.current() != 0;
       zoneList.next())
    {
      QpZone *thisZone = (QpZone *) zoneList.current();
      
      if (thisZone->isTouched(_x,_y))
	retval = thisZone;
      
    }
  
  /*
    TODO
  QPtrListIterator<QpObject> iGO(list); 
  for( ; iGO.current(); ++iGO ) {
    if (iGO.current()->isA(QString("QpZone"))) {
      QpZone *zp = (QpZone*)iGO.current();
      if (zp->isTouched(_x,_y))
	retval = zp;
    }
  }
  */
  return retval;
}

void QpPage::updateListener(QEvent *ev)
{
  
  if (ev->type() == QEvent::Paint) {
  
    listener->setEventType(QpListener::Display);
    listener->setZoneCoords(0.0,0.0);
    listener->setZoneTitle("-");
    listener->setPicked(0);
    listener->setPickedNode(0);
    //listener->setEventType(QpListener::None);
  }
  else {
  
    QMouseEvent *e = (QMouseEvent*)ev;
    listener->setPixelCoords(e->x(),e->y());
    // Picking point in page coordinates
    double x = transformation->invXTransform(e->x());
    double y = transformation->invYTransform(e->y());
    // Did we click on a zone?
    QpZone *zone = touchedZone(x,y);
    QpObject *node = zone;
    listener->setPageCoords(x,y);
    if (e->type() ==  QEvent::MouseMove) {      
      
      listener->setEventType(QpListener::Move);
    }    
    if (e->button() == LeftButton) {      
      if (e->type() ==  QEvent::MouseButtonPress) 
	listener->setEventType(QpListener::LeftPressed);
      if (e->type() ==  QEvent::MouseButtonRelease) 
	listener->setEventType(QpListener::LeftReleased);
      if (e->type() ==  QEvent::MouseButtonDblClick) 
	listener->setEventType(QpListener::LeftDouble);
    }
    else if (e->button() == RightButton) {
      
      if (e->type() ==  QEvent::MouseButtonPress) 
	listener->setEventType(QpListener::RightPressed);
      if (e->type() ==  QEvent::MouseButtonRelease) 
	listener->setEventType(QpListener::RightReleased);
      //if (e->type() ==  QEvent::MouseButtonDblClick) 
      //listener->setEventType(QpListener::RightDouble);
    }
    if (zone != 0) {
      
      // Pick an object which is contained in the rectangle 
      // [xPickRange,yPickRange]
      // (page coordinates) centered on the picking point.
      // The range is 0.5% of page dimensions
      double xPickRange = getWidth()/200;
      
      double yPickRange = getHeight()/200;
      
      ZoneStrategy *t = 
	(ZoneStrategy*)zone->getTransformation(QpTransformation::Zone);
      
      listener->
	setZoneCoords(t->invXTransform(e->x()), t->invYTransform(e->y()));
      
      listener->setZoneTitle(zone->getTitle());
      
      QpObject *picked = 0;
      
      // Traverse the tree looking for the touched object (if required)
      if (finePicking)
	picked = zone->touched(x,y,xPickRange,yPickRange,node);
      
      listener->setPicked(picked); 
      
      listener->setPickedNode((QpNode<QpObject>*)node); 
      
    } else {
      
      // Modifications asked by Andreas: do not reset wheh leaving the zone
      // listener->setZoneCoords(0.0,0.0); 
      listener->setZoneTitle("-");
      listener->setPicked(0);
      listener->setPickedNode(0);
    }
  }
  
  // Explicitly notify all observers
  listener->notify();
  
}

// Creates nbX * nbY zones, symmetrical placement 
void QpPage::createZones(int nbX, int nbY)
{
  clearList();  // Warning -> Existing zone objects are deleted 
  nbXZones = nbX; nbYZones = nbY;
  double pageWidth = getWidth();
  double pageHeight = getHeight();
  double zoneWidth = rint((pageWidth - (nbXZones + 1) * leftMargin)/nbXZones);
  double zoneHeight = rint((pageHeight-(nbYZones + 1)*bottomMargin)/nbYZones);
  double xPosition = leftMargin;
  double yPosition = pageHeight - bottomMargin - zoneHeight;
  QpMap *xMap=0, *yMap=0;

  uint zoneIndex = 0;
  zones = QArray<QpZone *> (nbXZones*nbYZones);
  
  for(int y=0;y<nbYZones;y++)
    {
      for(int x=0;x<nbXZones;x++)
	{
	  QpZone *newZone;
	  
	  xMap = new QpMap();
	  // Keep the default content limits
	  xMap->setContainerRange(xPosition,xPosition+zoneWidth); 
	  yMap = new QpMap();
	  yMap->setContainerRange(yPosition,yPosition+zoneHeight);
	  newZone = new QpZone(this, xMap, yMap);
	  addElement(newZone);
	  zones[zoneIndex++] = newZone;
	  
	  xPosition += zoneWidth + leftMargin;
	}
      yPosition -= zoneHeight + bottomMargin;
      xPosition = leftMargin;
    }
}

QpZone* QpPage::createIrregularZone(double x , double y,
				    double w_, double h_)
{
  QpMap *xMap;
  QpMap *yMap;
  
  // Retrieve page size and compute 10% margins
  double pageWidth = getWidth();
  double pageHeight = getHeight();
  double leftMargin_ = rint(pageWidth / 10);
  double bottomMargin_ = rint(pageHeight / 10);

  double xPosition = leftMargin_   + x * (pageHeight - 2*leftMargin_);
  double yPosition = bottomMargin_ + y * (pageWidth  - 2*bottomMargin_);
  double zoneWidth  = w_ * (pageWidth  - 2*bottomMargin_);
  double zoneHeight = h_ * (pageHeight - 2*leftMargin_);
  
  xMap = new QpMap();
  xMap->setContainerRange(xPosition,xPosition+zoneWidth);
  yMap = new QpMap();
  yMap->setContainerRange(yPosition,yPosition+zoneHeight);
  QpZone *result = new QpZone(this, xMap, yMap);
  addElement(result);

  return result;
  
}



QpText *QpPage::setTitle(QString _title, int _align)
{
  // Retrieve zone width
  double pageWidth = getWidth();
  double pageHeight = getHeight();  

  //The rectangle is going to have the same width of our page, and the height of the font. 
  //Change from  Pixels to Page Coordinates.
  double titWidth = pageWidth - 2*leftMargin;
    
  //Title position. 
  double xTitle = leftMargin; 
  double yTitle = pageHeight - bottomMargin/2;
   
  if (pagetitle) 
    removeElement(pagetitle);
  // Create text with proper bounding width, and with height=0 (font's height).
  // Also this object is added to his parent.
  pagetitle = new QpText(_title,this,xTitle ,yTitle ,titWidth,0.,true,
			 QpTransformation::Page);
  // With this I allow the different types of Qt alignments.
  pagetitle->getTextObject()->setAlignment(_align);
  return pagetitle;
}


QpText *QpPage::addText(double xPosition,double yPosition,QString text)
{
   // Creating the text with width and height of the text.
   QpText* pagetext = new QpText( text,this,xPosition,yPosition,0.,0.,true,
				  QpTransformation::Page);
   return pagetext;
}

void QpPage::flushZone(int index)
{
  QpZone *old = 0, *z=0;
  old = getZone(index);
  if (old) {
    z = new QpZone(*old);
    removeElement(old);
    addElement(z);
  }
}

void QpPage::redrawZone(QpZone *z)
{
  if (z) {
    QBrush b = getPainter()->brush();
    QPen p = getPainter()->pen();
    getPainter()->setBrush(Qt::white);
    getPainter()->setPen(Qt::white);
    QpAxisNode *axisnode = z->getAxisNode();
    bool left=false,right=false,top=false,bottom=false;
    if (axisnode != 0) {
      left = axisnode->getAxis("y1") != 0;
      right = axisnode->getAxis("y2") != 0;
      top = axisnode->getAxis("x2") != 0;
      bottom = axisnode->getAxis("x1") != 0 ;
    }

    // Heuristic to cleanup labels & ticks.
    // Should be replaced by something better like 
    // (distance between axis and label) + (heigth of text)
    // or redraw the whole zone in white...
    QRect r = z->getLimits();
    if (left) {
      (r.left()-35>=0) ? r.setLeft(r.left()-35) : r.setLeft(0);
    }
    else r.setLeft(r.left()-7);
    if (right) {
      r.setWidth(r.width()+30);
    }
    else r.setWidth(r.width()+7);
    if (top) {
      (r.top()-25>=0) ? r.setTop(r.top()-25) : r.setTop(0);
    }
    else r.setTop(r.top()-7);
    if (bottom) {
      r.setHeight(r.height()+25);
    }
    getPainter()->drawRect(r);
    getPainter()->setPen(p);
    getPainter()->setBrush(b);
    z->draw();
    update();
  }
}

/****************************************/
/*************** EVENTS *****************/
/****************************************/

void QpPage::paintEvent(QPaintEvent* event)
{
  updateListener(event);
  bitBlt(this, 0, 0, _buffer);
}

void QpPage::resizeEvent(QResizeEvent* event)
{
  screenWidth  = event->size().width() ;
  screenHeight = event->size().height() ;

  QPixmap save(*_buffer);
  if (getPainter()) {
    // Any resources used are released by end().
    getPainter()->end();
  }

  _buffer->resize(event->size().width(), event->size().height());
  _buffer->fill(white);

  if (getPainter()) {
    getPainter()->begin(getBuffer());
  }

  transformation->getXpMap()->setIntRange(0, event->size().width());
  transformation->getYpMap()->setIntRange(0, event->size().height());

  if (outline)
    outline->updateLimit();
  
  redraw();
}

void QpPage::mouseMoveEvent(QMouseEvent *e)
{
  updateListener(e);
}

void QpPage::mousePressEvent(QMouseEvent *e)
{
  updateListener(e);
}

void QpPage::mouseReleaseEvent(QMouseEvent *e)
{
  updateListener(e);
}


