#include <iostream>

#include "Qplotter/qp/qpplaintext.h"
#include "Qplotter/qp/qpqwt_math.h"
#include "Qplotter/qp/qptextmetrics.h"


QpPlainText::QpPlainText(QString _data, QpObject *z, double _x, double _y, double _w, double _h, bool addOnParent,QpTransformation::tStrategy st) :
  QpCommonText(_data, z, _x, _y, _w, _h,addOnParent,st)
{
}

int QpPlainText::getPixWidth(QPainter *p) const
{
  // getPrinter() or getPainter()
  if (p != 0) {    
    return QpTextMetrics::getPixWidth(getData(),p);
  }
  std::cerr << " QpPlainText::getPixWidth Painter is null !" << std::endl ;
  return -1 ;  
}

int QpPlainText::getPixHeight(QPainter *p) const
{
  return QpTextMetrics::getPixHeight(getData(),p);
}

int QpPlainText::getLineSpacing(QPainter *p) const
{
  return (p) ? QpTextMetrics::getLineSpacing(p) : 0 ;
}

void QpPlainText::paint(QPainter *painter)
{
  QpTransformation *transf = getTransformation(getTransfStrategy());  
  int xCoord = transf->xTransform(getX());
  int yCoord = transf->yTransform(getY());
  int xOffset=0;

  // Rotate painter if necessary
  if (getW() > 0.) {
    int boundWidth = transf->xTransform(getX() + getW()) - xCoord;
    xOffset = getXOffset(boundWidth, painter);
  }
  int boundHeight = yCoord - transf->yTransform(getY() + getH());
  rotatePainter(painter,xCoord,yCoord);
  xCoord += xOffset;
  yCoord += getYOffset(boundHeight,painter);

  int fs = getQpFont()->pointSize();
  QFont painterFont = painter->font();
  QFont fo = QFont(painterFont);
  fo.setPointSize (fs);
  painter->setFont(fo);
  painter->drawText(xCoord,yCoord,getData());
  painter->setFont(painterFont);
  
  //br=painter->boundingRect(_x,_y ,_w,_h,getAlignment(),getData());
  // Restore painter if necessary
  restorePainter(painter);
}

void QpPlainText::draw() 
{
  paint(getPainter());
}

void QpPlainText::print() 
{
  //DEBUG  std::cerr << getData() << std::endl; 
  paint(getPrinter());
}

QpObject *QpPlainText::touched(double _x, double _y, double errorX, double errorY, QpObject *& /* node */)
{
  bool res = false;
  QpTransformation *tp = getTransformation(QpTransformation::Page); 

  res = tp->xTransform(_x-errorX) <= qwtMax(br.right(),br.left());
  res = res && tp->xTransform(_x+errorX) >= qwtMin(br.left(),br.right());
  res = res && tp->yTransform(_y-errorY) >= qwtMin(br.bottom(),br.top());
  res = res && tp->yTransform(_y+errorY) <= qwtMax(br.bottom(),br.top());

  if (res) return this;
  else return 0;
}











