#include "Qplotter/qp/qpplot.h"
#include "fileprint.xpm"
#include "zoom.xpm"
#include "unzoom.xpm"
#include "pointer.xpm"

const int pixSize = 600;


QpPlot::QpPlot(QWidget *p, const char *name) : QFrame(p,name)
{
  wFrame = pixSize;         // in pixels
  hFrame = pixSize+50;
  wPanel = pixSize;         // in pixels
  hPanel = pixSize;
  setGeometry(1,1,wFrame,hFrame);
  //setMinimumSize(wFrame,hFrame);
  //setMaximumSize(wFrame,hFrame);

  // Use default pagesize 210X210
  page = new QpPage(this,QRect(0,0,wPanel,hPanel));

  plotLayout = new QHBoxLayout(this, 4);

  leftToolbar = 0;  
  addLeftToolbar();

  drawingLayout = new QVBoxLayout( plotLayout, 10 );  
  //  drawingLayout->addMultiCellWidget(page,0,4,0,4); //,Qt::AlignTop | Qt::AlignLeft);
  drawingLayout->addWidget(page,0,0); //,Qt::AlignTop | Qt::AlignLeft);

  bottomBar = new QHBoxLayout(drawingLayout);
  bottomBar->addStretch(0);
  bottomBar->setAlignment(Qt::AlignRight);
  quitButton = new QPushButton("Quit",this);  
  bottomBar->addWidget(quitButton);

  QObject::connect(quitButton, SIGNAL(clicked()), this, SLOT(quitButtonPressed()));

  rightToolbar = 0;  
  //  addRightToolbar();

  //  drawingLayout->addColSpacing (5, 12);

  page->createZones(1,1);
}

void QpPlot::addLeftToolbar()
{
  leftToolbar = new QVBoxLayout(plotLayout, 5);
  leftToolbar->setResizeMode(QLayout::Fixed);  
  leftToolbar->setGeometry(QRect(0,0,20,40));
  leftToolbar->setMargin(2);
  leftToolbar->setSpacing(2);
  leftToolbar->setAlignment(Qt::AlignTop);

  addButtonToolbar (selectButton, QPixmap(pointer_xpm), leftToolbar);
  addButtonToolbar (zoomButton, QPixmap(zoom_xpm), leftToolbar);
  addButtonToolbar (unzoomButton, QPixmap(unzoom_xpm), leftToolbar);
  leftToolbar->addSpacing(8);  
  addButtonToolbar (printButton, QPixmap(fileprint), leftToolbar);

  selectButton->setToggleButton(true);
  zoomButton->setToggleButton(true);
  
  QObject::connect(printButton, SIGNAL(clicked()),
		   (QObject*)page, SLOT(slotPrint()));
  QObject::connect(selectButton, SIGNAL(toggled(bool)),
		   this, SLOT(SelectPressed(bool)));
  QObject::connect(this, SIGNAL(SelectSignal(bool)),
		   (QObject*)page, SLOT(slotSelect(bool)));
  QObject::connect(unzoomButton, SIGNAL(clicked()),
		   this, SLOT(unzoomPressed()));
  QObject::connect(this, SIGNAL(PageUnzoom()),
		   (QObject*)page, SLOT(slotUnZoom()));
  QObject::connect(zoomButton, SIGNAL(toggled (bool)),
		   this, SLOT(ZoomPressed (bool)));
  QObject::connect(this, SIGNAL(PageZoom (bool)),
		   (QObject*)page, SLOT(slotZooming (bool)));

  leftToolbar->addStretch(0);  // prevent resizing
}

void QpPlot::addRightToolbar()
{
  rightToolbar = new QVBoxLayout(plotLayout, 1);
  rightToolbar->addStretch(0);  // prevent resizing
}



void QpPlot::paintEvent(QPaintEvent*) 
{} 

void QpPlot::resizeEvent(QResizeEvent* /* e */) 
{ 
  /*
    //QSize s = e->size();
    //page->resize(e->size().width()-20,e->size().height()-60);
    drawingLayout->activate();*/
}


void QpPlot::addButtonToolbar (QPushButton* &button, QPixmap img, QVBoxLayout* &toolbar)
{
  button = new QPushButton("",this);
  button->setPixmap(img);
  button->setMaximumSize(QSize(24,24));

  toolbar->addWidget(button);  
}





