#ifndef QPPLOT_H
#define QPPLOT_H

/** %%%

    QpPlot is the frame containing the QpPage.
    
  Usage:

   The QpPlot class is the highest object in the hierarchy. It
  contains buttons, page and so on.  

  TODO:

*/


#include <qframe.h>
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qlayout.h> 
#include <qgroupbox.h>
#include "Qplotter/qp/qppage.h"

class QApplication;


class QpPlot : public QFrame
{ 
  Q_OBJECT
    public:
  /// Constructor
  QpPlot(QWidget *p=0, const char *name=0);
  /// Retrieve the page
  QpPage  *getPage() const {return page;}
  /// Draw the plot
  void draw()    { page->draw(); }
  /// Redraw the plot
  void refresh() { page->redraw();    }
  void addLeftToolbar() ;
  void addRightToolbar() ;
  void addButtonToolbar (QPushButton* &button, QPixmap img, QVBoxLayout* &toolbar) ;
  
 private:
  /// The page
  QpPage  *page;  
  QPushButton *selectButton,*printButton, *quitButton, *zoomButton, *unzoomButton;
  QBoxLayout *plotLayout;  
  QVBoxLayout *leftToolbar, *drawingLayout,*rightToolbar;
  QHBoxLayout *bottomBar ;  

  int wFrame ;         // in pixels
  int hFrame;
  int wPanel ;         // in pixels
  int hPanel;
  double wPage ;     // in page coordinates
  double hPage ;

 protected:
  /// Events processing
  void paintEvent(QPaintEvent*);
  void resizeEvent(QResizeEvent* e);

  private slots:
    void quitButtonPressed() { emit quit(); }

  void unzoomPressed()
    {
      if (selectButton->isOn()) {
	selectButton->toggle();    
	selectButton->setOn(false);
      }
      
      if (zoomButton->isOn()) {
	zoomButton->toggle();    
	zoomButton->setOn(false);
      }
      
      emit PageUnzoom();    
    }

  
  void SelectPressed(bool toggle)
    {
      if (toggle) {
	// Must be done in that order to show labels
	
	if (zoomButton->isOn()) {
	  zoomButton->toggle();    
	  zoomButton->setOn(false);
	}
      
	if (unzoomButton->isOn()) {
	  unzoomButton->toggle();    
	  unzoomButton->setOn(false);
	}
	/*
	  xzl->show();	
	  yzl->show();
	  xpl->show();
	  ypl->show();
	  xz->show();
	  yz->show();
	  xp->show();
	  yp->show();
	*/
      } /*
	  else {
	  xzl->hide();	
	  yzl->hide();
	  xpl->hide();
	  ypl->hide();
	  xz->hide();
	  yz->hide();
	  xp->hide();
	  yp->hide();
	  }
	*/
      emit SelectSignal(toggle);      
    }

  void ZoomPressed(bool toggle) 
    {
      /* To show coordinates while zooming
	 Then why not to always show it ?
      */
      
      if (toggle) {
	// Must be done in that order to show labels

	if (selectButton->isOn()) {
	  selectButton->toggle();    
	  selectButton->setOn(false);
	}

	if (unzoomButton->isOn()) {
	  unzoomButton->toggle();    
	  unzoomButton->setOn(false);
	}
	/*
	xzl->show();	
	yzl->show();
	xpl->show();
	ypl->show();
	xz->show();
	yz->show();
	xp->show();
	yp->show();
	*/
      }
      /*
      else {
	xzl->hide();	
	yzl->hide();
	xpl->hide();
	ypl->hide();
	xz->hide();
	yz->hide();
	xp->hide();
	yp->hide();
      }
      */

      emit PageZoom(toggle);
    }



 signals:
  void SelectSignal(bool);
  void PageUnzoom();
  void quit();

  void PageZoom(bool);
}; 


#endif


