#include "Qplotter/qp/qpnode.cpp"
#include "Qplotter/qp/qppolyface.h"

QpPolyFace::QpPolyFace(int nbElement, double *_x, double *_y, double *_w, double *_h,
		       bool center, QpObject *z, bool addOnParent, 
		       QpTransformation::tStrategy st) :
  QpNode<QpFace>(new QString("QpPolyFace"), z, addOnParent,st)
{
  nFaces = nbElement;
  FaceMode = 0;
  //Creating the arrays for face properties.
  x= new double [nbElement]; 
  y= new double [nbElement];
  w= new double [nbElement]; 
  h= new double [nbElement];
  //Filling the arrays
  for(int i=0;i<nbElement;i++) {
    x[i]=_x[i];
    y[i]=_y[i];
    w[i]=_w[i];
    h[i]=_h[i];
  }
  //Creating the list of faces to have a polyface.
  QpFace *f = 0;
  for(int i=0;i<nbElement;i++) {
    f = new QpFace(this,true,st);
    f->setGeometry(x[i],y[i],w[i],h[i],center);
  }
}

QpPolyFace::~QpPolyFace() {
    if (x) delete [] x; 
    if (y) delete [] y; 
    if (w) delete [] w; 
    if (h) delete [] h;
}

void QpPolyFace::setFaceSize(double _w, double _h)
{
  QPtrList<QpObject> objectList = findType(QString("QpFace"));
  for (objectList.first();
       objectList.current() != 0;
       objectList.next())
    {
      //Changing the size of all the faces in the list.
      ((QpFace *) objectList.current())->setW(_w);
      ((QpFace *) objectList.current())->setH(_h); 
    }
}

void QpPolyFace::draw() {
  paint(getPainter());
}

void QpPolyFace::print() {
  paint(getPrinter());
}

// If we're painting inside the zone, a pre-clipping is applied to shorten
// drawing time in case of zooming on a very large polyface
void QpPolyFace::paint(QPainter *painter)
{
  // Possibly change default pen/brush/font
  changeStyle(painter);

  // Getting zone limits
  ZoneStrategy *zt = (ZoneStrategy*)getTransformation(getTransfStrategy());
  ZoneStrategy *ztz = (ZoneStrategy*)getTransformation(QpTransformation::Zone);
  // If we're painting inside the zone, optimized = true  
  bool optimized = (zt == ztz);
  double xmin = zt->getXzMap()->getContent1();
  double xmax = zt->getXzMap()->getContent2();
  double ymin = zt->getYzMap()->getContent1();
  double ymax = zt->getYzMap()->getContent2();

  // Scanning the list with all the Faces.
  QPtrList<QpObject> objectList = findType(QString("QpFace"));
  QpFace *f ;

  for (objectList.first();
       objectList.current() != 0;
       objectList.next())
    { 
      //Creating QpFace Object.
      f = ((QpFace *) objectList.current());
      f->FaceMode = FaceMode;
      if (optimized) {
	//Getting the Face different variables.
	double facexmin = f->getX();
	double faceymin = f->getY()-(f->getH());
	double facexmax = f->getX()+(f->getW());
	double faceymax = f->getY();
	//The comparison
	if (!((faceymax < ymin) || (faceymin > ymax) ||
			   (facexmax < xmin) || (facexmin > xmax )))
	  f->paint(painter);
      } else
	  f->paint(painter);	
     }
  // Possibly restore default pen/brush/font
  restoreStyle(painter);
}
