#ifndef QPPOLYLINE_H
#define QPPOLYLINE_H

#include <qpointarray.h>
#include "Qplotter/qp/qpobject.h"

/** %%%
  
    The abstract  PolyShape class.
    
  Usage:

    This class consists basically in a set of points which can be
  drawn according t different strategies.  Real classes derived from
  this are PolyLine and Polygon, which implement different "basic"
  methods to draw themselves.

  TODO:

*/


class QpPolyShape : public QpObject
{
public:
  /// Constructor
  QpPolyShape(const char *className,int nbElement, double *_x, double *_y, 
	      QpObject *z, bool addOnParent=true,
	      QpTransformation::tStrategy st=QpTransformation::Zone);
  /// Destructor
  virtual ~QpPolyShape();
  /// Returns number of points
  int getNb() const {return nb;}
  /// Get X coordinate of one point
  double getX(int indice=0) const {return (indice<getNb()) ? x[indice] : 0;}
  /// Get T coordinate of one point
  double getY(int indice=0) const {return (indice<getNb()) ? y[indice] : 0;}
  /// Set coordinates
  void setCoords(int nbElements, double *_x, double *_y);
  /// Draw polyshape
  void draw();
  /// Print polyshape
  void print();
  /// Set the smooth flag
  void setSmooth(bool flag=true) {boolSmooth = flag;}
  /// Is this object touched by this point ?
  QpObject *touched(double _x, double _y, double errorX, double errorY, 
		    QpObject *&node);
protected:
  virtual void basicDraw(QPainter *painter,QPointArray &qpa) = 0;
  virtual void smoothDraw(QPainter *painter, 
			  double *_x, double *_y, int npt) = 0;
private:
  void paint(QPainter *painter);
  void computeCoeffs(double x1,double y1,double x2,double y2,
		     double *a,double *b);
  bool boolSmooth;
  double *x;
  double *y;
  int nb;
};

/** The simple polyline object.
   This object will often be used as a QpNode<QpPolyLine> because the
   QpNode has a 'drawing attributes' feature.
*/
 
class QpPolyLine : public QpPolyShape
{
public:
  /// Constructor
  QpPolyLine(int nbElement, double *_x, double *_y, QpObject *z, 
	     bool addOnParent=true,
	     QpTransformation::tStrategy st=QpTransformation::Zone);
  /// Destructor
  virtual ~QpPolyLine(){};
protected:
  virtual void basicDraw(QPainter *painter,QPointArray &qpa);
  virtual void smoothDraw(QPainter *painter, double *_x, double *_y, int npt);
};

/** The simple polygon object. The polygon draw will automatically close
  the shape by connecting the first and the last points. If using the smooth
  option, the polygon will be closed by the X axis, unless the client
  explicitly adds a closing point at the end (this is a limitation but
  i don't know how to solve it best right now...).
*/
 
class QpPolygon : public QpPolyShape 
{
public:
  /// Constructor
  QpPolygon(int nbElement, double *_x, double *_y, QpObject *z, 
	    bool addOnParent=true,
	    QpTransformation::tStrategy st=QpTransformation::Zone);
  /// Destructor
  virtual ~QpPolygon() {}
protected:
  virtual void basicDraw(QPainter *painter,QPointArray &qpa);
  virtual void smoothDraw(QPainter *painter, double *_x, double *_y, int npt);
};

#endif
