#include "Qplotter/qp/qpscatterplot2d.h"
#include "Qplotter/qp/qpzone.h"

QpScatterPlot2D::QpScatterPlot2D(QpObject *_z):
  QpNode<QpObject>(new QString("QpScatterPlot2D"),_z), st (0) , 
  markerSize(0) 
{
}

QpScatterPlot2D::QpScatterPlot2D(QpObject *_z, Anaphe::IScatter2D *_v):
  QpNode<QpObject>(new QString("QpScatterPlot2D"),_z), st (0), 
  markerSize(0) {
    init(_v);
}

void QpScatterPlot2D::newCurve(Anaphe::IScatter2D *_v) {
    init(_v);
}

void QpScatterPlot2D::init(Anaphe::IScatter2D *sp2)
{
   theData = sp2;
   double xmin = sp2->minX();
   double xmax = sp2->maxX();
   double ymin = sp2->minY();
   double ymax = sp2->maxY();
   // Give the zone a chance to update axis
   getZone()->updateX(xmin,xmax);
   getZone()->updateY(ymin,ymax);
}


/// The draw/print function
void QpScatterPlot2D::paint (QPainter *painter)
{
  changeStyle(painter);

  // Getting zone limits
  ZoneStrategy *zt = (ZoneStrategy*)getTransformation(QpTransformation::Zone);
  double xmin = zt->getXzMap()->getContent1();
  double xmax = zt->getXzMap()->getContent2();
  double ymin = zt->getYzMap()->getContent1();
  double ymax = zt->getYzMap()->getContent2();

  double xC,yC;
  int pixX, pixY;
  // Transform coords in pixel 
  QpTransformation *transf = getTransformation(getTransfStrategy());  

  for (theData->begin() ; theData->nextPoint(xC,yC) ; ) {
    if (!((yC < ymin) || (yC > ymax) || (xC < xmin) || (xC > xmax ))) {
      pixX = transf->xTransform(xC);
      pixY = transf->yTransform(yC);
      if (markerSize != 0) {
	d_pa.resize(4);
	d_pa.setPoint(0, pixX-markerSize, pixY-markerSize);
	d_pa.setPoint(1, pixX+markerSize, pixY-markerSize);
	d_pa.setPoint(2, pixX+markerSize, pixY+markerSize);
	d_pa.setPoint(3, pixX-markerSize, pixY+markerSize);
	painter->drawPolygon(d_pa);
      }
      else
	painter->drawPoint(pixX , pixY);
    }
  }
  // Restore default pen/brush/font
  restoreStyle(painter);
}

/// The draw function
void QpScatterPlot2D::draw()
{
  paint(getPainter());
}

/// The print function
void QpScatterPlot2D::print()
{
  paint(getPrinter());
}

/// Determine the point [x +/- errorX, y +/- errorY] is in the object
QpObject *QpScatterPlot2D::touched(double _x, double _y, double errorX, double errorY, QpObject *&node)
{
  QpObject *res = QpNode<QpObject>::touched(_x,_y,errorX,errorY,node);
  if (res != 0) 
    node=this; // The returned node will be the curve
  return res;
}

void QpScatterPlot2D::updateSt()
{
  if (st != 0) 
    st->setPen(*(getQpPen()));
}
