#ifndef QPSMOOTHER_H
#define QPSMOOTHER_H

/** %%%
    
    The smoother object.
    
     Author - Marlow etc.   Modified by - P. Ward     Date -  3.10.1973

     Extracted from the ROOT package by O.Couet.

     Re-structured to remove most gotos by Dino Ferrero Merlino
     (the numerical part has not been touched)

  Usage:

      Helps to compute a smooth representation of a curve.
   The history of this code is quite long...
  
   This routine draws a smooth tangentially continuous curve through
   the sequence of data points P(I) I=1,N where P(I)=(X(I),Y(I))
   the curve is approximated by a polygonal arc of short vectors .
   the data points can represent open curves, P(1) != P(N) or closed
   curves P(2) == P(N) . If a tangential discontinuity at P(I) is
   required , then set P(I)=P(I+1) . loops are also allowed .
  
   Reference Marlow and Powell,Harwell report No.R.7092.1972
   MCCONALOGUE,Computer Journal VOL.13,NO4,NOV1970PP392 6
  
   The routine was then adapted as the IGRAP1 routine of HIGZ.
   The FORTRAN code was translated with f2c and adapted to ROOT.
  
   
  
  TODO:

*/


class QpSmoother 
{
public:
  /**Constructor.
  Input parameters:
  npoints    : Number of data points.
  _x         : Abscissa
  _y         : Ordinate
  */  
  QpSmoother(int nPoints, double *_x, double *_y) ;

  /// Destructor
  ~QpSmoother();
  /// Transform the polyline in a smooth curve.
  int smooth();
  int getNPoint() const { return npoints;} 
  /// SHOULD make them const!!!
  double* getX () const {return qlx;}
  double* getY () const {return qly;}
private:
  // Find zero of a continuous function
  void zero(int &k,double AZ,double BZ,double E2,double &X,double &Y ,int maxiter);
  void computeCoeffs(double x1, double y1, double x2, double y2, double *a, double *b);
  bool scaleRatio();
  bool continousTangentAtEndPoints();
  int skipConsecutiveEqualPoints(int start);
  int resizeSmoothArrays() ;
  void computeStraightLine(int k);
  void computeDirectionCosine (int kMinus1, int k, int kPlus1);
  void newPoint();
  void dirCosL150();
  void dirCosL130();
  void computeCubicCoeffs();
  void checkMaxIterations(int &k, int &IT, int J1, double &X, 
			  double X2,int maxiterations);
private:
  int npoints;
  int k, kp, NpointsMax, banksize, npt;
  double xratio, yratio;
  double sxmin, symin;
  int finished;
  int jtype;
  bool  closed; // Curve closed/open
  int flgis;
  int iw;
  double P1, P2, P3, P4, P5, P6;
  double W1, W2, W3;
  double A, B, C, R, S, T, Z;
  double CO, SO, CT, ST, CTU, STU, XNT;
  double DX1, DY1, DX2, DY2;
  double XO, YO, DX, DY, XT, YT;
  double XA, XB, YA, YB;
  double U1, U2, U3, TJ;
  double SB, STH;
  double wsign, tsquare, tcube;  
  double *qlx;
  double *qly;
  double *x;
  double *y;

protected:
 /// Copy constructor
 QpSmoother(QpSmoother &copy) {
  QpSmoother unuse = copy;
 }

 QpSmoother& operator= (QpSmoother& arg) {
   return arg;   
 }

};

#endif
