#include "qpsummarybox.h"


QpSummaryBox::QpSummaryBox(QpObject *p, SummaryTable *st,
			   ZrStrategy *tr,
			   double xStart, double yStart,
			   double w, double lh, double inc,bool title,
			   bool frameVisible)
  : QpNode<QpObject>(new QString("QpSummaryBox"),p,true)
{
  table = st;
  transf = tr;
  isFrameVisible = frameVisible;
  
  
  /* Usually in Zone coordinates */
  
  x = xStart;
  y = yStart;  
  width = w;  
  lineHeight = lh ;
  margin = inc ;
  
  height = 0;
  visible = true;
  titleOnly = title;
  fillColor = QColor(222,222,222);
  
  
  /* Here the list of attributes */
    
  QPtrListIterator<StPair> it(*table);
  for ( ; it.current(); ++it ) {
    if (it.current()->getKey() == "separator") {
      /* Create separator */
      if (!titleOnly) height += lineHeight/2 ;      
    } else if (it.current()->getKey() == "legend") {
      /* Create legend */
      height += 13*lineHeight/8 ;
    } else {
      /* Create texts */
      if (!titleOnly) height += lineHeight ;
    }
    
    if (!titleOnly) height += lineHeight/2 ;    
  }
  
  Init();  
}

QpSummaryBox::~QpSummaryBox()
{
}

void  QpSummaryBox::Init()
{

  //  if (isFrameVisible)
    createFrame();
    
  /* Summary Lines */
  double currentLine = 0.0;
    
  QPtrListIterator<StPair> it(*table);

  //  Title ?
 
  for ( ; it.current(); ++it ) {
    if (it.current()->getKey() == "separator") {
      /* Create separator */
      if (!titleOnly) 
	createSeparator(it.current()->getValue(), currentLine);	
    } else if (it.current()->getKey() == "legend") {
      /* Create legend */
      QpNode<QpPolyLine> *pl = createLegend(it.current()->getValue(),currentLine);
      pl->setQpBrush(QBrush(table->getBrush()));

      if (table->isMarker)
	createMarker(pl,currentLine);

      if (table->isFilled)
      	createBrush(pl,currentLine);

    } else {
      /* Create texts */
      if (!titleOnly) 
	createTexts(it.current()->getKey(),it.current()->getValue(),currentLine);	
    }
    
    if (!titleOnly) 
      currentLine += lineHeight/2 ;    
  }

  
}

void QpSummaryBox::draw()
{
  if (visible) {
    QpNode<QpObject>::draw();
    
    /*
    changeStyle(getPainter());

    for (list.first();
	 list.current() != 0;
	 list.next())
      {
	list.current()->draw();
      }
  
    restoreStyle(getPainter());
    */
  }
}

void QpSummaryBox::print()
{
  if (visible) {
    QpNode<QpObject>::print();  
  }
}

void QpSummaryBox::createFrame()
{
  setQpPen(QPen(Qt::black,1));
  //  setQpBrush(QBrush(Qt::lightGray));
  setQpBrush(fillColor);

  frameFace = new QpFace(this, true, QpTransformation::ZrTopRight);  
  frameFace->setGeometry(x,y,width,height,false);
}

void QpSummaryBox::createSeparator(QString value, double &currentLine)
{
  currentLine += lineHeight/4;
  
  if (value == "-") {
    // Horizontal line
    double xv[2];
    double yv[2];

    xv[0] = x;
    xv[1] = x + width;
    yv[0] = y - currentLine;
    yv[1] = y - currentLine;
    
    new QpPolyLine(2, xv, yv, this, true, QpTransformation::ZrTopRight);
  }
  currentLine += lineHeight/4;
}

QpNode<QpPolyLine> *QpSummaryBox::createLegend(QString value, double &currentLine)
{
  /* TODO: Need some code cleanup here.. */

  QpNode<QpPolyLine> *pl= 0;
  
  double tabx[4], taby[4];
  
  currentLine += lineHeight ;

  QpText *valueText = new QpText(value,this,x,y-currentLine,width-margin,0,
				 true,QpTransformation::ZrTopRight);
  valueText->getTextObject()->setAlignment(Qt::AlignRight | Qt::AlignBottom);	


  // if we want --<>-- in the legend this is the length of --<
  markerWidth = (int) rint(transf->getXpMap()->invTransform(transf->getXpMap()->i1()+10/2+10) - transf->getXpMap()->d1());

  tabx[0]=x+margin;
  tabx[1]=x + 2*markerWidth+margin;
  taby[0]=y-currentLine;
  taby[1]=y-currentLine;

  pl = new QpNode<QpPolyLine>(new QString("QpPolyLine"),
			      this,true,QpTransformation::ZrTopRight);

  if (table->getPen())
    pl->setQpPen(QPen(*(table->getPen())));
  new QpPolyLine(2,tabx,taby,pl,true,QpTransformation::ZrTopRight);

  return pl;
}

void QpSummaryBox::createMarker(QpNode<QpPolyLine> *pl, double &currentLine)
{
  QpFixedSizeMarker *m ;
  
  m = new QpFixedSizeMarker(table->getMarker(), QpMarker::NoLine, 
			    pl,true,QpTransformation::ZrTopRight);
  m->setGeometry(x+markerWidth+margin,y-currentLine,10,10); 

  
  currentLine += 5*lineHeight/8 ;  

}

void QpSummaryBox::createBrush(QpNode<QpPolyLine> *pl, double &currentLine)
{
  pl->setQpPen(QPen(Qt::black,1));
  pl->setQpBrush(QBrush(table->getBrush()));

  QpFace *face = new QpFace(pl, true, QpTransformation::ZrTopRight);  
  face->setGeometry(x+4*markerWidth+margin,y-currentLine+3,5,3,false);
}

void QpSummaryBox::createTexts(QString key, QString value, double &currentLine)
{
  currentLine += lineHeight ;
  
  QpText *keyText = new QpText(key,this,x+margin,y-currentLine,width,0,
			       true,QpTransformation::ZrTopRight);
  keyText->getTextObject()->setAlignment(Qt::AlignLeft | Qt::AlignBottom);

  QpText *valueText = new QpText(value,this,x,y-currentLine,width-margin,0,
				 true,QpTransformation::ZrTopRight);
  valueText->getTextObject()->setAlignment(Qt::AlignRight | Qt::AlignBottom);	
}


