#ifndef QPSUMMARYBOX_H
#define QPSUMMARYBOX_H

/** %%%
    
  Usage:

    A summary box is a box with a solid background in which there is
  text lines for each property pair.

  TODO:
    (--) Why not make it shaded when clicked on it ?
    (--) Add a box background property ? (color, pattern)
*/


#include <qptrlist.h>
#include "Qplotter/qp/qpobject.h"
#include "Qplotter/qp/qpnode.cpp"
#include "Qplotter/qp/qpsummaryinfo.h"
#include "Qplotter/qp/qpface.h"
#include "Qplotter/qp/qppolyface.h"
#include "Qplotter/qp/qppolyline.h"
#include "Qplotter/qp/qptext.h"



class QpSummaryBox : public QpNode<QpObject>
{
 public:
  QpSummaryBox  (QpObject *p, SummaryTable *st,
		 ZrStrategy *tr,
		 double xStart, double yStart,
		 double w, double lh, double inc,bool title=false,
		 bool frameVisible=true);
  ~QpSummaryBox ();

  void draw();
  void print();

  void setFontSize(int size) 
    {
      setQpFont(QFont( "helvetica", size));
    }
  
  
  void setVisible(bool vis) { visible = vis; }
  void setX(double xStart) { x = xStart; }
  void setWidth(double w) { width = w; }
  double  getWidth() { return width; }
  double  getHeight() { return height; }

  //  void setFrameVisible(bool b) {isFrameVisible = b;}
  void setColor(QColor col) {fillColor = col;}

 private:

  void Init();
  
  void createFrame();
  void createSeparator (QString value, double &currentLine);
  void createTexts     (QString key, QString value, double &currentLine);
  QpNode<QpPolyLine> *createLegend    (QString value, double &currentLine);
  void createMarker (QpNode<QpPolyLine> *pl,double &currentLine);
  void createBrush  (QpNode<QpPolyLine> *pl,double &currentLine);
  
  
  /* Attributes */
  SummaryTable *table;
  ZrStrategy *transf;

  QColor fillColor;
  bool isFrameVisible;
  QpFace *frameFace ;
  
  
  bool visible ;
  bool titleOnly;  
  double lineHeight;  
  double x,y;
  double height,width;
  double margin;
  int markerWidth;
  
};


#endif



