#include "Qplotter/qp/qptext.h"
#include "Qplotter/qp/qptextmetrics.h"
#include "Qplotter/qp/qppolyline.h"
#include "Qplotter/qp/qpnode.cpp"
#include "Qplotter/qp/qpsummaryinfo.h"
#include "Qplotter/qp/qpsummarybox.h"
#include "Qplotter/qp/qpzone.h"
#include "Qplotter/qp/qpface.h"
#include "Qplotter/qp/qppolyface.h"

const int textMarginInPixel = 3;

//-ap: for now ...
using std::vector;

StPair::StPair(QString _key, QString _value) : key(_key), value(_value)
{
}

bool SummaryTable::removeByKey(QString keyVal)
{
  QPtrListIterator<StPair> it(*this);
  for ( ; it.current(); ++it ) {
    if (it.current()->getKey() == keyVal) {
      remove(it.current());
      return true;
    }
  }
  return false;
}

int SummaryTable::getMaxStringWidth(QpSummaryInfo *qps,QPainter *painter)
{
  // Is there any way to do this without temporary QpText instances ?
  
  int mklength=0, mvlength=0;
  QPtrListIterator<StPair> it(*this);
  for ( ; it.current(); ++it ) {
    int klen,vlen;
    // To find out the width of keys and values, we've to create the
    // corresponding QpText object, since math formulas have to be
    // taken into account according to their final size
    QpText *qpk = new QpText(it.current()->getKey(),qps,0,0,0,0,true);
    klen = qpk->getTextObject()->getPixWidth(painter);
    QpText *qpv = new QpText(it.current()->getValue(),qps,0,0,0,0,true);
    vlen = qpv->getTextObject()->getPixWidth(painter);

    if (mklength < klen) 
      mklength = klen;
    if (mvlength < vlen) 
      mvlength = vlen;
  }
  // Once got the lengths, destroy the temporary QpText instances
  // WARNING !! Clear everything here ..
  qps->clearList(); 
  // Compute the length of spacing between key and value
  int kvSpacing = QpTextMetrics::getPixWidth(QString("W "),/*qps->getPainter()*/painter);

  return (kvSpacing+mklength+mvlength);
}


QpSummaryInfo::QpSummaryInfo(QpZone *z, QPSTD::vector <SummaryTable *> _vst,
			     QPainter *painter, QpSummaryInfo::Visible v, bool titleOnly)
  : QpNode<QpSummaryBox>(new QString("QpSummaryInfo"), z, true,QpTransformation::ZrTopRight)
{  
  visible=v;
  vst = _vst;  
  xOffset=0; 
  yOffset=0;
  currentPainter = painter ;  
  createBoxes(titleOnly);
}

void QpSummaryInfo::createBoxes(bool titleOnly)
{
  /*
    Foreach SummaryTable st in vst 
    do
    Draw a SummaryBox of st
    done
  */

  // Compute font size to keep boxes inside zone
  zoneWidth = getZoneWidth();
  
  // Compute the boxes width with the current font
  ZrStrategy *tr = (ZrStrategy *) getTransformation(QpTransformation::ZrTopRight);
  margin = tableMargin(tr);
  
  tWidth = tableWidth(tr);
  double lh = tr->getYpMap()->invWidthTransform(QpTextMetrics::getLineSpacing(getPainter()));
  double x = -tWidth - margin - xOffset;
  double y = -margin - yOffset;

  
  _width = tr->xTransform(x+tWidth) - tr->xTransform(x);

  QpSummaryBox *currentBox;
  
  for(uint i=0;
      i < vst.size();
      i++)
    {
      currentBox = new QpSummaryBox(this, vst[i],tr, x, y, tWidth, lh, margin, titleOnly);
      
      y -= (currentBox->getHeight() + 0.6*lh); 
    }
  
}


int QpSummaryInfo::maxStringWidth()
{
  int maxWidth=0;
  int curmaxWidth = -1;
  size_t maxTables = 0;
  if (visible == All) 
    maxTables = vst.size();
  else {
    if (visible == First)
      maxTables = 1;
  }
  for(size_t i=0;i<maxTables;i++) {
    curmaxWidth = vst[i]->getMaxStringWidth(this,currentPainter);
    if (curmaxWidth > maxWidth)
      maxWidth = curmaxWidth;
  }

  return maxWidth;
}



void QpSummaryInfo::setSummaryTableVector(QPSTD::vector <SummaryTable *> _vst)
{
  vst = _vst;  
}

/* Why not create the box and let the draw() methods to draw them ? */
void QpSummaryInfo::update(QPSTD::vector <SummaryTable *> _vst, QPainter *painter)
{
  currentPainter = painter ;
  vst = _vst;

  if (currentPainter != 0) {
    paint();
  }
}


double QpSummaryInfo::getZoneWidth()
{
  double zWidth ;

  QpObject *obj ;
  obj = this;

  while (obj != 0)
    {
      if (obj->isA() == "QpZone")	
	zWidth = ((QpZone *) obj)->getScreenWidth();      

      obj = obj->getParent();
    }

  return zWidth ;  
}

		    

void QpSummaryInfo::paint()
{
  double newWidth ;
  // So that _width is the same as before
  newWidth = tWidth * (zoneWidth/getZoneWidth());
  //  newWidth = tWidth;
  
  int fontSize = 12;
  // recompute font size
  
  setQpFont(QFont("helvetica",fontSize));

  QPtrList<QpObject> boxList = findType("QpSummaryBox");
  
  for (boxList.first();
       boxList.current() != 0;
       boxList.next())
    {
      ((QpSummaryBox *)boxList.current())->setX(xStart(newWidth));	
      ((QpSummaryBox *)boxList.current())->setWidth(newWidth);
    }
}



double QpSummaryInfo::tableWidth(QpTransformation *tr)
{
  double w;

  // Width in pixel coordinates of the widest line
  w = maxStringWidth();

  // Width in page coordinates of the widest line
  w = tr->getXpMap()->invWidthTransform( int(w) );

  return w;
}

double QpSummaryInfo::tableMargin(QpTransformation *tr)
{
  return tr->getXpMap()->invWidthTransform(textMarginInPixel);
}





