#ifndef QPSUMMARYINFO_H
#define QPSUMMARYINFO_H

#include "Qplotter/qp/qpmarker.h"
#include "Qplotter/qp/qpsummarybox.h"
#include <vector>

#if defined __hpux || (defined __sun && __SUNPRO_CC < 0x500)
#define QPSTD
#else
#define QPSTD std
#endif

/** %%%

    StPair : couple of key and value.
    
  Usage:

    Use for summary information such as histogram statistics, fit
  parameters etc.  StPair can be a separator if key = separator and
  value = " " (for blank separator) or "-" for a line separator

  TODO:

  Examples:
    summaryt, markerr, oneZone1D, simpleviewer
*/


class StPair
{
public:
  StPair(QString _key, QString _value);
  QString getKey()   const { return key;   }
  QString getValue() const { return value; }
private :
  QString key, value;  
};

/** SummaryTable : list of StPair representing the summary information
  plus marker type and pen used to draw the related curve, so that a 
  legend can be shown if necessary.
*/

class QpSummaryInfo;

class SummaryTable : public QPtrList<StPair>
{
public:
  SummaryTable() : QPtrList<StPair>(), pen() , brush()  { 
    setAutoDelete(true); 
    marker=QpMarker::None; 
  }
  ~SummaryTable()
    {
      delete pen ;      
    }
  /** Pixel size of the combination of the largest key string + largest value string
    It is used as the maximum width required to represent the content of the summary
    table using the current font.
   */
  int getMaxStringWidth(QpSummaryInfo *qps, QPainter *painter);

  void addEntry (QString _key, QString _value)
    {
      QPtrList<StPair>::append (new StPair(_key, _value));
    }
  
  bool removeByKey(QString keyVal);
  void setPen(QPen _p)                 { delete pen; pen=new QPen(_p); }
  void setBrush(QBrush _p)                 { brush=_p; }
  void setMarker(QpMarker::SymStyle s) { marker = s; }
  QpMarker::SymStyle getMarker() const { return marker; }
  QPen* getPen()                  const { return pen; }
  QBrush getBrush()              const { return brush; }
  bool isFilled ;
  bool isMarker ;

 private:
  QpMarker::SymStyle marker;
  QPen *pen;
  QBrush brush;
  
};


class QpSummaryBox;

/** QpSummaryInfo : summary info of a zone
    (regroup summary tables of each curve)
*/

class QpSummaryInfo : public QpNode<QpSummaryBox>
{
public:
  /// Which summary tables are visible ? 
  enum Visible {All, First, None};
  /// Ctor. By default all summary tables are visible
  QpSummaryInfo(QpZone *z, QPSTD::vector <SummaryTable *> _vst, QPainter *painter,
		QpSummaryInfo::Visible v=All, bool titleOnly=false);
  virtual ~QpSummaryInfo() {}
  
  void setSummaryTableVector(QPSTD::vector <SummaryTable *> vst) ;
  
  /// Update must be called after the first draw and will build the QpTexts
  void update(QPSTD::vector <SummaryTable *> _vst, QPainter *painter);
  void paint();
  
  /// Change visibility attributes
  void setVisible(QpSummaryInfo::Visible v=All) {visible=v;}
  /// Set the distance from upRight zones corner (in page coordinates)
  void setOffsets(double _x, double _y) { xOffset=_x; yOffset=_y; }
  double xStart(double w) { return -(margin + w + xOffset);}
      
protected:
  int maxStringWidth();

private:
  QPSTD::vector <SummaryTable *> vst;
  // This one is needed to compute max string width
  QPainter *currentPainter ;   
  
    
  Visible visible;
  /// Distance to right and up borders in % of width/height of zone
  /// This feature is not implemented yet

  double xOffset, yOffset;
  double margin ;
  double tWidth,zoneWidth;
  
  int _width ;
  
  
  void createBoxes(bool titleOnly);  
  double tableWidth(QpTransformation *tr);
  double tableMargin(QpTransformation *tr);
  double getZoneWidth();
  
};


#endif

