#if defined __hpux || (defined __sun && __SUNPRO_CC < 0x500)
#define QPSTD
#else
#define QPSTD std
#endif

#include <vector>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpzone.h"
#include "Qplotter/qp/qptext.h"
#include "Qplotter/qp/qpaxis.h"
#include "Qplotter/qp/qpsummaryinfo.h"
#include "Qplotter/qp/qpcurve1D.h"
#include "Qplotter/qp/qpcurve2D.h"
#include "Qplotter/qp/qpqwt_math.h"
#include <qcolor.h>



//-ap: for now ...
using std::vector;

const unsigned int defTicks = 606U;


QpZone::QpZone(QpObject *p, QpMap *_xMap, QpMap *_yMap):QpNode<QpObject>(new QString("QpZone"),0,false,QpTransformation::Page)
{
  //xMap -> Width
  //yMap -> Height
  
  transformation   = 0;
  zrtransformation[0] = 0;
  zrtransformation[1] = 0;
  zrtransformation[2] = 0;
  zrtransformation[3] = 0;

  setParent(p);

  updateTransf(_xMap, _yMap);  // Creation of transformation objects
  init();
}

QpZone::QpZone(const QpZone &z):QpNode<QpObject>(new QString("QpZone"),0,false)
{
  setParent(z.getParent());
  QpMap *x = ((ZoneStrategy*)(z.getTransformation(QpTransformation::Zone)))->getXzMap(),
        *y = ((ZoneStrategy*)(z.getTransformation(QpTransformation::Zone)))->getYzMap();
  updateTransf(new QpMap(x->getContent1(), x->getContent2(), x->getContainer1(), x->getContainer2()), 
               new QpMap(y->getContent1(), y->getContent2(), y->getContainer1(), y->getContainer2()));
  init();
}

QpZone::~QpZone() {
  // Delete zone-to-page maps
  delete (transformation->getXzMap());
  delete (transformation->getYzMap());
  // Delete zone transformation and zone corners transformation
  delete transformation;
  delete zrtransformation[0];
  delete zrtransformation[1];
  delete zrtransformation[2];
  delete zrtransformation[3];
  // Delete the stack of zoom transformations (if any)
  while (!mapStack.isEmpty())  {
    ZoneStrategy *s = mapStack.pop();
    // Delete zone-to-page maps and zone transformation
    delete s->getXzMap(); 
    delete s->getYzMap();
    delete s;
  }
  delete mapStack ;
}

void QpZone::init() 
{
  fixedCoords=false;
  limits = true;
  title = 0;  
  sinfo=0;
  gridX=false;
  gridY=false;
  logX=false;
  logY=false;
  qpaxis = 0;
  thickness = 0;
  is2D = true ;  
} 

void QpZone::null(double xmin, double xmax, double ymin, double ymax)
{
  setFixedCoords(false);
  updateX(xmin,xmax,true);
  updateY(ymin,ymax,true);
  //transformation->getXzMap()->setContentRange(xmin,xmax);
  //transformation->getYzMap()->setContentRange(ymin,ymax);
  setFixedCoords(true);
}


QpText *QpZone::setTitle(QString _title, int _align)
{
  // Retrieve zone width
  double xx = transformation->getXzMap()->getContainer1();
  double ww = transformation->getXzMap()->getContainer2() - xx ;
  if (title) 
    removeElement(title);
  // Create text with proper bounding width
  title = new QpText(_title,this,0.,0.,ww,0.,true,QpTransformation::ZrTopLeft);
  title->getTextObject()->setAlignment(_align);
  // Take into account font baseline
  title->getTextObject()->setBaseline(true);
  return title;
}


QString QpZone::getTitle() const
{ 
  if (title) 
    return title->getTextObject()->getData(); 
  else
    return QString("");
}

void QpZone::enableGridX(bool maj, bool min)
{ 
  gridX = maj||min;
  setGridX();
}

void QpZone::setGridX()
{ 
  QpAxis *axis = getAxisNode()->getAxis("x1");
  if (axis != 0) {
    if ( gridX ) {
      // Retrieve zone width
      double xx = transformation->getYzMap()->getContainer1();
      double ww = transformation->getYzMap()->getContainer2() - xx ;
      axis->setGridLength(ww);
      axis->setOption(QpAxis::GridMajor,true);
    } else {
      axis->setOption(QpAxis::GridMajor,false);    
    }    
  }
}
      
void QpZone::enableGridY(bool maj, bool min)
{ 
  gridY = maj||min;
  setGridY();
}

void QpZone::setGridY()
{ 
  QpAxis *axis = getAxisNode()->getAxis("y1");
  if (axis != 0) {
    if ( gridY ) {
      // Retrieve zone width
      double xx = transformation->getXzMap()->getContainer1();
      double ww = transformation->getXzMap()->getContainer2() - xx ;
      axis->setGridLength(ww);
      axis->setOption(QpAxis::GridMajor,true);
    } else {
      axis->setOption(QpAxis::GridMajor,false);    
    }    
  }
}

void QpZone::setXRange(double min, double max, bool log)
{
  transformation->getXzMap()->setContentRange(min,max,log);
}

void QpZone::setYRange(double min, double max, bool log)
{
  transformation->getYzMap()->setContentRange(min,max,log);
}

void QpZone::getXRange(double& min, double& max) const
{
  min = transformation->getXzMap()->getContent1(); 
  max = transformation->getXzMap()->getContent2();
}

void QpZone::getYRange(double& min, double& max) const
{
  min = transformation->getYzMap()->getContent1(); 
  max = transformation->getYzMap()->getContent2();
}

bool QpZone::getXLog() const {
  return logX;
}

bool QpZone::getYLog() const {
  return logY;
}

QpAxis* QpZone::makeAxis (AxisPosition edge) {
  QpAxis* axis;
  double leftX =   transformation->getXzMap()->getContainer1();
  double rightX =  transformation->getXzMap()->getContainer2();
  double bottomY = transformation->getYzMap()->getContainer1();
  double topY =    transformation->getYzMap()->getContainer2(); 
  double xMin,xMax;
  double yMin,yMax;
  getXRange(xMin,xMax);
  getYRange(yMin,yMax);
  switch (edge) {
  case Top: 
    axis = new QpAxis(leftX ,topY ,rightX, topY, 
		      xMin, xMax ,
		      QpAxis::TickNegative+QpAxis::Optimized+QpAxis::Unlabeled,
		      defTicks, qpaxis, true);
    axis->setFindHint("x2");
    axis->setOption(QpAxis::Logarithmic,logX);
    break;
  case Bottom: 
    axis = new QpAxis(leftX, bottomY, rightX, bottomY,
		      xMin, xMax ,QpAxis::TickPositive+QpAxis::Optimized,
		      defTicks, qpaxis, true);
    axis->setFindHint("x1");
    axis->setOption(QpAxis::Logarithmic,logX);
    break;
  case Left: 
    axis = new QpAxis(leftX, bottomY, leftX, topY, yMin, yMax ,
		      QpAxis::TickNegative+QpAxis::Optimized,
		      defTicks, qpaxis, true);
    axis->setFindHint("y1");
    axis->setOption(QpAxis::Logarithmic,logY);
    break;
  case Right: 
    axis = new QpAxis(rightX, bottomY, rightX, topY,
		      yMin, yMax , 
		      QpAxis::TickPositive+QpAxis::Optimized+QpAxis::Unlabeled,
		      defTicks, qpaxis, true);
    axis->setFindHint("y2");
    axis->setOption(QpAxis::Logarithmic,logY);
    break;
  }
  return axis;
}

// Update zone's coordinates and axis
// The force boolean will allow to change them even if that's not needed 
// i.e. if the new system of coordinates is covered by the older one
// If there is no axis, this function will create one and return a pointer 
// on it.

QpAxisNode *QpZone::updateX(double xmin, double xmax, bool force)
{
  if (xmin >= xmax) return 0;
  QpAxis *axis = getAxisNode()->getAxis("x1");
  if (axis == 0) {
    setXRange(xmin,xmax,logX);
    // X axis does not exist yet
    axis = makeAxis(QpZone::Bottom); 
    axisLinLog(axis, transformation->getXzMap(),logX);
  } else {
    // X axis existing
    if (force) {
      setXRange(xmin,xmax);
      axis->update(transformation->getXzMap()->getContent1(),
		   transformation->getXzMap()->getContent2());
    } else {
      if (!fixedCoords ) {
	// Automatic update of zone limits
	double curMin = transformation->getXzMap()->getContent1(); 
	double curMax = transformation->getXzMap()->getContent2();
	if ((xmin < curMin) || (xmax > curMax) ) {      
	  setXRange(qwtMin(xmin,curMin),qwtMax(xmax,curMax));
	  curMin = transformation->getXzMap()->getContent1(); 
	  curMax = transformation->getXzMap()->getContent2();
 	  axis->update(curMin,curMax);
	}
      }
    }
  }
  setGridX();
  setGridY();
  return qpaxis;
}

QpAxisNode *QpZone::updateY(double ymin, double ymax, bool force)
{
  if (ymin >= ymax) return 0;
  QpAxis *axis = getAxisNode()->getAxis("y1");
  if (axis == 0) {
    setYRange(ymin,ymax,logY);
    // Y axis does not exist yet
    axis = makeAxis(QpZone::Left); 
    axisLinLog(axis,transformation->getYzMap(),logY);
  } else {
    // Y axis existing
    if (force) {
      setYRange(ymin,ymax);
      axis->update(transformation->getYzMap()->getContent1(),
		   transformation->getYzMap()->getContent2());
    } else {
      if (!fixedCoords ) {
	// Automatic update of zone limits
	double curMin = transformation->getYzMap()->getContent1(); 
	double curMax = transformation->getYzMap()->getContent2();
	if ((ymin < curMin) || (ymax > curMax) ) {
	  setYRange(qwtMin(ymin,curMin),qwtMax(ymax,curMax));
	  curMin = transformation->getYzMap()->getContent1(); 
	  curMax = transformation->getYzMap()->getContent2();
	  axis->update(curMin,curMax);
	}      
      }
    }
  }
  setGridX();
  setGridY();
  return qpaxis;
}


QRect QpZone::getLimits() const
{
  int x,y,w,h;
  PageStrategy *pt = (PageStrategy*)getTransformation(QpTransformation::Page);
  x = pt->xTransform(transformation->getXzMap()->getContainer1());
  y = pt->yTransform(transformation->getYzMap()->getContainer2());
  w = pt->xTransform(transformation->getXzMap()->getContainer2()) - x ;
  h = pt->yTransform(transformation->getYzMap()->getContainer1()) - y ;
  QRect res(x,y,w,h);
  return res;
}

double QpZone::getHeight() const {
  QpMap *yMap = transformation->getYzMap();
  return (yMap->getContainer2()-yMap->getContainer1());
  /*
  return (transformation->getYzMap()->getContainer2() -
	  transformation->getYzMap()->getContainer1());
  */
}

double QpZone::getWidth() const {
  QpMap *xMap = transformation->getXzMap();
  return (xMap->getContainer2()-xMap->getContainer1());
  /*
  return  (transformation->getXzMap()->getContainer2() -
	   transformation->getXzMap()->getContainer1());
  */
}

double QpZone::getScreenHeight() const {
  double h = getHeight();  

  QpPage *page = (QpPage *)getParent();
  
  return h*(page->getScreenHeight())/((page->getHeight()));
}

double QpZone::getScreenWidth() const {
  double w = getWidth();

  QpPage *page = (QpPage *)getParent();

  return w*(page->getScreenWidth())/((page->getWidth()));
}


/** How to retrieve bounding limits including axis text ?
 */

void QpZone::boundingBox(QPainter *painter, int frame)
{
  //getPainter()->drawRect(getLimits());
  // Buggy drawRect() ?
  /* How to know about the axis area of a zone ??
  QPen pn=painter->pen();
  pn.setWidth(2);
  pn.setColor(QColor(255,0,0));
  painter->setPen(pn);
  */
  QRect r = getLimits();
  QPen zonePen = painter->pen();
  zonePen.setWidth(frame);
  painter->setPen(zonePen);

  painter->drawLine(r.x(),r.y(),r.x(),r.y()+r.height());
  painter->drawLine(r.x(),r.y()+r.height(),r.x()+r.width(),r.y()+r.height());  
  painter->drawLine(r.x()+r.width(),r.y()+r.height(),r.x()+r.width(),r.y());  
  painter->drawLine(r.x()+r.width(),r.y(),r.x(),r.y());

  zonePen.setWidth(1);
  painter->setPen(zonePen); 
}

/** Add a legend below the drawing area
    TODO: height of axis have to be computed..
*/
QpText *QpZone::addLegend(QString text)
{
  /*
    QRect r = getLimits();
    Why not using painter ?
    painter->setPen( Qt::blue );
    painter->drawText(, ,  QString ("The Text")) ;
  */
  // Retrieve zone width
  double minZoneX = transformation->getXzMap()->getContainer1();
  double maxZoneX = transformation->getXzMap()->getContainer2();
  double minZoneY = transformation->getYzMap()->getContainer1();
  double maxZoneY = transformation->getYzMap()->getContainer2();

  double _width = maxZoneX - minZoneX ;
  double _height = minZoneY - maxZoneY ;
  double fontHeight = 8.0 ; // How to compute that one ?
  double axisHeight = 8.0 ; // How to compute that one ?
  
  
  // Create text with proper bounding width, centered in rectangle
  // just behind axis

  QpText *legend = new QpText(text,this,
			      0.,_height - fontHeight - axisHeight,
			      _width,0., true,QpTransformation::ZrTopLeft);
  // Take into account font baseline
  legend->getTextObject()->setBaseline(true);

  return legend;
}


QpTransformation *QpZone::getTransformation(QpTransformation::tStrategy s) const
{
  switch (s)
    {
    case QpTransformation::Zone:
      return transformation; break; 
    case QpTransformation::ZrBottomLeft:
      return zrtransformation[0];break;
    case QpTransformation::ZrBottomRight:
      return zrtransformation[1];break;
    case QpTransformation::ZrTopLeft:
      return zrtransformation[2];break;
    case QpTransformation::ZrTopRight:
      return zrtransformation[3];break;      
    
    default:
      return QpObject::getTransformation(s);
    }
  return 0;  
}

#include <iostream>

// Update the list of summary tables by scanning all children objects
// Warning : only the first level of objects is scanned
void QpZone::updateSt(QPainter *painter)
{
  // Add a summary mode to display only titles with line/marker without separator
  QPSTD::vector <SummaryTable *> allTables;
  SummaryTable *st=0;
  QPtrList<QpObject> curveList ;
  
  // Retrieve pointers to summary tables attached to any curve
  curveList = search("QpCurve1D");
  for (curveList.first();
       curveList.current() != 0;
       curveList.next())
    {
      st = ((QpCurve1D *) curveList.current())->getSummaryTable();
      ((QpCurve1D *) curveList.current())->updateSt();
      if (st) {
	allTables.push_back(st);
	st = 0;
      }
    }
  
  curveList = search("QpCurve2D");
  for (curveList.first();
       curveList.current() != 0;
       curveList.next())
    {
      st = ((QpCurve2D *) curveList.current())->getSummaryTable();
      ((QpCurve2D *) curveList.current())->updateSt();
      if (st) {
	allTables.push_back(st);
	st = 0;
      }
    }
  /*
  
  QPtrListIterator<QpObject> iGO(list); 
  for( ; iGO.current(); ++iGO ) {
    if (iGO.current()->isA(QString("QpCurve1D"))) {
      st=((QpCurve1D*)iGO.current())->getSummaryTable(); 
      ((QpCurve1D*)iGO.current())->updateSt();
    }
    else if (iGO.current()->isA(QString("QpCurve2D"))) {
      st=((QpCurve2D*)iGO.current())->getSummaryTable(); 
      ((QpCurve2D*)iGO.current())->updateSt();
    }
    if (st) {
      allTables.push_back(st);
      st = 0;
    }
  }
  */
  
  if (allTables.size() > 0 ) {
    if (sinfo == 0)
      sinfo = new QpSummaryInfo(this, allTables, painter);
    sinfo->update(allTables, painter); 
  }
}

// To avoid copy/paste of QpNode::draw code, I will reuse it here
// So the limits of the zone will use the PAGE drawing attributes
// and not the zone's ones.
void QpZone::draw()
{
  updateSt(getPainter());

  QpNode<QpObject>::draw();
  if (limits)
    boundingBox(getPainter(),thickness);
}

void QpZone::print()
{
  updateSt(getPainter());
  
  QpNode<QpObject>::print();
  if (limits) 
    boundingBox(getPrinter());
}

bool QpZone::checkLimits()
{
  // Check all mins are less than the corresponding max
  double minZoneX = transformation->getXzMap()->getContent1();
  double maxZoneX = transformation->getXzMap()->getContent2();
  double minZoneY = transformation->getYzMap()->getContent1();
  double maxZoneY = transformation->getYzMap()->getContent2();

  return ((minZoneX <= maxZoneX) && (minZoneY <= maxZoneY));
}

void QpZone::setXLog(bool log) {
  logX = log;
  // If the axis exists, update it
  QpAxis *axis = getAxisNode()->getAxis("x1");
  if (axis != 0) {
    transformation->getXzMap()->setLog(log);
    axisLinLog (axis,transformation->getXzMap(),logX);
  }
}

void QpZone::setYLog(bool log){
  logY = log;
  // If the axis exists, update it
  QpAxis *axis = getAxisNode()->getAxis("y1");
  if (axis != 0) {
    transformation->getYzMap()->setLog(log);
    axisLinLog (axis,transformation->getYzMap(),logY);
  }
}

QpAxisNode *QpZone::getAxisNode() {
  if (qpaxis == 0 )
    qpaxis = new QpAxisNode(this,true);
  return qpaxis;
} 

void QpZone::updateTransf(QpMap *xz, QpMap *yz)
{
  // Retrieve page-to-pixel maps
  QpQwtDiMap *x = getTransformation(QpTransformation::Page)->getXpMap();
  QpQwtDiMap *y = getTransformation(QpTransformation::Page)->getYpMap();
  // Create the zone transformation
  transformation = new ZoneStrategy(x, y, xz, yz) ;
  // The transformation with respect to zone corners is always re-created
  delete zrtransformation[0];
  delete zrtransformation[1];
  delete zrtransformation[2];
  delete zrtransformation[3];

  // zrtransformation = new ZrStrategy(x, y, xz, yz, QpTransformation::Zone) ;

  zrtransformation[0] = new ZrStrategy(x, y, xz, yz, QpTransformation::ZrBottomLeft) ;
  zrtransformation[1] = new ZrStrategy(x, y, xz, yz, QpTransformation::ZrBottomRight) ;
  zrtransformation[2] = new ZrStrategy(x, y, xz, yz, QpTransformation::ZrTopLeft) ;
  zrtransformation[3] = new ZrStrategy(x, y, xz, yz, QpTransformation::ZrTopRight) ;
}

void QpZone::zoom(double x1, double y1, double x2, double y2)
{
  QpMap *xMap=0, *yMap=0;
  double xmin, xmax, ymin, ymax;
  xmin = qwtMin(x1,x2) ; ymin = qwtMin(y1,y2);
  xmax = qwtMax(x1,x2) ; ymax = qwtMax(y1,y2);
  // Change the current maps
  xMap = new QpMap(xmin,xmax,transformation->getXzMap()->getContainer1(),
		   transformation->getXzMap()->getContainer2());
  yMap = new QpMap(ymin,ymax,transformation->getYzMap()->getContainer1(),
		   transformation->getYzMap()->getContainer2());
  // Save the current transformation object (i.e. the maps)
  mapStack.push(transformation); transformation=0;
  // Reconstruct transformation objects
  updateTransf(xMap, yMap);
  // Update axis 
  QpAxis *axis = getAxisNode()->getAxis("x1");
  if (axis != 0) 
    axis->update(xmin,xmax);
  axis = getAxisNode()->getAxis("y1");
  if (axis != 0) 
    axis->update(ymin,ymax);

  // Update the grid
  setGridX();
  setGridY();
}

void QpZone::unZoom(bool original)
{
  QpMap *xMap=0, *yMap=0;
  ZoneStrategy *s = 0;

  if (original) {
    while (!mapStack.isEmpty())  {
      // Go back to the original transformation (getting rid of all
      // intermediate zoom states)
      // Delete zone-to-page maps and zone transformation
      if (s) {
	delete s->getXzMap(); 
	delete s->getYzMap();
	delete s;
      }
      s = mapStack.pop();
    }
    // s is the last transformation in the stack
  }
  else if (!mapStack.isEmpty()) {
    // Back one state
    s = mapStack.pop();
  }
  if (s != 0) {
    // Retrieve maps (will be reused to re-create the transformation)
    xMap = s->getXzMap();
    yMap = s->getYzMap();
    // Delete current transformation
    delete transformation;
    updateTransf(xMap, yMap);
    // Update axis
    QpAxis *axis = getAxisNode()->getAxis("x1");
    if (axis != 0) 
      axis->update(xMap->getContent1(),xMap->getContent2());
    axis = getAxisNode()->getAxis("y1");
    if (axis != 0) 
      axis->update(yMap->getContent1(),yMap->getContent2());
    // Update the grid
    setGridX();
    setGridY();
  }
  if (mapStack.isEmpty()) setThickness(1);
  delete s;
}

void QpZone::zoom(double x, double y, double factor)
{
  double w = transformation->getXzMap()->getContent2() - transformation->getXzMap()->getContent1();
  double h = transformation->getYzMap()->getContent2() - transformation->getYzMap()->getContent1();
  double x2 = x+(w/factor);
  double y2 = y+(h/factor);
  zoom(x,y,x2,y2);
}

bool QpZone::isTouched(double _x, double _y)
{
  bool testX = (transformation->getXzMap()->getContainer1() <= _x) && 
    ( _x <= transformation->getXzMap()->getContainer2());
  bool testY = (transformation->getYzMap()->getContainer1() <= _y) && 
    ( _y <= transformation->getYzMap()->getContainer2());
  return testX && testY;
}

void QpZone::axisLinLog (QpAxis *axis, QpMap *theMap, bool logV) {
  double datamin = theMap->getContent1();
  double datamax = theMap->getContent2();
  axis->setOption(QpAxis::Logarithmic, logV);
  axis->update(datamin,datamax);  
}

QpText * QpZone::addText(double xPosition,double yPosition,QString text,QpTransformation::tStrategy st)
{
  // Creating the text with width and height of the text.
  QpText* zonetext = new QpText( text,this,xPosition,yPosition,0.,0.,true,st);
  return zonetext;
}

bool QpZone::isInvalid() {
    if (transformation == 0) return true ;
    if (zrtransformation[0] == 0) return true ;
    if (zrtransformation[1] == 0) return true ;
    if (zrtransformation[2] == 0) return true ;
    if (zrtransformation[3] == 0) return true ;
    return (transformation->isInvalid()
	    || zrtransformation[0]->isInvalid()
	    || zrtransformation[1]->isInvalid()
	    || zrtransformation[2]->isInvalid()
	    || zrtransformation[3]->isInvalid());
  }
