#include "Qplotter/qwt/qpqwt_grid.h"
//------------------------------------------------------------
//
//.H QpQwtGrid | 3 | 01/01/98 | Qwt Widget Library | Qwt Programmer's Manual
//
//.U NAME
//	QpQwtGrid - A class which draws a coordinate grid
//
//.U SYNOPSIS
//	#include <qwt_grid.h>
//
//.U DESCRIPTION
//	The QpQwtGrid class can be used to draw a coordinate grid.
//	A coordinate grid consists of major and minor vertical
//	and horizontal gridlines. The locations of the gridlines
//	are determined by the X and Y scale divisions which can
//	be assigned with @QpQwtGrid::setXDiv@ and @QpQwtGrid::setYDiv@
//	The @QpQwtGrid::draw@ member draws the grid within a bounding
//	rectangle.
//
//.U PUBLIC MEMBERS 
//.R
//	QpQwtGrid::QpQwtGrid (1) --	Constructor
//	QpQwtGrid::QpQwtGrid (2) -- Constructor
//	QpQwtGrid::~QpQwtGrid -- Destructor
//	QpQwtGrid::draw -- Draw the grid 
//	QpQwtGrid::enableX -- Enable vertical gridlines
//	QpQwtGrid::enableY -- Enable horizontal gridlines
//	QpQwtGrid::enableXMin -- Enable minor vertical gridlines
//	QpQwtGrid::enableYMin -- Enable minor horizontal gridlines
//	QpQwtGrid::setXDiv -- Assign a scale division for vertical gridlines 
//	QpQwtGrid::setYDiv -- Assign a scale division for horizontal gridlines
//	QpQwtGrid::setPen -- Set a pen for major and minor gridlines
//	QpQwtGrid::setMajPen -- Set a pen for major gridlines
//	QpQwtGrid::setMinPen -- Set a pen for minor gridlines
//	QpQwtGrid::majPen -- Return pen for major gridlines
//	QpQwtGrid::minPen -- Return pen for minor gridlines
//	QpQwtGrid::xEnabled -- Return TRUE if vertical gridlines enabled
//	QpQwtGrid::xMinEnabled -- Return TRUE if minor vertical gridlines enabled
//	QpQwtGrid::yEnabled -- Return TRUE if horizontal gridlines enabled
//	QpQwtGrid::yMinEnabled -- Return TRUE if vertical gridlines enabled
//	QpQwtGrid::xScaleDiv -- Return x scale division
//	QpQwtGrid::yScaleDiv -- Return y scale division
//
//.U PROTECTED MEMBERS 
//.R
//	QpQwtGrid::gridChanged -- Called when an attribute has changed
//
//.-
//.U COPYING
//
//	Copyright (C) 1997  Josef Wilgen
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License, version 2,
//	as published by	the Free Software Foundation.
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// 	 
//------------------------------------------------------------

//------------------------------------------------------------
//.U MEMBER FUNCTION DESCRIPTION
//------------------------------------------------------------


//------------------------------------------------------------
//
//.F	QpQwtGrid::QpQwtGrid
//	Construct a QpQwtGrid instance
//
//.u	Syntax
//.f	 QpQwtGrid::QpQwtGrid()
//
//------------------------------------------------------------
QpQwtGrid::QpQwtGrid()
{
    d_xMinEnabled = FALSE;
    d_yMinEnabled = FALSE;
    d_xEnabled = TRUE;
    d_yEnabled = TRUE;
}

//------------------------------------------------------------
//
//.F	QpQwtGrid::~QpQwtGrid
//	Destroy a QpQwtGrid instance
//
//.u	Syntax
//.f	 QpQwtGrid::~QpQwtGrid()
//
//------------------------------------------------------------
QpQwtGrid::~QpQwtGrid()
{
    
}

//------------------------------------------------------------
//
//.F	QpQwtGrid::enableX
//	Enable or disable vertical gridlines
//
//.u	Syntax
//.f	void QpQwtGrid::enableX(bool tf)
//
//.u	Parameters
//.p	bool tf -- TRUE (enable) or FALSE (disable)
//
//.u    See also
//	Minor gridlines can be enabled or disabled using
//	@QpQwtGrid::enableXMin@ 
//------------------------------------------------------------
void QpQwtGrid::enableX(bool tf)
{
    d_xEnabled = tf;
    gridChanged();
}

//------------------------------------------------------------
//
//.F	QpQwtGrid::enableY
//	Enable or disable horizontal gridlines
//
//.u	Syntax
//.f	void QpQwtGrid::enableY(bool tf)
//
//.u	Parameters
//.p	bool tf -- Enable (TRUE) or disable (FALSE)
//
//.u    See also
//	Minor gridlines can be enabled or disabled using
//	@QpQwtGrid::enableYMin@ 
//------------------------------------------------------------
void QpQwtGrid::enableY(bool tf)
{
    d_yEnabled = tf;  
    gridChanged();
}

//------------------------------------------------------------
//
//.F	QpQwtGrid::enableXMin
//      Enable or disable  minor vertical gridlines.
//
//.u	Syntax
//.f	void QpQwtGrid::enableXMin(bool tf)
//
//.u	Parameters
//.p	bool tf -- Enable (TRUE) or disable (FALSE)
//
//.u    See also
//	@QpQwtGrid::enableX@
//
//------------------------------------------------------------
void QpQwtGrid::enableXMin(bool tf)
{
    d_xMinEnabled = tf;
    gridChanged();
}

//------------------------------------------------------------
//
//.F	QpQwtGrid::enableYMin
//	Enable or disable minor horizontal gridlines
//
//.u	Syntax
//.f	void QpQwtGrid::enableYMin(bool tf)
//
//.u	Parameters
//.p	bool tf -- Enable (TRUE) or disable (FALSE)
//
//.u    See also
//	@QpQwtGrid::enableY@
//------------------------------------------------------------
void QpQwtGrid::enableYMin(bool tf)
{
    d_yMinEnabled = tf;
    gridChanged();
}

//------------------------------------------------------------
//
//.F	QpQwtGrid::setXDiv
//	Assign an x axis scale division
//
//.u	Syntax
//.f	void QpQwtGrid::setXDiv(const QpQwtScaleDiv &sx)
//
//.u	Parameters
//.p	const QpQwtScaleDiv &sx -- scale division
//
//.u	Note
//	QpQwtGrid uses implicit sharing (see Qt Manual) for
//	the scale divisions.
//
//------------------------------------------------------------
void QpQwtGrid::setXDiv(const QpQwtScaleDiv &sx)
{
    d_sdx = sx;
    gridChanged();
}

//------------------------------------------------------------
//
//.F	QpQwtGrid::setYDiv
//	Assign a y axis division
//
//.u	Syntax
//.f	void QpQwtGrid::setYDiv(const QpQwtScaleDiv &sy)
//
//.u	Parameters
//.p	const QpQwtScaleDiv &sy -- scale division
//
//.u	Note
//	QpQwtGrid uses implicit sharing (see Qt Manual) for
//	the scale divisions.
//
//------------------------------------------------------------
void QpQwtGrid::setYDiv(const QpQwtScaleDiv &sy)
{
    d_sdy = sy;    
    gridChanged();
}

//------------------------------------------------------------
//
//.F	QpQwtGrid::setPen
//	Assign a pen for both major and minor gridlines
//
//.u	Parameters
//.p	const QPen &p -- pen
//
//.u	Syntax
//.f	void QpQwtGrid::setPen(const QPen &p)
//
//------------------------------------------------------------
void QpQwtGrid::setPen(const QPen &p)
{
    d_majPen = p;
    d_minPen = p;
    gridChanged();
}

//------------------------------------------------------------
//
//.F	QpQwtGrid::setMajPen
//	Assign a pen for the major gridlines
//
//.u	Syntax
//.f	void QpQwtGrid::setMajPen(const QPen &p)
//
//------------------------------------------------------------
void QpQwtGrid::setMajPen(const QPen &p)
{
    d_majPen = p;
    gridChanged();
}

//------------------------------------------------------------
//
//.F	QpQwtGrid:.setMinPen
//	Assign a pen for the minor gridlines
//
//.u	Syntax
//.f	void QpQwtGrid:.setMinPen(const QPen &p)
//
//------------------------------------------------------------
void QpQwtGrid::setMinPen(const QPen &p)
{
    d_minPen = p;  
    gridChanged();
}

//------------------------------------------------------------
//
//.F	QpQwtGrid::draw
//	Draw the grid
//
//.u	Syntax
//.f	void QpQwtGrid::draw(QPainter *p, const QRect &r, const QwtDiMap &mx, const QwtDiMap &my)
//
//.u	Parameters
//.p	QPainter *p -- Qt painter
//	const QRect &r -- bounding rectangle
//	const QwtDiMap &mx -- x axis map
//	const QwtDiMap &my -- y axis map
//
//.u	Description
//	The grid is drawn into the bounding rectangle such that the
//	gridlines begin and end at the rectangle's borders. The X and Y
//	maps are used to map the scale divisions into the drawing region
//	screen.
//
//------------------------------------------------------------
void QpQwtGrid::draw(QPainter *p, const QRect &r, const QpTransformation &t)
{

    int i, x = 0, y = 0;
    int x1, x2, y1, y2;
    x1 = r.left();
    x2 = r.right();
    y1 = r.top();
    y2 = r.bottom();

    //
    //  draw minor gridlines
    //
    p->setPen(d_minPen);
    
    if (d_xEnabled && d_xMinEnabled)
    {
	for (i=0;i<d_sdx.minCnt();i++)
	{
	    x = t.xTransform(d_sdx.minMark(i));
	    if ((x >= x1) && (x <= x2))
	       p->drawLine(x, y1, x, y2);
	}
	
    }

    if (d_yEnabled && d_yMinEnabled)
    {
	for (i=0;i<d_sdy.minCnt();i++)
	{
	    y = t.yTransform(d_sdy.minMark(i));
	    if ((y >= y1) && (y <= y2))
	       p->drawLine(x1, y, x2, y);
	}
	
    }

    //
    //  draw major gridlines
    //
    p->setPen(d_majPen);
    
    if (d_xEnabled)
    {
	for (i=0;i<d_sdx.majCnt();i++)
	{
	    x = t.xTransform(d_sdx.majMark(i));
	    if ((x >= x1) && (x <= x2))
	       p->drawLine(x, y1, x, y2);
	}
	
    }
    
    if (d_yEnabled)
    {
	for (i=0;i<d_sdy.majCnt();i++)
	{
	    y = t.yTransform(d_sdy.majMark(i));
	    if ((y >= y1) && (y <= y2))
	       p->drawLine(x1, y, x2, y);
	}
    }
    
}

//------------------------------------------------------------
//
//.F	QpQwtGrid::operator=
//	Assignment operator
//
//.u	Syntax
//.f	const QpQwtGrid & QpQwtGrid::operator=(const QpQwtGrid &g)
//
//.u	Parameters
//.p	const QpQwtGrid &g -- Grid to be assigned
//
//------------------------------------------------------------
const QpQwtGrid & QpQwtGrid::operator=(const QpQwtGrid &g)
{
    d_xEnabled = g.d_xEnabled;
    d_yEnabled = g.d_yEnabled;
    d_xMinEnabled = g.d_xMinEnabled;
    d_yMinEnabled = g.d_yMinEnabled;
    d_sdx = g.d_sdx;
    d_sdy = g.d_sdy;

    d_majPen = g.d_majPen;
    d_minPen = g.d_minPen;
    
    return *this;
}


//------------------------------------------------------------
//
//.F	QpQwtGrid::majPen
//	Return the pen for the major gridlines
//
//.u	Syntax
//.f	const QPen & QpQwtGrid::majPen()
//
//.u	See Also
//	@QpQwtGrid::setMinPen@, @QpQwtGrid:.setPen@
//
//------------------------------------------------------------

//------------------------------------------------------------
//
//.F	QpQwtGrid::minPen
//	Return the pen for the minor gridlines
//
//.u	Syntax
//.f	const QPen & QpQwtGrid::minPen()
//
//.u	See Also
//	@QpQwtGrid::setMinPen@, @QpQwtGrid:.setPen@
//------------------------------------------------------------

//------------------------------------------------------------
//
//.F	QpQwtGrid::xEnabled
//	Return TRUE if vertical gridlines are enabled
//
//
//.u	Syntax
//.f	bool QpQwtGrid::xEnabled()
//
//.u    See Also
//	@QpQwtGrid::enableY@
//
//------------------------------------------------------------

//------------------------------------------------------------
//
//.F	QpQwtGrid::xMinEnabled
//	Return TRUE if minor vertical gridlines are enabled
//
//.u	Syntax
//.f	bool QpQwtGrid::xMinEnabled()
//
//.u	See Also
//	@QpQwtGrid::enableXMin@
//
//------------------------------------------------------------

//------------------------------------------------------------
//
//.F	QpQwtGrid::yEnabled
//	Return TRUE if horizontal gridlines are enabled
//
//.u	Syntax
//.f	bool QpQwtGrid::yEnabled()
//
//.u    See Also
//	@QpQwtGrid::enableY@
//
//------------------------------------------------------------

//------------------------------------------------------------
//
//.F	QpQwtGrid::yMinEnabled
//	Return TRUE if minor horizontal gridlines are enabled
//
//.u	Syntax
//.f	bool QpQwtGrid::yMinEnabled()
//
//.u	See Also
//	@QpQwtGrid::enableYMin@
//
//------------------------------------------------------------

//------------------------------------------------------------
//
//.F	QpQwtGrid::xScaleDiv
//	Return the scale division of the x axis
//
//.u	Syntax
//.f	const QpQwtScaleDiv & QpQwtGrid::xScaleDiv()
//
//.u	Description
//.u    See Also
//	@^QpQwtScaleDiv@
//
//------------------------------------------------------------

//------------------------------------------------------------
//
//.F	QpQwtGrid::yScaleDiv
//	Return the scale division of the y axis
//
//.u	Syntax
//.f	const QpQwtScaleDiv & QpQwtGrid::yScaleDiv()
//
//.u    See Also
//	@^QpQwtScaleDiv@
//
//------------------------------------------------------------

//------------------------------------------------------------
//
//.F	QpQwtGrid::gridChanged
//	Notify a change of attributes
//
//.u	Syntax
//.f	void QpQwtGridy::gridChanged()
//
//.u	Access
//	protected virtual
//
//.u	Description
//	This virtual function is called when an attribute of the grid
//	has changed. It can be redefined by derived classes. The
//	default implementation does nothing.
//
//------------------------------------------------------------











