#include "Qplotter/qwt/qpqwt_scale.h"
#include "Qplotter/qwt/qpqwt_math.h"

//------------------------------------------------------------
//
//.H QpQwtScale | 3 | 02/05/98 | QpQwt Widget Library | QpQwt Programmer's Manual
//
//
//.U NAME
//	QpQwtScale - A Widget which contains a scale
//
//.U SYNOPSIS 
//	#include <qwt_scale.h>
//
//.U INHERITED CLASSES
//	QWidget
//
//.U PUBLIC MEMBERS
//.R
//	QpQwtScale::QpQwtScale --	Constructor
//	QpQwtScale::~QpQwtScale --	Destructor
//	QpQwtScale::enableTitle --  Enable or disable the title
//	QpQwtScale::setBaselineDist -- Specify the border distance of the base line 
//	QpQwtScale::setBorderDist -- Set the border distances of the scale's endpoints
//	QpQwtScale::setScale --	Assign a scale
//	QpQwtScale::setTitle --	Assign a title
//	QpQwtScale::setLabel --	Assign a label
//	QpQwtScale::setTitleFont -- Set the title font
//	QpQwtScale::setTitleColor -- Set the title color
//	QpQwtScale::title -- Return the title
//	QpQwtScale::label -- Return the label
//	QpQwtScale::titleFont -- Return the title font
//	QpQwtScale::titleColor -- Return the title color
//	QpQwtScale::minBorderDist -- Return a recommended minimum offset
//	QpQwtScale::sizeHint -- Return a size hint
//
//.U PROTECTED MEMBERS
//.R
//	QpQwtScale::scaleChange -- Notify a change of the scale
//	QpQwtScale::fontChange --	Notify a change of the font
//
//.U DESCRIPTION
//	This Widget can be used to decorate composite widgets with
//	a scale.
//------------------------------------------------------------

//------------------------------------------------------------
//.U MEMBER FUNCTION DESCRIPTION
//------------------------------------------------------------

//------------------------------------------------------------
//
//.F	QpQwtScale::QpQwtScale
//	Construct a QpQwtScale object
//
//.u	Syntax
//.f	 QpQwtScale::QpQwtScale(QpQwtScale::Orientation ori, QWidget *parent, const char *name = 0)
//
//.u	Parameters
//.p	QpQwtScale::Orientation ori	-- Orientation. Must be one of
//					   QpQwtScale::Left, QpQwtScale::Right,
//					   QpQwtScale::Bottom, or QpQwtScale::Top
//	QWidget *parent		-- Parent widget
//	const char *name	-- name
//
//------------------------------------------------------------
QpQwtScale::QpQwtScale( QpQwtScale::Orientation ori)   //, QWidget *parent, const char *name)
//: QWidget(parent,name)
{
    QpQwtScaleDraw::Orientation sdo;
    d_borderDist[0] = 0;
    d_borderDist[1] = 0;
    d_baseDist = 4;
    d_titleOffset = 0;
    d_titleDist = 8;

    oparent=0;

    switch(ori)
    {
    case QpQwtScale::Left:
	sdo = QpQwtScaleDraw::Left;
	break;
    case QpQwtScale::Right:
	sdo = QpQwtScaleDraw::Right;
	break;
    case QpQwtScale::Bottom:
	sdo = QpQwtScaleDraw::Bottom;
	break;
    case QpQwtScale::Top:
	sdo = QpQwtScaleDraw::Top;
	break;
    default:
	sdo = QpQwtScaleDraw::Right;
    }
    
    d_scale.setGeometry(0,0,400,sdo);
    //setMinimumSize(30,30);
}

QpQwtScale::Orientation QpQwtScale::getOrientation() {
    switch(d_scale.orientation())
    {
    case QpQwtScaleDraw::Left:
	return QpQwtScale::Left;
	break;
    case QpQwtScaleDraw::Right:
	return QpQwtScale::Right;
	break;
    case QpQwtScaleDraw::Bottom:
	return QpQwtScale::Bottom;
	break;
    case QpQwtScaleDraw::Top:
	return QpQwtScale::Top;
	break;
    default:
	return QpQwtScale::Right;
    }
}

//------------------------------------------------------------
//
//.F	QpQwtScale::~QpQwtScale
//	Destroy a QpQwtScale object
//
//.u	Syntax
//.f	 QpQwtScale::~QpQwtScale()
//------------------------------------------------------------
QpQwtScale::~QpQwtScale()
{
}


//------------------------------------------------------------
//
//.F	QpQwtScale::setTitle
//	Assign a title
//
//.u	Syntax
//.f	void QpQwtScale::setTitle(const char *title)
//
//.u	Parameters
//.p	const char *title -- new title
//
//------------------------------------------------------------
void QpQwtScale::setTitle(const char *title)
{
    d_title = QString(title);
    repaint();
}
void QpQwtScale::setLabel(const char *label)
{
    d_label = QString(label);
    repaint();
}

//------------------------------------------------------------
//
//.F	QpQwtScale::setBorderDist
//	Specify distances of the scale's endpoints from the
//	widget's borders.
//
//
//.u	Syntax
//.f	void QpQwtScale::setBorderDist(int dist1, int dist2)
//
//.u	Parameters
//.p	int dist1  --   Left or top Distance
//	int dist2 --	Right or bottom distance
//
//
//------------------------------------------------------------
void QpQwtScale::setBorderDist(int dist1, int dist2)
{
    d_borderDist[0] = dist1;
    d_borderDist[1] = dist2;
    resizeScale();
    repaint();
}


//------------------------------------------------------------
//
//.F	QpQwtScale::setBaselineDist
//      Specify the border distance of the scale's base line.
//
//.u	Syntax
//.f	void QpQwtScale::setBaselineDist(int bd)
//
//.u	Parameters
//.p	int bd	-- border distance of the baseline
//
//------------------------------------------------------------
void QpQwtScale::setBaselineDist(int bd)
{
    d_baseDist = qwtMax(0,bd);
    repaint();
}

//------------------------------------------------------------
//
//.F	QpQwtScale::setTitleFont
//	Change the title font
//
//.u	Syntax
//.f	void QpQwtScale::setTitleFont(const QFont &f)
//
//.u	Parameters
//.p	const QFont &f	-- new font
//
//------------------------------------------------------------
void QpQwtScale::setTitleFont(const QFont &f)
{
    d_titleFont = f;
    repaint();
}

//------------------------------------------------------------
//
//.F	QpQwtScale::setTitleColor
//	Change the title color
//
//.u	Syntax
//.f	void QpQwtScale::setTitleColor(const QColor &c)
//
//.u	Parameters
//.p	const QColor &c  --    new title color
//
//------------------------------------------------------------
void QpQwtScale::setTitleColor(const QColor &c)
{
    d_titleColor = c;
    repaint();
}


//------------------------------------------------------------
//.-
//.F	QpQwtScale::paintEvent
//	Paint Event
//
//.u	Syntax
//.f	void QpQwtScale::paintEvent(QPaintEvent *e)
//
//.u	Parameters
//.p	QPaintEvent *e
//
//------------------------------------------------------------
void QpQwtScale::draw(QPainter* p, QRect r, int leftMargin, int bottomMargin)
{
  p->save();
  int th, ta, ttop;
   	 
  //QRect r = w->rect();
  
  //QPainter p;
  //p.begin(this);
  
  ta = p->fontMetrics().ascent();
  th = p->fontMetrics().height();
  ttop = ta + d_titleOffset;
  
  // draw scale
  d_scale.draw(p);
  
  // draw title
  //p->setFont(d_titleFont); // Fonts are managed by QpNode
  switch(d_scale.orientation())
    {
    case QpQwtScaleDraw::Left:
      p->rotate(-90.0);
      ttop = qwtMin(ttop + (r.width() - ttop) / 4, r.width());
      p->drawText(-r.bottom(), r.right() - ttop,
		  r.height(), 2.5*th,QPainter::AlignCenter, d_title);
      p->drawText(-r.bottom()+bottomMargin, r.right() - ttop,
		  r.height()-2.5*bottomMargin, 2*th, QPainter::AlignBottom|QPainter::AlignRight, d_label);
      break;
    case QpQwtScaleDraw::Right:
      p->rotate(90.0);
      ttop = qwtMin(ttop + (r.width() - ttop) / 4, r.width());
      p->drawText(r.top(), -r.left()-ttop,
		  r.height(), 2.5*th, QPainter::AlignCenter, d_title);
      p->drawText(r.top()+bottomMargin, -r.left()-ttop,
		  r.height()-2*bottomMargin, 2.5*th, QPainter::AlignBottom|QPainter::AlignRight, d_label);
      break;
    case QpQwtScaleDraw::Top:
      p->drawText(r.x(), d_titleOffset - 2*ta -3, // r.top()-th,
		  r.width(), 2*th,
		  QPainter::AlignHCenter|QPainter::AlignTop ,d_title);
      p->drawText(r.x()+leftMargin, d_titleOffset - 2*ta-1, // r.top()-th,
		  r.width()-2*leftMargin, 1.5*th,
		  QPainter::AlignRight|QPainter::AlignVCenter,d_label);
      break;
    case QpQwtScaleDraw::Bottom:
      p->drawText(r.x(), d_titleOffset + ta +1,// r.bottom()-th,  // r.top() -50 - d_titleOffset - th ,
		  r.width(), 2*th,
		  QPainter::AlignHCenter|QPainter::AlignTop,d_title);
      p->drawText(r.x()+leftMargin, d_titleOffset + ta-1,  // r.bottom()-th,  // r.top() -50 - d_titleOffset - th ,
		  r.width()-2*leftMargin, (int)1.5*th,
		  QPainter::AlignRight|QPainter::AlignVCenter,d_label);
      break;
    default:
      break;
    }
  p->restore();
  //p.end();
  
}

//------------------------------------------------------------
//.-
//.F	QpQwtScale::resizeEvent
//	Resize Event
//
//.u	Syntax
//.f	void QpQwtScale::resizeEvent(QResizeEvent *e)
//
//.u	Parameters
//.p	QResizeEvent *e
//
//------------------------------------------------------------
/*void QpQwtScale::resizeEvent(QResizeEvent *e)
{
    resizeScale();
}
*/

//------------------------------------------------------------
//.-
//.F	QpQwtScale::resizeScale
//	Resize the scale
//
//.u	Syntax
//.f	void QpQwtScale::resizeScale()
//
//------------------------------------------------------------
void QpQwtScale::resizeScale(QPainter* p, QWidget* w)
{
  if (!p) { if (oparent) {p=oparent->getPainter(); w=(QWidget*)(oparent->getPage());} else return; }
  p->save();
    QRect r = w->rect();
    //QPainter p;
    //p.begin(this);
    switch(d_scale.orientation())
    {
    case QpQwtScaleDraw::Bottom:
	d_scale.setGeometry(r.x() + d_borderDist[0],
			    r.y() + d_baseDist,
			    r.width() - d_borderDist[1] - d_borderDist[0],
			    QpQwtScaleDraw::Bottom);

	//d_titleOffset = d_titleDist + d_baseDist + d_scale.maxHeight(p);
	d_titleOffset = r.y() + d_baseDist + 12;
	break;
	
    case QpQwtScaleDraw::Top:
	d_scale.setGeometry(r.x() + d_borderDist[0],
			    r.bottom() - d_baseDist,
			    r.width() - d_borderDist[1] - d_borderDist[0],
			    QpQwtScaleDraw::Top);
	//d_titleOffset =  d_titleDist + d_baseDist + d_scale.maxHeight(p);
	d_titleOffset = r.bottom() - d_baseDist - 12;
	break;
	
    case QpQwtScaleDraw::Left:
	d_scale.setGeometry(r.right() - d_baseDist,
			    r.y() + d_borderDist[0],
			    r.height() - d_borderDist[0] - d_borderDist[1],
			    QpQwtScaleDraw::Left);
	d_titleOffset = d_baseDist + d_titleDist + d_scale.maxWidth(p,FALSE);
	break;
	
    case QpQwtScaleDraw::Right:
	d_scale.setGeometry(r.x() + d_baseDist,
			    r.y() + d_borderDist[0],
			    r.height() - d_borderDist[0] - d_borderDist[1],
			    QpQwtScaleDraw::Right);
	d_titleOffset = d_baseDist + d_titleDist + d_scale.maxWidth(p,FALSE);
	break;
    default:
	break;
    }
    //p.end();
  p->restore();
    
}

//------------------------------------------------------------
//
//.F	QpQwtScale::scaleChange			
//	Notify a change of the scale
//
//.u	Syntax
//.f	void QpQwtScale::scaleChange()
//
//.u	Description
//	This virtual function can be overloaded by derived
//	classes. The default implementation repaints the widget.
//
//------------------------------------------------------------
void QpQwtScale::scaleChange()
{
    repaint();
}



//------------------------------------------------------------
//
//.F	QpQwtScale::sizeHint
//	Return a size hint
//
//.u	Syntax
//.f	QSize QpQwtScale::sizeHint() const
//
//.u	Note
//	This function uses a QPainter internally. Don't call
//	it while another QPainter is active.
//
//------------------------------------------------------------
QSize QpQwtScale::sizeHint(QPainter* p) const
{
  QSize rv;
  if (!p) { if (oparent) {p=oparent->getPainter();} else return(rv); }
  p->save();
    int th,ta, tw;
    
    //QPainter p;
    //p.begin(this);

    //p->setFont(d_titleFont);
    QFontMetrics fm = p->fontMetrics();
   
    th = fm.height();
    ta = fm.ascent();
    tw = fm.width(d_title);
    
    switch(d_scale.orientation())
    {
    case QpQwtScaleDraw::Left:
    case QpQwtScaleDraw::Right:
	rv.setWidth(d_scale.maxWidth(p,TRUE) + d_baseDist + ta);
	rv.setHeight(d_scale.scaleDiv().majCnt() * 2 * d_scale.maxHeight(p));
	break;
    case QpQwtScaleDraw::Top:
    case QpQwtScaleDraw::Bottom:
	rv.setHeight(d_scale.maxHeight(p) + d_baseDist + th + d_titleDist);
	rv.setWidth(d_scale.scaleDiv().majCnt()  * d_scale.maxWidth(p,TRUE));
	break;
    default:
	break;
    }
    //p.end();
    
    p->restore();
    return rv;
    
}

//------------------------------------------------------------
//
//.F	QpQwtScale::minBorderDist
//	Determine the minimum Border distance
//
//.u	Syntax
//.f	int QpQwtScale::minBorderDist()
//
//.u	Description
//	This member function returns the minimum distance
//	of the scale's endpoints from the widget borders which
//	is required for the mark labels to fit into the widget.
//
//.u	Notes
//.i
//	-- This function uses a QPainter internally. Don't call it
//	while another QPainter is active.
//	-- The minimum border distance depends on the font.
//
//.u	See Also
//	@QpQwtScale::setBorderDist@
//
//------------------------------------------------------------
int QpQwtScale::minBorderDist(QPainter* p) const
{
  if (!p) { if (oparent) {p=oparent->getPainter(); } else return (0); }
    int rv = 0;
    
    //QPainter p;
    //p.begin(this);
    QFontMetrics fm = p->fontMetrics();
    

    switch(d_scale.orientation())
    {
    case QpQwtScaleDraw::Left:
    case QpQwtScaleDraw::Right:
	rv = (fm.ascent() + fm.descent() + 1) / 2;
	break;
    case QpQwtScaleDraw::Top:
    case QpQwtScaleDraw::Bottom:
	rv = (d_scale.maxLabelWidth(p,TRUE) + 1) / 2;
	break;
    default:
	break;
    }
    //p.end();
    
    return rv;
    
}

//------------------------------------------------------------
//
//.F	QpQwtScale::fontChange
//	Notify a change of the font
//
//.u	Syntax
//.f	void QpQwtScale::fontChange(const QFont &oldfont)
//
//.u	Parameters
//.p	const QFont &oldfont -- old font
//
//.u	Description
//	This virtual function may be overloaded by derived widgets.
//	The default implementation resizes the scale and repaints
//	the widget.
//------------------------------------------------------------
void QpQwtScale::fontChange(const QFont &oldfont)
{
    resizeScale();
    repaint();
}



//------------------------------------------------------------
//
//.F	QpQwtScale::setScale
//	Assign a scale division
//
//.u	Syntax
//.f	void QpQwtScale::setScale(const QpQwtScaleDiv &sd)
//
//.u	Parameters
//.p	const QpQwtScaleDiv &sd -- Scale Division
//
//.u	Description
//	The scale division determines where to set the tick
//	marks. For more information about scale divisions, see
//	@^QpQwtScaleDiv@.
//
//------------------------------------------------------------
void QpQwtScale::setScale(const QpQwtScaleDiv &sd)
{
    if (d_scale.scaleDiv() != sd)
    {
    	d_scale.setScale(sd);
    	//resize(this->size());
    	repaint();
    }
}









