#include "test10.h"
#include "Qplotter/qp/qpcurve1D.h"

/** General zone test (from nonregzones)
 */

bool Test10::execute(MyWidget & mW) {
  bool retval = true;
  mW.createZones(1,1);

  QpPage *page = mW.getPage();

  QpMap *xMap;
  QpMap *yMap;
  
  // Retrieve page size and compute 10% margins
  double pageWidth = page->getWidth();
  double pageHeight = page->getHeight();
  double leftMargin = rint( pageWidth / 8);
  double bottomMargin = rint(pageHeight  / 8);
  
  // First zone: full height, half width
  double zoneWidth = rint((pageWidth - 3*leftMargin) / 2);
  double zoneHeight = rint((pageHeight - 2*bottomMargin));
  double xPosition = leftMargin;
  double yPosition = pageHeight - bottomMargin - zoneHeight;
  xMap = new QpMap();
  xMap->setContainerRange(xPosition,xPosition+zoneWidth);
  yMap = new QpMap();
  yMap->setContainerRange(yPosition,yPosition+zoneHeight);
  page->addElement(new QpZone(page, xMap, yMap));
  
  // Second zone: half height, half width, top
  zoneHeight = rint((pageHeight - 3*bottomMargin)/2);
  xPosition = leftMargin*2 + zoneWidth;
  yPosition = pageHeight - bottomMargin - zoneHeight;
  xMap = new QpMap();
  xMap->setContainerRange(xPosition,xPosition+zoneWidth);
  yMap = new QpMap();
  yMap->setContainerRange(yPosition,yPosition+zoneHeight);
  page->addElement(new QpZone(page, xMap, yMap));
  
  // Third zone half height, half width, bottom
  yPosition = pageHeight - bottomMargin*2 - zoneHeight*2;
  xMap = new QpMap();
  xMap->setContainerRange(xPosition,xPosition+zoneWidth);
  yMap = new QpMap();
  yMap->setContainerRange(yPosition,yPosition+zoneHeight);
  page->addElement(new QpZone(page, xMap, yMap));
  
  // Fourth small zone (10% of page dimensions) overlaying the first one
  zoneWidth = rint( pageWidth / 10);
  zoneHeight = rint(pageHeight  / 10);
  xPosition = rint((pageWidth - 3*leftMargin) / 2) ;
  yPosition = pageHeight - bottomMargin*2 ;
  xMap = new QpMap();
  xMap->setContainerRange(xPosition,xPosition+zoneWidth);
  yMap = new QpMap();
  yMap->setContainerRange(yPosition,yPosition+zoneHeight);
  page->addElement(new QpZone(page, xMap, yMap));

  QpZone *z1 =  page->getZone(1) ;
  
  // Insert one curve 
  new QpCurve1D( z1, *(mW.getV1()) ,QpCurve1D::HISTO); 
  z1 = page->getZone(2);
  // Insert another curve
  new QpCurve1D( z1, *(mW.getV1()) ,QpCurve1D::HISTO); 
  z1 = page->getZone(3);
  // Insert another curve
  new QpCurve1D( z1, *(mW.getV1()) ,QpCurve1D::HISTO); 
  // Insert another curve
  z1 = page->getZone(4);
  new QpCurve1D( z1, *(mW.getV1()) ,QpCurve1D::HISTO); 

  
  mW.print(fileName.c_str());
  return retval ;  
}





