#include <qapplication.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qppolyface.h"
#include "VectorOfPoints/Vector.h"

/** A Qplotter program to draw a chessboard with QpFaces
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->draw();}
  void chessBoard();
private:
  QpPage  *page;
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(1,1);
}

void MyWidget::chessBoard () {
  QpZone *z1 = page->getZone(0);
  z1->setXRange(0.0,8.001);
  z1->setYRange(0.0,8.0);
  
  // Create a QpPolyface simulating a chessboard (black brush)
  //Arrays to be filled
  double x[32],y[32],w[32],h[32];
  //Variable to fill the array
  double _x,_y,_w,_h;
  //Initializating the Weight and Height of the faces.
  _w= 1.0;
  _h= 1.0;
  
  bool pairs= false;
  int index = 0;
  
  //Arrays filling
  for(_x=0;_x<8;_x++){
    for(_y=1;_y<9;_y+=2){
      if ((_y==1)&&(pairs)) _y=_y+1;
      //Filling all the coordinates
      x[index]=_x;
      y[index]=_y;
      w[index]=_w;
      h[index]=_h;
      index++;
    }
    if (!pairs) pairs=true;
    else pairs=false;
  }
  //Building the QpPolyFace
  QpPolyFace *qppf= new QpPolyFace (index,x,y,w,h,false,z1);
  qppf->setQpBrush(QBrush(Qt::black));
  qppf->setQpPen(QPen(Qt::black,1)); 
  
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.chessBoard();
  myFrame.show();
  myFrame.draw();
  // Enter event loop
  myapp.exec();
  return 1;
}
