#include <qapplication.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpcurve1D.h"
#include "VectorOfPoints/Vector.h"

/** A Qplotter program to show how to deal with zone limits
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->draw();}
  void newCurves();
private:
  QpPage  *page;
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(2,2);
}

void MyWidget::newCurves () {
  Anaphe::AIDAVector vp1,vp2;
  // Read data from file
  vp1.fromAscii("ex2.dat");
  vp2.fromAscii("ex2.dat");
  vp2.mul(2.);
  QpZone *z = page->getZone(0);
  // Insert two curves on the first zone (coordinates not fixed)
  new QpCurve1D( z, vp1 ,QpCurve1D::HISTO); 
  new QpCurve1D( z, vp2 ,QpCurve1D::HISTO); 
  // Insert two curves on the second zone (fixed coordinates)
  z = page->getZone(1);
  z->setFixedCoords(true);
  new QpCurve1D( z, vp1 ,QpCurve1D::HISTO); 
  new QpCurve1D( z, vp2 ,QpCurve1D::HISTO); 
  // Now set arbitrary Y coordinates (fixed)
  z = page->getZone(2);
  z->setFixedCoords(true);
  new QpCurve1D( z, vp1 ,QpCurve1D::HISTO); 
  new QpCurve1D( z, vp2 ,QpCurve1D::HISTO); 
  // Override fixed coordinates
  z->updateY(0,1000.,true);
  // Now set arbitrary X coordinates (not fixed)
  z = page->getZone(3);
  z->updateX(-50,0.);
  new QpCurve1D( z, vp1 ,QpCurve1D::HISTO); 
  new QpCurve1D( z, vp2 ,QpCurve1D::HISTO); 
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.newCurves();
  myFrame.show();
  myFrame.draw();
  // Enter event loop
  myapp.exec();
  return 1;
}
