#include <qapplication.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpcurve1D.h"
#include "VectorOfPoints/Vector.h"

/** A Qplotter program to draw a curve using several representations
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->draw();}
  void newCurve();
private:
  QpPage  *page;
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(3,3);
}

void MyWidget::newCurve () {
  QpCurve1D *qpc;
  Anaphe::AIDAVector vp;
  // Read data from file
  vp.fromAscii("ex2.dat");
  // Insert one curve as Line
  QpZone *z1 = page->getZone(0);
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::LINE); 
  // Insert one curve as Histo
  z1 = page->getZone(1);
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::HISTO); 
  // Insert one curve as error bars
  z1 = page->getZone(2);
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::ERROR); 
  // Insert one curve as a set of markers
  z1 = page->getZone(3);
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::MARKER); 
  qpc->setSymSize(2);
  // Insert one curve as a smooth line
  z1 = page->getZone(4);
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::SMOOTH); 
  // Insert one curve as a filled histo
  z1 = page->getZone(5);
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::FILLED_HISTO);
  qpc->setQpBrush(QBrush(Qt::red));
  // Insert one curve as a set of markers plus Histo
  z1 = page->getZone(6);
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::MARKER_ERROR); 
  qpc->setSymSize(6);
  // Insert one curve as  a filled smooth line
  z1 = page->getZone(7);
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::FILLED_SMOOTH); 
  qpc->setQpBrush(QBrush(Qt::blue));
  // Insert one curve as as Line plus set of markers
  z1 = page->getZone(8);
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::LINE_MARKER); 
  qpc->setSymStyle(QpMarker::Cross);
  qpc->setSymSize(3);
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.newCurve();
  myFrame.show();
  myFrame.draw();
  // Enter event loop
  myapp.exec();
  return 1;
}
