#include <qapplication.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpcurve1D.h"
#include "VectorOfPoints/Vector.h"

/** A Qplotter program showing how to define "non-regular" zones 
  on the same page.

  BUG:
  Seg Fault with the current nonregular zone implementation
  problem with the index of a non reg zone in the page
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->draw();}
  void customZones();
  void newCurve();
private:
  QpPage  *page;
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  customZones();
}

void MyWidget::customZones()
{
  QpMap *xMap;
  QpMap *yMap;
  // Retrieve page size and compute 10% margins
  double pageWidth = page->getWidth();
  double pageHeight = page->getHeight();
  double leftMargin = rint( pageWidth / 10);
  double bottomMargin = rint(pageHeight  / 10);
  // First zone: full height, half width
  double zoneWidth = rint((pageWidth - 3*leftMargin) / 2);
  double zoneHeight = rint((pageHeight - 2*bottomMargin));
  double xPosition = leftMargin;
  double yPosition = pageHeight - bottomMargin - zoneHeight;
  xMap = new QpMap();
  xMap->setContainerRange(xPosition,xPosition+zoneWidth);
  yMap = new QpMap();
  yMap->setContainerRange(yPosition,yPosition+zoneHeight);
  page->addElement(new QpZone(page, xMap, yMap));
  // Second zone: half height, half width, top
  zoneHeight = rint((pageHeight - 3*bottomMargin)/2);
  xPosition = leftMargin*2 + zoneWidth;
  yPosition = pageHeight - bottomMargin - zoneHeight;
  xMap = new QpMap();
  xMap->setContainerRange(xPosition,xPosition+zoneWidth);
  yMap = new QpMap();
  yMap->setContainerRange(yPosition,yPosition+zoneHeight);
  page->addElement(new QpZone(page, xMap, yMap));
  // Third zone half height, half width, bottom
  yPosition = pageHeight - bottomMargin*2 - zoneHeight*2;
  xMap = new QpMap();
  xMap->setContainerRange(xPosition,xPosition+zoneWidth);
  yMap = new QpMap();
  yMap->setContainerRange(yPosition,yPosition+zoneHeight);
  page->addElement(new QpZone(page, xMap, yMap));
  // Fourth small zone (10% of page dimensions) overlaying the first one
  zoneWidth = rint( pageWidth / 10);
  zoneHeight = rint(pageHeight  / 10);
  xPosition = rint((pageWidth - 3*leftMargin) / 2) ;
  yPosition = pageHeight - bottomMargin*2 ;
  xMap = new QpMap();
  xMap->setContainerRange(xPosition,xPosition+zoneWidth);
  yMap = new QpMap();
  yMap->setContainerRange(yPosition,yPosition+zoneHeight);
  page->addElement(new QpZone(page, xMap, yMap));
}

void MyWidget::newCurve () {
  Anaphe::AIDAVector vp;
  // Read data from file
  vp.fromAscii("ex2.dat");
  QpZone *z1 = page->getZone(0);
  // Insert one curve 
  new QpCurve1D( z1, vp ,QpCurve1D::HISTO); 
  z1 = page->getZone(1);
  // Insert another curve
  new QpCurve1D( z1, vp ,QpCurve1D::HISTO); 
  z1 = page->getZone(2);
  // Insert another curve
  new QpCurve1D( z1, vp ,QpCurve1D::HISTO); 
  // Insert another curve
  z1 = page->getZone(3);
  new QpCurve1D( z1, vp ,QpCurve1D::HISTO); 
  z1->setQpFont( QFont( "helvetica", 8 ));
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.newCurve();
  myFrame.show();
  myFrame.draw();
  // Enter event loop
  myapp.exec();
  return 1;
}
