#include <qapplication.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpcurve2D.h"
#include "VectorOfPoints/Vector.h"

/** A Qplotter program to draw a 2D curve on screen.
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->draw();}
  void newCurve();
private:
  QpPage  *page;
  QpCurve2D *qpc;
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(1,1);
}

void MyWidget::newCurve () {
  Anaphe::AIDAVector vp;
  // Read data from file
  vp.fromAscii("ex2D.dat");
  QpZone *z1 = page->getZone(0);
  // Insert a 2D curve
  qpc = new QpCurve2D( z1, vp ,QpCurve2D::BOX); 
  qpc->setQpBrush(QBrush(Qt::red));
  qpc->setQpPen(QPen(Qt::blue,1));
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.newCurve();
  myFrame.show();
  myFrame.draw();
  // Enter event loop
  myapp.exec();
  return 1;
}
