#include <qapplication.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpcurve1D.h"
#include "VectorOfPoints/Vector.h"

/** A Qplotter program to show how to use picking information to change
  curve color.
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->redraw();}
  void newCurve();
  void toggleColor();
  QpPage  *getPage() {return page;}
private:
  QpPage  *page;
  QpCurve1D *qpc;
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(1,1);
  page->setFinePicking(true);
}

void MyWidget::newCurve () {
  Anaphe::AIDAVector vp;
  // Read data from file
  vp.fromAscii("ex2.dat");
  QpZone *z1 = page->getZone(0);
  // Insert one curve for Monte Carlo values
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::LINE); 
}

void MyWidget::toggleColor () {
  if (qpc->getQpPen()->color() == Qt::red)
    qpc->setQpPen(QPen(Qt::black,1));
  else
    qpc->setQpPen(QPen(Qt::red,1));
  draw();
}

/**
  A small class implementing the locator for this frame.
  Whenever the mouse is clicked on a curve, the color is toggled between
  red and black.
 */

class SmallLocator : public QpEvObserver {
public:
  SmallLocator(MyWidget *_parent) : 
    QpEvObserver((QpListener::EventType)(QpListener::LeftPressed)) {
      mParent = _parent;
  }
  /// On notification toggle color
  virtual void update();
  /// Termination
  virtual void end() {
    listener->removeObserver(this);
  }
private:
  // Empty copy ctor
  SmallLocator(const SmallLocator& rhs):QpEvObserver(QpListener::LeftPressed) {}
  MyWidget *mParent;
};  

void SmallLocator::update() {
  if (listener->getPicked() != 0) {
    std::cerr << listener->getPicked()->isA() << std::endl;
    if (QString("QpPolyLine") == listener->getPicked()->isA())
      mParent->toggleColor();
  }
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.newCurve();
  myFrame.show();
  myFrame.draw();
  SmallLocator *locator = new SmallLocator(&myFrame);
  locator->lRegister(myFrame.getPage());
  // Enter event loop
  myapp.exec();
  delete locator;
  return 1;
}
