#include <qapplication.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpcurve1D.h"
#include "VectorOfPoints/Vector.h"

/** A Qplotter program to show how to use selective zone redrawing
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->redraw();}
  void newCurve();
  void redrawZone(QpZone *z);
  QpPage  *getPage() {return page;}
private:
  QpPage  *page;
  QpCurve1D *qpc;
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(2,2);
}

void MyWidget::newCurve () {
  Anaphe::AIDAVector vp;
  // Read data from file
  vp.fromAscii("ex2.dat");
  QpZone *z1 = page->getZone(0);
  // Insert one curve 
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::HISTO); 
  z1 = page->getZone(1);
  // Insert one curve 
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::HISTO); 
  z1 = page->getZone(2);
  // Insert one curve 
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::HISTO); 
  z1 = page->getZone(3);
  // Insert one curve 
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::HISTO); 
}

void MyWidget::redrawZone(QpZone *z) {
  page->redrawZone(z);
}

/**
  A small class implementing the locator for this frame.
  Whenever the mouse is clicked on a curve, the color is toggled between
  red and black.
 */

class SmallLocator : public QpEvObserver {
public:
  SmallLocator(MyWidget *_parent) : 
    QpEvObserver((QpListener::EventType)(QpListener::LeftPressed)) {
      mParent = _parent;
  }
  /// On notification execute this
  virtual void update();
  /// Termination
  virtual void end() {
    listener->removeObserver(this);
  }
private:
  // Empty copy ctor
  SmallLocator(const SmallLocator& rhs):QpEvObserver(QpListener::LeftPressed) {}
  MyWidget *mParent;
};  

void SmallLocator::update() {
  // Check we picked on a zone
  if (listener->getPickedNode() != 0) {
    // Retrieve zone pointer
    QpZone *z1 = listener->getPickedNode()->getZone();
    // Toggle zone pen color
    if (z1->getQpPen()->color() == Qt::red)
      z1->setQpPen(QPen(Qt::black,1));
    else
      z1->setQpPen(QPen(Qt::red,1));
    // Redraw the touched zone only
    mParent->redrawZone(z1);
  }
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.newCurve();
  myFrame.show();
  myFrame.draw();
  SmallLocator *locator = new SmallLocator(&myFrame);
  locator->lRegister(myFrame.getPage());
  // Enter event loop
  myapp.exec();
  delete locator;
  return 1;
}
