#include <qapplication.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpcurve1D.h"
#include "VectorOfPoints/Vector.h"

/** A Qplotter program showing how to use summary information
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->draw();page->print("summaryt.eps");}
  void newCurve();
private:
  QpPage  *page;
  QpCurve1D *qpc;
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(1,1);
}

void MyWidget::newCurve () {
  Anaphe::AIDAVector vp1,vp2;
  // Read data from file
  vp1.fromAscii("ex2.dat");
  vp2.fromAscii("ex2.dat");
  vp2.mul(2.);
  QpZone *z1 = page->getZone(0);
  // Insert one curve for Monte Carlo values
  qpc = new QpCurve1D( z1, vp1 ,QpCurve1D::LINE); 
  qpc->setQpPen(QPen(Qt::red,1));
  SummaryTable* st1 = new SummaryTable;
  st1->append( new StPair( "Max", "100"));
  st1->append( new StPair("legend", "Data"));
  
  qpc->setSummaryTable(st1);
  qpc = new QpCurve1D( z1, vp2 ,QpCurve1D::LINE_MARKER); 
  SummaryTable* st2 = new SummaryTable;
  st2->append( new StPair( "Max", "200"));
  st2->append( new StPair("legend", "Monte Carlo"));
  qpc->setSummaryTable(st2);
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.newCurve();
  myFrame.show();
  myFrame.draw();
  // Enter event loop
  myapp.exec();
  return 1;
}
