#include <qapplication.h>
#include "Qplotter/qp/qppage.h"

/** A Qplotter program to show how to use the different text and titles
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->draw();}
  void print() {page->print("text.ps");}
  void addTexts();
private:
  QpPage  *page;
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(1,2);
}

void MyWidget::addTexts () {
  
  // Insert the page title in the middle of the page.
  page->setTitle("PAGE TITLE", Qt::AlignCenter);
  // Insert a Text in Zone Coordinates.
  QpZone *z = page->getZone(0);
  z->updateX(0,10.,true);
  z->updateY(0,100.,true);
  z->addText (5,50,"(5,50) Zone Coordinates",QpTransformation::Zone);
  //Insert a text in the same Zone Coordinates.
  z->addText (-150,-20,"(-150,-20) Page coords with respect to zone top right corner",QpTransformation::ZrTopRight);

  //Insert the Zone Title aligned on the left.
  z = page->getZone(1);
  z->setTitle("ZONE TITLE",Qt::AlignLeft); 
  // Changing the Coordinates
  z->updateX(0,100.,true);
  z->updateY(0,500.,true);
  
  //Insert a text in the same Zone Coordinates.
  z->addText (5,50,"(5,50) Zone Coordinates",QpTransformation::Zone);

  //Insert a text in the same Zone Coordinates.
  z->addText (-150,-20,"(-150,-20) Page coords with respect to zone top right corner",QpTransformation::ZrTopRight);
 
  //Insert a text with Page coordinates.   
  QpText *pgT = page->addText(90.0,90.0,"(90,90) Page Coordinates");
  pgT->getTextObject()->setAngle(45);
  pgT->setQpPen(QPen(Qt::blue,1));
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.addTexts();
  myFrame.show();
  myFrame.print();
  // Enter event loop
  myapp.exec();
  return 1;
}



