#include <qapplication.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpaxis.h"
#include "Qplotter/qp/qpcurve1D.h"
#include "VectorOfPoints/Vector.h"

/** A Qplotter program showing how to deal with axis
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->draw();}
  void newCurve();
  void changeAxis();
private:
  QpPage  *page;
  QpCurve1D *qpc;
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(1,1);
}

void MyWidget::newCurve () {
  Anaphe::AIDAVector vp;
  // Read data from file
  vp.fromAscii("ex2.dat");
  QpZone *z1 = page->getZone(0);
  // Insert one curve for Monte Carlo values
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::LINE); 
}

void MyWidget::changeAxis () {
  QpZone *z1 = page->getZone(0);

  // Change axis appearance via QpZone methods
  // Set logarithmic scale on both axis
  z1->setXLog(true);
  z1->setYLog(true);
  // Grid management
  z1->enableGridX(true,true);
  z1->enableGridY(true,true);

  // Change axis directly
  // Get a pointer to the system of axis
  QpAxisNode *axis = z1->getAxisNode();
  // Change axis titles and labels
  axis->getAxis("x1")->setTitle("Energy");
  axis->getAxis("x1")->setTitle("Luminosity");
  axis->getAxis("y1")->setLegend("MeV");
  axis->getAxis("x1")->setLegend("pBarn");
  // Create a new logarithmic scale on the top 
  z1->makeAxis(QpZone::Top);
  axis->getAxis("x2")->update(1.,10000000.);
  axis->getAxis("x2")->setOption(QpAxis::Logarithmic,true);
  // Create a new linear scale on the right
  z1->makeAxis(QpZone::Right);
  axis->getAxis("y2")->update(-50.,50.);
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.newCurve();
  myFrame.changeAxis();
  myFrame.show();
  myFrame.draw();
  // Enter event loop
  myapp.exec();
  return 1;
}
