#include <qapplication.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qptext.h"

/** A Qplotter program to show how to use math stuff.
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->draw();}
  void print() {page->print("math.eps");}
  void makeMath();
private:
  QpPage  *page;
  void pawExample(QpZone *z);
  void sqrtExample(QpZone *z);
  void integralExample(QpZone *z);
  void rotateExample(QpZone *z);
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(1,1);
}

void MyWidget::makeMath () {
  QpZone *z = page->getZone(0);
  z->setXRange(0.0,9.0);
  z->setYRange(0.0,9.0);
  z->setQpFont( QFont( "helvetica", 24 ));
  pawExample(z);
  sqrtExample(z);
  integralExample(z);
  rotateExample(z);
}

void MyWidget::pawExample (QpZone *z) {
  // A formula used in the PAW manual
  // No alignment (i.e. left alignment on the X coordinate
  new QpText
    ("<qpmath><mrow> <msub><mi>L</mi> <mi>em</mi></msub><mo>=</mo> <mi>&nbsp;</mi> <mi>e</mi><mi>&nbsp;</mi> <msubsup><mi>J</mi><mi>em</mi> <mi>&mu;</mi></msubsup><msub><mi>A</mi> <mi>&mu;</mi></msub><mi>&nbsp;</mi><mi>,</mi><mi>&nbsp;</mi><msubsup><mi>J</mi><mi>em</mi> <mi>&mu;</mi></msubsup> <mo>=</mo> <mover><mi>l</mi><mo>_</mo></mover><mi>&nbsp;</mi><msub><mi>&gamma;</mi> <mi>&mu;</mi></msub><mi>l</mi><mi>&nbsp;</mi><mi>,</mi><mi>&nbsp;</mi><msubsup><mi>M</mi><mi>i</mi> <mi>j</mi></msubsup> <mo>=</mo> <munderover><mi>&sum;</mi> <mrow><mi>&nbsp;</mi><mi>&alpha;</mi></mrow><mrow><mi>&nbsp;</mi><mi>&infin;</mi></mrow></munderover><msub><mi>A</mi> <mi>&alpha;</mi></msub><msubsup><mi>&tau;</mi><mrow><mi>&alpha;</mi><mi>j</mi></mrow> <mi>i</mi></msubsup> </mrow></qpmath>",
     z,1.,7.);
}

void MyWidget::sqrtExample (QpZone *z) {
  // Two square roots
  QpText *myMath = new QpText ("<qpmath><mrow>Hello world<mi>&plusmn;</mi><msqrt><mrow><msup><mi>a</mi> <mn>2</mn> </msup> <mo>+</mo><msup><mi>b</mi> <mn>2</mn> </msup></mrow></msqrt><msqrt><mi>abc</mi></msqrt></mrow></qpmath>" , z, 1.,5.,8.,0.);
  //    QpText *myMath = new QpText ("<qpmath><msqrt><mrow><msup><mi>a</mi><mn>2</mn></msup><mo>+</mo><msup><mi>b</mi><mn>2</mn></msup></mrow></msqrt></qpmath>" , z, 1.,5.,8.,0.);
  //    QpText *myMath = new QpText ("<qpmath><msqrt><mrow>alpha beta</mrow></msqrt></qpmath>" , z, 1.,5.,8.,0.);
  //    QpText *myMath = new QpText ("<qpmath><msqrt><mo>foobar</mo></msqrt></qpmath>" , z, 1.,5.,8.,0.);
  // Align the formula on the center of a rectangle whose width is
  // 8 units in zone coordinates
  myMath->getTextObject()->setAlignment (Qt::AlignCenter);
}

void MyWidget::integralExample (QpZone *z) {
  // An integral
  QpText *myMath = new QpText("<qpmath><mrow> <munderover><mi>&int;</mi><mrow><mo>-</mo><mi>&infin;</mi></mrow><mrow><mi>&nbsp;</mi><mi>&nbsp;</mi><mi>0</mi></mrow></munderover><mrow><mi>&nbsp;</mi><msup><mi>e</mi><mrow><mo>-</mo><mi>x</mi></mrow></msup><mi>dx</mi></mrow></mrow></qpmath>" , z,1.,3.,8.,0.);
  // Align the formula on the right of a rectangle whose width is
  // 8 units in zone coordinates
  myMath->getTextObject()->setAlignment (Qt::AlignRight);
}

void MyWidget::rotateExample (QpZone *z) {
  // Two square roots rotated and coloured
  //  QpText *myMath = new QpText("<qpmath><mrow> <mi>&plusmn;</mi><msqrt><mrow><msup><mi>a</mi> <mn>2</mn> </msup> <mo>+</mo><msup><mi>b</mi> <mn>2</mn> </msup></mrow></msqrt><msqrt><mi>abc</mi></msqrt></mrow></qpmath>" , z, 0.,0.,0.,0.);
  QpText *myMath = new QpText("<qpmath><mrow> <mi>&plusmn;</mi><msqrt><mrow><msup><mi>a</mi> <mn>2</mn> </msup> <mo>+</mo><msup><mi>b</mi> <mn>2</mn> </msup></mrow></msqrt></mrow></qpmath>" , z, 0.,0.,0.,0.);
  myMath->getTextObject()->setCoords(1.,1.);
  myMath->getTextObject()->setAngle(45);
  myMath->setQpPen(QPen(Qt::red,1));
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.makeMath();
  myFrame.show();
  myFrame.print();
  // Enter event loop
  myapp.exec();
  return 1;
}
