#include <qapplication.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qppolyline.h"

/** A Qplotter program to show how to use the scene graph.
*/

// Dat to draw three polygons
double polyX1[] = {3.,5.,7.,7.,5.,3.,1.,1.};
double polyY1[] = {1.,1.,3.,5.,7.,7.,5.,3.};
double polyX2[] = {4.,6.,6.,4.};
double polyY2[] = {6.,6.,8.,8.};
double polyX3[] = {6.,8.,8.,6.};
double polyY3[] = {2.,2.,4.,4.};

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->draw();page->print("polygon.eps");}
  void newCurve();
private:
  QpPage  *page;
  void newPolygons(QpZone *z);
  static bool cmpPolygon(QpObject *obj)   { return (obj->isA("QpNode<QpPolyShape>"));   }
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(2,1);
}

void MyWidget::newPolygons (QpZone *z) {
  z->setXRange(0.0,9.0);
  z->setYRange(0.0,9.0);
  // Insert one polygon
  // Create a QpPolyShape node first (so that we can assign it a pen attribute)
  QpNode<QpPolyShape> *pl=new QpNode<QpPolyShape>(new QString("QpNode<QpPolyShape>"),z,true);
  pl->setQpBrush(QBrush(Qt::red));
  // Now create a new polygon and insert it in the QpNode
  new QpPolygon(8, polyX1, polyY1, pl, true);
  // Insert Another polygon
  // Create a QpPolyShape node first (so that we can assign it a pen attribute)
  pl=new QpNode<QpPolyShape>(new QString("QpNode<QpPolyShape>"),z,true);
  pl->setQpBrush(QBrush(Qt::blue));
  // Now create two polygons and insert them in the QpNode
  new QpPolygon(4, polyX2, polyY2, pl, true);
  new QpPolygon(4, polyX3, polyY3, pl, true);
}

void MyWidget::newCurve () {
  // Create the set of polygons in the two zones
  newPolygons(page->getZone(0));
  newPolygons(page->getZone(1));
  // Change the order in the list of polygons of the second zone
  QpZone *z = page->getZone(1);
  // Retrieve the first polygon
  QpObject *first = z->searchElement(cmpPolygon);
  // Bring to front the last polygon
  // (i.e. put it as last element of the list of the zone)
  z->addElement(first);
  // Remove the last polygon
  z->setAutoDelete(false);
  z->removeElement(first);
  z->setAutoDelete(true);
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.newCurve();
  myFrame.show();
  myFrame.draw();
  // Enter event loop
  myapp.exec();
  return 1;
}
