#include <qapplication.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpcurve1D.h"
#include "VectorOfPoints/Vector.h"

/** Another Qplotter program to show how to use zooming
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->draw();}
  void newCurve();
  void zoomBy();
private:
  QpPage  *page;
  QpCurve1D *qpc;
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(1,2);
}

void MyWidget::newCurve () {
  Anaphe::AIDAVector vp;
  // Read data from file
  vp.fromAscii("ex2.dat");
  QpZone *z1 = page->getZone(0);
  z1->setTitle("Original data");
  // Insert one curve 
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::HISTO);
  z1 = page->getZone(1);
  // Insert one curve 
  qpc = new QpCurve1D( z1, vp ,QpCurve1D::HISTO);
}

void MyWidget::zoomBy () {
  QpZone *z1 = page->getZone(1);
  z1->setTitle("Reverse zooming from {-100,0} to {100,120}");
  z1->zoom(-100.,0.,100.,120.);
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.newCurve();
  myFrame.zoomBy();
  myFrame.show();
  myFrame.draw();
  // Enter event loop
  myapp.exec();
  return 1;
}
