#include <qapplication.h>
#include <qpushbutton.h>
#include "Qplotter/qp/qppage.h"
#include "Qplotter/qp/qpcurve1D.h"
#include "VectorOfPoints/Vector.h"

/** A Qplotter program to show how to use zooming
*/

const int pixDim=600;

// Subclass Qt QWidget to make our window

class MyWidget : public QWidget
{
public:
  MyWidget( QWidget *parent=0, const char *name=0 );
  void draw() {page->draw();page->print("zoom3.eps");}
  void newCurve();
private:
  QpPage  *page;
  QPushButton *zoomButton, *unzoomButton;
protected:
  /// Events processing
  void resizeEvent(QResizeEvent* e){page->resize(e->size());}
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
  setGeometry(1,1,pixDim,pixDim);
  page = new QpPage(this,QRect(0,0,pixDim,pixDim),210,210);
  page->createZones(1,1);
  zoomButton = new QPushButton("Zoom", this);
  QObject::connect(zoomButton, SIGNAL(clicked()), page, SLOT(slotZoom()));
  unzoomButton = new QPushButton("Unzoom", this);
  unzoomButton->setGeometry(110,0,100,30);
  QObject::connect(unzoomButton, SIGNAL(clicked()), page, SLOT(slotUnZoom()));
}

void MyWidget::newCurve () {
  Anaphe::AIDAVector vp;
  // Read data from file
  vp.fromAscii("ex2.dat");
  QpZone *z1 = page->getZone(0);
  // Insert one curve 
  new QpCurve1D( z1, vp ,QpCurve1D::HISTO);
}

int main(int argc, char* argv[]) {
  // QApplication object manages the event loop
  QApplication myapp(argc, argv);
  // Our widget corresponds to a window
  MyWidget myFrame;
  // make the frame visible
  myapp.setMainWidget(&myFrame);
  myFrame.newCurve();
  myFrame.show();
  myFrame.draw();
  // Enter event loop
  myapp.exec();
  return 1;
}
