#ifndef _IVIEWER_
#define _IVIEWER_

#if defined __hpux || (defined __sun && __SUNPRO_CC < 0x500)
#define QPSTD
#else
#define QPSTD std
#endif

#include <string>

// type ahead declarations only !!!
namespace Anaphe 
{
  class IVector;
  class IScatter2D;
}


/** Interface class to drawing style object. Pure abstract class.
 */
class IDrawStyle {
public:
  /// Change text property
  virtual void setProperty (QPSTD::string name, QPSTD::string value) = 0;
  /// List supported text options
  virtual void listOptions(QPSTD::ostream &os = QPSTD::cout)= 0; 
};  

/** Interface class to text style object. Pure abstract class.
 */
class ITextStyle {
public:
  /// Change text property
  virtual void setProperty (QPSTD::string name, QPSTD::string value) = 0;
  /// List supported text options
  virtual void listOptions(QPSTD::ostream &os = QPSTD::cout)= 0; 
};  

/** Interface class to text object. Pure abstract class.
 */
class IText {
public:
  /// Change dataset property
  virtual void setProperty (QPSTD::string name, QPSTD::string value) = 0;
  /// List supported dataset options
  virtual void listOptions(QPSTD::ostream &os = QPSTD::cout) = 0; 
  /// Get text style
  virtual ITextStyle* getStyle() = 0;
};  

/** Interface class to an axis object. Pure abstract class.
 */
class ViewerAxis {
public:
  /// Change axis property
  virtual void setProperty (QPSTD::string name, QPSTD::string value) = 0;
  /// List supported axis options
  virtual void listOptions(QPSTD::ostream &os = QPSTD::cout)= 0; 
};  

/** Interface class to a generic dataset. Pure abstract class.
 */
class IDataset {
public:
  /// Change dataset property
  virtual void setProperty (QPSTD::string name, QPSTD::string value) = 0;
  /// Change integer property
  virtual void setProperty (QPSTD::string name, int value)= 0;
  /// List supported dataset options
  virtual void listOptions(QPSTD::ostream &os = QPSTD::cout)= 0; 
  /// Get draw style
  virtual IDrawStyle* getStyle() = 0;
};  
  
/** Interface class to a generic zone. Pure abstract class.
 */
class IZone {
public:
  /// Set title
  virtual IText *setTitle(QPSTD::string newTitle) = 0;
  /// Put text in the zone at {x,y} coordinates
  virtual IText *addText(double x, double y, QPSTD::string text) = 0;
  /// Change zone property
  virtual void setProperty (QPSTD::string name, QPSTD::string value)= 0;
  /// List supported zone options
  virtual void listOptions(QPSTD::ostream &os = QPSTD::cout)= 0; 
  /// Add a dataset to the zone
  virtual IDataset* addDataset (Anaphe::IVector& VoP) = 0;
  /// Add a scatterplot to the zone
  virtual void addScatter (Anaphe::IScatter2D *scat) = 0;
  /// Retrieve a dataset from the zone. Index is 0 to N-1
  virtual IDataset* getDataset (unsigned int ) = 0;
  /// Set min/max on X axis
  virtual void xMinMax (double min, double max) = 0;
  /// Set min/max on Y axis
  virtual void yMinMax (double min, double max) = 0;
  /// Retrieve axis
  virtual ViewerAxis *getAxis (QPSTD::string name) = 0;
  /// Get text style
  virtual ITextStyle* getTextStyle() = 0;
  /// Get draw style
  virtual IDrawStyle* getDrawStyle() = 0;

  /// Clear all objects
  virtual void clear() = 0;

  ///
  virtual void createHistogram (Anaphe::IVector *data) = 0;

  // Todo: Add/Remove histo
}; 

/** Interface class to a generic page. Pure abstract class.
 */
class IPage {
public:
  /// Set title
  virtual IText *setTitle(QPSTD::string newTitle) = 0;
  /// Put text in the page at {x,y} coordinates
  virtual IText *addText(double x, double y,QPSTD::string text) = 0;
  /// Retrieve a zone from the page. Index is 0 to N-1
  virtual IZone* selectZone(unsigned int)= 0;
  /// Get text style
  virtual ITextStyle* getTextStyle() = 0;
  /// Get draw style
  virtual IDrawStyle* getDrawStyle() = 0;

  /// Create Zone
  virtual void createZones(int nbX, int nbY) = 0;
  virtual IZone* createIrregularZone(double x, double y,
				     double w, double h) = 0;
  /// Clear page
  virtual void clear() = 0;

};

/** Interface class to a generic viewer. Pure abstract class.
 */
class IViewer {
 public:
  /// Clear the Viewer
  virtual void clear() = 0;
  /// Show the Viewer
  virtual void show() = 0;
  /// Create a new page with nx*ny zones
  virtual IPage *createPage(int nx, int ny) = 0;
  /// Dump viewer content to file as Postscript
  virtual void makePS(const char *filename) = 0;
  /// Dump viewer content to file as PNG bitmap
  virtual void makePicture(const char *filename) = 0;
  /// Draw the current page on the Viewer window
  virtual void draw()= 0;
  /// Change viewer property
  virtual void setProperty (QPSTD::string name, QPSTD::string value) = 0;
  /// List supported viewer properties
  virtual void listOptions(QPSTD::ostream &os = QPSTD::cout)= 0; 
  /// Refresh the viewer
  virtual void refresh() = 0;
  virtual void reset() = 0;
  virtual ~IViewer(){};

  // Are this methods to be here ?
  /// Create coordinate window
  virtual void createCoordWindow() = 0;  

  virtual void createRegion  (double x, double y, double w, double h) = 0;
  virtual void createRegions (int columns, int rows) = 0;
  virtual void plot (Anaphe::IVector *data, int index = 0) = 0;
  virtual void clearRegion(int index) = 0;
  virtual void activateOutline(bool On = true) = 0;
  
};

#endif
