#ifndef NO_THREADED_PLOTTER
#include "Qplotter/viewer/qpViewThread.h"
#endif

#include <iostream>

#include <assert.h>
#include "Qplotter/viewer/applInstance.h"

// Initialize static instance of the singleton

QApplicationInstance* QApplicationInstance::_instance = 0;
#ifndef NO_THREADED_PLOTTER
int QApplicationInstance::count=0;
GUIThread* QApplicationInstance::guiThread=0;
#endif

// Create the application instance
void QApplicationInstance::createInstance() {
  if (_instance == 0)
    {
      int argc=1;
      char *argv[1]; 
      argv[0] =((char *) "Qplotter");
  
      _instance = new QApplicationInstance(argc,argv);

      //  For threaded configurations (i.e. when Qt has been built as
      //  a threaded library), the application global mutex will be
      //  locked in the constructor and unlocked when entering the
      //  event loop with exec(). You must unlock the mutex explicitly
      //  if you don't call exec(), otherwise you might get warnings
      //  on application exit.

      //  We call exec() in the guiThread but as it is another thread
      //  we must unlock herequit
      //  and lock again in the guiThread

#ifndef NO_THREADED_PLOTTER
      _instance->unlock();
#endif
      
    }
}

void QApplicationInstance::cancelThreads() {
  _instance->lock();
  guiThread->cancel();
  delete guiThread;
  guiThread = 0;
  _instance->unlock();
}


// Create the application instance if needed and/or make a new client.
QApplicationInstance* QApplicationInstance::Instance() {
  if (_instance == 0) {
    QApplicationInstance::createInstance();
  } else {
#ifndef NO_THREADED_PLOTTER
    if (count > 0) {
      // In Qt, the QThread is the running thread. If a thread object
      // is started again then it waits for the object end and then
      // restart again. With one QThread object there is only one
      // thread running.
      QApplicationInstance::cancelThreads();
    }
#endif
  }

  // Any Instance request starts the only GUI thread 
  QApplicationInstance::makeThread();
  return _instance;
}

// Static method to get the unique instance
QApplicationInstance* QApplicationInstance::retrieveInstance() {
  assert(_instance != 0);
  return _instance;
}

// When a client is destroyed, reference counting is decremented. The
// last client exits the thread as well.
void QApplicationInstance::deref() {
#ifndef NO_THREADED_PLOTTER
  count--;
  std::cout << "count = " << count << std::endl ;
  
  // The last client of this Instance exits the thread and destroys it
  if (count == 0) QApplicationInstance::cancelThreads();
#endif
}

/** Create a the GUI thread that runs the current QApplication instance
 */
void QApplicationInstance::makeThread() {
#ifndef NO_THREADED_PLOTTER
  if (guiThread == 0)
    {
      // Create a new thread to manage GUI
      guiThread = new GUIThread(_instance);
      // Start the thread
      guiThread->start();
      // Update no. of thread clients
      count++;
    } else {
      std::cerr << "QApplicationInstance::makeThread()>  Thread exists already." << std::endl ;
    }
#endif
}
