#ifndef _PROPERTYSET_
#define _PROPERTYSET_

#include <string>
#include <list>
#include <iterator>

#if defined __hpux || (defined __sun && __SUNPRO_CC < 0x500)
#define QPSTD
#else
#define QPSTD std
#endif

/** A class implementing very basic property management for the 
  QpViewer::  classes.
  */

const char* const ColorProperties = 
" blue white black red green yellow magenta cyan darkgray lightgray gray darkred darkgreen darkblue darkcyan darkmagenta darkyellow " ;

class StringPropertySet {
public:
  enum Existance {FOUND,KEYNOTFOUND,VALUENOTFOUND};
  /// Constructor
  StringPropertySet() {properties_.clear();};
  /// Destructor
  ~StringPropertySet() {};
  /** Add a property. The property name is key. The property values
    are specified as blank separated literals in values.
    An empty values string means any string value will be accepted as it is.
  */
  void addProperty ( QPSTD::string key, QPSTD::string values );
  /// Color properties are a special case of general properties
  void addColorProperty ( QPSTD::string key);
  /**Does the property named key exist? Is so, is value a valid property
    instance?
  */
  Existance find ( QPSTD::string key, QPSTD::string value );
  /// Stream out the property list
  void print (QPSTD::ostream& s) ;
private:
  QPSTD::list< QPSTD::pair < QPSTD::string, QPSTD::string > > properties_;
};

template < class T > class BasicProperty {
 private:
  T value;
  bool changed;
 public:
  BasicProperty() : changed(false) {};
  void setValue(T newVal) {value=newVal;changed=true;}
  const T getValue() const {return value;}
  const bool hasChanged() const { return changed;}
};

#endif // _PROPERTYSET_
