#include <qfontdatabase.h>
#include "Qplotter/viewer/qpViewer.h"
#include <algorithm>

#if (defined __sun && __SUNPRO_CC > 0x500)
# include <strstream>
#else
# include <sstream>
#endif

QpViewer::VDrawStyle::VDrawStyle(IStyleOwner *iso) : 
  lineColor(Qt::black) , fillColor(Qt::white) , lineWidth(1) , 
  lineShape (Qt::SolidLine)  , bStyle(Qt::NoBrush) ,theStyleOwner(iso) {
    vDrawStyleProperties.addProperty("linecolor","");
    vDrawStyleProperties.addProperty("fillcolor","");
    vDrawStyleProperties.addProperty("linewidth","");
    vDrawStyleProperties.addProperty("lineshape",
				     "none solid dash dot dashdot dashdotdot");
    vDrawStyleProperties.addProperty("fillstyle",
				     "none solid dense94 dense88 dense63 dense50 dense37 dense12 dense06 horiz vert cross bdiag fdiag diagcross");
}

QpViewer::VDrawStyle::~VDrawStyle() {
}

void QpViewer::VDrawStyle::setProperty (QPSTD::string name, 
					QPSTD::string newValue) {
  QPSTD::string value(newValue);
  QPSTD::transform(name.begin(),name.end(),name.begin(),tolower);
  QPSTD::transform(value.begin(),value.end(),value.begin(),tolower);
  StringPropertySet::Existance result = vDrawStyleProperties.find(name,value);
  switch (result) {
  case(StringPropertySet::FOUND) :
      if (name == "lineshape") {
	  if(value == "none") lineShape = Qt::NoPen;
	  if(value == "solid") lineShape = Qt::SolidLine;
	  if(value == "dash") lineShape = Qt::DashLine;
	  if(value == "dot") lineShape = Qt::DotLine;
	  if(value == "dashdot") lineShape = Qt::DashDotLine;
	  if(value == "dashdotdot") lineShape = Qt::DashDotDotLine;
      }
      if (name == "fillstyle") {
	  if(value == "none") bStyle = Qt::NoBrush;
	  if(value == "solid") bStyle = Qt::SolidPattern;
	  if(value == "dense94") bStyle = Qt::Dense1Pattern;
	  if(value == "dense88") bStyle = Qt::Dense2Pattern;
	  if(value == "dense63") bStyle = Qt::Dense3Pattern;
	  if(value == "dense50") bStyle = Qt::Dense4Pattern;
	  if(value == "dense37") bStyle = Qt::Dense5Pattern;
	  if(value == "dense12") bStyle = Qt::Dense6Pattern;
	  if(value == "dense06") bStyle = Qt::Dense7Pattern;
	  if(value == "horiz") bStyle = Qt::HorPattern;
	  if(value == "vert") bStyle = Qt::VerPattern;
	  if(value == "cross") bStyle = Qt::CrossPattern;
	  if(value == "bdiag") bStyle = Qt::BDiagPattern;
	  if(value == "fdiag") bStyle = Qt::FDiagPattern;
	  if(value == "diagcross") bStyle = Qt::DiagCrossPattern;
      }
      if (name == "linecolor") lineColor = MapColor(value);
      if (name == "fillcolor") fillColor = MapColor(value);
      if (name == "linewidth") {
	QPSTD::istringstream is(value.c_str());
	is >> lineWidth;      
      }
      // Notify change to style owner
      theStyleOwner->update(QpViewer::IStyleOwner::DrawStyle);
      break;
  case(StringPropertySet::KEYNOTFOUND):
      QPSTD::cerr << "Unknown property: " << name << QPSTD::endl;
      break;
  case(StringPropertySet::VALUENOTFOUND):
      QPSTD::cerr << "Unknown value: " << value << " for property" ;
      QPSTD::cerr << name <<QPSTD::endl; 
  }
}

void QpViewer::VDrawStyle::listOptions(QPSTD::ostream &os) {
  QpViewer::listOptions("DrawStyle",vDrawStyleProperties, os);
} 

QBrush QpViewer::VDrawStyle::getBrush() {
  return QBrush(fillColor,bStyle);
}

QPen QpViewer::VDrawStyle::getPen() {
  return QPen(lineColor,lineWidth,lineShape);
}

QpViewer::VTextStyle::VTextStyle(IStyleOwner *iso) : 
  fontName("Helvetica") , fontSize(12), textColor(Qt::black) , 
  textWeight(QFont::Normal) , italic (false),  theStyleOwner(iso) {
    vTextStyleProperties.addProperty("fontname","");
    vTextStyleProperties.addProperty("fontsize","");
    vTextStyleProperties.addProperty("color","");
    vTextStyleProperties.addProperty("bold","yes no");
    vTextStyleProperties.addProperty("italic","yes no");
}
 
QpViewer::VTextStyle::~VTextStyle() {
}

void QpViewer::VTextStyle::setProperty (QPSTD::string name, QPSTD::string value) {
  QPSTD::transform(name.begin(),name.end(),name.begin(),tolower);
  if (vTextStyleProperties.find(name,value) == StringPropertySet::FOUND) {
    if (name == "fontsize" ) {
      QPSTD::istringstream is(value.c_str());
      is >> fontSize;      
    }
    if (name == "fontname" ) fontName = value;
    if (name == "bold" ) {
      if (value == "yes")    textWeight = QFont::Bold;
      if (value == "no")     textWeight = QFont::Normal;
    }
    if (name == "italic" )   {
      if (value == "yes")   italic = true;
      if (value == "no")    italic = false;
    }
    if (name == "color" )    textColor = MapColor(value);
    // Notify change to style owner
    theStyleOwner->update(QpViewer::IStyleOwner::TextStyle);
  } else
    QPSTD::cerr << "Unknown property: " << name <<QPSTD::endl;
}

void QpViewer::VTextStyle::listOptions(QPSTD::ostream &os) {
  QpViewer::listOptions("TextStyle",vTextStyleProperties, os);
  os << "--------------------- ";
  os << " Available fonts  -------------------"<< QPSTD::endl;
  // Acquire lock on the library
  (QApplicationInstance::retrieveInstance())->lock();
  {
    QFontDatabase qdfb;
    QStringList fam = qdfb.families();
    for ( QStringList::Iterator it = fam.begin(); it != fam.end(); ++it )
      os <<(*it).latin1() << QPSTD::endl;
  }
  // Release lock on the library
  (QApplicationInstance::retrieveInstance())->unlock();
  os << "------------------------------------------------------------" ;
  os << QPSTD::endl << QPSTD::endl;
} 

QFont QpViewer::VTextStyle::getFont() {
  return QFont(fontName.c_str(),
	       fontSize,
	       textWeight,
	       italic
	       /* ::Qt 2::,QFont::AnyCharSet */);
}

QPen QpViewer::VTextStyle::getPen() {
  return QPen(textColor);
}

QpViewer::VText::VText(QpText *qpto) : theText (qpto) {
  vTextProperties.addProperty("angle","");
  theTextStyle = new VTextStyle(this);
}
 
QpViewer::VText::~VText() {
  delete theTextStyle;
}

void QpViewer::VText::setProperty (QPSTD::string name, QPSTD::string value) {
  QPSTD::transform(name.begin(),name.end(),name.begin(),tolower);
  if (vTextProperties.find(name,value) == StringPropertySet::FOUND) {
    if (name == "angle" ) {
      int angle;
      QPSTD::istringstream is(value.c_str());
      is >> angle;
      theText->getTextObject()->setAngle(angle);
    }
  } else
    QPSTD::cerr << "Unknown property: " << name <<QPSTD::endl;
}

void QpViewer::VText::listOptions(QPSTD::ostream &os) {
  QpViewer::listOptions("Text",vTextProperties, os);
} 

ITextStyle* QpViewer::VText::getStyle() {
  return theTextStyle;
} 

void QpViewer::VText::update(QpViewer::IStyleOwner::styleId sti ) {
  if (sti == QpViewer::IStyleOwner::TextStyle) {
    theText->setQpFont(theTextStyle->getFont());
    theText->setQpPen(theTextStyle->getPen());
  }
} 

QpViewer::VAxis::VAxis() : axisTitle("") , axisLabel("") {
  vAxisProperties.addProperty("title","");
  vAxisProperties.addProperty("label","");
}
 
QpViewer::VAxis::~VAxis() {
}

const QPSTD::string QpViewer::VAxis::getLabel() {
  return axisLabel;
}

const QPSTD::string QpViewer::VAxis::getTitle() {
  return axisTitle;
}
void QpViewer::VAxis::setProperty (QPSTD::string name, QPSTD::string value) {
  QPSTD::transform(name.begin(),name.end(),name.begin(),tolower);
  if (vAxisProperties.find(name,value) == StringPropertySet::FOUND) {
    if (name == "title")  axisTitle = value;
    if (name == "label")  axisLabel = value;
  } else
    QPSTD::cerr << "Unknown property: " << name <<QPSTD::endl;
}

void QpViewer::VAxis::listOptions(QPSTD::ostream &os) {
  QpViewer::listOptions("Axis",vAxisProperties, os);
}

