#ifndef _QPVIEWEREXTH_
#define _QPVIEWEREXTH_

#if defined __hpux || (defined __sun && __SUNPRO_CC < 0x500)
#define QPSTD
#else
#define QPSTD std
#endif

namespace QpViewer {

/** Class to implement notification change for style management.
 */

class IStyleOwner {
public:
  enum styleId {TextStyle,DrawStyle,AllStyles};
  virtual void update(styleId) = 0;
};
    
/** Class to implement a draw style object for Qplotter.
 */

class VDrawStyle : public IDrawStyle {
public:
  /// Constructor 
  VDrawStyle(QpViewer::IStyleOwner *)  ;
  /// Destructor
  virtual ~VDrawStyle();
  /// Change style property
  virtual void setProperty (QPSTD::string name, QPSTD::string value);
  /// List supported style properties
  virtual void listOptions(QPSTD::ostream &os = QPSTD::cout); 
  /// Return corresponding brush
  QBrush getBrush();
  /// Return corresponding pen
  QPen getPen();
private:
  QColor lineColor;
  QColor fillColor;
  int lineWidth;
  Qt::PenStyle lineShape;
  Qt::BrushStyle bStyle;
  StringPropertySet vDrawStyleProperties;
  IStyleOwner *theStyleOwner;
};  

/** Class to implement a text style object for Qplotter.
 */

class VTextStyle : public ITextStyle {
public:
  /// Constructor 
  VTextStyle(QpViewer::IStyleOwner *)  ;
  /// Destructor
  virtual ~VTextStyle();
  /// Change style property
  virtual void setProperty (QPSTD::string name, QPSTD::string value);
  /// List supported style properties
  virtual void listOptions(QPSTD::ostream &os = QPSTD::cout); 
  /// Return corresponding font
  QFont getFont();
  /// Return corresponding pen
  QPen getPen();
private:
  QPSTD::string fontName;
  int fontSize;
  QColor textColor;
  QFont::Weight textWeight;
  bool italic;
  StringPropertySet vTextStyleProperties;
  IStyleOwner *theStyleOwner;
};  

/** Class to implement a generic axis object for Qplotter.
 */

class VAxis : public ViewerAxis {
public:
  /// Constructor 
  VAxis()  ;
  /// Destructor
  virtual ~VAxis();
  /// Change axis property
  virtual void setProperty (QPSTD::string name, QPSTD::string value);
  /// List supported axis properties
  virtual void listOptions(QPSTD::ostream &os = QPSTD::cout); 
  const QPSTD::string getTitle();
  const QPSTD::string getLabel();
private:
  QPSTD::string axisTitle;
  QPSTD::string axisLabel;
  StringPropertySet vAxisProperties;
};  

/** Class to implement a generic text object for Qplotter.
 */

class VText : public IText , IStyleOwner {
public:
  /// Constructor 
  VText(QpText* qpto)  ;
  /// Destructor
  virtual ~VText();
  /// Change text property
  virtual void setProperty (QPSTD::string name, QPSTD::string value);
  /// List supported text properties
  virtual void listOptions(QPSTD::ostream &os = QPSTD::cout); 
  /// Update the style on property change (upon notification)
  void update(IStyleOwner::styleId) ;
  /// Get text style
  ITextStyle *getStyle();
private:
  StringPropertySet vTextProperties;
  QpText *theText;
  VTextStyle *theTextStyle;
};  

}
#endif
