#include <stdio.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qtooltip.h> 
#include <qfont.h> 
#include "Qplotter/viewer/viewFrame.h"
#include "Qplotter/qp/fileprint.xpm"
#include "Qplotter/qp/zoom.xpm"
#include "Qplotter/qp/unzoom.xpm"
#include "Qplotter/qp/pointer.xpm"

#define VERBOSE cout << __FILE__ << ":" << __LINE__ << endl ;

const int defaultTimerInterval=500;

/**
  A small class implementing the locator for this frame.
  Whenever the mouse moves, the labels in the frame are updated.

  Todo:
  (--) Put the coordinates inside a little frame box around the page (Like WMaker)
*/

class SmallLocator : public QpEvObserver {
public:
  SmallLocator(ViewFrame *_parent) : 
    QpEvObserver((QpListener::EventType)(QpListener::Move)) {
      mParent = _parent;
  }
  /// On notification update the labels in the frame
  virtual void update();
  /// Termination
  virtual void end();
private:
  // Empty copy ctor
  SmallLocator(const SmallLocator& rhs) :  QpEvObserver(QpListener::Move) {}
  ViewFrame *mParent;
};  

void SmallLocator::update() {
  if (listener) {
    if (listener->getPickedNode()) {
      if (listener->getPickedNode()->getZone()) {
	if (!listener->getPickedNode()->getZone()->isInvalid())
	  {
	    mParent->setXz(listener->getXz());
	    mParent->setYz(listener->getYz());
	  } else {
	    // Invalid coordinates
	  }
      } else {
	// Out of a zone
      }     
    }
    mParent->setXp(listener->getXp());
    mParent->setYp(listener->getYp());
  }      
}

void SmallLocator::end() {
  listener->removeObserver(this);
}

ViewFrame::ViewFrame(QApplication& app, QWidget *p, const char *name) :   QFrame(p,name), 
  theApp(app) , drawTimer(0), locator(0)
{
  wFrame = 600;         // in pixels
  hFrame = 650;
  wPanel = 600;         // in pixels
  hPanel = 600;
  wPage = 210.0;     // in page coordinates
  hPage = 210.0;
  coordWindow = 0;  
#ifdef NO_THREADED_PLOTTER
  setGeometry(1,1,wFrame,hFrame);
  page = new QpPage(this,QRect(0,0,wPanel,hPanel),wPage,hPage);
  page->createZones(1,1); 
  drawTimer = new QTimer( &app ,"drawTimer");
  //  QObject::connect(drawTimer , SIGNAL(timeout()),&app, SLOT(quit()) );
  waitTime = defaultTimerInterval;
#else
  // The locking is already done, so is it really needed ?
  app.lock();
  setGeometry(1,1,wFrame,hFrame);
  //  grid = new QGridLayout( this, 16, 4, 10 );
  plotLayout = new QHBoxLayout(this, 4);
  leftToolbar = new QVBoxLayout(plotLayout, 5);
  leftToolbar->setResizeMode(QLayout::Fixed);
  leftToolbar->setMargin(2); // Seg Fault ?
  leftToolbar->setSpacing(2);
  leftToolbar->setAlignment(Qt::AlignTop);

  addButtonToolbar (selectButton, QPixmap(pointer_xpm), leftToolbar);
  addButtonToolbar (zoomButton, QPixmap(zoom_xpm), leftToolbar);
  addButtonToolbar (unzoomButton, QPixmap(unzoom_xpm), leftToolbar);
  leftToolbar->addSpacing(8);  
  addButtonToolbar (printButton, QPixmap(fileprint), leftToolbar);

  QToolTip::add( selectButton, "Locate" );
  QToolTip::add( zoomButton, "Zoom in a zone (Right zoom/Left unzoom)" );
  QToolTip::add( unzoomButton, "Unzoom all (on a zone)" );
  QToolTip::add( printButton, "Print the page" );
  
  selectButton->setToggleButton(true);
  zoomButton->setToggleButton(true);

  selectButton->setFocusPolicy(QWidget::ClickFocus );
  zoomButton->setFocusPolicy(QWidget::ClickFocus );
  unzoomButton->setFocusPolicy(QWidget::ClickFocus );
  printButton->setFocusPolicy(QWidget::ClickFocus );
  

  //  leftToolbar->addStretch(0);
  
  drawingLayout = new QVBoxLayout( plotLayout, 10 );  
  page = new QpPage(this,QRect(0,0,wPanel,hPanel),wPage,hPage);
  page->createZones(1,1);
  drawingLayout->addWidget(page,0,0);

  commandLine = new ViewLine(this);
  drawingLayout->addWidget(commandLine,0,0);
  commandLine->setFocusPolicy(QWidget::ClickFocus );
  commandLine->show();

  /*
    text()
    void QLineEdit::returnPressed () [signal] 
  */

  QObject::connect(commandLine, SIGNAL(returnPressed ()),
		   this, SLOT(command()));
  
  QObject::connect(printButton, SIGNAL(clicked()),
		   (QObject*)page, SLOT(slotPrint()));
  QObject::connect(selectButton, SIGNAL(toggled(bool)),
		   this, SLOT(SelectPressed(bool)));
  QObject::connect(this, SIGNAL(SelectSignal(bool)),
		   (QObject*)page, SLOT(slotSelect(bool)));
  QObject::connect(zoomButton, SIGNAL(toggled (bool)),
		   this, SLOT(ZoomPressed (bool)));
  QObject::connect(this, SIGNAL(PageZoom (bool)),
		   (QObject*)page, SLOT(slotZooming (bool)));
  QObject::connect(unzoomButton, SIGNAL(clicked()),
		   this, SLOT(unzoomPressed()));
  QObject::connect(this, SIGNAL(PageUnzoom()),
		   (QObject*)page, SLOT(slotUnZoom()));

  //  bottomBar = new QHBoxLayout(drawingLayout);
  //  bottomBar->addStretch(0);
  //  bottomBar->setAlignment(Qt::AlignLeft);
  
  //  quitButton = new QPushButton("Quit",this);  
  //  bottomBar->addWidget(quitButton);
  //  bottomGrid = new QGridLayout(bottomBar,3,5); **

  ///
  
  bottomGrid = new QVBoxLayout( plotLayout, 10 );  
  //  bottomGrid = new QGridLayout (plotLayout,3,5);

  bottomGrid->setAlignment(Qt::AlignLeft);
       

  //  QObject::connect(quitButton, SIGNAL(clicked()), this, SLOT(quitButtonPressed()));

  // Toolbar

  /*
    QpFrameLocator fl(0, "locator");
    fl.lRegister(page);
    page->setFinePicking(true);

    QLabel label(page,"Hello World");
    label.setFrameStyle( QFrame::Panel | QFrame::Raised );
    label.setLineWidth( 2 );
    label.show();
  */

  
  // Create labels for the locator

  /*
    xzl = new QLabel(QString("X (Zone): "), this, "xzl");
    yzl = new QLabel(QString("Y (Zone): "), this, "yzl");
    xpl = new QLabel(QString("X (Page): "), this, "xpl");
    ypl = new QLabel(QString("Y (Page): "), this, "ypl");
    xz = new QLabel(QString("-"), this, "xz");
    yz = new QLabel(QString("-"), this, "yz");
    xp = new QLabel(QString("-"), this, "xp");
    yp = new QLabel(QString("-"), this, "yp");
  */
  
  
  QFont labelFont = QFont ("Courier", 12);
  labelFont.setFixedPitch(true);

  /*
    xz->setFont(labelFont);  
    yz->setFont(labelFont);  
    xp->setFont(labelFont);  
    yp->setFont(labelFont);  

   
    xzl->setAlignment(Qt::AlignLeft);
    xzl->setMaximumHeight(24);  
    yzl->setAlignment(Qt::AlignLeft);
    yzl->setMaximumHeight(24);  
    xpl->setAlignment(Qt::AlignLeft);
    xpl->setMaximumHeight(24);  
    ypl->setAlignment(Qt::AlignLeft);
    ypl->setMaximumHeight(24);  
    xz->setAlignment(Qt::AlignLeft);
    xz->setMaximumHeight(24);  
    yz->setAlignment(Qt::AlignLeft);
    yz->setMaximumHeight(24);  
    xp->setAlignment(Qt::AlignLeft);
    xp->setMaximumHeight(24);  
    yp->setAlignment(Qt::AlignLeft);
    yp->setMaximumHeight(24);  
  
    bottomGrid->addWidget(xzl,0,0);
    bottomGrid->addWidget(xz,0,1);
    bottomGrid->addWidget(yzl,1,0);
    bottomGrid->addWidget(yz,1,1);
    bottomGrid->addWidget(xpl,0,3);
    bottomGrid->addWidget(xp,0,4);
    bottomGrid->addWidget(ypl,1,3);
    bottomGrid->addWidget(yp,1,4);

    //  bottomGrid->addStretch(0);

    xzl->hide();	
    yzl->hide();
    xpl->hide();
    ypl->hide();
    xz->hide();
    yz->hide();
    xp->hide();
    yp->hide();
  
  */

  // Create the locator
  locator = new SmallLocator(this);
  locator->lRegister(page);
  // Activate Cross outline on the page
  //  page->getOutLine()->setType((QpOutLine::Type)(QpOutLine::Cross)); 

  app.unlock();
#endif
}
 
ViewFrame::~ViewFrame() {
  locator->end();
  delete locator;
  delete page;
}

void ViewFrame::paintEvent(QPaintEvent*) {} 

void ViewFrame::closeEvent( QCloseEvent *e )
{
  e->ignore();
}

void ViewFrame::refresh() { 
  page->redraw();
  theApp.flushX();
#ifdef NO_THREADED_PLOTTER
  if (waitTime >= 0)
    drawTimer->start( waitTime , TRUE );
#endif
}

void ViewFrame::setTimerInterval(int interval) {
  waitTime = interval;
}

void ViewFrame::setXz(double _xz)  { 
#ifndef NO_THREADED_PLOTTER
  theApp.lock();
  char xz_[16];
  QString xzString(xz_) ;
  sprintf(xz_,"%.2f",_xz);  
  //  xz->setText(QString(xz_).rightJustify(8, ' '));  
  theApp.unlock();
#endif
}

void ViewFrame::setYz(double _yz)  { 
#ifndef NO_THREADED_PLOTTER
  theApp.lock();
  char yz_[16];  
  sprintf(yz_,"%.2f",_yz);  
  //  yz->setText(QString(yz_).rightJustify(8, ' '));  
  theApp.unlock();
#endif
}

void ViewFrame::setXp(double _xp)  { 
#ifndef NO_THREADED_PLOTTER
  theApp.lock();
  char xp_[16];  
  sprintf(xp_,"%.2f",_xp);  
  //  xp->setText(QString(xp_).rightJustify(8, ' '));  
  theApp.unlock();
#endif
}

void ViewFrame::setYp(double _yp)  { 
#ifndef NO_THREADED_PLOTTER
  theApp.lock();
  char yp_[16];  
  sprintf(yp_,"%.2f",_yp);  
  //  yp->setText(QString(yp_).rightJustify(8, ' '));  
  theApp.unlock();
#endif
}

void ViewFrame::createCoordWindow() 
{
  theApp.lock();
  coordWindow = new QpFrameLocator(0, "Coordinate Window");
  coordWindow->lRegister(page);
  theApp.unlock();
}


void ViewFrame::addButtonToolbar (QPushButton* &button, QPixmap img, QVBoxLayout* &toolbar)
{
  button = new QPushButton("",this);
  button->setPixmap(img);
  button->setMaximumSize(QSize(24,24));

  toolbar->addWidget(button);  
}
