#ifndef VIEWFRAME_H
#define VIEWFRAME_H

#include <qframe.h>
#include <qpushbutton.h>
#include <qtimer.h>
#include <qlabel.h> 
#include <qlayout.h> 
#include "viewline.h"
#include <qlineedit.h> 
#include <qapplication.h> 
#include "Qplotter/qp/qppage.h"
#include "viewline.h"
#include <qlineedit.h>



/*
class QpFrameLocator;
class QpEvObserver;
*/

/** ViewFrame is the frame containing the QpPage. ViewFrame provides
  labels where X and Y zone coordinates are shown, by implementing his
  own observer.
*/

class ViewFrame : public QFrame
{
  Q_OBJECT
    public:
  /// Constructor
  ViewFrame (QApplication& app, QWidget *p=0, const char *name=0);
  /// Destructor
  ~ViewFrame() ;
  /// Retrieve the page
  QpPage  *getPage()  {return page;}
  /// Draw the plot
  void draw()    { page->draw(); }  
  /// Redraw the plot
  void refresh();
  /// Create coordinate window
  virtual void createCoordWindow();  

  /** Set the timer timeout. Control goes back to user application when
      the time expires. A negative interval means forever (timer is not started).
      In this case the user application mut provide a way to get out from Qt
      QApplication::exec() method.
  */
  void setTimerInterval(int interval);
  /// Change value of label X
  virtual void setXz(double _xz=0.0) ;
  /// Change value of label Y
  virtual void setYz(double _yz=0.0) ;
  /// Change value of label X (page)
  virtual void setXp(double _xz=0.0) ;
  /// Change value of label Y (page)
  virtual void setYp(double _yz=0.0) ;

  void addButtonToolbar (QPushButton* &button, QPixmap img, QVBoxLayout* &toolbar) ;
  
 private:
  /// The page
  QpPage  *page;
  QApplication& theApp;
  QpFrameLocator *coordWindow;  
  int wFrame ;         // in pixels
  int hFrame;
  int wPanel ;         // in pixels
  int hPanel;
  double wPage ;     // in page coordinates
  double hPage ;
  int waitTime;
  QTimer *drawTimer; 

  /*
    QLabel *xzl, *yzl;
    QLabel *xpl, *ypl;
    QLabel *xz , *yz;
    QLabel *xp , *yp;
  */
  ViewLine *commandLine ;
  
  QGridLayout* grid;
  
  QBoxLayout *plotLayout;
  QVBoxLayout *leftToolbar, *drawingLayout;
  QHBoxLayout *bottomBar ;
  //  QGridLayout *bottomGrid ;
  QVBoxLayout *bottomGrid ;
  QpEvObserver* locator;

  QPushButton *selectButton,*printButton, *quitButton,
    *zoomButton, *unzoomButton;
  
 protected:
  /// Events processing
  void paintEvent(QPaintEvent*);
  void closeEvent( QCloseEvent *e );

  
  private slots: 
    void selectPressed() {;} 
  void unzoomPressed()
    {
      theApp.lock();

      if (selectButton->isOn()) {
	selectButton->toggle();    
	selectButton->setOn(false);
      }
      
      if (zoomButton->isOn()) {
	zoomButton->toggle();    
	zoomButton->setOn(false);
      }
      
      emit PageUnzoom();    
      theApp.unlock();
    }
  void ZoomPressed(bool toggle) 
    {
      theApp.lock();
      /* To show coordinates while zooming
	 Then why not to always show it ?
      */
      
      if (toggle) {
	// Must be done in that order to show labels

	if (selectButton->isOn()) {
	  selectButton->toggle();    
	  selectButton->setOn(false);
	}

	if (unzoomButton->isOn()) {
	  unzoomButton->toggle();    
	  unzoomButton->setOn(false);
	}
	/*
	  xzl->show();	
	  yzl->show();
	  xpl->show();
	  ypl->show();
	  xz->show();
	  yz->show();
	  xp->show();
	  yp->show();
	*/
      } else {
	/*
	  xzl->hide();	
	  yzl->hide();
	  xpl->hide();
	  ypl->hide();
	  xz->hide();
	  yz->hide();
	  xp->hide();
	  yp->hide();
	*/
      }

      emit PageZoom(toggle);
      theApp.unlock();
    }

  void SelectPressed(bool toggle)
    {
      theApp.lock();

      if (toggle) {
	// Must be done in that order to show labels
	
	if (zoomButton->isOn()) {
	  zoomButton->toggle();    
	  zoomButton->setOn(false);
	}
      
	if (unzoomButton->isOn()) {
	  unzoomButton->toggle();    
	  unzoomButton->setOn(false);
	}

	/*
	  xzl->show();	
	  yzl->show();
	  xpl->show();
	  ypl->show();
	  xz->show();
	  yz->show();
	  xp->show();
	  yp->show();
	*/
      } else {
	/*
	  xzl->hide();	
	  yzl->hide();
	  xpl->hide();
	  ypl->hide();
	  xz->hide();
	  yz->hide();
	  xp->hide();
	  yp->hide();
	*/
      }

      emit SelectSignal(toggle); 
      theApp.unlock();
    }

  void command()
    {
      std::cout << "Command Line: " << commandLine->text() << std::endl ;
      emit commandSignal(commandLine->text());
      commandLine->clear();
    }
  
  
 signals:
  void PageZoom(bool);
  void PageUnzoom();
  void SelectSignal(bool);
  void commandSignal (QString);  

};


#endif






