 /**********************************************************************
  *                                                                    *
  * Copyright (c) 2002 Andreas Pfeiffer, CERN/IT                       *
  *                    <andreas.pfeiffer@cern.ch>                      *
  *                                                                    *
  * This library is free software; you can redistribute it and/or      *
  * modify it under the terms of the GNU Lesser General Public         *
  * License as published by the Free Software Foundation; either       *
  * version 2.1 of the License, or (at your option) any later version. *
  *                                                                    *
  * This library is distributed in the hope that it will be useful,    *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of     *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
  * Lesser General Public License for more details.                    *
  *                                                                    *
  * You should have received a copy of the GNU Lesser General Public   *
  * License along with this library (see file COPYING); if not, write  *
  * to the Free Software Foundation, Inc., 59 Temple Place, Suite      *
  * 330, Boston, MA 02111-1307 USA, or contact the author              *
  **********************************************************************/

// implementation file for class AIDAVectorFactory 
// 
// Created by: Andreas Pfeiffer
// 
// Last update: 
// 

#ifndef INCLUDED_AIDAVECTORFACTORY_H
#include "VectorOfPoints/AIDAVectorFactory.h"
#endif

#include "VectorOfPoints/Vector.h"

#include "Interfaces/IAnnotation.h"
#include "Interfaces/IAnnotationFactory.h"

#include "Interfaces/IVectorManager.h"

Anaphe::IVectorFactory * createIVectorFactory() { return new Anaphe::AIDAVectorFactory(); }

Anaphe::AIDAVectorFactory::AIDAVectorFactory() : myManager(0) { /* nop */ }

Anaphe::IScalar * Anaphe::AIDAVectorFactory::createScalar(double x, double xp, double xm)
{
  Anaphe::IScalar *s = new Anaphe::AIDAScalar(x, xp, xm);
  return s;
}

Anaphe::IPoint * Anaphe::AIDAVectorFactory::createPoint()
{
  Anaphe::IPoint *p = new Anaphe::AIDAPoint();
  return p;
}

Anaphe::IVector * Anaphe::AIDAVectorFactory::create()
{
  Anaphe::AIDAVector * v = new Anaphe::AIDAVector();
  v->setFactory(this);
  return v;
}

//-apAnaphe::IVector * Anaphe::AIDAVectorFactory::from1D(Anaphe::IHistogram1D *h)
//-ap{
//-ap  Anaphe::AIDAVector * v = new Anaphe::AIDAVector(h,0, this);
//-ap  return v;
//-ap}
//-ap
//-apAnaphe::IVector * Anaphe::AIDAVectorFactory::from2D(Anaphe::IHistogram2D *h)
//-ap{
//-ap  Anaphe::AIDAVector * v = new Anaphe::AIDAVector(0,h, this);
//-ap  return v;
//-ap}
//-ap
//-apAnaphe::IVector * Anaphe::AIDAVectorFactory::fromProfile(Anaphe::IProfileHistogram *h, const char * options)
//-ap{
//-ap  Anaphe::AIDAVector * v = new Anaphe::AIDAVector(h, options, this);
//-ap  return v;
//-ap}

void  Anaphe::AIDAVectorFactory::setManager(Anaphe::IVectorManager * vm)
{
  myManager = vm;
}

void  Anaphe::AIDAVectorFactory::unregister(Anaphe::IVector * v)
{
  if (myManager != 0) myManager->unregister(v);
}
