/**********************************************************************
  *                                                                    *
  * Copyright (c) 2002 Andreas Pfeiffer, CERN/IT                       *
  *                    <andreas.pfeiffer@cern.ch>                      *
  *                                                                    *
  * This library is free software; you can redistribute it and/or      *
  * modify it under the terms of the GNU Lesser General Public         *
  * License as published by the Free Software Foundation; either       *
  * version 2.1 of the License, or (at your option) any later version. *
  *                                                                    *
  * This library is distributed in the hope that it will be useful,    *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of     *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
  * Lesser General Public License for more details.                    *
  *                                                                    *
  * You should have received a copy of the GNU Lesser General Public   *
  * License along with this library (see file COPYING); if not, write  *
  * to the Free Software Foundation, Inc., 59 Temple Place, Suite      *
  * 330, Boston, MA 02111-1307 USA, or contact the author              *
  **********************************************************************/

// 
// header file for class AIDAVectorManager 
// Created by: Andreas Pfeiffer  at Wed Jan 19 20:35:12 2000
// 
// Last update: 
//              
// 
#ifndef INCLUDED_AIDAVECTORMANAGER_H
#define INCLUDED_AIDAVECTORMANAGER_H

#include <iostream>
#include <vector>

#include "Interfaces/IVector.h"
#include "Interfaces/IVectorFactory.h"
#include "Interfaces/IVectorManager.h"

namespace Anaphe {

  class AIDAVectorManager : virtual public IVectorManager { 

  public: 
    AIDAVectorManager(IVectorFactory *f = 0);
    virtual ~AIDAVectorManager() { clearAndDestroy(); }

  public:
    // let these be done by the compiler
    //  AIDAVectorManager(const AIDAVectorManager &); 
    //  AIDAVectorManager & operator = (const AIDAVectorManager &); 

  public: 		// public methods
    void setFactory(IVectorFactory * f) { vFact = f; }

  public:				// scalar- and point- creators
    virtual IScalar * createScalar(double x=0, double xp=0, double xm=0);
    virtual IPoint  * createPoint();

  public:				// vector-creators
    IVector * create();

  public: 		// public methods
    void enroll(IVector *v) { 
      vectorList.push_back( v ); 
    }

    void unregister(IVector * v);

    IVector * retrieve1D(int i);
    IVector * retrieve2D(int i);

    void delete1D(int i);
    void delete2D(int i);

    int  size() const;

    void clearAll();

    void list(std::ostream& os=std::cout);

  private:		// private methods
    void clearAndDestroy(); 

  private:		// private attributes
    std::vector<IVector *> vectorList;

    IVectorFactory *vFact;
  }; 

} //end namespace Anaphe

#endif /* INCLUDED_AIDAVECTORMANAGER_H */ 
