# Dependencies calculations for a package. It relies on g++ since I could not
# figure out the equivalent of -MM option (exclude system files included
# in angular brackets <...> ) with makedepend.
# Dependencies on absolute paths are removed (i.e. all /afs/...)

MAKEDEPEND=g++

# Output file containing dependencies

ifndef DEP_MAKEFILE
DEP_MAKEFILE=./depend.mk
endif

# awk filter for dependencies: drops dependencies starting with absolute path
# like those in /afs/cern.ch etc.
# prepend $(OS) to object file, i.e. Linux/qpobject.o:

AWK_DEP_FILTER='$$1 !~ /^\// { \
if ($$1 ~ /:$$/) {\
print ""; print osname $$0; \
}else print \
$$0}'

# Targets. firstdepend builds the file if it does not exist
#          depend can be used to recompute dependencies on demand
#
# The list of source files is specified in DEP_SRCS.
# They expect C_INCLUDES being defined as when compiling the code

ifndef DEP_SRCS
DEP_SRCS=*.cpp
endif


firstdepend: 
	@echo "Computing dependencies file" ; 
	@test -r $(DEP_MAKEFILE) || \
	$(MAKEDEPEND) -MM $(DEP_SRCS) $(C_INCLUDES) |\
	awk $(AWK_DEP_FILTER) osname=$(OS)/ 1>$(DEP_MAKEFILE)

depend: 
	@echo "Computing dependencies file on demand" ; \
	rm -rf $(DEP_MAKEFILE)
	$(MAKEDEPEND) -MM $(DEP_SRCS) $(C_INCLUDES) |\
	awk $(AWK_DEP_FILTER) osname=$(OS)/ 1>$(DEP_MAKEFILE)


ifdef VERBOSE
  MSG:=$(shell echo including "${ETCDIR}"/deps.mk  >& 2 )  
endif 
